package biz.capitalware.mqrc2;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import javax.swing.*;

import com.ibm.mq.data.AMQMessageNumber;
import com.ibm.mq.data.AMQMessageNumbers;
import com.ibm.mq.data.ReasonCode;
import com.ibm.mq.data.ReasonCodes;

import biz.capitalware.mqrc2.popups.AboutBox;

/**
 * This class will interpret an MQ reason code number (i.e. 2035), MQ reason code symbol 
 * (i.e. MQRC_OBJECT_IN_USE) or AMQ message number (i.e. AMQ4005) and display the relevant 
 * information from the WebSphere MQ Messages manual.
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.0.0
 * @license Apache 2 License
 */
public class MQRC2GUI extends JFrame implements ActionListener
{
   public static final String VERSION        = "1.0.1";
   public static final String RELEASE_DATE   = "2011/04/25";
   public static final String COPYRIGHT_DATE = "2011";
   public static final String WHO_AM_I_NAME  = "MQRC2-GUI";

   private static final int   minWidth = 600;
   private static final int   minHeight = 800;;

   private JTextField         reasonCode;
   private JButton            rcFindButton;
   
   private JComboBox          reasonCode_CB;
   private JButton            rcsFindButton;

   private JTextField         AMQMsgNum;
   private JButton            amqFindButton;

   private JTextArea          firstTA;

   private JLabel             secondTA_Lbl;
   private JTextArea          secondTA;

   private JLabel             thirdTA_Lbl;
   private JTextArea          thirdTA;

   private JLabel             fourthTA_Lbl;
   private JTextArea          fourthTA;
   
   private ReasonCodes        rcs = new ReasonCodes();
   private AMQMessageNumbers  amqMN = new AMQMessageNumbers();

   /**
    * The constructor
    */
   public MQRC2GUI()
   {
      super();

      try
      {
         UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      }
      catch (Exception e1)
      {
         e1.printStackTrace();
      }
      
      addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent e)
         {
            System.exit(0);
         }
      } );

      // set up menus
      menuBar();
      
      getContentPane().setLayout(new GridBagLayout());
      GridBagConstraints gbc = new GridBagConstraints();

      JPanel bgP = new JPanel(new GridBagLayout());
      gbc.insets = new Insets(0,0,0,0);
      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      add(bgP,  gbc, 0, 0, 1, 1);  // c , r, cw, rh

      // header label

      gbc.insets = new Insets(5,10, 0, 10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.NONE;
      gbc.weightx = 0.0;
      gbc.weighty = 0.0;
      JLabel lblF1 = new JLabel(WHO_AM_I_NAME);
      lblF1.setForeground(Color.blue);
      lblF1.setFont(new Font("Serif", Font.PLAIN, 24));
      addToPane(bgP, lblF1, gbc, 0, 0, 10, 1); // c , r, cw, rh

      // Text label

      gbc.insets = new Insets(5,10, 0, 10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;
      gbc.weightx = 0.0;
      gbc.weighty = 0.0;

      JLabel lblRC = new JLabel("Reason Code:");
      lblRC.setForeground(Color.black);
      addToPane(bgP, lblRC,  gbc, 0, 1, 2, 1); // c , r, cw, rh

      // Text field

      gbc.insets = new Insets(2,10, 0, 5);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;

      reasonCode = new JTextField(6);
      reasonCode.setText("");
      addToPane(bgP, reasonCode, gbc, 0, 2, 1, 1); // c , r, cw, rh

      // Find button

      gbc.insets = new Insets(2,0,0,10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;

      rcFindButton = new JButton("Find");
      rcFindButton.addActionListener(this);
      addToPane(bgP, rcFindButton, gbc, 1, 2, 1, 1); // c , r, cw, rh

      // text label

      JLabel conTypeLbl = new JLabel("Reason Code Symbol:");
      conTypeLbl.setForeground(Color.black);

      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;

      addToPane(bgP, conTypeLbl, gbc, 3, 1, 2, 1); // c , r, cw, rh

      // drop downs

      gbc.insets = new Insets(2,10,0,5);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;

      reasonCode_CB = new JComboBox();
      reasonCode_CB.setEditable(false);
      
      Object[] allRCs = rcs.getAllRCs();
      
      for (int i=0; i < allRCs.length; i++)
      {
         reasonCode_CB.addItem( allRCs[i] );
      }
      
      reasonCode_CB.addActionListener(this);
      addToPane(bgP, reasonCode_CB, gbc, 3, 2, 3, 1); // c , r, cw, rh

      // Find button

      gbc.insets = new Insets(2,0,0,10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;

      rcsFindButton = new JButton("Find");
      rcsFindButton.addActionListener(this);
      addToPane(bgP, rcsFindButton, gbc, 6, 2, 1, 1); // c , r, cw, rh

      // text label

      JLabel amqMsgNumLbl = new JLabel("AMQ Message Number:");
      amqMsgNumLbl.setForeground(Color.black);

      gbc.insets = new Insets(10,10,0,10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;

      addToPane(bgP, amqMsgNumLbl, gbc, 0, 5, 2, 1); // c , r, cw, rh

      // Text field

      gbc.insets = new Insets(2,10, 0, 5);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;

      AMQMsgNum = new JTextField(6);
      AMQMsgNum.setText("");
      addToPane(bgP, AMQMsgNum, gbc, 0, 6, 1, 1); // c , r, cw, rh
      
      // Find button

      gbc.insets = new Insets(2,0,0,10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;

      amqFindButton = new JButton("Find");
      amqFindButton.addActionListener(this);
      addToPane(bgP, amqFindButton, gbc, 1, 6, 1, 1); // c , r, cw, rh

      //////////////////////
      
      JPanel resultsPanel = new JPanel(new GridBagLayout());
      gbc.insets = new Insets(15,5,5,5);
      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      resultsPanel.setBorder(BorderFactory.createTitledBorder(" Results "));
      addToPane(bgP, resultsPanel,  gbc, 0, 10, 10, 1);  // c , r, cw, rh
      
      /////////////////////////

      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.insets = new Insets(2,5,0,5);  // Top, Left, Bottom, Right

      /*
       * First Text Area
       */
      firstTA = new JTextArea("");;
      firstTA.setLineWrap(true);
      firstTA.setFont(new Font("Arial", Font.PLAIN, 12));

      JScrollPane firstJS = new JScrollPane(firstTA,   //place the text area in a scroll pane
                                          JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                          JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      firstJS.setPreferredSize(new Dimension(600,30));  // w * h

      addToPane(resultsPanel, firstJS, gbc, 0, 21, 10, 2);   // c , r, cw, rh

      // Text label

      gbc.insets = new Insets(5, 5, 0, 10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;
      gbc.weightx = 0.0;
      gbc.weighty = 0.0;

      secondTA_Lbl = new JLabel("Explanation:");
      secondTA_Lbl.setForeground(Color.black);
      addToPane(resultsPanel, secondTA_Lbl,  gbc, 0, 30, 2, 1); // c , r, cw, rh

      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.insets = new Insets(2,5,0,5);  // Top, Left, Bottom, Right

      /*
       * Second Text Area
       */
      secondTA = new JTextArea("");;
      secondTA.setLineWrap(true);
      secondTA.setFont(new Font("Arial", Font.PLAIN, 12));

      JScrollPane secondJS = new JScrollPane(secondTA,   //place the text area in a scroll pane
                                          JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                          JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      secondJS.setPreferredSize(new Dimension(600,150));  // w * h

      addToPane(resultsPanel, secondJS, gbc, 0, 31, 10, 8);   // c , r, cw, rh

      // Text label

      gbc.insets = new Insets(5, 5, 0, 10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;
      gbc.weightx = 0.0;
      gbc.weighty = 0.0;

      thirdTA_Lbl = new JLabel("Completion Code:");
      thirdTA_Lbl.setForeground(Color.black);
      addToPane(resultsPanel, thirdTA_Lbl,  gbc, 0, 40, 2, 1); // c , r, cw, rh

      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.insets = new Insets(2,5,0,5);  // Top, Left, Bottom, Right

      /*
       * third Text Area
       */
      thirdTA = new JTextArea("");;
      thirdTA.setLineWrap(true);
      thirdTA.setFont(new Font("Arial", Font.PLAIN, 12));

      JScrollPane thirdJS = new JScrollPane(thirdTA,   //place the text area in a scroll pane
                                          JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                          JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      thirdJS.setPreferredSize(new Dimension(600,30));  // w * h

      addToPane(resultsPanel, thirdJS, gbc, 0, 41, 10, 3);   // c , r, cw, rh

      // Text label

      gbc.insets = new Insets(5, 5, 0, 10);  // Top, Left, Bottom, Right
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.NONE;
      gbc.weightx = 0.0;
      gbc.weighty = 0.0;

      fourthTA_Lbl = new JLabel("Programmer Response:");
      fourthTA_Lbl.setForeground(Color.black);
      addToPane(resultsPanel, fourthTA_Lbl,  gbc, 0, 50, 2, 1); // c , r, cw, rh

      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.insets = new Insets(2,5,5,5);  // Top, Left, Bottom, Right

      /*
       * fourth Text Area
       */
      fourthTA = new JTextArea("");;
      fourthTA.setLineWrap(true);
      fourthTA.setFont(new Font("Arial", Font.PLAIN, 12));

      JScrollPane fourthJS = new JScrollPane(fourthTA,   //place the text area in a scroll pane
                                          JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                          JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      fourthJS.setPreferredSize(new Dimension(600,150));  // w * h

      addToPane(resultsPanel, fourthJS, gbc, 0, 51, 10, 8);   // c , r, cw, rh

      //////////////
      KeyListener keyListener = (new KeyAdapter()
      {
         public void keyTyped(KeyEvent e)
         {
            if ( ( (e.getSource() == reasonCode) ) && (e.getKeyChar() == KeyEvent.VK_ENTER) )
            {
               handleRCFind();
            }
            if ( ( (e.getSource() == AMQMsgNum) ) && (e.getKeyChar() == KeyEvent.VK_ENTER) )
            {
               handleAMQFind();
            }
         }
      });
      
      reasonCode.addKeyListener(keyListener);
      AMQMsgNum.addKeyListener(keyListener);

      setSize(minWidth, minHeight); // w * h
      
      setTitle(WHO_AM_I_NAME + " v" + VERSION);
      
      setLocationRelativeTo(null);

      URL url = Anchor.class.getResource("gif/mqrc2gui.gif");
      ImageIcon icon = new ImageIcon(url);
      this.setIconImage(icon.getImage());
      
      pack();
      this.setVisible(true);
   }
   
   /**
    * Create a menu bar for the application.
    */
   private void menuBar( )
   {
      // create menu - File, Edit, View & Help
      JMenuBar menuBar = new JMenuBar();
      setJMenuBar(menuBar);
      
      JMenu fileMenu = new JMenu("File");
      menuBar.add(fileMenu);

      JMenuItem exitItem = new JMenuItem("Exit");
      exitItem.addActionListener(this);
      fileMenu.add(exitItem);

      JMenu HelpMenu = new JMenu("Help");
      menuBar.add(HelpMenu);

      JMenuItem aboutItem = new JMenuItem("About");
      aboutItem.addActionListener(this);
      HelpMenu.add(aboutItem);
   }
   
   /**
    * Method to add a component to the main window
    * @param c
    * @param gbc
    * @param x
    * @param y
    * @param w
    * @param h
    */
   private void add(Component c, GridBagConstraints gbc, int x, int y, int w, int h)
   {
      gbc.gridx = x;
      gbc.gridy = y;
      gbc.gridwidth = w;
      gbc.gridheight = h;
      getContentPane().add(c, gbc);
   }

   /**
    * Add a component to a panel.
    * @param p
    * @param c
    * @param gbc2
    * @param x
    * @param y
    * @param w
    * @param h
    */
   private void addToPane(JPanel p, Component c, GridBagConstraints gbc2, int x, int y, int w, int h)
   {
      gbc2.gridx = x;
      gbc2.gridy = y;
      gbc2.gridwidth = w;
      gbc2.gridheight = h;
      p.add(c, gbc2);
   }

   /**
    * actionPerformed is called by the JVM Gui handler
    * @param evt
    */
   public void actionPerformed(ActionEvent evt)
   {
      String  arg = evt.getActionCommand();
      Object  source = evt.getSource();

      if (source == rcFindButton)
      {
         handleRCFind();
      }
      else if ( (source == reasonCode_CB) || (source == rcsFindButton) )
      {
         if (reasonCode_CB.getSelectedIndex() != -1)
         {
            setLabelsForRC();
            String rcSymbol = ((String)(reasonCode_CB.getSelectedItem())).trim();
            int rcNum = rcs.getReasonCodeNumber(rcSymbol);
            reasonCode.setText(rcNum+"");
            setFieldsForRC(rcNum);
            
         }
         else
         {
            JOptionPane.showMessageDialog( this,
                                          "Please select an item from the dropdown.",
                                          "Warning",
                                          JOptionPane.WARNING_MESSAGE);
            return;
         }
      }
      else if (source == amqFindButton)
      {
         handleAMQFind();
      }
      else if (arg.equals("About"))
      {
         AboutBox about = new AboutBox( this,
                                        WHO_AM_I_NAME,
                                        VERSION,
                                        RELEASE_DATE,
                                        COPYRIGHT_DATE,
                                        "gif/mqrc2gui.gif");
         about.show();
      }
      else if (arg.equals("Exit"))
      {
         System.exit(0);
      }
      else
      {
         System.out.println( arg + " - NOT defined!!!" );
      }
   }

   /**
    * Handle the user action of user selecting reason code
    */
   private void handleRCFind()
   {
      int x = 0;
      
      setLabelsForRC();
      String num = reasonCode.getText();

      try
      {
         if (num.startsWith("0x") || num.startsWith("0X"))
            x = Integer.parseInt(num.substring(2),16);
         else
            x = Integer.parseInt(num);
         
         setFieldsForRC(x);
      }
      catch (NumberFormatException e)
      {
         JOptionPane.showMessageDialog( this,
                                        "Invalid number inputted on the Reason Code field.\n Use 2033 or 0x091E",
                                        "Warning",
                                        JOptionPane.WARNING_MESSAGE);
         return;
      }
   }

   /**
    * Handle the user action of user selecting AMQ message number
    */
   private void handleAMQFind()
   {
      setLabelsForAMQ();
      String num = AMQMsgNum.getText();

      try
      {
         if (num.startsWith("0x") || num.startsWith("0X"))
         {
            Integer.parseInt(num.substring(2),16);
            setFieldsForAMQ(num.substring(2));
         }
         else
         {
            Integer.parseInt(num);
            setFieldsForAMQ(num);
         }
      }
      catch (NumberFormatException e)
      {
         JOptionPane.showMessageDialog( this,
                                        "Invalid number inputted on the AMQ Message Number field.\n Use 4042 or 0x7B18",
                                        "Warning",
                                        JOptionPane.WARNING_MESSAGE);
         return;
      }
   }

   /**
    * Set the fields with reason codes related values.
    * @param rcNum
    */
   private void setFieldsForRC(int rcNum)
   {
      ReasonCode rc = rcs.getReasonCode(rcNum+"");
      
      if (rc != null)
      {
         firstTA.setText(rcNum + "  (0x'" + Integer.toHexString(rcNum).toUpperCase()  + "')  " + rc.getRCSymbol());
         secondTA.setText(rc.getExplanation());
         thirdTA.setText(rc.getCompCode());
         fourthTA.setText(rc.getProgResp());
         
         setCursorPosition();
         
         reasonCode_CB.setSelectedItem(rc.getRCSymbol());
      }
      else
         throw new NumberFormatException();
   }

   /**
    * Set the fields with AMQ related values.
    * @param amqNum
    */
   private void setFieldsForAMQ(String amqNum)
   {
      AMQMessageNumber amq = amqMN.getAMQMessageNumber(amqNum);
      
      if (amq != null)
      {
         firstTA.setText("AMQ" + amqNum + " " + amq.getHeader());
         secondTA.setText(amq.getExplanation());
         thirdTA.setText(amq.getSeverity());
         fourthTA.setText(amq.getResponse());
         
         setCursorPosition();
      }
      else
         throw new NumberFormatException();
   }

   /**
    * Set cursor position to the beginning of the field.
    */
   private void setCursorPosition()
   {
      firstTA.setCaretPosition(0);
      secondTA.setCaretPosition(0);
      thirdTA.setCaretPosition(0);
      fourthTA.setCaretPosition(0);
   }

   /**
    * Set the labels for Reason Code
    */
   private void setLabelsForRC()
   {
      thirdTA_Lbl.setText("Completion Code:");
      fourthTA_Lbl.setText("Programmer Response:");
   }

   /**
    * Set the labels for AMQ
    */
   private void setLabelsForAMQ()
   {
      thirdTA_Lbl.setText("Severity:");
      fourthTA_Lbl.setText("Response:");
   }
   
   /**
    * Mainline - entry point.
    * 
    * @param args - not used
    */
   public static void main(String[] args)
   {
      new MQRC2GUI();
   }
}
