package com.ibm.mq.data;

import java.util.Hashtable;

/**
 * This class handles all AMQ Message Numbers.  
 * 
 * <p>The information belongs to IBM. Capitalware takes no responsibility for the content.</p>
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.0.0
 * @license Apache 2 License
 */
public class AMQMessageNumbers
{
   private Hashtable amqMN_Ht = new Hashtable(2500);

   /**
    * The construct
    */
   public AMQMessageNumbers()
   {
      super();
      
      init4s();
      init5s();
      init6s();
      init7s();
      init8s();
      init9s();
      init7Xs();
   }

   /**
    * Initialize the AMQ 4000's message number.
    */
   public void init4s()
   {
      amqMN_Ht.put("4000",new AMQMessageNumber(4000,
                  "New object not created because the default object for the object type could not be found.",
                  "10 : Warning",
                  "The creation of an object requires a default template for each object type. The required default template for this object type could not be found.",
                  "Determine why the default object is not available, or create a new one. Then try the request again."));
      amqMN_Ht.put("4001",new AMQMessageNumber(4001,
                  "The queue manager specified is already shown in the console.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4002",new AMQMessageNumber(4002,
                  "Are you sure you want to delete the <insert_0> named <insert_1>?",
                  "10 : Warning",
                  "A confirmation is required before the specified object is deleted. The type of object and name are provided in the message.",
                  "Continue only if you want to permanently delete the object."));
      amqMN_Ht.put("4003",new AMQMessageNumber(4003,
                  "WebSphere MQ system objects are used internally by WebSphere MQ. You are advised not to delete them. Do you want to keep the WebSphere MQ system object?",
                  "0 : Information",
                  "A confirmation is required before an internal WebSphere MQ system object (for example SYSTEM.DEFAULT.LOCAL.QUEUE) is deleted.",
                  "Continue only if you want to permanently delete the system object."));
      amqMN_Ht.put("4004",new AMQMessageNumber(4004,
                  "Clear all messages from the queue?",
                  "10 : Warning",
                  "The removal of the messages from the queue is an irreversible action. If the command is allowed to proceed the action cannot be undone.",
                  "Continue only if you want to permanently delete the messages."));
      amqMN_Ht.put("4005",new AMQMessageNumber(4005,
                  "The object has been replaced or deleted. The properties could not be applied.",
                  "10 : Warning",
                  "During the process of updating the properties of the object, it was determined that the object has either been deleted or replaced by another instance. The properties have not been applied.",
                  ""));
      amqMN_Ht.put("4006",new AMQMessageNumber(4006,
                  "WebSphere MQ successfully sent data to the remote queue manager and received the data returned.",
                  "0 : Information",
                  "An open channel has been successfully verified by WebSphere MQ as the result of a user request.",
                  "Message for information only."));
      amqMN_Ht.put("4007",new AMQMessageNumber(4007,
                  "The message sequence number for the channel was reset.",
                  "0 : Information",
                  "A channel has had its sequence number successfully reset by WebSphere MQ as the result of a user request.",
                  "Message for information only."));
      amqMN_Ht.put("4008",new AMQMessageNumber(4008,
                  "The request to start the channel was accepted.",
                  "0 : Information",
                  "A channel has been started successfully by WebSphere MQ as the result of a user request.",
                  "Message for information only."));
      amqMN_Ht.put("4009",new AMQMessageNumber(4009,
                  "The request to stop the channel was accepted.",
                  "0 : Information",
                  "A channel has been stopped successfully by WebSphere MQ as the result of a user request.",
                  "Message for information only."));
      amqMN_Ht.put("4010",new AMQMessageNumber(4010,
                  "The 'in-doubt' state was resolved.",
                  "0 : Information",
                  "A channel has had its 'in-doubt' state resolved successfully by WebSphere MQ as the result of a user request.",
                  "Message for information only"));
      amqMN_Ht.put("4011",new AMQMessageNumber(4011,
                  "The queue has been cleared of messages.",
                  "0 : Information",
                  "The CLEAR command completed successfully and has removed all messages from the target queue. If the CLEAR was performed using the MQGET API command, uncommitted messages might still be on the queue.",
                  ""));
      amqMN_Ht.put("4012",new AMQMessageNumber(4012,
                  "The object was created successfully but it is not visible with the current settings for visible objects.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4014",new AMQMessageNumber(4014,
                  "The character <insert_0> is not valid.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4015",new AMQMessageNumber(4015,
                  "Supply a non-blank name.",
                  "0 : Information",
                  "",
                  "Enter a valid name."));
      amqMN_Ht.put("4016",new AMQMessageNumber(4016,
                  "The test message was put successfully.",
                  "0 : Information",
                  "The request to place a message on the target queue has completed successfully. The queue now contains the message.",
                  "Message for information only."));
      amqMN_Ht.put("4019",new AMQMessageNumber(4019,
                  "An object called <insert_0> already exists. Do you want to replace the definition of the existing object?",
                  "0 : Information",
                  "",
                  "Confirm that you want to replace the definition."));
      amqMN_Ht.put("4020",new AMQMessageNumber(4020,
                  "The changes you are making to the attributes of page \"<insert_0>\" will affect the operation of the queue manager or another program currently using the object. Do you want to force the change to the object's attributes?",
                  "10 : Warning",
                  "You are trying to change a object that cannot be changed because the object is in use, or the change will affect other programs or queue managers. Some changes can be forced anyway.",
                  "Select Yes to try forcing the changes, or No to abandon the change."));
      amqMN_Ht.put("4021",new AMQMessageNumber(4021,
                  "Failed to access one or more WebSphere MQ objects.",
                  "10 : Warning",
                  "The objects' icons have been marked to indicate the objects in error.",
                  ""));
      amqMN_Ht.put("4022",new AMQMessageNumber(4022,
                  "The name specified for the initiation queue is the same as the name of the queue itself.",
                  "0 : Information",
                  "",
                  "Specify a different name to that of the object being created or altered."));
      amqMN_Ht.put("4023",new AMQMessageNumber(4023,
                  "The queue manager <insert_0> does not exist on this computer.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4024",new AMQMessageNumber(4024,
                  "The object cannot be replaced.",
                  "0 : Information",
                  "The request to replace the object was unsuccessful.",
                  "To define this object, delete the existing object and try the operation again."));
      amqMN_Ht.put("4025",new AMQMessageNumber(4025,
                  "The changes made to the cluster attributes of the queue will take effect once they have propagated across the network.",
                  "0 : Information",
                  "",
                  "Refresh any views containing the cluster queues in the affected clusters to show the changes."));
      amqMN_Ht.put("4026",new AMQMessageNumber(4026,
                  "You have created a queue which is shared in one or more clusters. The queue will be available as a cluster queue once its definition has propagated across the network.",
                  "0 : Information",
                  "",
                  "Refresh any views containing the cluster queues in the affected clusters to show the cluster queue."));
      amqMN_Ht.put("4027",new AMQMessageNumber(4027,
                  "An error occurred connecting to the queue manager. Are you sure you want to show this queue manager in the folder anyway?",
                  "10 : Warning",
                  "A connection could not be made to the specified remote queue manager.",
                  "Ensure that the named queue manager is running on the host and port specified, and has a channel corresponding to the specified name. Ensure that you have the authority to connect to the remote queue manager, and ensure that the network is running. Select Yes if you believe that the problem can be resolved later. Select No if you want to correct the problem now and try again."));
      amqMN_Ht.put("4028",new AMQMessageNumber(4028,
                  "Platform not supported. This queue manager cannot be administered by the WebSphere MQ Explorer because it is running on an unsupported platform. The value <insert_0> for the Platform attribute of the queue manager is not supported by the WebSphere MQ Explorer.",
                  "20 : Error",
                  "",
                  ""));
      amqMN_Ht.put("4029",new AMQMessageNumber(4029,
                  "Command level too low. This queue manager cannot be administered by the WebSphere MQ Explorer because the command level of the queue manager is less than <insert_0>.",
                  "20 : Error",
                  "",
                  "If you want to administer this queue manager, you must upgrade it to a newer version of WebSphere MQ."));
      amqMN_Ht.put("4030",new AMQMessageNumber(4030,
                  "Queue manager cannot be administered because codepage conversion table not found.",
                  "20 : Error",
                  "This queue manager cannot be administered by the WebSphere MQ Explorer because a codepage conversion table was not found.",
                  "Install a codepage conversion table from CCSID <insert_0> to CCSID <insert_1> on the computer on which the WebSphere MQ Explorer is running."));
      amqMN_Ht.put("4031",new AMQMessageNumber(4031,
                  "Queue manager cannot be administered because CCSID not found.",
                  "20 : Error",
                  "This queue manager cannot be administered by the WebSphere MQ Explorer because CCSID <insert_0> cannot be found in the CCSID table. The WebSphere MQ Explorer cannot convert character data to or from the unrecognized CCSID.",
                  ""));
      amqMN_Ht.put("4032",new AMQMessageNumber(4032,
                  "Command server not responding within timeout period.",
                  "10 : Warning",
                  "",
                  "Ensure that the command server is running and that the queue called 'SYSTEM.ADMIN.COMMAND.QUEUE' is configured to enable programs to get messages from it."));
      amqMN_Ht.put("4033",new AMQMessageNumber(4033,
                  "Cannot get messages from the queue.",
                  "0 : Information",
                  "A reason code returned when the object was opened for input indicated that the queue is disabled for MQGET request.",
                  "To get messages from this queue, enable it for GET requests."));
      amqMN_Ht.put("4034",new AMQMessageNumber(4034,
                  "Message too long. You tried to put a message on a queue that was bigger than the maximum allowed for the queue or queue manager.",
                  "10 : Warning",
                  "The request to put a message on a queue returned a reason code indicating that the data length of the message exceeds the maximum allowed in the definition of the queue.",
                  "Either change the MAXMSGL attribute of the queue so that it is equal to or greater than the length of the message, or reduce the length of the message being put on the queue."));
      amqMN_Ht.put("4035",new AMQMessageNumber(4035,
                  "No message available. The response message did not arrive within a reasonable amount of time.",
                  "0 : Information",
                  "The request to get a message from a queue returned a reason code indicating that there are currently no messages on the queue that meet the selection criteria specified on the GET request.",
                  ""));
      amqMN_Ht.put("4036",new AMQMessageNumber(4036,
                  "Access not permitted. You are not authorized to perform this operation.",
                  "10 : Warning",
                  "The queue manager's security mechanism has indicated that the userid associated with this request is not authorized to access the object.",
                  ""));
      amqMN_Ht.put("4037",new AMQMessageNumber(4037,
                  "Object definition changed since it was opened.",
                  "0 : Information",
                  "Object definitions that affect this object have been changed since the Hobj handle used on this call was returned by the MQOPEN call.",
                  "Issue an MQCLOSE call to return the handle to the system. It is then usually sufficient to reopen the object and try the operation again."));
      amqMN_Ht.put("4038",new AMQMessageNumber(4038,
                  "Object damaged.",
                  "10 : Warning",
                  "The object is damaged and cannot be accessed.",
                  "The object should be deleted. Alternatively, it might be possible to recover it from a media image or backup."));
      amqMN_Ht.put("4039",new AMQMessageNumber(4039,
                  "Object in use. The object is already opened by another application.",
                  "10 : Warning",
                  "An MQOPEN call was issued, but the object in question has already been opened by this or another application with options that conflict with those specified in the Options parameter. This arises if the request is for shared input, but the object is already open for exclusive input. It also arises if the request is for exclusive input, but the object is already open for input (of any sort).",
                  "To change the attributes of an object, specify the Force option as 'Yes' to apply the changes. If you do this, any applications using the object must close and reopen the object to proceed."));
      amqMN_Ht.put("4040",new AMQMessageNumber(4040,
                  "Cannot put messages on this queue.",
                  "0 : Information",
                  "MQPUT and MQPUT1 calls are currently inhibited for the queue, or for the queue to which this queue resolves.",
                  ""));
      amqMN_Ht.put("4042",new AMQMessageNumber(4042,
                  "Queue full. The queue contains the maximum number of messages.",
                  "10 : Warning",
                  "On an MQPUT or MQPUT1 call, the call failed because the queue is full; that is, it already contains the maximum number of messages possible.",
                  ""));
      amqMN_Ht.put("4043",new AMQMessageNumber(4043,
                  "Queue manager not available for connection.",
                  "10 : Warning",
                  "",
                  "Ensure that the queue manager is running. If the queue manager is running on another computer, ensure it is configured to accept remote connections."));
      amqMN_Ht.put("4044",new AMQMessageNumber(4044,
                  "Queue manager stopping.",
                  "0 : Information",
                  "An MQI call was issued, but the call failed because the queue manager is shutting down. If the call was an MQGET call with the MQGMO_WAIT option, the wait has been canceled.",
                  "You cannot issue any more MQI calls."));
      amqMN_Ht.put("4045",new AMQMessageNumber(4045,
                  "Queue not empty. The queue contains one or more messages or uncommitted PUT or GET requests.",
                  "0 : Information",
                  "An operation that requires the queue to be empty has failed because the queue either contains messages or has uncommitted PUT or GET requests outstanding.",
                  ""));
      amqMN_Ht.put("4046",new AMQMessageNumber(4046,
                  "Insufficient system resources available.",
                  "20 : Error",
                  "",
                  ""));
      amqMN_Ht.put("4047",new AMQMessageNumber(4047,
                  "Insufficient storage available.",
                  "20 : Error",
                  "",
                  ""));
      amqMN_Ht.put("4048",new AMQMessageNumber(4048,
                  "The request received an unexpected reason code from an underlying API or command request. The reason code was <insert_0>",
                  "20 : Error",
                  "While executing the requested operation, an unexpected return code was received. This has resulted in the operation not completing as expected.",
                  "Use the appropriate documentation to determine why the reason code was returned."));
      amqMN_Ht.put("4049",new AMQMessageNumber(4049,
                  "Unknown object name.",
                  "10 : Warning",
                  "A command or API request was issued, but the object cannot be found.",
                  ""));
      amqMN_Ht.put("4050",new AMQMessageNumber(4050,
                  "Allocation failed. An attempt to allocate a conversation to a remote system failed.",
                  "10 : Warning",
                  "The error might be due to an incorrect entry in the channel definition or it might be that the listening program on the remote system was not running.",
                  ""));
      amqMN_Ht.put("4051",new AMQMessageNumber(4051,
                  "Bind failed. The bind to a remote system during session negotiation failed.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4052",new AMQMessageNumber(4052,
                  "Coded character-set ID error. Cannot convert a command message to the CCSID of the target queue manager.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4053",new AMQMessageNumber(4053,
                  "Channel in doubt. Operation not completed.",
                  "10 : Warning",
                  "The operation could not complete because the channel was in doubt.",
                  ""));
      amqMN_Ht.put("4054",new AMQMessageNumber(4054,
                  "Channel in use.",
                  "10 : Warning",
                  "An attempt was made to perform an operation on a channel, but the channel is currently active.",
                  ""));
      amqMN_Ht.put("4055",new AMQMessageNumber(4055,
                  "Channel status not found.",
                  "10 : Warning",
                  "No channel status is available for this channel. This might indicate that the channel has not been used.",
                  ""));
      amqMN_Ht.put("4056",new AMQMessageNumber(4056,
                  "Command failed.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4057",new AMQMessageNumber(4057,
                  "Configuration error in the channel definition or communication subsystem.",
                  "10 : Warning",
                  "Allocation of a conversation is not possible.",
                  ""));
      amqMN_Ht.put("4058",new AMQMessageNumber(4058,
                  "Connection closed.",
                  "10 : Warning",
                  "The connection to a remote system has unexpectedly broken while receiving data.",
                  ""));
      amqMN_Ht.put("4059",new AMQMessageNumber(4059,
                  "Could not establish a connection to the queue manager.",
                  "10 : Warning",
                  "The attempt to connect to the queue manager failed. This could be because the queue manager is incorrectly configured to allow a connection from this system, or the connection has been broken.",
                  "Try the operation again. If the error persists, examine the problem determination information to see if any information has been recorded."));
      amqMN_Ht.put("4060",new AMQMessageNumber(4060,
                  "Dynamic queue scope error.",
                  "10 : Warning",
                  "The Scope attribute of the queue was set to MQSCO_CELL but this is not allowed for a dynamic queue.",
                  ""));
      amqMN_Ht.put("4061",new AMQMessageNumber(4061,
                  "Remote system not available. Could not allocate a conversation to a remote system.",
                  "10 : Warning",
                  "",
                  "The error might be transitory; try again later."));
      amqMN_Ht.put("4062",new AMQMessageNumber(4062,
                  "An MQINQ call failed when the queue manager inquired about a WebSphere MQ object.",
                  "10 : Warning",
                  "",
                  "Check the queue manager's error log for more information about the error."));
      amqMN_Ht.put("4063",new AMQMessageNumber(4063,
                  "An MQOPEN call failed when the queue manager tried to open a WebSphere MQ object.",
                  "10 : Warning",
                  "",
                  "Check the queue manager's error log for more information about the error."));
      amqMN_Ht.put("4064",new AMQMessageNumber(4064,
                  "An MQSET call failed when the queue manager tried to set the values of the attributes of a WebSphere MQ object.",
                  "10 : Warning",
                  "",
                  "Check the queue manager's error log for more information about the error."));
      amqMN_Ht.put("4065",new AMQMessageNumber(4065,
                  "Message sequence number error.",
                  "10 : Warning",
                  "The message sequence number parameter was not valid.",
                  ""));
      amqMN_Ht.put("4066",new AMQMessageNumber(4066,
                  "Message truncated because it is larger than the command server's maximum valid message size.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4067",new AMQMessageNumber(4067,
                  "Communications manager not available.",
                  "20 : Error",
                  "The communications subsystem is not available.",
                  ""));
      amqMN_Ht.put("4068",new AMQMessageNumber(4068,
                  "Queue is not a transmission queue.",
                  "10 : Warning",
                  "The queue specified in the channel definition was not a transmission queue.",
                  ""));
      amqMN_Ht.put("4069",new AMQMessageNumber(4069,
                  "Object already exists.",
                  "10 : Warning",
                  "Could not create object because the object already existed.",
                  ""));
      amqMN_Ht.put("4070",new AMQMessageNumber(4070,
                  "Object is open.",
                  "10 : Warning",
                  "An attempt was made to delete, change or clear an object that is in use.",
                  "Wait until the object is not in use, then try again."));
      amqMN_Ht.put("4071",new AMQMessageNumber(4071,
                  "Object has wrong type. Could not replace a queue object of a different type.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4072",new AMQMessageNumber(4072,
                  "Queue already exists in cell.",
                  "10 : Warning",
                  "Cannot define a queue with cell scope or change the scope of an existing queue from queue-manager scope to cell scope, because a queue with that name already exists in the cell.",
                  ""));
      amqMN_Ht.put("4073",new AMQMessageNumber(4073,
                  "Ping error. You can only ping a sender or server channel. If the local channel is a receiver channel, ping from the remote queue manager.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4074",new AMQMessageNumber(4074,
                  "Receive failed, possibly due to a communications failure.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4075",new AMQMessageNumber(4075,
                  "Error while receiving data from a remote system, possibly due to a communications failure.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4076",new AMQMessageNumber(4076,
                  "Remote queue manager terminating.",
                  "10 : Warning",
                  "The channel stopped because the remote queue manager was terminating.",
                  ""));
      amqMN_Ht.put("4077",new AMQMessageNumber(4077,
                  "Remote queue manager not available.",
                  "10 : Warning",
                  "The channel could not be started because the remote queue manager was not available.",
                  "Ensure that the remote queue manager is started, and that it is configured to accept incoming communication requests."));
      amqMN_Ht.put("4078",new AMQMessageNumber(4078,
                  "Send failed. An error occurred while sending data to a remote system, possibly due to a communications failure.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4079",new AMQMessageNumber(4079,
                  "Channel closed by security exit.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4080",new AMQMessageNumber(4080,
                  "Remote channel not known.",
                  "10 : Warning",
                  "There is no definition of this channel on the remote system.",
                  ""));
      amqMN_Ht.put("4081",new AMQMessageNumber(4081,
                  "User exit not available.",
                  "10 : Warning",
                  "The channel was closed because the user exit specified does not exist.",
                  ""));
      amqMN_Ht.put("4082",new AMQMessageNumber(4082,
                  "Unexpected WebSphere MQ error (<insert_0>).",
                  "20 : Error",
                  "",
                  ""));
      amqMN_Ht.put("4083",new AMQMessageNumber(4083,
                  "Queue manager name not known.",
                  "10 : Warning",
                  "If the queue manager is remote, this might indicate that another queue manager is incorrectly using the same connection name. Queue managers using TCP/IP on the same computer must listen on different port numbers. This means that they will also have different connection names.",
                  ""));
      amqMN_Ht.put("4084",new AMQMessageNumber(4084,
                  "Cell directory is not available.",
                  "10 : Warning",
                  "The Scope attribute of the queue was set to MQSCO_CELL but no name service supporting a cell directory has been configured.",
                  "Configure a name service to support the cell directory."));
      amqMN_Ht.put("4085",new AMQMessageNumber(4085,
                  "No name supplied for transmission queue.",
                  "10 : Warning",
                  "",
                  "Supply a non-blank transmission queue name for this channel type."));
      amqMN_Ht.put("4086",new AMQMessageNumber(4086,
                  "No connection name supplied.",
                  "10 : Warning",
                  "",
                  "Supply a non-blank connection name for this channel type."));
      amqMN_Ht.put("4087",new AMQMessageNumber(4087,
                  "An error occurred while trying to use a cluster resource.",
                  "10 : Warning",
                  "",
                  "Check that the queues whose names start with 'SYSTEM.CLUSTER.' are not full and that messages are allowed to be put on them."));
      amqMN_Ht.put("4088",new AMQMessageNumber(4088,
                  "Cannot share transmission queue in cluster.",
                  "10 : Warning",
                  "The queue is a transmission queue and cannot be shared in a cluster.",
                  ""));
      amqMN_Ht.put("4089",new AMQMessageNumber(4089,
                  "PUT commands inhibited for system command queue called 'SYSTEM.ADMIN.COMMAND.QUEUE'.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4090",new AMQMessageNumber(4090,
                  "Are you sure you want to inhibit PUT and GET commands for the queue called 'SYSTEM.ADMIN.COMMAND.QUEUE'? If you do, you will no longer be able to administer the queue manager using the WebSphere MQ Explorer.",
                  "10 : Warning",
                  "WebSphere MQ Explorer uses the queue called 'SYSTEM.ADMIN.COMMAND.QUEUE' to administer the queue manager.",
                  "Continue only if you really want to inhibit PUT or GET commands for this queue and stop using the WebSphere MQ Explorer to administer the queue manager."));
      amqMN_Ht.put("4091",new AMQMessageNumber(4091,
                  "Cannot connect to remote queue manager.",
                  "10 : Warning",
                  "The remote queue manager is using an unsupported protocol for connections. The WebSphere MQ Explorer only supports connections to remote queue managers using the TCP/IP protocol.",
                  ""));
      amqMN_Ht.put("4092",new AMQMessageNumber(4092,
                  "The queue manager could not be removed from the cluster because its membership of the cluster is defined using namelist <insert_0>.",
                  "10 : Warning",
                  "",
                  "To remove the queue manager from the cluster, remove it from the namelist. Ensure that you do not inadvertently affect the definitions of other objects using the namelist."));
      amqMN_Ht.put("4093",new AMQMessageNumber(4093,
                  "The cluster specified is already shown in the console.",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("4094",new AMQMessageNumber(4094,
                  "An error occurred adding this cluster to the console. Are you sure you want to show this cluster in the console anyway?",
                  "10 : Warning",
                  "",
                  "Select Yes if you believe that the problem can be resolved later. Select No if you want to correct the problem now and try again."));
      amqMN_Ht.put("4095",new AMQMessageNumber(4095,
                  "Queue manager <insert_0> is not a repository queue manager for cluster <insert_1>.",
                  "0 : Information",
                  "To administer a cluster, the WebSphere MQ Explorer needs a connection to the repository queue manager.",
                  ""));
      amqMN_Ht.put("4096",new AMQMessageNumber(4096,
                  "Are you sure you want to clear the password for this channel?",
                  "0 : Information",
                  "",
                  "Check with the user before clearing the password from the channel. Continue only if you really want to clear the password."));
      amqMN_Ht.put("4097",new AMQMessageNumber(4097,
                  "Unmatched quotation mark.",
                  "10 : Warning",
                  "An unmatched quotation mark has been found in a list of attributes. Each value in the list can be enclosed in a pair of single or double quotation marks. (Only required for values which contain spaces, commas or quotation marks.)",
                  "Check that all opening and closing quotation marks are in pairs. (To include a quotation mark within an attribute, use two together with no space between.)"));
      amqMN_Ht.put("4098",new AMQMessageNumber(4098,
                  "Incorrect list format.",
                  "10 : Warning",
                  "The attribute can contain a list of values which must be separated by a space or a comma. Each value in the list can be enclosed in a pair of single or double quotation marks. (Only required for values which contain spaces, commas or quotation marks.)",
                  "Check that values are separated by a space or a comma, and that all opening and closing quotation marks are in pairs. (To include a quotation mark within an attribute, use two together with no space between.)"));
      amqMN_Ht.put("4099",new AMQMessageNumber(4099,
                  "Cannot communicate with one or more repository queue managers. Cluster <insert_0> is configured to use one or more repository queue managers which communicate using a protocol other than TCP/IP.",
                  "10 : Warning",
                  "The WebSphere MQ Explorer can only establish connections to remote queue managers using TCP/IP.",
                  "To complete removal of the queue manager from the cluster, issue the RESET CLUSTER ACTION(FORCEREMOVE) command from the repository queue manager."));
      amqMN_Ht.put("4103",new AMQMessageNumber(4103,
                  "An error occurred connecting to the queue manager. Are you sure you want to show this queue manager in the folder?",
                  "10 : Warning",
                  "A connection could not be made to the specified remote queue manager.",
                  "Ensure that the named queue manager is running on the machine specified in the selected channel definition table. Ensure that you have the authority to connect to the remote queue manager, and ensure that the network is up and running. Select Yes if you believe that the problem can be resolved later. Select No if you want to correct the problem now and try again."));
      amqMN_Ht.put("4104",new AMQMessageNumber(4104,
                  "The specified file <insert_0> does not contain a client definition table in the correct format.",
                  "10 : Warning",
                  "The given channel definition table is not in the correct format.",
                  "Specify a file in the correct format."));
      amqMN_Ht.put("4105",new AMQMessageNumber(4105,
                  "The remote queue manager has not been removed because it is still required by other plug-ins.",
                  "10 : Warning",
                  "Other plug-ins have responded to the attempted removal of this queue manager by indicating that they are still using it.",
                  "Ensure that the other plug-ins have finished using the queue manager before trying to delete it again."));
      amqMN_Ht.put("4117",new AMQMessageNumber(4117,
                  "This action cannot be undone. Are you sure you want to delete the WebSphere MQ queue manager <insert_0> from your system?",
                  "10 : Warning",
                  "A confirmation is required before the queue manager is deleted.",
                  "Continue only if you want to permanently delete the queue manager."));
      amqMN_Ht.put("4121",new AMQMessageNumber(4121,
                  "The MQGET request received an unexpected reason code of <insert_0>.",
                  "10 : Warning",
                  "An unexpected reason code was returned from a MQGET API request. Use the reason code to determine the underlying reason why the request failed.",
                  "The MQGET request was not successful. Some messages might not have been retrieved."));
      amqMN_Ht.put("4122",new AMQMessageNumber(4122,
                  "The MQPUT request received an unexpected reason code of <insert_0>.",
                  "10 : Warning",
                  "An unexpected reason code was returned from a MQPUT API request. Use the reason code to determine the underlying reason why the request failed.",
                  "MQPUT processing was unsuccessful. No message was placed on the queue."));
      amqMN_Ht.put("4123",new AMQMessageNumber(4123,
                  "The <insert_0> <insert_1> was deleted successfully.",
                  "0 : Information",
                  "The object of the specified type and name has been successfully deleted.",
                  "none."));
      amqMN_Ht.put("4124",new AMQMessageNumber(4124,
                  "The MQOPEN request received an unexpected reason code of <insert_0>.",
                  "10 : Warning",
                  "An unexpected reason code was returned from an MQOPEN API request. The queue has not been opened.",
                  "Use the reason code to determine the underlying reason for the failure."));
      amqMN_Ht.put("4125",new AMQMessageNumber(4125,
                  "Putting a test message on the queue received an unexpected reason code <insert_0>.",
                  "10 : Warning",
                  "One of the underlying API requests was unsuccessful. The test message was not placed on the queue.",
                  ""));
      amqMN_Ht.put("4126",new AMQMessageNumber(4126,
                  "The value of one of the properties specified is not valid. The request was not processed.",
                  "20 : Error",
                  "",
                  "Specify a different value."));
      amqMN_Ht.put("4127",new AMQMessageNumber(4127,
                  "WebSphere MQ failed to read queue manager information from disk because the file format is not valid. The request was not processed.",
                  "20 : Error",
                  "The format of the WebSphere MQ_Handles file is incorrect. This file has been backed up and removed, meaning that any remote queue manager definitions are lost. All local queue managers should be detected automatically and displayed in the WebSphere MQ Explorer.",
                  "Ensure that the Eclipse workspace has not been corrupted."));
      amqMN_Ht.put("4128",new AMQMessageNumber(4128,
                  "Could not start the iKeyMan program.",
                  "30 : Severe error",
                  "An error was encountered when trying to execute the iKeyMan program.",
                  "Try again, and if symptoms persist contact the System Administrator."));
      amqMN_Ht.put("4129",new AMQMessageNumber(4129,
                  "Could not query the user ID from Java.",
                  "10 : Warning",
                  "The Java API System.getProperty(\"user.id\") threw a SecurityException.",
                  "Configure your Java security environment using the 'policytool' to allow WebSphere MQ Explorer to query the 'user.id'."));
      amqMN_Ht.put("4130",new AMQMessageNumber(4130,
                  "A Browser Control could not be opened. Make sure Mozilla has been installed.",
                  "10 : Warning",
                  "The SWT Browser control depends on Mozilla being installed.",
                  "Ensure that the Mozilla browser is correctly installed."));
      amqMN_Ht.put("4131",new AMQMessageNumber(4131,
                  "A Browser Control could not be opened.",
                  "10 : Warning",
                  "The SWT Browser control depends on the system browser being installed.",
                  "Ensure that the system browser is correctly installed."));
      amqMN_Ht.put("4132",new AMQMessageNumber(4132,
                  "Are you sure you want to stop the <insert_0> named \"<insert_1>\"?",
                  "10 : Warning",
                  "A confirmation is required before the specified object is stopped. The type of object and name are provided in the message.",
                  "Continue only if you want to stop the object."));
      amqMN_Ht.put("4133",new AMQMessageNumber(4133,
                  "When a queue manager is removed, WebSphere MQ Explorer destroys the connection information for that queue manager.",
                  "10 : Warning",
                  "",
                  "Continue only if you want to remove the queue manager."));
      amqMN_Ht.put("4134",new AMQMessageNumber(4134,
                  "The default channel used by remote queue managers to administer this queue manager does not exist.",
                  "0 : Information",
                  "",
                  "Select Yes to create the channel."));
      amqMN_Ht.put("4135",new AMQMessageNumber(4135,
                  "The default channel used by remote queue managers to administer this queue manager is SYSTEM.ADMIN.SVRCONN.",
                  "0 : Information",
                  "",
                  "Select Yes to delete the channel."));
      amqMN_Ht.put("4136",new AMQMessageNumber(4136,
                  "Are you sure you want to clear all FFSTs and Trace from this machine? This operation cannot be undone.",
                  "10 : Warning",
                  "Deleting all FFSTs and Trace from this machine means that any historical error logs and trace will be lost.",
                  "Select Yes to clear FFSTs and Trace"));
      amqMN_Ht.put("4137",new AMQMessageNumber(4137,
                  "The default remote administration channel SYSTEM.ADMIN.SVRCONN has been deleted successfully.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4138",new AMQMessageNumber(4138,
                  "Are you sure you want to import new settings that will overwrite the current settings? This operation cannot be undone.",
                  "10 : Warning",
                  "Importing settings into the WebSphere MQ Explorer will overwrite the current settings.",
                  "Continue only if you want to overwrite the current settings."));
      amqMN_Ht.put("4139",new AMQMessageNumber(4139,
                  "The default remote administration channel SYSTEM.ADMIN.SVRCONN was created successfully.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4140",new AMQMessageNumber(4140,
                  "The custom CipherSpec is not valid.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4141",new AMQMessageNumber(4141,
                  "The Distinguished Names specification is not valid.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4142",new AMQMessageNumber(4142,
                  "The default remote administration channel SYSTEM.ADMIN.SVRCONN could not be created.",
                  "10 : Warning",
                  "A problem has occurred when issuing a command to the command server to create the channel.",
                  "Try again, and if symptoms persist contact the Systems Administrator."));
      amqMN_Ht.put("4143",new AMQMessageNumber(4143,
                  "The default remote administration channel SYSTEM.ADMIN.SVRCONN could not be created.",
                  "10 : Warning",
                  "A problem has occurred with the underlying data model, and the channel could not be created.",
                  "Try again, and if symptoms persist contact the Systems Administrator."));
      amqMN_Ht.put("4144",new AMQMessageNumber(4144,
                  "The default remote administration channel SYSTEM.ADMIN.SVRCONN could not be deleted.",
                  "10 : Warning",
                  "A problem has occurred issuing a command to the command server to delete the channel.",
                  "Ensure that the channel is not in use and try again. If symptoms persist, contact the Systems Administrator."));
      amqMN_Ht.put("4145",new AMQMessageNumber(4145,
                  "An error occurred connecting to the remote queue manager using the intermediate queue manager. Are you sure you want to show this queue manager in the folder anyway?",
                  "10 : Warning",
                  "A connection could not be made to the specified remote queue manager.",
                  "Ensure that the intermediate queue manager is available and that the named remote queue manager is running, and is accessible from the intermediate queue manager. Ensure that you have the authority to connect to the remote queue manager, and ensure that the network is up and running. Select Yes if you believe that the problem can be resolved later. Select No if you want to correct the problem now and try again."));
      amqMN_Ht.put("4146",new AMQMessageNumber(4146,
                  "Eclipse cannot create or read the workspace for WebSphere MQ Explorer.",
                  "40 : Stop Error",
                  "To load the WebSphere MQ Explorer, a valid workspace is required.",
                  "Ensure that you can write to the Eclipse workspace."));
      amqMN_Ht.put("4147",new AMQMessageNumber(4147,
                  "Eclipse cannot write to the workspace for WebSphere MQ Explorer in <insert_0>.",
                  "40 : Stop Error",
                  "To load the WebSphere MQ Explorer, write access to the workspace is required.",
                  "Ensure that you can write to the Eclipse workspace."));
      amqMN_Ht.put("4148",new AMQMessageNumber(4148,
                  "The object was created successfully.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4149",new AMQMessageNumber(4149,
                  "The request to start the listener was accepted.",
                  "0 : Information",
                  "A user request to start the listener was accepted by WebSphere MQ.",
                  "Message for information only."));
      amqMN_Ht.put("4150",new AMQMessageNumber(4150,
                  "The request to stop the listener was accepted.",
                  "0 : Information",
                  "A user request to stop the listener was accepted by WebSphere MQ.",
                  "Message for information only."));
      amqMN_Ht.put("4151",new AMQMessageNumber(4151,
                  "The request to start the service was accepted.",
                  "0 : Information",
                  "A user request to start the service was accepted by WebSphere MQ.",
                  "Message for information only."));
      amqMN_Ht.put("4152",new AMQMessageNumber(4152,
                  "The request to stop the service was accepted.",
                  "0 : Information",
                  "A user request to stop the service was accepted by WebSphere MQ.",
                  "Message for information only."));
      amqMN_Ht.put("4153",new AMQMessageNumber(4153,
                  "WebSphere MQ cannot stop the listener because it is not running.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4154",new AMQMessageNumber(4154,
                  "WebSphere MQ cannot start the service because no start command has been specified.",
                  "10 : Warning",
                  "",
                  "Ensure that the service has a start command specified."));
      amqMN_Ht.put("4155",new AMQMessageNumber(4155,
                  "WebSphere MQ cannot stop the service because no stop command has been specified.",
                  "10 : Warning",
                  "",
                  "Ensure that the service has a stop command specified."));
      amqMN_Ht.put("4156",new AMQMessageNumber(4156,
                  "WebSphere MQ cannot stop the service because the service is not running.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4157",new AMQMessageNumber(4157,
                  "WebSphere MQ cannot start the service because the services is already running.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4158",new AMQMessageNumber(4158,
                  "WebSphere MQ cannot start the listener because it is already running.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4159",new AMQMessageNumber(4159,
                  "WebSphere MQ cannot start the client connection channel because one or more of the properties are incorrectly specified.",
                  "10 : Warning",
                  "",
                  "Ensure that the client connection has the correct queue manager name and connection name before trying to start."));
      amqMN_Ht.put("4160",new AMQMessageNumber(4160,
                  "WebSphere MQ cannot process the request because the executable specified cannot be started.",
                  "10 : Warning",
                  "The requested was unsuccessful because the program which was defined to be run to complete the action could not be started. Reasons why the program could not be started are :- The program does not exist at the specified location. The WebSphere MQ user does not have sufficient access to execute the program. If StdOut or StdErr are defined for the program, the WebSphere MQ user does not have sufficient access to the locations specified.",
                  "Check the Queue Manager error logs for further details on the cause of the failure, correct the problem and try again."));
      amqMN_Ht.put("4161",new AMQMessageNumber(4161,
                  "The parameter specified is not valid.",
                  "20 : Error",
                  "The parameter specified when trying to create or alter an object is not valid.",
                  "Ensure that valid parameters are specified, then try again."));
      amqMN_Ht.put("4162",new AMQMessageNumber(4162,
                  "The password cannot be cleared.",
                  "0 : Information",
                  "",
                  "Try to clear the password again later."));
      amqMN_Ht.put("4163",new AMQMessageNumber(4163,
                  "The password cannot be changed.",
                  "10 : Warning",
                  "The attempt to change the password failed because of an error.",
                  "Try a different password"));
      amqMN_Ht.put("4164",new AMQMessageNumber(4164,
                  "The password was successfully changed.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4165",new AMQMessageNumber(4165,
                  "No password entered in the new password field. No change applied.",
                  "10 : Warning",
                  "You must enter a new password in both the new and confirm password fields.",
                  "Enter a new password in the new password field."));
      amqMN_Ht.put("4166",new AMQMessageNumber(4166,
                  "No password entered in the confirm new password field. No change applied.",
                  "10 : Warning",
                  "You must enter a new password in both the new and confirm password fields.",
                  "Re-enter the new password in the confirm new password field."));
      amqMN_Ht.put("4167",new AMQMessageNumber(4167,
                  "Passwords do not match. No change applied.",
                  "10 : Warning",
                  "You must enter the same new password in both the new and confirm password fields.",
                  "Ensure that the passwords in the new and confirm fields match."));
      amqMN_Ht.put("4168",new AMQMessageNumber(4168,
                  "WebSphere MQ failed to start listening for objects.",
                  "20 : Error",
                  "No objects will be displayed in the currently selected view.",
                  "Check the problem determination information, and ensure that WebSphere MQ and the queue manager in question are both running correctly."));
      amqMN_Ht.put("4169",new AMQMessageNumber(4169,
                  "WebSphere MQ failed to set the object filter.",
                  "20 : Error",
                  "The WebSphere MQ Explorer cannot listen for objects, so no objects will be displayed in the currently selected view.",
                  "Check the problem determination information, and ensure that WebSphere MQ and the queue manager in question are both running correctly."));
      amqMN_Ht.put("4170",new AMQMessageNumber(4170,
                  "The object name specified is not valid.",
                  "20 : Error",
                  "The object name specified when trying to create or alter an object is not valid.",
                  "Ensure that a valid object name is specified, then try again."));
      amqMN_Ht.put("4171",new AMQMessageNumber(4171,
                  "There was an error when communicating with the queue manager.",
                  "20 : Error",
                  "A request for information from the queue manager failed.",
                  "Try the operation again. If the error persists, examine the problem determination information to see if any details have been recorded."));
      amqMN_Ht.put("4172",new AMQMessageNumber(4172,
                  "There was an error when trying to set or retrieve information.",
                  "20 : Error",
                  "There was an error when trying to set or retrieve information from the queue manager. This might have happened because you specified incorrect or inconsistent attributes when trying create or update an object.",
                  "If this error occurred during object creation or modification, ensure that the attributes specified are correct for this type of object. If the error persists, examine the problem determination information to see if any details have been recorded."));
      amqMN_Ht.put("4173",new AMQMessageNumber(4173,
                  "WebSphere MQ cannot clear one or more Trace and FFST files.",
                  "10 : Warning",
                  "WebSphere MQ cannot clear some files, possibly because these files are currently in use, or the WebSphere MQ Explorer does not have the appropriate access permission.",
                  "Check that tracing is disabled, and that the WebSphere MQ Explorer has appropriate access permission to delete the Trace and FFST files."));
      amqMN_Ht.put("4174",new AMQMessageNumber(4174,
                  "FFSTs and Trace were cleared successfully.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4175",new AMQMessageNumber(4175,
                  "WebSphere MQ cannot process your request because the value specified is not valid.",
                  "20 : Error",
                  "Only certain combinations and values are valid for the object your are trying to alter or create. Please check the values and try again.",
                  "Specify a valid value and try again."));
      amqMN_Ht.put("4176",new AMQMessageNumber(4176,
                  "WebSphere MQ cannot process your request because the object name specified is not valid.",
                  "20 : Error",
                  "Only certain combinations and values are valid for the object your are trying to alter or create. You might also see this message if you have specified an invalid QSG disposition.",
                  "Check all values are valid for this type of object and try again. If you have altered the disposition of this object, check that the value is correct."));
      amqMN_Ht.put("4177",new AMQMessageNumber(4177,
                  "The WebSphere MQ Explorer cannot process your request because the connection to WebSphere MQ is quiescing.",
                  "20 : Error",
                  "The connection to WebSphere MQ is quiescing, so no new information can be queried.",
                  "Wait for the connection to end, then try reconnecting."));
      amqMN_Ht.put("4178",new AMQMessageNumber(4178,
                  "WebSphere MQ cannot process your request because there was a disposition conflict detected.",
                  "20 : Error",
                  "A disposition conflict was detected. Please ensure that all disposition related fields are correct for this type of object.",
                  "Ensure that all disposition related fields are correct for this type of object and try again."));
      amqMN_Ht.put("4179",new AMQMessageNumber(4179,
                  "WebSphere MQ cannot process your request because the string provided was of an incorrect length.",
                  "20 : Error",
                  "A string value has been modified or supplied that is too long when creating or modifying an object.",
                  "Check the values being supplied and try again."));
      amqMN_Ht.put("4180",new AMQMessageNumber(4180,
                  "WebSphere MQ cannot process your request because there was a parameter conflict.",
                  "20 : Error",
                  "When creating or modifying an object, the combination of parameters specified is not valid.",
                  "Check that the combination specified is valid for the object and try again."));
      amqMN_Ht.put("4181",new AMQMessageNumber(4181,
                  "WebSphere MQ is not responding. Do you want to continue waiting?",
                  "10 : Warning",
                  "WebSphere MQ does not appear to be responding. This could be because of a heavily loaded remote system, or a slow network connection. However there could have been a system failure. Choosing not to continue could leave the WebSphere MQ Explorer in an unknown state, so you should restart it.",
                  "If you choose not to continue waiting, restart the WebSphere MQ Explorer, if the problem persists check for problem determination information."));
      amqMN_Ht.put("4182",new AMQMessageNumber(4182,
                  "No objects were found.",
                  "10 : Warning",
                  "The query did not find any objects.",
                  "If you were expecting objects to be found, check the problem determination information, and ensure that WebSphere MQ and the queue manager in question are both running correctly."));
      amqMN_Ht.put("4183",new AMQMessageNumber(4183,
                  "Query failed because the queue manager is not in a queue-sharing group.",
                  "10 : Warning",
                  "WebSphere MQ issued a query that required the queue manager to be a member of a queue-sharing group.",
                  "Try the operation again, if the problem persists check the problem determination information for more details."));
      amqMN_Ht.put("4184",new AMQMessageNumber(4184,
                  "WebSphere MQ is unable to perform your request because the channel is not active.",
                  "20 : Error",
                  "",
                  ""));
      amqMN_Ht.put("4185",new AMQMessageNumber(4185,
                  "WebSphere MQ failed to import your settings.",
                  "20 : Error",
                  "",
                  "Try again. If the error persists, examine the problem determination information to see if any details have been recorded."));
      amqMN_Ht.put("4186",new AMQMessageNumber(4186,
                  "WebSphere MQ failed to export your settings.",
                  "20 : Error",
                  "",
                  "Try again. If the error persists, examine the problem determination information to see if any details have been recorded."));
      amqMN_Ht.put("4187",new AMQMessageNumber(4187,
                  "WebSphere MQ has succesfully imported your settings. (You must restart WebSphere MQ Explorer to apply the imported settings.)",
                  "0 : Information",
                  "",
                  "Restart WebSphere MQ explorer to apply the imported settings"));
      amqMN_Ht.put("4188",new AMQMessageNumber(4188,
                  "Are you sure you want to remove queue manager <insert_0> from cluster <insert_1>?",
                  "10 : Warning",
                  "A confirmation is required before the queue manager is removed from the cluster.",
                  "Continue only if you want to permanently remove the queue manger from the cluster."));
      amqMN_Ht.put("4189",new AMQMessageNumber(4189,
                  "The queue manager could not be suspended from the cluster. The operation failed with error <insert_0>.",
                  "20 : Error",
                  "The queue manager has not been removed from the cluster.",
                  "Try the operation again. If the error persists, examine the problem determination information to see if any information has been recorded."));
      amqMN_Ht.put("4190",new AMQMessageNumber(4190,
                  "An error occurred when clearing the queue manager's REPOS field. The operation failed with error <insert_0>.",
                  "20 : Error",
                  "The queue manager has only partially been removed from the cluster. The queue manager has been suspended from the cluster. The REPOS field of the queue manager and the CLUSTER fields of the associated cluster channels have not been cleared.",
                  "Try the operation again. If the error persists, examine the problem determination information to see if any information has been recorded."));
      amqMN_Ht.put("4191",new AMQMessageNumber(4191,
                  "An error occurred when clearing the CLUSTER field of channel <insert_0>. The operation failed with error <insert_1>.",
                  "20 : Error",
                  "The queue manager has only partially been removed from the cluster. The queue manager has been suspended from the cluster and the queue manager's REPOS field has been cleared. Some of the CLUSTER fields of other associated cluster channels might also have been cleared.",
                  "To completely remove the queue manager, ensure that all the CLUSTER fields of associated cluster channels are cleared."));
      amqMN_Ht.put("4192",new AMQMessageNumber(4192,
                  "The queue manager could not be removed from a cluster because channel <insert_0> is using cluster namelist <insert_1>.",
                  "10 : Warning",
                  "",
                  "Remove the cluster channel from the cluster namelist. Ensure that you do not inadvertently affect the definitions of other objects using the namelist. Then try removing the queue manager again."));
      amqMN_Ht.put("4193",new AMQMessageNumber(4193,
                  "The information supplied could not be correctly converted to the required code page.",
                  "20 : Error",
                  "All or part of the information entered required conversion to a different code page. One or more characters could not be converted to an equivalent character in the new code page.",
                  "Change the characters used, then try the operation again."));
      amqMN_Ht.put("4194",new AMQMessageNumber(4194,
                  "Request failed because the queue manager attempted to use a default transmission queue which is not valid.",
                  "20 : Error",
                  "An MQOPEN or MQPUT1 call specified a remote queue as the destination. The queue manager used the default transmission queue, as there is no queue defined with the same name as the destination queue manager, but the attempt failed because the default transmission queue is not a valid local queue.",
                  "Check that the queue manager's default transmission queue property (DefXmitQName) specifies a valid local queue."));
      amqMN_Ht.put("4195",new AMQMessageNumber(4195,
                  "WebSphere MQ Explorer is now in an unknown state and should be restarted. Do you want to restart WebSphere MQ Explorer?",
                  "10 : Warning",
                  "You have chosen not to wait for WebSphere MQ to respond to a request. WebSphere MQ Explorer is therefore in an unknown state and should be restarted.",
                  "Restart the WebSphere MQ Explorer and try the operation again. If the problem persists check for problem determination information."));
      amqMN_Ht.put("4196",new AMQMessageNumber(4196,
                  "The command or operation is not valid against the type of object or queue specified",
                  "20 : Error",
                  "You have attempted a command or operation against an object or queue whose type is not valid for the operation specified. For instance: browsing a remote queue; issuing the clear command against a queue whose type is not QLOCAL; clearing by API calls, a queue who type cannot be opened for input.",
                  "Retry the command or operation against an object or queue whose type is valid for the operation requested."));
      amqMN_Ht.put("4197",new AMQMessageNumber(4197,
                  "An MQOPEN or MQPUT1 call was issued specifying an alias queue as the target, but the BaseQName in the alias queue attributes is not recognized as a queue name.",
                  "20 : Error",
                  "An MQOPEN or MQPUT1 call was issued specifying an alias queue as the target, but the BaseQName in the alias queue attributes is not recognized as a queue name. This reason code can also occur when BaseQName is the name of a cluster queue that cannot be resolved successfully.",
                  "Correct the queue definitions."));
      amqMN_Ht.put("4207",new AMQMessageNumber(4207,
                  "The path specified is not valid.",
                  "20 : Error",
                  "",
                  "Check the path specified and try again."));
      amqMN_Ht.put("4400",new AMQMessageNumber(4400,
                  "Explorer cannot administer the queue manager because the queue <insert_0> is not defined.",
                  "10 : Warning",
                  "Explorer uses the queue <insert_0> to administer queue managers.",
                  "Define the queue <insert_0> and retry."));
      amqMN_Ht.put("4401",new AMQMessageNumber(4401,
                  "Explorer cannot administer the queue manager because the user is not authorised to open the queue <insert_0>.",
                  "10 : Warning",
                  "Explorer uses the queue <insert_0> to administer this queue manager.",
                  "Allow Explorer to open the queue <insert_0> and retry."));
      amqMN_Ht.put("4402",new AMQMessageNumber(4402,
                  "The queue <insert_0> could not be opened for reason <insert_1>.",
                  "10 : Warning",
                  "Explorer uses the queue <insert_0> to administer this queue manager.",
                  "Allow Explorer to open the queue <insert_0> and retry."));
      amqMN_Ht.put("4500",new AMQMessageNumber(4500,
                  "Are you sure you want to forcibly remove queue manager <insert_0> from cluster <insert_1>?",
                  "10 : Warning",
                  "You should only forcibly remove a queue manager from a cluster when it has already been deleted and cannot be removed from the cluster in the normal way. A confirmation is required before the queue manager is forcibly removed.",
                  "Continue only if you want to forcibly remove the queue manager."));
      amqMN_Ht.put("4501",new AMQMessageNumber(4501,
                  "The queue manager was successfully removed from the cluster.",
                  "0 : Information",
                  "The queue manager will still appear as a member of the cluster until the configuration changes have been sent across the network and the cluster channels to the queue manager have become inactive. This might take a long time.",
                  ""));
      amqMN_Ht.put("4502",new AMQMessageNumber(4502,
                  "You have shared the queue in cluster <insert_0>. The queue manager is not a member of this cluster.",
                  "10 : Warning",
                  "",
                  "To make the queue available to the members of this cluster, you must join the queue manager to the cluster."));
      amqMN_Ht.put("4503",new AMQMessageNumber(4503,
                  "The list of values is too long.",
                  "10 : Warning",
                  "The list of values that you have entered is too long. The maximum number of characters allowed for this value is <insert_0>.",
                  ""));
      amqMN_Ht.put("4504",new AMQMessageNumber(4504,
                  "The value is too long.",
                  "10 : Warning",
                  "You have entered a value containing too many characters. The maximum number of characters allowed for each value of this attribute is <insert_0>.",
                  ""));
      amqMN_Ht.put("4505",new AMQMessageNumber(4505,
                  "There are too many entries in the list.",
                  "10 : Warning",
                  "You have entered too many values in the list. The maximum number of values is <insert_0>.",
                  ""));
      amqMN_Ht.put("4506",new AMQMessageNumber(4506,
                  "Cannot connect to queue manager <insert_0>. It cannot be removed from the cluster in the normal way.",
                  "10 : Warning",
                  "",
                  "Try the operation again when the queue manager is available. If the queue manager no longer exists, you can choose to forcibly remove the queue manager from the cluster."));
      amqMN_Ht.put("4507",new AMQMessageNumber(4507,
                  "The remote queue manager is not using TCP/IP.",
                  "10 : Warning",
                  "The connection information available for the remote queue manager uses a communication protocol other than TCP/IP. The WebSphere MQ Explorer cannot connect to the queue manager to remove it from the cluster in the normal way.",
                  "If the queue manager no longer exists, you can choose to forcibly remove the queue manager from the cluster."));
      amqMN_Ht.put("4508",new AMQMessageNumber(4508,
                  "The queue manager successfully left the cluster.",
                  "0 : Information",
                  "The queue manager will still appear as a member of the cluster until the configuration changes have been sent across the network and the cluster channels to the queue manager have become inactive. This might take a long time.",
                  ""));
      amqMN_Ht.put("4509",new AMQMessageNumber(4509,
                  "The request to suspend membership of the cluster has been accepted.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4510",new AMQMessageNumber(4510,
                  "The request to resume membership of the cluster has been accepted.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4511",new AMQMessageNumber(4511,
                  "The queue manager is not a member of the cluster.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4513",new AMQMessageNumber(4513,
                  "The request to refresh the information about the cluster has been accepted.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4514",new AMQMessageNumber(4514,
                  "The queue manager is not a member of cluster <insert_0>.",
                  "10 : Warning",
                  "The object that you have shared in the cluster will not be available to other members of the cluster until you make this queue manager a member of the cluster.",
                  ""));
      amqMN_Ht.put("4515",new AMQMessageNumber(4515,
                  "The repository queue manager for cluster <insert_0> is not available for connection.",
                  "10 : Warning",
                  "Views showing cluster queues in this cluster might not be complete.",
                  ""));
      amqMN_Ht.put("4516",new AMQMessageNumber(4516,
                  "Cluster workload exit error.",
                  "10 : Warning",
                  "The queue manager's cluster workload exit failed unexpectedly or did not respond in time.",
                  ""));
      amqMN_Ht.put("4517",new AMQMessageNumber(4517,
                  "Cluster resolution error.",
                  "10 : Warning",
                  "The definition of the cluster queue could not be resolved correctly because a response from a repository queue manager was not available.",
                  ""));
      amqMN_Ht.put("4518",new AMQMessageNumber(4518,
                  "AMQ4518=A call was stopped by the cluster exit.",
                  "10 : Warning",
                  "The queue manager's cluster workload exit rejected a call to open or put a message onto a cluster queue.",
                  ""));
      amqMN_Ht.put("4519",new AMQMessageNumber(4519,
                  "No destinations are available.",
                  "10 : Warning",
                  "At the time that the message was put, there were no longer any instances of the queue in the cluster.",
                  ""));
      amqMN_Ht.put("4520",new AMQMessageNumber(4520,
                  "The WebSphere MQ Explorer could not initialize TCP/IP. Administration of remote queue managers and clusters is not possible.",
                  "10 : Warning",
                  "",
                  ""));
      amqMN_Ht.put("4521",new AMQMessageNumber(4521,
                  "The text you entered contained a comma (,) which is used as a list separator character.",
                  "10 : Warning",
                  "This value does not accept lists.",
                  "If you want to use a comma as part of a value, enclose the value in double quotes."));
      amqMN_Ht.put("4522",new AMQMessageNumber(4522,
                  "The wizard was unable to add the queue manager to the cluster.",
                  "10 : Warning",
                  "A problem occurred while defining objects or modifying the queue manager's properties.",
                  "Ensure that the default objects exist for the queue manager."));
      amqMN_Ht.put("4523",new AMQMessageNumber(4523,
                  "The wizard was unable to add one of the queue managers to the cluster.",
                  "10 : Warning",
                  "A problem occurred while defining objects or modifying one of the queue managers' properties.",
                  "Ensure that the default objects exist for the queue manager."));
      amqMN_Ht.put("4571",new AMQMessageNumber(4571,
                  "Are you sure you want to change the location of the Key Repository for queue manager <insert_0>?",
                  "10 : Warning",
                  "You might prevent the queue manager from starting if you change the Key Repository field to a location which is not valid.",
                  "Ensure that the location specified is correct before continuing."));
      amqMN_Ht.put("4572",new AMQMessageNumber(4572,
                  "The request to refresh the information about all clusters has been accepted.",
                  "0 : Information",
                  "",
                  "Message for information only."));
      amqMN_Ht.put("4574",new AMQMessageNumber(4574,
                  "IBM WebSphere Explorer is already running.",
                  "30 : Severe error",
                  "",
                  ""));
      amqMN_Ht.put("4575",new AMQMessageNumber(4575,
                  "An error occurred initializing the data model.",
                  "30 : Severe error",
                  "",
                  ""));
      amqMN_Ht.put("4576",new AMQMessageNumber(4576,
                  "The working directory <insert_0> is not valid.",
                  "30 : Severe error",
                  "",
                  ""));
      amqMN_Ht.put("4577",new AMQMessageNumber(4577,
                  "An error occurred initializing the process.",
                  "30 : Severe error",
                  "",
                  ""));
      amqMN_Ht.put("4578",new AMQMessageNumber(4578,
                  "An error occurred loading the messages file <insert_0>.",
                  "30 : Severe error",
                  "",
                  ""));
      amqMN_Ht.put("4579",new AMQMessageNumber(4579,
                  "An error occurred loading the system libraries.",
                  "30 : Severe error",
                  "",
                  ""));
      amqMN_Ht.put("4580",new AMQMessageNumber(4580,
                  "An internal method detected an unexpected system return code. The method <insert_0> returned <insert_1>.",
                  "30 : Severe error",
                  "",
                  "Examine the problem determination information on this computer to establish the cause of the error."));
      amqMN_Ht.put("4581",new AMQMessageNumber(4581,
                  "Parameter check failed on the internal function <insert_0>. The error was <insert_1>.",
                  "30 : Severe error",
                  "",
                  "Examine the problem determination information on this computer to establish the cause of the error."));
      amqMN_Ht.put("4582",new AMQMessageNumber(4582,
                  "Queue manager <insert_0> is not available for client connection.",
                  "30 : Severe error",
                  "",
                  "Ensure the queue manager is running and is configured to accept remote connections."));
      amqMN_Ht.put("4583",new AMQMessageNumber(4583,
                  "Queue manager <insert_0> is not available for connection.",
                  "30 : Severe error",
                  "",
                  "Ensure the queue manager is running."));
      amqMN_Ht.put("4584",new AMQMessageNumber(4584,
                  "Queue manager <insert_0> is not available for cluster connection.",
                  "30 : Severe error",
                  "",
                  "Ensure that the queue manager is running. If the queue manager has been deleted it might continue to be displayed as a member of a cluster for up to 30 days."));
      amqMN_Ht.put("4585",new AMQMessageNumber(4585,
                  "An internal method <insert_0> encountered an unexpected error.",
                  "30 : Severe error",
                  "",
                  "Examine the problem determination information on this computer to establish the cause of the error."));
      amqMN_Ht.put("4586",new AMQMessageNumber(4586,
                  "The attempt to create the URL for file <insert_0> failed.",
                  "30 : Severe error",
                  "The file name specified was not recognized.",
                  "Ensure that the file exists at the specified location and can be read."));
      amqMN_Ht.put("4587",new AMQMessageNumber(4587,
                  "The attempt to read from URL <insert_0> failed.",
                  "30 : Severe error",
                  "There was an error when the system tried to read the Client channel definition table.",
                  "Ensure that the file exists at the specified location and can be read."));
      amqMN_Ht.put("4588",new AMQMessageNumber(4588,
                  "The attempt to read from URL <insert_0> failed.",
                  "30 : Severe error",
                  "There was an error when the system tried to read the file.",
                  "Ensure that the file exists at the specified location and can be read."));
      amqMN_Ht.put("4589",new AMQMessageNumber(4589,
                  "No connection was found to application <insert_0>.",
                  "10 : Warning",
                  "The connection was not found. Possibly the connection was closed before the command was issued.",
                  "Check that the application connection has not been closed in the background."));
      amqMN_Ht.put("4590",new AMQMessageNumber(4590,
                  "The queue manager connection to application <insert_0> could not be closed.",
                  "20 : Error",
                  "The connection could not be closed due to a PCF error.",
                  "Check for FFSTs."));
      amqMN_Ht.put("4591",new AMQMessageNumber(4591,
                  "The command server for <insert_0> is not running.",
                  "30 : Severe error",
                  "The command server has stopped for some reason, so the request cannot be processed.",
                  "Start the command server. If the error persists, examine the problem determination information to see if any details have been recorded."));
      amqMN_Ht.put("4592",new AMQMessageNumber(4592,
                  "The connection was closed successfully.",
                  "0 : Information",
                  "The request to close the connection to an application was successful.",
                  "Message for information only."));
      amqMN_Ht.put("4593",new AMQMessageNumber(4593,
                  "Do you really want to stop the connection to application \"<insert_0>\"",
                  "0 : Information",
                  "WebSphere MQ explorer is about to stop a connection, stopping the connection will prevent further communication between MQ and the application in question.",
                  "Select yes if you want to stop the connection."));
      amqMN_Ht.put("4700",new AMQMessageNumber(4700,
                  "PCF command identifier (<insert_0>) not valid for queue manager <insert_1>.",
                  "10 : Warning",
                  "The specified PCF command is not supported by this queue manager.",
                  ""));
      amqMN_Ht.put("4800",new AMQMessageNumber(4800,
                  "Error initializing <insert_0>.",
                  "30 : Severe error",
                  "An error occurred while starting this application.",
                  "Check that the WebSphere MQ runtime libraries are available. Check that the PATH system environment variable includes the directory for these runtime libraries.)"));
      amqMN_Ht.put("4801",new AMQMessageNumber(4801,
                  "Error getting the location of the help system.",
                  "10 : Warning",
                  "To launch the standalone Eclipse help system, the WebSphere MQ file transfer application needs to know where it is installed.",
                  "Check that Eclipse has been installed with WebSphere MQ."));
      amqMN_Ht.put("4802",new AMQMessageNumber(4802,
                  "Error launching the help system.",
                  "10 : Warning",
                  "The WebSphere MQ file transfer application failed to create an instance of the Eclipse standalone help system.",
                  "Check that Eclipse has been installed with WebSphere MQ."));
      amqMN_Ht.put("4803",new AMQMessageNumber(4803,
                  "Error starting the help system.",
                  "10 : Warning",
                  "The WebSphere MQ file transfer application failed to start the standalone Eclipse system.",
                  "Check that Eclipse has been installed with WebSphere MQ."));
      amqMN_Ht.put("4805",new AMQMessageNumber(4805,
                  "Error saving the history log file.",
                  "10 : Warning",
                  "The WebSphere MQ file transfer application could not read the history log file. This file is called com.ibm.mq.fta.log.ser and is in your home directory. On Windows, this is %APPDATA%\\IBM\\WebSphere MQ FileTransferApp On Unix, this is $HOME/.mqdata",
                  "Check that the read/write properties on this file allow you to write to it."));
      amqMN_Ht.put("4806",new AMQMessageNumber(4806,
                  "Error reading the history log.",
                  "10 : Warning",
                  "The WebSphere MQ file transfer application could not read the history log file. This file is called com.ibm.mq.fta.log.ser and is in your home directory. On Windows, this is %APPDATA%\\IBM\\WebSphere MQ FileTransferApp On Unix, this is $HOME/.mqdata",
                  "Check that the read/write properties on this file allow you to write to it."));
      amqMN_Ht.put("4807",new AMQMessageNumber(4807,
                  "The message size specified (<insert_0>) is outside the permitted range.",
                  "10 : Warning",
                  "",
                  "Specify a value of 1000 to 100 000 000."));
   }

   /**
    * Initialize the AMQ 5000's message number.
    */
   public void init5s()
   {
      amqMN_Ht.put("5005",new AMQMessageNumber(5005,
            "Unexpected error",
            "20 : Error",
            "An unexpected error occurred in an internal function of the product.",
            "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5006",new AMQMessageNumber(5006,
                  "Unexpected error: rc = <insert_1>",
                  "20 : Error",
                  "An unexpected error occurred in an internal function of the product.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5008",new AMQMessageNumber(5008,
                  "An essential WebSphere MQ process <insert_1> (<insert_3>) cannot be found and is assumed to be terminated.",
                  "40 : Stop Error",
                  "1) A user has inadvertently terminated the process. 2) The system is low on resources. Some operating systems terminate processes to free resources. If your system is low on resources, it is possible it has terminated the process so that a new process can be created.",
                  "WebSphere MQ will stop all MQ processes. Inform your systems administrator. When the problem is rectified WebSphere MQ can be restarted."));
      amqMN_Ht.put("5009",new AMQMessageNumber(5009,
                  "WebSphere MQ agent process <insert_1> has terminated unexpectedly.",
                  "40 : Stop Error",
                  "WebSphere MQ has detected that an agent process has terminated unexpectedly. The queue manager connection(s) that this process is responsible for will be broken.",
                  "Use any previous FFSTs to determine the reason for the failure. Try to eliminate the following reasons before contacting your IBM support center. 1) A user has inadvertently terminated the process. 2) The system is low on resources. Some operating systems terminate processes to free resources. If your system is low on resources, it is possible that the operating system has terminated the process so that a new process can be created."));
      amqMN_Ht.put("5010",new AMQMessageNumber(5010,
                  "The system is restarting the WorkLoad Management Server process.",
                  "10 : Warning",
                  "The system has detected that the WorkLoad Management server process (amqzlwa0, pid:<insert_1>) has stopped and is restarting it.",
                  "Save the generated output files which may indicate the reason why the WorkLoad Management process stopped. If the reason the WorkLoad Management Server process stopped is a problem in a WorkLoad Management user exit, correct the problem, otherwise contact your IBM support center."));
      amqMN_Ht.put("5011",new AMQMessageNumber(5011,
                  "The Queue Manager ended for reason <insert_1> <insert_3>",
                  "10 : Warning",
                  "The Queue Manager ended because of a previous error <insert_1> or <insert_3>",
                  "This message should be preceded by a message or FFST information from the internal routine that detected the error. Take the action associated with the earlier error information."));
      amqMN_Ht.put("5019",new AMQMessageNumber(5019,
                  "Unable to access program <insert_3>.",
                  "40 : Stop Error",
                  "A request was made to execute the program <insert_3>, however the operation was unsuccessful because the program could not be found in the specified location.",
                  "Check the definition of the service specifies the correct and full path to the program to run. If the path is correct then verify that the program exists in the specified location and that WebSphere MQ userid has permission to access it."));
      amqMN_Ht.put("5020",new AMQMessageNumber(5020,
                  "Permission denied attempting to execute program <insert_3>.",
                  "40 : Stop Error",
                  "A request was made to execute the program <insert_3>, however the operation was unsuccessful because the WebSphere MQ operating environment has insufficient permissions to access the program file.",
                  "Check the access permissions of the of the program to be executed and if necessary alter them to include execute permission for the WebSphere MQ userId. Also check that the WebSphere MQ userId has search access on all directories which compose the path to the program file."));
      amqMN_Ht.put("5021",new AMQMessageNumber(5021,
                  "Unable to start program <insert_3>.",
                  "40 : Stop Error",
                  "A request was made to execute the program <insert_3> however the operation was unsuccessful. Reasons for the failure may include a shortage of available system resources a problem with the program to be started",
                  "If the problem persists then the WebSphere MQ error logs should be consulted for further information related to this error. The Operating System error recording facilities should also be consutled for information relating to shortage of system resources."));
      amqMN_Ht.put("5022",new AMQMessageNumber(5022,
                  "The Channel Initiator has started. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Channel Initiator process has started.",
                  "None."));
      amqMN_Ht.put("5023",new AMQMessageNumber(5023,
                  "The Channel Initiator has ended. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Channel Initiator process has ended.",
                  "None."));
      amqMN_Ht.put("5024",new AMQMessageNumber(5024,
                  "The Command Server has started. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Command Server process has started.",
                  "None."));
      amqMN_Ht.put("5025",new AMQMessageNumber(5025,
                  "The Command Server has ended. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Command Server process has ended.",
                  "None."));
      amqMN_Ht.put("5026",new AMQMessageNumber(5026,
                  "The Listener <insert_3> has started. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Listener process has started.",
                  "None."));
      amqMN_Ht.put("5027",new AMQMessageNumber(5027,
                  "The Listener <insert_3> has ended. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Listener process has ended.",
                  "None."));
      amqMN_Ht.put("5028",new AMQMessageNumber(5028,
                  "The Server <insert_3> has started. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Server process has started.",
                  "None."));
      amqMN_Ht.put("5029",new AMQMessageNumber(5029,
                  "The Server <insert_3> has ended. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Server process has ended.",
                  "None."));
      amqMN_Ht.put("5030",new AMQMessageNumber(5030,
                  "The Command <insert_3> has started. ProcessId(<insert_1>).",
                  "0 : Information",
                  "The Command has started.",
                  "None."));
      amqMN_Ht.put("5032",new AMQMessageNumber(5032,
                  "Error (<insert_4>) accessing file <insert_3>.",
                  "40 : Stop Error",
                  "While attempting to access the file <insert_3> the error <insert_4> occurred.",
                  "Use the information contained in the error to locate and correct the cause of the failure."));
      amqMN_Ht.put("5036",new AMQMessageNumber(5036,
                  "Error detected processing line <insert_1>, position <insert_2> in service environment file.",
                  "40 : Stop Error",
                  "While processing the environment file <insert_3> an error was detected on line <insert_1> at position <insert_2>. Possible causes are Variable name too long Variable value too long Incorrectly formed line. Lines must be in the format <name>=<value>. There should be no blank characters in name field. All characters following the '=' are part of the value field.",
                  "This error will not stop the command from executing but any data on the invalid line is not processed."));
      amqMN_Ht.put("5037",new AMQMessageNumber(5037,
                  "The Queue Manager task <insert_3> has started.",
                  "0 : Information",
                  "The Utility Task Manager, processId(<insert_1>) type(<insert_2>), has started the <insert_3> task.",
                  "None."));
      amqMN_Ht.put("5038",new AMQMessageNumber(5038,
                  "The Queue Manager task <insert_3> failed to start with error-code <insert_1>.",
                  "40 : Stop Error",
                  "The Utility Task Manager, attempted to start the task <insert_3> but the start request failed with error code <insert_1>.",
                  "The failure to start the identified task may not be critical to queue-manager operation however all of the queue manager functionality may not be available. Further details of the failure are available in WebSphere MQ error logs."));
      amqMN_Ht.put("5041",new AMQMessageNumber(5041,
                  "The Queue Manager task <insert_3> has ended.",
                  "0 : Information",
                  "The Queue Manager task <insert_3> has ended.",
                  "None."));
      amqMN_Ht.put("5042",new AMQMessageNumber(5042,
                  "Request to start <insert_3> failed.",
                  "40 : Stop Error",
                  "The request to start the process <insert_3> failed.",
                  "Consult the Queue Manager error logs for further details on the cause of the failure."));
      amqMN_Ht.put("5043",new AMQMessageNumber(5043,
                  "Statistics recording is unavailable due to error code <insert_1>.",
                  "40 : Stop Error",
                  "The statistics collection task was unable to start due the error code <insert_1>. Statistics collection will be unavailable until the problem is rectified and the Queue Manager is restarted.",
                  "Consult the Queue Manager error logs for further details on the cause of the failure."));
      amqMN_Ht.put("5044",new AMQMessageNumber(5044,
                  "<insert_3> task operation restricted due to Reason Code <insert_1>.",
                  "10 : Warning",
                  "The <insert_3> task encountered a non-fatal error which may effect the operation of the task.",
                  "Using the Reason Code <insert_1> and any previous messages recorded in the Error Logs correct the error. It may be necessary to restart the Queue Manager in order remove the restriction caused by the failure."));
      amqMN_Ht.put("5045",new AMQMessageNumber(5045,
                  "System reconfiguration event received",
                  "0 : Information",
                  "The Queue Manager received a system re-configuration event. This is likely to have been caused by an administrative change in the configuration of the machine (for example dynamically adding or removing resources such as memory or processors).",
                  "No action is required unless this notification was unexpected."));
      amqMN_Ht.put("5203",new AMQMessageNumber(5203,
                  "An error occurred calling the XA interface.",
                  "0 : Information",
                  "The error number is <insert_2> where a value of 1 indicates the supplied flags value of <insert_1> was invalid, 2 indicates that there was an attempt to use threaded and non-threaded libraries in the same process, 3 indicates that there was an error with the supplied queue manager name <insert_3>, 4 indicates that the resource manager id of <insert_1> was invalid, 5 indicates that an attempt was made to use a second queue manager called <insert_3> when another queue manager was already connected, 6 indicates that the Transaction Manager has been called when the application isn't connected to a queue manager, 7 indicates that the XA call was made while another call was in progress, 8 indicates that the xa_info string <insert_3> in the xa_open call contained an invalid parameter value for parameter name <insert_4>, 9 indicates that the xa_info string <insert_3> in the xa_open call is missing a required parameter, parameter name <insert_4>, and 10 indicates that MQ was called in dynamic registration mode but cannot find the ax_reg and ax_unreg functions ! Either call MQ in non-dynamic registration mode or supply the correct library name via the AXLIB parameter in the xa_open string.",
                  "Correct the error and try the operation again."));
      amqMN_Ht.put("5204",new AMQMessageNumber(5204,
                  "A non-threaded application tried to run as a Trusted application.",
                  "10 : Warning",
                  "Only applications linked with the threaded MQ libraries can run as Trusted applications.",
                  "Make sure that the application is relinked with the threaded MQ libraries, or set the the environment variable MQ_CONNECT_TYPE to STANDARD."));
      amqMN_Ht.put("5205",new AMQMessageNumber(5205,
                  "File or directory <insert_3> not owned by user <insert_4>.",
                  "40 : Stop Error",
                  "WebSphere MQ has detected that the file or directory <insert_3> is not owned by the user <insert_4>. This is not necessarily an error but you should investigate further if this is unexpected.",
                  "\n If this is unexpected then you should alter the ownership of the file or directory back to the user <insert_4>. If this is expected, then WebSphere MQ will continue however WebSphere MQ will be unable to verify the security of this file or directory. If the access permissions are too strict then you may encounter problems if WebSphere MQ cannot access the contents of the file or directory. If the access permissions are too relaxed then there may be an increased risk to the security of the WebSphere MQ system."));
      amqMN_Ht.put("5206",new AMQMessageNumber(5206,
                  "Duplicate parameters detected.",
                  "10 : Warning",
                  "WebSphere MQ has detected that the activity about to be displayed contains two or more parameters in the same group with the same parameter identifier. The activity may be displayed incorrectly.",
                  "Inform the author of the activity that there may be an error in it."));
      amqMN_Ht.put("5358",new AMQMessageNumber(5358,
                  "WebSphere MQ could not load AX support module <insert_3>.",
                  "20 : Error",
                  "An error has occurred loading the AX support module <insert_3>. This module needs to be loaded so that dynamically-registering resource managers, such as DB2, can participate in global units of work.",
                  "Look for a previous message outlining the reason for the load failure. Message AMQ6175 should have been issued if the load failed because of a system error. If this is the case then follow the guidance given in message AMQ6175 to resolve the problem. In the absence of prior messages or FFST information related to this problem check that the AX support module and the mqmax library have been correctly installed on your system."));
      amqMN_Ht.put("5501",new AMQMessageNumber(5501,
                  "There was not enough storage to satisfy the request",
                  "20 : Error",
                  "An internal function of the product attempted to obtain storage, but there was none available.",
                  "Stop the product and restart it. If this does not resolve the problem, save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5502",new AMQMessageNumber(5502,
                  "The CDS directory name <insert_3> is not in the correct format.",
                  "20 : Error",
                  "An internal function of the DCE Naming service found a CDS directory name in the wrong format. The name was expected to start with either '/...' for a fully qualified name (from global root), or '/.:' for a partially qualified name (from local cell root).",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5503",new AMQMessageNumber(5503,
                  "The name of the local DCE cell cannot be determined, status = <insert_1>",
                  "20 : Error",
                  "The DCE Naming Service attempted to determine the name of the local DCE cell by calling 'dce_cf_get_cell_name()', which returned a nonzero return code.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5504",new AMQMessageNumber(5504,
                  "DCE error. No value for the XDS attribute found.",
                  "20 : Error",
                  "The DCE Naming service called om_get() to get the entry from the object returned by ds_read(). Although the status was correct, no objects were returned.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5505",new AMQMessageNumber(5505,
                  "DCE error. No value for the XDS attribute number <insert_1> found.",
                  "20 : Error",
                  "The DCE Naming service called om_get() to get the entry from the object returned by ds_read(). Although the status was correct, no objects were returned.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5506",new AMQMessageNumber(5506,
                  "DCE error. <insert_3> returned <insert_1> for attribute number <insert_2>.",
                  "20 : Error",
                  "The DCE Naming service queried an object by calling <insert_3> which returned a nonzero return code.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5507",new AMQMessageNumber(5507,
                  "DCE error. <insert_3> failed for an unknown reason.",
                  "20 : Error",
                  "An unexpected error occurred in an internal function of the DCE Naming service.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5508",new AMQMessageNumber(5508,
                  "DCE error. The requested attribute is not present.",
                  "20 : Error",
                  "The DCE Naming service was attempting to extract the value from an attribute, but the attribute cannot be found in the XDS object.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5509",new AMQMessageNumber(5509,
                  "DCE error. The XDS workspace cannot be initialized.",
                  "20 : Error",
                  "The DCE Naming service called 'ds_initialize()' to initialize the XDS workspace, but 'ds_initialize()' returned a nonzero return code.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5510",new AMQMessageNumber(5510,
                  "DCE error. <insert_3> returned with problem <insert_1>.",
                  "20 : Error",
                  "The DCE Naming service found an unexpected XDS error.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5511",new AMQMessageNumber(5511,
                  "Installable service component <insert_3> returned <insert_4>.",
                  "20 : Error",
                  "The internal function, that adds a component to a service, called the component initialization process. This process returned an error.",
                  "Check the component was installed correctly. If it was, and the component was supplied by IBM, then save the generated output files and contact your IBM support center. If the component was not supplied by IBM, save the generated output files and follow the support procedure for that component."));
      amqMN_Ht.put("5511",new AMQMessageNumber(5511,
                  "(iSeries) An installable service component returned an error.",
                  "20 : Error",
                  "Installable service component <insert_3> returned <insert_4>. The internal function, that adds a component to a service, called the component initialization process. This process returned an error.",
                  "Check the component was installed correctly. If it was, and the component was supplied by IBM, then save the generated output files and contact your IBM support center. If the component was not supplied by IBM, save the generated output files and follow the support procedure for that component."));
      amqMN_Ht.put("5512",new AMQMessageNumber(5512,
                  "Installable service component <insert_3> returned <insert_4> for queue manager name = <insert_5>.",
                  "20 : Error",
                  "An installable service component returned an unexpected return code.",
                  "Check the component was installed correctly. If it was, and the component was supplied by IBM, then save the generated output files and contact your IBM support center. If the component was not supplied by IBM, save the generated output files and follow the support procedure for that component."));
      amqMN_Ht.put("5512",new AMQMessageNumber(5512,
                  "(iSeries) An installable service component returned an unexpected return code.",
                  "20 : Error",
                  "Installable service component <insert_3> returned <insert_4> for queue manager name = <insert_5>.",
                  "Check the component was installed correctly. If it was, and the component was supplied by IBM, then save the generated output files and contact your IBM support center. If the component was not supplied by IBM, save the generated output files and follow the support procedure for that component."));
      amqMN_Ht.put("5513",new AMQMessageNumber(5513,
                  "<insert_3> returned <insert_1>.",
                  "20 : Error",
                  "An unexpected error occurred.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("5519",new AMQMessageNumber(5519,
                  "Bad DCE identity. Status = <insert_1>, auth = <insert_2>, keytab file = <insert_3>, principal = <insert_4>.",
                  "20 : Error",
                  "The keytab file was not installed correctly, or the WebSphere MQ user ID has a different password from that used to create the keytab file.",
                  "Make sure that the MQ user ID defined when the product was installed has the same password as that defined by the keytab file, and that the keytab file has been installed correctly."));
      amqMN_Ht.put("5519",new AMQMessageNumber(5519,
                  "(iSeries) Bad DCE identity.",
                  "20 : Error",
                  "Status = <insert_1>, auth = <insert_2>, keytab file = <insert_3>, principal = <insert_4>. The keytab file was not installed correctly, or the WebSphere MQ user ID has a different password from that used to create the keytab file.",
                  "Make sure that the MQ user ID defined when the product was installed has the same password as that defined by the keytab file, and that the keytab file has been installed correctly."));
      amqMN_Ht.put("5520",new AMQMessageNumber(5520,
                  "The system could not load the module <insert_5> for the installable service <insert_3> component <insert_4>. The system return code was <insert_1>. The Queue Manager is continuing without this component.",
                  "10 : Warning",
                  "The queue manager configuration data included a stanza for the installable service <insert_3> component <insert_4> with the module <insert_5>. The system returned <insert_1> when it tried to load this module. The Queue Manager is continuing without this component.",
                  "Make sure that the module can be loaded. Put the module into a directory where the system can load it, and specify its full path and name in the configuration data . Then stop and restart the queue manager."));
      amqMN_Ht.put("5520",new AMQMessageNumber(5520,
                  "(iSeries) The system could not load a module. The Queue Manager is continuing without this component.",
                  "10 : Warning",
                  "The queue manager configuration data included a stanza for the installable service <insert_3> component <insert_4> with the module <insert_5>. The system returned <insert_1> when it tried to load this module. The Queue Manager is continuing without this component.",
                  "Make sure that the module can be loaded. Put the module into a directory where the system can load it, and specify its full path and name in the configuration data . Then stop and restart the queue manager."));
      amqMN_Ht.put("5521",new AMQMessageNumber(5521,
                  "The system could not open \"<insert_3>\".",
                  "10 : Warning",
                  "The system failed to open the default object \"<insert_3>\" at connect time for reason <insert_4>. This may be because \"<insert_3>\" has been deleted or changed.",
                  "Recreate the default objects by running \"strmqm -c <qmgr>\" (where <qmgr> is the name of the queue manager) and retry the application."));
      amqMN_Ht.put("5522",new AMQMessageNumber(5522,
                  "A WebSphere MQ installable service component could not be initialized.",
                  "20 : Error",
                  "An installable service component returned an unexpected return code.",
                  "Check the queue manager error logs for messages explaining which installable service could not be initialized and why that service could not be initialized. Check the component was installed correctly. If it was, and the component was supplied by IBM, then save the generated output files and contact your IBM support center. If the component was not supplied by IBM, save the generated output files and follow the support procedure for that component."));
      amqMN_Ht.put("5524",new AMQMessageNumber(5524,
                  "The WebSphere MQ Object Authority Manager has failed to migrate authority data.",
                  "20 : Error",
                  "The Object Authority Manager has attempted to migrate existing queue manager authority data from a previous version of an Object Authority Manager and failed.",
                  "Check this log for any previous related messages, follow their recommendations then restart the queue manager."));
      amqMN_Ht.put("5525",new AMQMessageNumber(5525,
                  "The WebSphere MQ Object Authority Manager has failed.",
                  "20 : Error",
                  "The Object Authority Manager has failed to complete an MQ request.",
                  "Check the queue manager error logs for messages explaining the failure and try to correct the problem accordingly."));
      amqMN_Ht.put("5526",new AMQMessageNumber(5526,
                  "The WebSphere MQ Object Authority Manager has failed with reason <insert_1>",
                  "20 : Error",
                  "The Object Authority Manager has failed an operation on the Object Authority Manager's data queue <insert_3> with reason <insert_1>.",
                  "Investigate why the error has occured and correct the problem."));
      amqMN_Ht.put("5527",new AMQMessageNumber(5527,
                  "The WebSphere MQ Object Authority Manager has failed to locate an essential authority file",
                  "20 : Error",
                  "The Object Authority Manager has failed to locate the authority file <insert_3>. The migration of authority data cannot continue until the file has been restored. The queue manager will shutdown.",
                  "Restore the authority file mentioned above and restart the queue manager."));
      amqMN_Ht.put("5528",new AMQMessageNumber(5528,
                  "The WebSphere MQ Object Authority Manager has failed to locate an object's authority file",
                  "20 : Error",
                  "The Object Authority Manager has failed to locate the authority file for the object <insert_3> of type (<insert_1>). The authority access to this object will initially be limited to members of the mqm group. Where type is one of the following: 1) Queue 2) Namelist 3) Process 5) Queue Manager",
                  "To extend access to this object use the setmqaut command, see the WebSphere MQ System Administration documentation for details."));
      amqMN_Ht.put("5529",new AMQMessageNumber(5529,
                  "The Remote OAM Service is not available.",
                  "20 : Error",
                  "The Remote OAM service is not available. The <insert_1> call returned <insert_1>, errno <insert_2> : <insert_3>. The context string is <insert_4>",
                  "To extend access to this object use the setmqaut command, see the WebSphere MQ System Administration documentation for details."));
      amqMN_Ht.put("5600",new AMQMessageNumber(5600,
                  "Usage: crtmqm [-z] [-q] [-c Text] [-d DefXmitQ] [-h MaxHandles]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5601",new AMQMessageNumber(5601,
                  "[-t TrigInt] [-u DeadQ] [-x MaxUMsgs] [-lp LogPri] [-ls LogSec]",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("5602",new AMQMessageNumber(5602,
                  "[-lc | -ll] [-lf LogFileSize] [-ld LogPath] QMgrName",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5602",new AMQMessageNumber(5602,
                  "(iSeries) [-ll] [-lf LogFileSize] [-ld LogPath] [-lz ASPNum] QMgrName",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5603",new AMQMessageNumber(5603,
                  "Usage: dltmqm [-z] QMgrName",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5604",new AMQMessageNumber(5604,
                  "Usage: dspmqaut [-m QMgrName] [-n ObjName] -t ObjType (-p Principal | -g Group) [-s ServiceComponent]",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("5605",new AMQMessageNumber(5605,
                  "Usage: endmqm [-z] [-c | -w | -i | -p] QMgrName",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5606",new AMQMessageNumber(5606,
                  "Usage: setmqaut [-m QMgrName] [-n ObjName] -t ObjType (-p Principal | -g Group) [-s ServiceComponent] Authorizations",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("5607",new AMQMessageNumber(5607,
                  "Usage: strmqm [-a|-c|-r][-d none|minimal|all][-z][-ns] [QMgrName]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5608",new AMQMessageNumber(5608,
                  "Usage: dspmqtrn [-m QMgrName] [-e] [-i]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5609",new AMQMessageNumber(5609,
                  "Usage: rsvmqtrn -m QMgrName (-a | ((-b | -c | -r RMId) Transaction,Number))",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5610",new AMQMessageNumber(5610,
                  "(iSeries) Usage: strmqtrc [-m QMgrName] [-e] [-t TraceType] [-o mqm|pex|all] [-x TraceType] [-l MaxFileSize] [-d UserDataSize]",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("5610",new AMQMessageNumber(5610,
                  " (Unix)Usage: strmqtrc [-m QMgrName] [-e] [-t TraceType] [-x TraceType]",
                  "0 : Information",
                  "This applies to UNIX systems. MaxFileSize is the maximum size of a trace file in millions of bytes. UserDataSize is the size of user data to be traced in bytes.",
                  "None."));
      amqMN_Ht.put("5610",new AMQMessageNumber(5610,
                  " (Windows)Usage: strmqtrc [-t TraceType] [-x TraceType] [-l MaxFileSize] [-d UserDataSize]",
                  "0 : Information",
                  "This applies to Windows NT and Windows 2000 systems only. MaxFileSize is the maximum size of a trace file in millions of bytes. UserDataSize is the size of user data to be traced in bytes.",
                  "None."));
      amqMN_Ht.put("5611",new AMQMessageNumber(5611,
                  " (Unix)Usage: endmqtrc [-m QMgrName] [-e] [-a]",
                  "0 : Information",
                  "This applies to UNIX systems.",
                  "None."));
      amqMN_Ht.put("5611",new AMQMessageNumber(5611,
                  " (Windows)Usage: endmqtrc",
                  "0 : Information",
                  "This applies to Windows NT and Windows 2000 systems only.",
                  "None."));
      amqMN_Ht.put("5612",new AMQMessageNumber(5612,
                  "Usage: dspmqtrc [-t TemplateFile] [-hs] [-o OutputFileName] [-C InputFileCCSID] InputFileName(s)",
                  "0 : Information",
                  "Options: -t Template file for formatting trace data -h Skip the trace file header -s Summary (format only the trace header) -o Save trace output to file -C Specifies the CCSID value for the input file",
                  "None."));
      amqMN_Ht.put("5613",new AMQMessageNumber(5613,
                  "Usage: dspmq [-m QMgrName] [-o status | -s] [-o default]",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5614",new AMQMessageNumber(5614,
                  "Usage: setmqtry",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5615",new AMQMessageNumber(5615,
                  "Default objects cannot be created: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "During the creation of a queue manager, using the crtmqm command, the default objects could not be created. Possible reasons for this include another command, issued elsewhere, quiescing or stopping the queue manager, or insufficient storage being available.",
                  "Use the Completion and Reason codes shown in the message to determine the cause of the failure, then re-try the command."));
      amqMN_Ht.put("5616",new AMQMessageNumber(5616,
                  "Usage: setmqprd LicenseFile",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5617",new AMQMessageNumber(5617,
                  "Default objects cannot be created.",
                  "20 : Error",
                  "During the creation of a queue manager using the crtmqm command, the default objects could not be created. The most likely reason for this error is that the queue manager was started before the crtmqm command had completed.",
                  "Ensure that the queue manager being created is not started before the create request completes. Stop the queue manager if it is already running. Restart the queue manager using the strmqm command with the '-c' option to request that the default objects are created."));
      amqMN_Ht.put("5618",new AMQMessageNumber(5618,
                  "integer",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5619",new AMQMessageNumber(5619,
                  "string",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5620",new AMQMessageNumber(5620,
                  "channel_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5621",new AMQMessageNumber(5621,
                  "process_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5622",new AMQMessageNumber(5622,
                  "q_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5623",new AMQMessageNumber(5623,
                  "connection_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5624",new AMQMessageNumber(5624,
                  "generic_channel_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5625",new AMQMessageNumber(5625,
                  "generic_process_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5626",new AMQMessageNumber(5626,
                  "generic_q_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5627",new AMQMessageNumber(5627,
                  "qalias_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5628",new AMQMessageNumber(5628,
                  "qmodel_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5629",new AMQMessageNumber(5629,
                  "qlocal_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5630",new AMQMessageNumber(5630,
                  "qremote_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5631",new AMQMessageNumber(5631,
                  "namelist_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5632",new AMQMessageNumber(5632,
                  "generic_namelist_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5633",new AMQMessageNumber(5633,
                  "generic_Q_Mgr_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5634",new AMQMessageNumber(5634,
                  "generic_cluster_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5635",new AMQMessageNumber(5635,
                  "The argument supplied with the -l flag is not valid.",
                  "20 : Error",
                  "The argument supplied with the -l flag must be in the range 1 - 4293.",
                  "Submit the command again with a valid argument."));
      amqMN_Ht.put("5636",new AMQMessageNumber(5636,
                  "cluster_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5637",new AMQMessageNumber(5637,
                  " (AIX)The environment variable EXTSHM is set to \"ON\". This is incompatable with the way WebSphere MQ uses shared memory. Reset the environment variable EXTSHM and retry the command.",
                  "20 : Error",
                  "On AIX the environment variable EXTSHM causes shared memory segments to be fixed size. WebSphere MQ expects to be able to extend shared memory segments.",
                  "Reset the environment variable EXTSHM and retry the command."));
      amqMN_Ht.put("5646",new AMQMessageNumber(5646,
                  "Usage: setmqcap Processors",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5647",new AMQMessageNumber(5647,
                  "Usage: dspmqcap",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5648",new AMQMessageNumber(5648,
                  "Usage: dmpmqaut [-m QMgrName] [-n Profile | -l] [-t ObjType] [-p Principal | -g Group] [-s ServiceComponent] [-e | -x]",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("5649",new AMQMessageNumber(5649,
                  "generic_authinfo_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5650",new AMQMessageNumber(5650,
                  "authinfo_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5651",new AMQMessageNumber(5651,
                  "qmname",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5652",new AMQMessageNumber(5652,
                  "The Deferred Message process failed to connect to the WebSphere MQ queue manager for reason <insert_1>.",
                  "30 : Severe error",
                  "The WebSphere MQ queue manager <insert_3> might have generated earlier messages or FFST information explaining why the deferred message process (amqzdmaa) could not connect.",
                  "Correct any configuration errors. Configuration errors that can cause this problem include badly configured CLWL Exit modules. If the problem persists contact your IBM service representative."));
      amqMN_Ht.put("5653",new AMQMessageNumber(5653,
                  "The mqm user is not defined.",
                  "30 : Severe error",
                  "The system call getpwnam(\"mqm\") failed with errno <insert_1>. The program was running as <insert_3>.",
                  "Create the mqm user as a member of the mqm group and retry the operation."));
      amqMN_Ht.put("5654",new AMQMessageNumber(5654,
                  "Usage: dspmqrte [-c] [-n] [-l Persistence] [-m QMgrName] [-o] [-p Priority]",
                  "0 : Information",
                  "This shows the correct usage of the DSPMQRTE command.",
                  "None."));
      amqMN_Ht.put("5655",new AMQMessageNumber(5655,
                  "[-rq ReplyQName [-rqm ReplyQMgrName]] [-ro ReportOptions]",
                  "0 : Information",
                  "This shows the correct usage of the DSPMQRTE command.",
                  "None."));
      amqMN_Ht.put("5656",new AMQMessageNumber(5656,
                  "[-xs Expiry] [-xp Pass] [-qm TargetQMgrName] [-ac [-ar]]",
                  "0 : Information",
                  "This shows the correct usage of the DSPMQRTE command.",
                  "None."));
      amqMN_Ht.put("5657",new AMQMessageNumber(5657,
                  "[-d Delivery] [-f Forwarding] [-s Activities] [-t Detail]",
                  "0 : Information",
                  "This shows the correct usage of the DSPMQRTE command.",
                  "None."));
      amqMN_Ht.put("5658",new AMQMessageNumber(5658,
                  "[-i CorrelId] [-b] [-v Verbosity] [-w WaitTime] -q TargetQName",
                  "0 : Information",
                  "This shows the correct usage of the DSPMQRTE command.",
                  "None."));
      amqMN_Ht.put("5700",new AMQMessageNumber(5700,
                  "listener_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5701",new AMQMessageNumber(5701,
                  "service_name",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5749",new AMQMessageNumber(5749,
                  "display_cmd",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5750",new AMQMessageNumber(5750,
                  "filter_keyword",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5751",new AMQMessageNumber(5751,
                  "operator",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5752",new AMQMessageNumber(5752,
                  "filter_value",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("5805",new AMQMessageNumber(5805,
                  "WebSphere MQ Publish/Subscribe broker currently running for queue manager.",
                  "10 : Warning",
                  "The command was unsuccessful because queue manager <insert_3> currently has an WebSphere MQ Publish/Subscribe broker running.",
                  "None."));
      amqMN_Ht.put("5806",new AMQMessageNumber(5806,
                  "WebSphere MQ Publish/Subscribe broker started for queue manager <insert_3>.",
                  "0 : Information",
                  "WebSphere MQ Publish/Subscribe broker started for queue manager <insert_3>.",
                  "None."));
      amqMN_Ht.put("5807",new AMQMessageNumber(5807,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> ended.",
                  "0 : Information",
                  "The WebSphere MQ Publish/Subscribe broker on queue manager <insert_3> has ended.",
                  "None."));
      amqMN_Ht.put("5808",new AMQMessageNumber(5808,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> is already quiescing.",
                  "10 : Warning",
                  "The endmqbrk command was unsuccessful because an orderly shutdown of the WebSphere MQ Publish/Subscribe broker running on queue manager <insert_3> is already in progress.",
                  "None."));
      amqMN_Ht.put("5808",new AMQMessageNumber(5808,
                  "(iSeries) WebSphere MQ Publish/Subscribe broker is already quiescing.",
                  "10 : Warning",
                  "The endmqbrk command was unsuccessful because an orderly shutdown of the broker, running on queue manager <insert_3>, is already in progress.",
                  "None."));
      amqMN_Ht.put("5809",new AMQMessageNumber(5809,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> starting.",
                  "0 : Information",
                  "The dspmqbrk command has been issued to query the state of the WebSphere MQ Publish/Subscribe broker. The WebSphere MQ Publish/Subscribe broker is currently initializing.",
                  "None."));
      amqMN_Ht.put("5810",new AMQMessageNumber(5810,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> running.",
                  "0 : Information",
                  "The dspmqbrk command has been issued to query the state of the WebSphere MQ Publish/Subscribe broker. The WebSphere MQ Publish/Subscribe broker is currently running.",
                  "None."));
      amqMN_Ht.put("5811",new AMQMessageNumber(5811,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> quiescing.",
                  "0 : Information",
                  "The dspmqbrk command has been issued to query the state of the WebSphere MQ Publish/Subscribe broker. The WebSphere MQ Publish/Subscribe broker is currently performing a controlled shutdown.",
                  "None."));
      amqMN_Ht.put("5812",new AMQMessageNumber(5812,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> stopping.",
                  "0 : Information",
                  "Either the dspmqbrk command or the endmqbrk command has been issued. The WebSphere MQ Publish/Subscribe broker is currently performing an immediate shutdown. If the endmqbrk command has been issued to request that the broker terminate, the command is unsuccessful because the broker is already performing an immediate shutdown.",
                  "None."));
      amqMN_Ht.put("5813",new AMQMessageNumber(5813,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> not active.",
                  "0 : Information",
                  "An WebSphere MQ Publish/Subscribe broker administration command has been issued to query or change the state of the broker. The WebSphere MQ Publish/Subscribe broker is not currently running.",
                  "None."));
      amqMN_Ht.put("5814",new AMQMessageNumber(5814,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> ended abnormally.",
                  "0 : Information",
                  "The dspmqbrk command has been issued to query the state of the WebSphere MQ Publish/Subscribe broker. The WebSphere MQ Publish/Subscribe broker has ended abnormally.",
                  "Refer to the queue manager error logs to determine why the broker ended abnormally."));
      amqMN_Ht.put("5815",new AMQMessageNumber(5815,
                  "Invalid WebSphere MQ Publish/Subscribe broker initialization file stanza for queue manager (<insert_3>).",
                  "20 : Error",
                  "The broker was started using the strmqbrk command. The broker stanza in the queue manager initialization file is not valid. The broker will terminate immediately. The invalid attribute is <insert_5>.",
                  "Correct the broker stanza in the queue manager initialization file."));
      amqMN_Ht.put("5815",new AMQMessageNumber(5815,
                  "(iSeries) Invalid WebSphere MQ Publish/Subscribe broker initialization file stanza.",
                  "20 : Error",
                  "The broker was started using the strmqbrk command. The Broker stanza in the queue manager(<insert_3>) initialization file is not valid. The broker will terminate immediately. The invalid attribute is <insert_5>.",
                  "Correct the Broker stanza in the queue manager initialization file."));
      amqMN_Ht.put("5815",new AMQMessageNumber(5815,
                  " (Windows)The WebSphere MQ Publish/Subscribe broker configuration for queue manager (<insert_3>) is not valid.",
                  "20 : Error",
                  "The broker was started using the strmqbrk command. The broker configuration information is not valid. The broker will terminate immediately. The invalid attribute is <insert_5>.",
                  "Correct the broker attribute using the cfgmqbrk configuration tool."));
      amqMN_Ht.put("5816",new AMQMessageNumber(5816,
                  "Unable to open WebSphere MQ Publish/Subscribe broker control queue for reason <insert_1>,<insert_2>.",
                  "20 : Error",
                  "The broker has failed to open the broker control queue (<insert_3>). The attempt to open the queue failed with completion code <insert_1> and reason <insert_2>. The most likely reasons for this error are that an application program has opened the broker control queue for exclusive access, or that the broker control queue has been defined incorrectly. The broker will terminate immediately.",
                  "Correct the problem and restart the broker."));
      amqMN_Ht.put("5817",new AMQMessageNumber(5817,
                  "An invalid stream queue has been detected by the broker.",
                  "10 : Warning",
                  "WebSphere MQ has detected an attempt to use a queue (<insert_3>) as a stream queue, but the attributes of the queue make it unsuitable for use as a stream queue. The most likely reason for this error is that the queue is: (1) Not a local queue; (2) A shareable queue; (3) A temporary dynamic queue. If the queue was created using implicit stream creation, the model stream might have been defined incorrectly. The message that caused the stream to be created will be rejected or put to the dead-letter queue, depending upon the message report options and broker configuration.",
                  "Correct the problem and resubmit the request."));
      amqMN_Ht.put("5818",new AMQMessageNumber(5818,
                  "Unable to open WebSphere MQ Publish/Subscribe broker stream queue.",
                  "10 : Warning",
                  "The broker has failed to open a stream queue (<insert_3>). The attempt to open the queue failed with completion code <insert_1> and reason <insert_2>. The most likely reason for this error is that an application has the queue open for exclusive access. The stream will be temporarily shut down and an attempt will be made to restart the stream after a short interval.",
                  "Correct the problem."));
      amqMN_Ht.put("5819",new AMQMessageNumber(5819,
                  "An WebSphere MQ Publish/Subscribe broker stream has ended abnormally.",
                  "10 : Warning",
                  "The broker stream (<insert_3>) has ended abnormally for reason <insert_1>. The broker will attempt to restart the stream. If the stream should repeatedly fail then the broker will progressively increase the time between attempts to restart the stream.",
                  "Investigate why the problem occurred and take appropriate action to correct the problem. If the problem persists, contact your IBM service representative."));
      amqMN_Ht.put("5820",new AMQMessageNumber(5820,
                  "WebSphere MQ Publish/Subscribe broker stream (<insert_3>) restarted.",
                  "0 : Information",
                  "The broker has restarted a stream that ended abnormally. This message will frequently be preceded by message AMQ5867 or AMQ5819 indicating why the stream ended.",
                  "Correct the problem."));
      amqMN_Ht.put("5821",new AMQMessageNumber(5821,
                  "WebSphere MQ Publish/Subscribe broker unable to contact parent broker.",
                  "10 : Warning",
                  "The broker has been started specifying a parent broker. The broker has been unable to send a message to the parent broker (<insert_3>) for reason <insert_1>. The broker will terminate immediately.",
                  "Investigate why the problem occurred and take appropriate action to correct the problem. The problem is likely to be caused by the parent broker name not resolving to the name of a transmission queue on the local broker."));
      amqMN_Ht.put("5822",new AMQMessageNumber(5822,
                  "WebSphere MQ Publish/Subscribe broker failed to register with parent broker.",
                  "10 : Warning",
                  "The broker has been started specifying a parent broker (<insert_3>). The broker attempted to register as a child of the parent broker, but received an exception response (<insert_1>) indicating that this was not possible. The broker will attempt to reregister as a child of the parent periodically. The child might not be able to process global publications or subscriptions correctly until this registration process has completed normally.",
                  "Investigate why the problem occurred and take appropriate action to correct the problem. The problem is likely to be caused by the parent broker not yet existing, or a problem with the SYSTEM.BROKER.INTER.BROKER.COMMUNICATIONS queue at the parent broker."));
      amqMN_Ht.put("5823",new AMQMessageNumber(5823,
                  "Exit path attribute invalid in WebSphere MQ Publish/Subscribe broker stanza.",
                  "10 : Warning",
                  "The broker exit path attribute <insert_3> is not valid. The attribute should be specified as: <path><module name>(<function name>). The broker will terminate immediately.",
                  "Correct the problem with the attribute and restart the broker."));
      amqMN_Ht.put("5824",new AMQMessageNumber(5824,
                  "WebSphere MQ Publish/Subscribe broker exit module could not be loaded.",
                  "10 : Warning",
                  "The broker exit module <insert_3> could not be loaded for reason <insert_1>:<insert_4>. The broker will terminate immediately.",
                  "Correct the problem with the broker exit module <insert_3> and restart the broker."));
      amqMN_Ht.put("5825",new AMQMessageNumber(5825,
                  "The address of the WebSphere MQ Publish/Subscribe broker exit function could not be found.",
                  "10 : Warning",
                  "The address of the broker exit function <insert_4> could not be found in module <insert_3> for reason <insert_1>:<insert_5>. The broker will terminate immediately.",
                  "Correct the problem with the broker exit function <insert_4> in module <insert_3>, and restart the broker."));
      amqMN_Ht.put("5826",new AMQMessageNumber(5826,
                  "The WebSphere MQ Publish/Subscribe broker has failed to propagate a subscription to another broker.",
                  "10 : Warning",
                  "The broker failed to propagate subscription to stream (<insert_4>) at broker (<insert_3>). Reason codes <insert_1> and <insert_2>. An application has either registered or deregistered a global subscription to stream (<insert_4>). The broker has attempted to propagate the subscription change to broker (<insert_3>) but the request has not been successful. The message broker will immediately attempt to refresh the state of the global subscriptions for stream (<insert_4>) at broker (<insert_3>). Until the subscription state has been successfully refreshed, messages published on stream (<insert_4>) through broker (<insert_3>) might not reach this broker.",
                  "Use the reason codes to investigate why the problem occurred and take appropriate action to correct the problem."));
      amqMN_Ht.put("5827",new AMQMessageNumber(5827,
                  "An WebSphere MQ Publish/Subscribe broker internal subscription has failed.",
                  "10 : Warning",
                  "The broker failed to subscribe to stream (<insert_4>) at broker (<insert_3>) with reason codes <insert_1> and <insert_2>. Related brokers learn about each others configuration by subscribing to information published by each other. A broker has discovered that one of these internal subscriptions has failed. The broker will reissue the subscription immediately. The broker cannot function correctly without knowing some information about neighboring brokers. The information that this broker has about broker (<insert_3>) is not complete and this could lead to subscriptions and publications not being propagated around the network correctly.",
                  "Investigate why the problem occurred and take appropriate action to correct the problem. The most likely cause of this failure is a problem with the SYSTEM.BROKER.CONTROL.QUEUE at broker (<insert_3>), or a problem with the definition of the route between this broker and broker (<insert_3>)."));
      amqMN_Ht.put("5828",new AMQMessageNumber(5828,
                  "WebSphere MQ Publish/Subscribe broker exit returned an ExitResponse that is not valid.",
                  "10 : Warning",
                  "The broker exit returned an ExitResponse <insert_1> that is not valid. The message has been allowed to continue and an FFST has been generated that contains the entire exit parameter structure.",
                  "Correct the problem with the broker exit."));
      amqMN_Ht.put("5829",new AMQMessageNumber(5829,
                  "Usage: <insert_3> [-m QMgrName] [-p ParentQMgrName]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5830",new AMQMessageNumber(5830,
                  "Usage: endmqbrk [-c | -i] [-m QMgrName]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5831",new AMQMessageNumber(5831,
                  "Usage: dspmqbrk [-m QMgrName]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5832",new AMQMessageNumber(5832,
                  "WebSphere MQ Publish/Subscribe broker failed to publish configuration information on SYSTEM.BROKER.ADMIN.STREAM.",
                  "10 : Warning",
                  "Related brokers learn about each others configuration by subscribing to information published by each other. A broker has discovered that one of these internal publications has failed. The broker will republish the information immediately. Brokers cannot function correctly without knowing some information about neighboring brokers. The information that neighboring brokers have of this broker might not be complete and this could lead to some subscriptions and publications not being propagated around the network.",
                  "Investigate why the problem occurred and take appropriate action to correct the problem."));
      amqMN_Ht.put("5833",new AMQMessageNumber(5833,
                  "A loop has been detected in the WebSphere MQ Publish/Subscribe broker hierarchy.",
                  "20 : Error",
                  "The broker, on queue manager (<insert_3>), introduced a loop in the broker hierarchy. This broker will terminate immediately.",
                  "Remove broker (<insert_3>) from the hierarchy, either by deleting the broker, or by removing knowledge of the broker's parent, using the clrmqbrk command."));
      amqMN_Ht.put("5834",new AMQMessageNumber(5834,
                  "Conflicting queue manager names in the WebSphere MQ Publish/Subscribe broker hierarchy.",
                  "10 : Warning",
                  "The names of the queue managers (<insert_3>) and (<insert_4>) in the broker hierarchy both start with the same 12 characters. The first 12 characters of a broker's queue manager name should be unique to ensure that no confusion arises within the broker hierarchy, and to guarantee unique message ID allocation.",
                  "Use a queue manager naming convention that guarantees uniqueness of the first 12 characters of the queue manager name."));
      amqMN_Ht.put("5835",new AMQMessageNumber(5835,
                  "WebSphere MQ Publish/Subscribe broker failed to inform its parent of a relation for reason <insert_1>.",
                  "0 : Information",
                  "The failed to notify its parent on queue manager (<insert_3>) of the relation (<insert_4>) in the broker hierarchy. The notification message will be put to the parent's dead-letter queue. A failure to notify a broker of a new relation will mean that no loop detection can be performed for the new relation.",
                  "Diagnose and correct the problem on the parent queue manager. One possible reason for this is that the parent broker does not yet exist."));
      amqMN_Ht.put("5836",new AMQMessageNumber(5836,
                  "Duplicate queue manager name located in the WebSphere MQ Publish/Subscribe hierarchy.",
                  "0 : Information",
                  "Multiple instances of the queue manager name (<insert_3>) have been located. This could either be the result of a previously resolved loop in the broker hierarchy, or multiple queue managers in the broker hierarchy having the same name.",
                  "If this broker introduced a loop in the hierarchy (typically identified by message AMQ5833), this message can be ignored. It is strongly recommended that every queue manager in a broker hierarchy has a unique name. It is not recommended that multiple queue managers use the same name."));
      amqMN_Ht.put("5837",new AMQMessageNumber(5837,
                  "WebSphere MQ Publish/Subscribe broker failed to quiesce queue (<insert_3>) for reason <insert_1>.",
                  "10 : Warning",
                  "When a broker is deleted, the broker's input queues are quiesced by making the queue get inhibited, and writing the contents of the queue to the dead-letter queue (depending upon the report options of the message). The broker was unable to quiesce the named queue for the reason shown. The attempt to delete the broker will fail.",
                  "Investigate why the problem occurred, take appropriate action to correct the problem, and reissue the dltmqbrk command. Likely reasons include the queue being open for input by another process, there being no dead-letter queue defined at this queue manager, or the operator setting the queue to get inhibited while the dltmqbrk command is running. If there is no dead-letter queue defined, the reason will be reported as MQRC_UNKNOWN_OBJECT_NAME. If the problem occurs because there is no dead-letter queue defined at this broker, the operator can either define a dead-letter queue, or manually empty the queue causing the problem."));
      amqMN_Ht.put("5837",new AMQMessageNumber(5837,
                  "(iSeries) WebSphere MQ Publish/Subscribe broker failed to quiesce queue.",
                  "10 : Warning",
                  "When a broker is deleted, the broker's input queues are quiesced by making the queue get inhibited, and writing the contents of the queue to the dead-letter queue (depending upon the report options of the message). The broker was unable to quiesce the queue (<insert_3>) for reason <insert_1>. The attempt to delete the broker will fail.",
                  "Investigate why the problem occurred, take appropriate action to correct the problem, and reissue the dltmqbrk command. Likely reasons include the queue being open for input by another process, there being no dead-letter queue defined at this queue manager, or the operator setting the queue to get inhibited while the dltmqbrk command is running. If there is no dead-letter queue defined, the reason will be reported as MQRC_UNKNOWN_OBJECT_NAME. If the problem occurs because there is no dead-letter queue defined at this broker, the operator can either define a dead-letter queue, or manually empty the queue causing the problem."));
      amqMN_Ht.put("5838",new AMQMessageNumber(5838,
                  "WebSphere MQ Publish/Subscribe broker cannot be deleted.",
                  "10 : Warning",
                  "The broker cannot be deleted as child (<insert_3>) is still registered. A broker cannot be deleted until all other brokers that have registered as children of that broker, have deregistered as its children.",
                  "Use the clrmqbrk and dltmqbrk commands to change the broker topology so that broker (<insert_3>) is not registered as a child of the broker being deleted."));
      amqMN_Ht.put("5839",new AMQMessageNumber(5839,
                  "WebSphere MQ Publish/Subscribe broker received an unexpected inter-broker communication.",
                  "10 : Warning",
                  "A broker has received an inter-broker communication that it did not expect. The message was sent by broker (<insert_3>). The message will be processed according to the report options in that message. The most likely reason for this message is that the broker topology has been changed while inter-broker communication messages were in transit (for example, on a transmission queue) and that a message relating to the previous broker topology has arrived at a broker in the new topology. This message may be accompanied by an informational FFST including details of the unexpected communication.",
                  "If the broker topology has changed and the broker named in the message is no longer related to the broker issuing this message, this message can be ignored. If the clrmqbrk command was issued to unilaterally remove knowledge of broker (<insert_3>) from this broker, the clrmqbrk command should also be used to remove knowledge of this broker from broker (<insert_3>). If the clrmqbrk command was issued to unilaterally remove knowledge of this broker from broker (<insert_3>), the clrmqbrk command should also be used to remove knowledge of broker (<insert_3>) at this broker."));
      amqMN_Ht.put("5840",new AMQMessageNumber(5840,
                  "WebSphere MQ Publish/Subscribe broker unable to delete queue.",
                  "10 : Warning",
                  "The broker has failed to delete the queue (<insert_3>) for reason <insert_2>. The broker typically attempts to delete queues during dltmqbrk processing, in which case the dltmqbrk command will fail.",
                  "The most likely reason for this error is that some other process has the queue open. Determine why the queue cannot be deleted, remove the inhibitor, and retry the failed operation. In a multi-broker environment, it is likely that a message channel agent might have queues open, which the broker needs to delete for a dltmqbrk command to complete."));
      amqMN_Ht.put("5841",new AMQMessageNumber(5841,
                  "WebSphere MQ Publish/Subscribe broker (<insert_3>) deleted.",
                  "0 : Information",
                  "The broker (<insert_3>) has been deleted using the dltmqbrk command.",
                  "None."));
      amqMN_Ht.put("5842",new AMQMessageNumber(5842,
                  "WebSphere MQ Publish/Subscribe broker (<insert_3>) cannot be deleted for reason <insert_1>:<insert_5>.",
                  "20 : Error",
                  "An attempt has been made to delete the broker (<insert_3>) but the request has failed for reason <insert_1>:<insert_5>.",
                  "Determine why the dltmqbrk command cannot complete successfully. The message logs for the queue manager might contain more detailed information on why the broker cannot be deleted. Resolve the problem that is preventing the command from completing and reissue the dltmqbrk command."));
      amqMN_Ht.put("5842",new AMQMessageNumber(5842,
                  "(iSeries) WebSphere MQ Publish/Subscribe broker cannot be deleted.",
                  "20 : Error",
                  "An attempt has been made to delete the WebSphere MQ Publish/Subscribe broker (<insert_3>) but the request has failed for reason <insert_1>:<insert_5>.",
                  "Determine why the dltmqbrk command cannot complete successfully. The message logs for the queue manager might contain more detailed information on why the broker cannot be deleted. Resolve the problem that is preventing the command from completing and reissue the dltmqbrk command."));
      amqMN_Ht.put("5843",new AMQMessageNumber(5843,
                  "WebSphere MQ Publish/Subscribe broker (<insert_3>) cannot be started as it is partially deleted.",
                  "10 : Warning",
                  "An attempt has been made to start a broker that is in a partially deleted state. An earlier attempt to delete the broker has failed. The broker deletion must be completed before the broker will be allowed to restart. When broker deletion is successful, message AMQ5841 is issued, indicating that the broker has been deleted. If this message is not received on completion of a dltmqbrk command, the broker deletion has not been completed and the command will have to be reissued.",
                  "Investigate why the earlier attempt to delete the broker failed. Resolve the problem and reissue the dltmqbrk command."));
      amqMN_Ht.put("5843",new AMQMessageNumber(5843,
                  "(iSeries) WebSphere MQ Publish/Subscribe broker cannot be started as it is partially deleted.",
                  "10 : Warning",
                  "An attempt has been made to start the broker <insert_3> that is in a partially deleted state. An earlier attempt to delete the broker has failed. The broker deletion must be completed before the broker will be allowed to restart. When broker deletion is successful, message AMQ5841 is issued, indicating that the broker has been deleted. If this message is not received on completion of a dltmqbrk command, the broker deletion has not been completed and the command will have to be reissued.",
                  "Investigate why the earlier attempt to delete the broker failed. Resolve the problem and reissue the dltmqbrk command."));
      amqMN_Ht.put("5844",new AMQMessageNumber(5844,
                  "The relation between two WebSphere MQ Publish/Subscribe brokers is unknown.",
                  "10 : Warning",
                  "The clrmqbrk command has been issued in an attempt to remove a brokers knowledge of a relation of that broker. The relative (<insert_4>) is unknown at broker (<insert_3>). If the \"-p\" flag was specified, the broker does not currently have a parent. If the \"-c\" flag was specified, the broker does not recognize the named child.",
                  "Investigate why the broker is unknown."));
      amqMN_Ht.put("5845",new AMQMessageNumber(5845,
                  "Usage: dltmqbrk -m QMgrName",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5846",new AMQMessageNumber(5846,
                  "Usage: clrmqbrk -p | -c ChildQMgrName -m QMgrName",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5847",new AMQMessageNumber(5847,
                  "WebSphere MQ Publish/Subscribe broker (<insert_3>) has removed knowledge of relation (<insert_4>).",
                  "0 : Information",
                  "The clrmqbrk command has been used to remove knowledge of broker (<insert_4>) from broker (<insert_3>).",
                  "None."));
      amqMN_Ht.put("5847",new AMQMessageNumber(5847,
                  "(iSeries) WebSphere MQ Publish/Subscribe broker relation removed.",
                  "0 : Information",
                  "The clrmqbrk command has been used to remove knowledge of broker (<insert_4>) from broker (<insert_3>).",
                  "None."));
      amqMN_Ht.put("5848",new AMQMessageNumber(5848,
                  "WebSphere MQ Publish/Subscribe broker (<insert_3>) has failed to remove references to relation (<insert_4>) for reason <insert_1>:<insert_5>.",
                  "20 : Error",
                  "An attempt has been made to remove references to broker (<insert_4>) from broker (<insert_3>) using the clrmqbrk command, but the request has been unsuccessful.",
                  "Determine why the clrmqbrk command cannot complete successfully. The message logs for the queue manager might contain more detailed information on why the broker cannot be deleted. Resolve the problem that is preventing the command from completing and then reissue the clrmqbrk command."));
      amqMN_Ht.put("5848",new AMQMessageNumber(5848,
                  "(iSeries) WebSphere MQ Publish/Subscribe broker has failed to remove references to a related broker.",
                  "20 : Error",
                  "An attempt has been made to remove references to broker (<insert_4>) from broker (<insert_3>) using the clrmqbrk command, but the request has been unsuccessful for reason <insert_1>:<insert_5>.",
                  "Determine why the clrmqbrk command cannot complete successfully. The message logs for the queue manager might contain more detailed information on why the broker cannot be deleted. Resolve the problem that is preventing the command from completing and then reissue the clrmqbrk command."));
      amqMN_Ht.put("5849",new AMQMessageNumber(5849,
                  "WebSphere MQ Publish/Subscribe broker may not change parent.",
                  "10 : Warning",
                  "An attempt has been made to start broker (<insert_3>), nominating broker (<insert_4>) as its parent. The broker (<insert_3>) has previously been started, nominating broker (<insert_5>) as its parent. The strmqbrk command cannot be used to change an existing relationship.",
                  "Do not attempt to change the broker topology by using the strmqbrk command. The dltmqbrk and clrmqbrk commands are the only supported means of changing the broker topology. Refer to the documentation of those commands for guidance on changing the broker topology."));
      amqMN_Ht.put("5850",new AMQMessageNumber(5850,
                  "WebSphere MQ Publish/Subscribe broker interrupted while creating queue.",
                  "10 : Warning",
                  "The broker was interrupted while creating queue (<insert_3>) for user ID (<insert_4>). When the broker creates a queue, it first creates the queue with default security attributes and it then sets the appropriate security attributes for the queue. If the broker should be interrupted during this operation (for example the queue manager is shut down), the broker cannot reliably detect that the security attributes have not been set correctly. The broker was creating a queue, but was interrupted before it could complete creation of the queue and setting the initial authority. If the interrupt occurred before the initial authority of the queue could be set, it might be necessary for the operator to set the appropriate authorities using the setmqaut command.",
                  "Confirm that the named queue has the appropriate security attributes and modify them as necessary."));
      amqMN_Ht.put("5851",new AMQMessageNumber(5851,
                  "WebSphere MQ Publish/Subscribe broker interrupted while creating internal queue.",
                  "10 : Warning",
                  "The broker was interrupted while creating internal queue (<insert_3>) for user ID (<insert_4>). When the broker creates an internal queue, it first creates the queue with default security attributes and it then sets the appropriate security attributes for the queue. If the broker should be interrupted during this operation (for example the queue manager is shut down), the broker attempts to delete and redefine the queue. If the internal queue is available to users (for example, the default stream or the administration stream), it is possible that a user will put a message on the queue while it is in this invalid state, or that a user application has the queue open. In this situation the broker does not automatically redefine the queue and cannot be restarted until the queue has been emptied or closed.",
                  "Examine any messages on the named queue and take appropriate action to remove them from the queue. Ensure that no applications have the queue open."));
      amqMN_Ht.put("5852",new AMQMessageNumber(5852,
                  "WebSphere MQ Publish/Subscribe broker failed to propagate delete publication command.",
                  "0 : Information",
                  "The broker failed to propagate delete publication command for stream (<insert_3>) to related broker (<insert_4>) for reason <insert_1>. When an application issues a delete publication command to delete a global publication, the command has to be propagated to all brokers in the sub-hierarchy supporting the stream. The broker reporting the error has failed to forward a delete publication command to a related broker (<insert_4>) who supports stream (<insert_3>). Delete publication commands are propagated without MQRO_DISCARD_MSG and the command message might have been written to a dead-letter queue. The topic for which the delete publication has failed is (<insert_5>).",
                  "If the delete publication has failed because the stream has been deleted at the related broker, this message can be ignored. Investigate why the delete publication has failed and take the appropriate action to recover the failed command."));
      amqMN_Ht.put("5853",new AMQMessageNumber(5853,
                  "WebSphere MQ Publish/Subscribe failed to propagate a delete publication command.",
                  "0 : Information",
                  "The broker failed to propagate a delete publication command for stream (<insert_3>) to a previously related broker. When an application issues a delete publication command to delete a global publication, the command is propagated to all brokers in the sub-hierarchy supporting the stream. The broker topology was changed after deleting the publication, but before a broker removed by the topology change processed the propagated delete publication message. The topic for which the delete publication has failed is (<insert_5>).",
                  "It is the user's responsibility to quiesce broker activity before changing the broker topology using the clrmqbrk command. Investigate why this delete publication activity was not quiesced. The delete publication command will have been written to the dead-letter queue at the broker that was removed from the topology. In this case, further action might be necessary to propagate the delete publication command that was not quiesced before the clrmqbrk command was issued. If this message occurs as a result of the dltmqbrk command, the publication will have been deleted as a result of the dltmqbrk command, and the delete publication message will have been written to the dead-letter queue at the queue manager where the broker was deleted. In this case the delete publication message on the dead-letter queue can be discarded."));
      amqMN_Ht.put("5854",new AMQMessageNumber(5854,
                  "WebSphere MQ Publish/Subscribe broker failed to propagate a delete publication command.",
                  "0 : Information",
                  "When an application issues a delete publication command to delete a global publication, the command has to be propagated to all brokers in the sub-hierarchy supporting the stream. At the time the delete publication was propagated, broker (<insert_4>) was a known relation of this message broker supporting stream (<insert_3>). Before the delete publication command arrived at the related broker, the broker topology was changed so that broker (<insert_4>) no longer supported stream (<insert_3>). The topic for which the delete publication has failed is (<insert_5>).",
                  "It is the user's responsibility to quiesce broker activity before changing the stream topology of the broker. Investigate why this delete publication activity was not quiesced. The delete publication command will have been written to the dead-letter queue at broker (<insert_4>)."));
      amqMN_Ht.put("5855",new AMQMessageNumber(5855,
                  "WebSphere MQ Publish/Subscribe broker ended.",
                  "10 : Warning",
                  "An attempt has been made to run the broker (<insert_3>) but the broker has ended for reason <insert_1>:<insert_5>.",
                  "Determine why the broker ended. The message logs for the queue manager might contain more detailed information on why the broker cannot be started. Resolve the problem that is preventing the command from completing and reissue the strmqbrk command."));
      amqMN_Ht.put("5856",new AMQMessageNumber(5856,
                  "Broker publish command message cannot be processed. Reason code <insert_1>.",
                  "10 : Warning",
                  "The WebSphere MQ Publish/Subscribe broker failed to process a publish message for stream (<insert_3>). The broker was unable to write the publication to the dead-letter queue and was not permitted to discard the publication. The broker will temporarily stop the stream and will restart the stream and consequently retry the publication after a short interval.",
                  "Investigate why the error has occurred and why the publication cannot be written to the dead-letter queue. Either manually remove the publication from the stream queue, or correct the problem that is preventing the broker from writing the publication to the dead-letter queue."));
      amqMN_Ht.put("5857",new AMQMessageNumber(5857,
                  "Broker control command message cannot be processed. Reason code <insert_1>.",
                  "10 : Warning",
                  "The WebSphere MQ Publish/Subscribe broker failed to process a command message on the SYSTEM.BROKER.CONTROL.QUEUE. The broker was unable to write the command message to the dead-letter queue and was not permitted to discard the command message. The broker will temporarily stop the stream and will restart the stream and consequently retry the command message after a short interval. Other broker control commands cannot be processed until this command message has been processed successfully or removed from the control queue.",
                  "Investigate why the error has occurred and why the command message cannot be written to the dead-letter queue. Either, manually remove the command message from the stream queue, or correct the problem that is preventing the broker from writing the command message to the dead-letter queue."));
      amqMN_Ht.put("5858",new AMQMessageNumber(5858,
                  "Broker could not send publication to subscriber queue.",
                  "10 : Warning",
                  "A failure has occurred sending a publication to subscriber queue (<insert_4>) at queue manager (<insert_3>) for reason <insert_1>. The broker configuration options prevent it from recovering from this failure by discarding the publication or by sending it to the dead-letter queue. Instead the broker will back out the unit of work under which the publication is being sent and retry the failing command message a fixed number of times. If the problem still persists, the broker will then attempt to recover by failing the command message with a negative reply message. If the issuer of the command did not request negative replies, the broker will either discard or send to the dead-letter queue the failing command message. If the broker configuration options prevent this, the broker will restart the affected stream, which will reprocess the failing command message again. This behavior will be repeated until such time as the failure is resolved. During this time the stream will be unable to process further publications or subscriptions.",
                  "Usually the failure will be due to a transient resource problem, for example, the subscriber queue, or an intermediate transmission queue, becoming full. Use reason code <insert_1> to determine what remedial action is required. If the problem persists for a long time, you will notice the stream being continually restarted by the broker. Evidence of this occurring will be a large number of AMQ5820 messages, indicating stream restart, being written to the error logs. In such circumstances, manual intervention will be required to allow the broker to dispose of the failing publication. To do this, you will need to end the broker using the endmqbrk command and restart it with appropriate disposition options. This will allow the publication to be sent to the rest of the subscribers, while allowing the broker to discard or send to the dead-letter queue the publication that could not be sent."));
      amqMN_Ht.put("5859",new AMQMessageNumber(5859,
                  "WebSphere MQ Publish/Subscribe broker stream is terminating due to an internal resource problem.",
                  "10 : Warning",
                  "The broker stream (<insert_3>) has run out of internal resources and will terminate with reason code <insert_1>. If the command in progress was being processed under syncpoint control, it will be backed out and retried when the stream is restarted by the broker. If the command was being processed out of syncpoint control, it will not be able to be retried when the stream is restarted.",
                  "This message should only be issued in very unusual circumstances. If this message is issued repeatedly for the same stream, and the stream is not especially large in terms of subscriptions, topics, and retained publications, save all generated diagnostic information and contact your IBM Support Center for problem resolution."));
      amqMN_Ht.put("5862",new AMQMessageNumber(5862,
                  "WebSphere MQ Publish/Subscribe broker for queue manager <insert_3> migrating.",
                  "0 : Information",
                  "The dspmqbrk command has been issued to query the state of the broker. The broker is currently being migrated.",
                  "None."));
      amqMN_Ht.put("5863",new AMQMessageNumber(5863,
                  "WebSphere MQ Integrator broker not ready for migration. See message logs for guidance.",
                  "10 : Warning",
                  "The migmqbrk command was unsuccessful because the WebSphere MQ Integrator broker was not ready to accept messages. The state of the WebSphere MQ Publish/Subscribe message broker is exported to the WebSphere MQ Integrator broker in a series of messages sent to queue SYSTEM.BROKER.INTERBROKER.QUEUE. Before migration commences the WebSphere MQ Publish/Subscribe broker checks whether the WebSphere MQ Integrator broker is ready to accept messages on this queue. This check has failed for reason <insert_1> so migration has been abandoned.",
                  "Reason code <insert_1> should be used to determine the nature of the problem. A value of 1 means that queue SYSTEM.BROKER.INTERBROKER.QUEUE does not exist. This is probably because no WebSphere MQ Integrator broker has been defined yet on this queue manager. A value of 2 means that the WebSphere MQ Integrator broker does not have the queue open probably because it hasn't been started or the first message flow has yet to be deployed for it. If both of these steps have been taken then the WebSphere MQ Integrator broker may have been created incorrectly. In particular, it should have been created in migration mode. If the broker was not created with the migration flag set then it will need to be deleted and recreated before migration can commence. Any other value in the reason code will need to be reported to your IBM Support Center for problem resolution. Note that until the problem has been resolved the WebSphere MQ Publish/Subscribe broker can still be restarted with the the strmqbrk command."));
      amqMN_Ht.put("5864",new AMQMessageNumber(5864,
                  "Broker reply message could not be sent. The command will be retried.",
                  "10 : Warning",
                  "While processing a publish/subscribe command, the WebSphere MQ Publish/Subscribe broker could not send a reply message to queue (<insert_4>) at queue manager (<insert_3>) for reason <insert_1>. The broker was also unable to write the message to the dead-letter queue. Since the command is being processed under syncpoint control, the broker will attempt to retry the command in the hope that the problem is only of a transient nature. If, after a set number of retries, the reply message still could not be sent, the command message will be discarded if the report options allow it. If the command message is not discardable, the stream will be restarted, and processing of the command message recommenced.",
                  "Use reason code <insert_1> to determine what remedial action is required. If the failure is due to a resource problem (for example, a queue being full), you might find that the problem has already cleared itself. If not, this message will be issued repeatedly each time the command is retried. In this case you are strongly advised to define a dead-letter queue to receive the reply message so that the broker can process other commands while the problem is being investigated. Check the application from which the command originated and ensure that it is specifying its reply-to queue correctly."));
      amqMN_Ht.put("5865",new AMQMessageNumber(5865,
                  "Broker reply message could not be sent.",
                  "10 : Warning",
                  "While processing a publish/subscribe command, the WebSphere MQ Publish/Subscribe broker could not send a reply message to queue (<insert_4>) at queue manager (<insert_3>) for reason <insert_1>. The broker was also unable to write the message to the dead-letter queue. As the command is not being processed under syncpoint control, the broker is not able to retry the command.",
                  "Use reason code <insert_1> to determine what remedial action is required. If the failure is due to a resource problem (for example, a queue being full), you might find that the problem has already cleared itself. If not, check the application from which the command originated and ensure that it is specifying its reply-to queue correctly. You might find that defining a dead-letter queue to capture the reply message on a subsequent failure will help you with this task."));
      amqMN_Ht.put("5866",new AMQMessageNumber(5866,
                  "Broker command message has been discarded. Reason code <insert_1>.",
                  "10 : Warning",
                  "The WebSphere MQ Publish/Subscribe broker failed to process a publish/subscribe command message, which has now been discarded. The broker will begin to process new command messages again.",
                  "Look for previous error messages to indicate the problem with the command message. Correct the problem to prevent the failure from happening again."));
      amqMN_Ht.put("5867",new AMQMessageNumber(5867,
                  "WebSphere MQ Publish/Subscribe broker stream has ended abnormally.",
                  "10 : Warning",
                  "The broker stream (<insert_3>) has ended abnormally for reason <insert_1>. The broker will attempt to restart the stream. If the stream should repeatedly fail, the broker will progressively increase the time between attempts to restart the stream.",
                  "Use the reason code <insert_1> to investigate why the problem occurred. A reason code of 1 indicates that the stream ended because a command message could not be processed successfully. Look in the error logs for earlier messages to determine the reason why the command message failed. A reason code of 2 indicates that the stream ended because the broker exit could not be loaded. Until the problem with the broker exit has been resolved, the stream will continue to fail."));
      amqMN_Ht.put("5868",new AMQMessageNumber(5868,
                  "User is no longer authorized to subscribe to stream.",
                  "0 : Information",
                  "The broker has attempted to publish a publication to a subscriber, but the subscriber no longer has browse authority to stream queue (<insert_4>). The publication is not sent to the subscriber and his subscription is deregistered. An event publication containing details of the subscription that was removed is published on SYSTEM.BROKER.ADMIN.STREAM. While user ID (<insert_3>) remains unauthorized, the broker will continue to deregister subscriptions associated with that user ID.",
                  "If the authority of user ID (<insert_3>) was intentionally removed, consider removing all of that user IDs subscriptions immediately by issuing an MQCMD_DEREGISTER_SUBSCRIBER command, specifying the MQREGO_DEREGISTER_ALL option on the subscriber's behalf. If the authority was revoked accidentally, reinstate it, but be aware that some, if not all, of the subscriber's subscriptions will have been deregistered by the broker."));
      amqMN_Ht.put("5869",new AMQMessageNumber(5869,
                  "WebSphere MQ Publish/Subscribe broker is checkpointing registrations.",
                  "0 : Information",
                  "A large number of changes have been made to the publisher and subscriber registrations of stream (<insert_3>). These changes are being checkpointed, in order to minimize both stream restart time and the amount of internal queue space being used.",
                  "None."));
      amqMN_Ht.put("5870",new AMQMessageNumber(5870,
                  "(Unexpected Error)",
                  "0 : Information",
                  "N/A",
                  "N/A"));
      amqMN_Ht.put("5871",new AMQMessageNumber(5871,
                  "(Resource Problem)",
                  "0 : Information",
                  "N/A",
                  "N/A"));
      amqMN_Ht.put("5872",new AMQMessageNumber(5872,
                  "(WebSphere MQ Publish/Subscribe broker has a known child)",
                  "0 : Information",
                  "N/A",
                  "N/A"));
      amqMN_Ht.put("5873",new AMQMessageNumber(5873,
                  "(WebSphere MQ Publish/Subscribe broker active)",
                  "0 : Information",
                  "N/A",
                  "N/A"));
      amqMN_Ht.put("5874",new AMQMessageNumber(5874,
                  "(One or more queues could not be quiesced)",
                  "0 : Information",
                  "N/A",
                  "N/A"));
      amqMN_Ht.put("5875",new AMQMessageNumber(5875,
                  "WebSphere MQ Publish/Subscribe broker cannot write a message to the dead-letter queue.",
                  "10 : Warning",
                  "The broker attempted to put a message to the dead-letter queue (<insert_3>) but the message could not be written to the dead-letter queue for reason <insert_1>:<insert_4>. The message was being written to the dead-letter queue with a reason of <insert_2>:<insert_5>.",
                  "Determine why the message cannot be written to the dead-letter queue. Also, if the message was not deliberately written to the dead-letter queue, for example by a message broker exit, determine why the message was written to the dead-letter queue and resolve the problem that is preventing the message from being sent to its destination."));
      amqMN_Ht.put("5876",new AMQMessageNumber(5876,
                  "A parent conflict has been detected in the WebSphere MQ Publish/Subscribe broker hierarchy.",
                  "20 : Error",
                  "The broker (<insert_3>) has been started, naming this broker as its parent. This broker was started naming broker (<insert_3>) as its parent. The broker will send an exception message to broker (<insert_3>) indicating that a conflict has been detected. The most likely reason for this message is that the broker topology has been changed while inter-broker communication messages were in transit (for example, on a transmission queue) and that a message relating to the previous broker topology has arrived at a broker in the new topology. This message may be accompanied by an informational FFST including details of the unexpected communication.",
                  "If the broker topology has changed and the broker named in the message no longer identifies this broker as its parent, this message can be ignored - for example, if the command \"clrmqbrk -m <insert_3> -p\" was issued. If broker (<insert_3>) has been defined as this broker's parent, and this broker has been defined as broker (<insert_3>)'s parent, the clrmqbrk or the dltmqbrk commands should be used to resolve the conflict."));
      amqMN_Ht.put("5877",new AMQMessageNumber(5877,
                  "WebSphere MQ Publish/Subscribe broker stream has ended abnormally.",
                  "10 : Warning",
                  "A broker stream (<insert_3>) has ended abnormally for reason <insert_1>. The broker recovery routines failed to reset the stream state and the stream cannot be restarted automatically.",
                  "Investigate why the stream failed and why the broker's recovery routine could not recover following the failure. Take appropriate action to correct the problem. Depending upon the broker configuration and the nature of the problem it will be necessary to restart either the broker,or both the queue manager and the broker, to make the stream available. If the problem persists contact your IBM service representative."));
      amqMN_Ht.put("5878",new AMQMessageNumber(5878,
                  "WebSphere MQ Publish/Subscribe broker recovery failure detected.",
                  "10 : Warning",
                  "An earlier problem has occurred with the broker, and either a stream has been restarted or the broker has been restarted. The restarted stream or broker has detected that the previous instance of the stream or broker did not clean up successfully and the restart will fail.",
                  "Investigate the cause of the failure that caused a stream or broker restart to be necessary, and why the broker or stream was unable to clean up its resources following the failure. When the broker processes with a non trusted routing exit (RoutingExitConnectType=STANDARD), the broker runs in a mode where it is more tolerant of unexpected failures and it is likely that the restart will succeed after a short delay. In the case of a stream restart, the broker will normally periodically retry the failing restart. In the case of a broker restart, it will be necessary to manually retry the broker restart after a short delay. When the broker processes without a routing exit, or with a trusted routine exit (RoutingExitConnectType=FASTPATH), the broker runs in a mode where it is less tolerant of unexpected failures and a queue manager restart will be necessary to resolve this problem. When the broker is running in this mode, it is important that the broker processes are not subjected to unnecessary asynchronous interrupts, for example, kill. If the problem persists, contact your IBM service representative."));
      amqMN_Ht.put("5879",new AMQMessageNumber(5879,
                  "WebSphere MQ Publish/Subscribe broker has been migrated.",
                  "10 : Warning",
                  "The command was unsuccessful because the MQ Pub/Sub broker at queue manager <insert_3> has been migrated. After migration the only command which can be issued against the migrated broker is the dltmqbrk command.",
                  "Issue the dltmqbrk command to delete the migrated broker."));
      amqMN_Ht.put("5880",new AMQMessageNumber(5880,
                  "User is no longer authorized to subscribe to stream.",
                  "0 : Information",
                  "The broker has attempted to publish a publication to a subscriber but the subscriber no longer has altusr authority to stream queue (<insert_4>). The publication is not sent to the subscriber and that user IDs subscription is deregistered. An event publication containing details of the subscription that was removed is published on SYSTEM.BROKER.ADMIN.STREAM. While user ID (<insert_3>) remains unauthorized, the broker will continue to deregister subscriptions associated with that user ID.",
                  "If the authority of user ID (<insert_3>) was intentionally removed, consider removing subscriptions immediately by issuing an MQCMD_DEREGISTER_SUBSCRIBER command for the appropriate topics on the subscriber's behalf. If the authority was revoked accidentally, reinstate it, but be aware that some, if not all, of the subscriber's subscriptions will have been deregistered by the broker."));
      amqMN_Ht.put("5881",new AMQMessageNumber(5881,
                  "The WebSphere MQ Publish/Subscribe broker configuration parameter combination <insert_1> is not valid.",
                  "20 : Error",
                  "A combination of Broker stanzas in the queue manager initialization file is not valid. The broker will not operate until this has been corrected. An combination of (1) indicates that SyncPointIfPersistent has been set to TRUE and DiscardNonPersistentInputMsg has been set to FALSE. DiscardNonPersistentInputMsg must be set to TRUE when SyncPointIfPersistent is set to TRUE. An combination of (2) indicates that SyncPointIfPersistent has been set to TRUE and DiscardNonPersistentResponse has been set to FALSE. DiscardNonPersistentResponse must be set to TRUE when SyncPointIfPersistent is set to TRUE. An combination of (3) indicates that SyncPointIfPersistent has been set to TRUE and DiscardNonPersistentPublication has been set to FALSE. DiscardNonPersistentPublication must be set to TRUE when SyncPointIfPersistent is set to TRUE.",
                  "Alter the message broker stanzas to comply with the above rules and retry the command."));
      amqMN_Ht.put("5881",new AMQMessageNumber(5881,
                  " (Windows)The WebSphere MQ Publish/Subscribe broker configuration parameter combination <insert_1> is not valid.",
                  "20 : Error",
                  "A combination of Broker parameters in the broker configuration information is not valid. The broker will not operate until this has been corrected. An combination of (1) indicates that SyncPointIfPersistent has been set to TRUE and DiscardNonPersistentInputMsg has been set to FALSE. DiscardNonPersistentInputMsg must be set to TRUE when SyncPointIfPersistent is set to TRUE. An combination of (2) indicates that SyncPointIfPersistent has been set to TRUE and DiscardNonPersistentResponse has been set to FALSE. DiscardNonPersistentResponse must be set to TRUE when SyncPointIfPersistent is set to TRUE. An combination of (3) indicates that SyncPointIfPersistent has been set to TRUE and DiscardNonPersistentPublication has been set to FALSE. DiscardNonPersistentPublication must be set to TRUE when SyncPointIfPersistent is set to TRUE.",
                  "Alter the message broker configuration information using the cfgmqbrk tool to comply with the above rules and retry the command."));
      amqMN_Ht.put("5882",new AMQMessageNumber(5882,
                  "WebSphere MQ Publish/Subscribe broker has written a message to the dead-letter queue.",
                  "10 : Warning",
                  "The broker has written a message to the dead-letter queue (<insert_3>) for reason <insert_1>:<insert_5>. Note. To save log space, after the first occurrence of this message for stream (<insert_4>), it will only be written periodically.",
                  "If the message was not deliberately written to the dead-letter queue, for example by a message broker exit, determine why the message was written to the dead-letter queue, and resolve the problem that is preventing the message from being sent to its destination."));
      amqMN_Ht.put("5883",new AMQMessageNumber(5883,
                  "WebSphere MQ Publish/Subscribe broker state not recorded.",
                  "10 : Warning",
                  "The broker state on stream (<insert_3>) not recorded while processing a publication outside of syncpoint. A nonpersistent publication has requested a change to either a retained message or a publisher registration. This publication is being processed outside of syncpoint because the broker has been configured with the SyncPointIfPersistent option set. A failure has occurred hardening either the publisher registration or the retained publication to the broker's internal queue. All state changes attempted as a result of this publication will be backed-out. Processing of the publication will continue and the broker will attempt to deliver it to all subscribers.",
                  "Investigate why the failure occurred. It is probably due to a resource problem occurring on the broker. The most likely cause is 'queue full' on a broker queue. If your publications also carry state changes, you are advised to send them either as persistent publications or turn off the SyncPointIfPersistent option. In this way, they will be carried out under syncpoint and the broker can retry them in the event of a failure such as this."));
      amqMN_Ht.put("5884",new AMQMessageNumber(5884,
                  "WebSphere MQ Publish/Subscribe broker control queue is not a local queue.",
                  "10 : Warning",
                  "WebSphere MQ Publish/Subscribe has detected that the queue 'SYSTEM.BROKER.CONTROL.QUEUE' exists and is not a local queue. This makes the queue unsuitable for use as the control queue of the broker. The broker will terminate immediately.",
                  "Delete the definition of the existing queue and, if required, re-create the queue to be of type MQQT_LOCAL. If you do not re-create the queue the broker will automatically create one of the correct type when started."));
      amqMN_Ht.put("5885",new AMQMessageNumber(5885,
                  "Usage: migmqbrk -m QMgrName",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("5886",new AMQMessageNumber(5886,
                  "WebSphere MQ Publish/Subscribe broker is being migrated.",
                  "10 : Warning",
                  "The command cannot be issued at this time because the MQ Pub/Sub broker at queue manager <insert_3> is being migrated.",
                  "Once migration has commenced then the only command which can be issued against the MQ Pub/Sub broker is the endmqbrk command to cancel the migration. Once the broker has ended if migration did not complete then it can be reattempted using the migmqbrk command again. Alternatively it can be cancelled by restarting the broker using the strmqbrk command."));
      amqMN_Ht.put("5887",new AMQMessageNumber(5887,
                  "Migration started for stream <insert_3>",
                  "0 : Information",
                  "Migration of stream <insert_3> has started.",
                  "None."));
      amqMN_Ht.put("5888",new AMQMessageNumber(5888,
                  "Migration complete for stream <insert_3>",
                  "0 : Information",
                  "All of the state of stream <insert_3> has been exported to the WebSphere MQ Integrator broker.",
                  "None."));
      amqMN_Ht.put("5889",new AMQMessageNumber(5889,
                  "WebSphere MQ Publish/Subscribe broker has been successfully migrated. It will now be deleted.",
                  "0 : Information",
                  "Migration of the broker has completed successfully. The broker will now be deleted.",
                  "The broker is no longer startable. If deletion fails then the dltmqbrk command will need to be re-issued to complete its deletion."));
      amqMN_Ht.put("5890",new AMQMessageNumber(5890,
                  "The migration of the WebSphere MQ Publish/Subscribe broker has failed.",
                  "10 : Warning",
                  "The WebSphere MQ Publish/Subscribe broker is being migrated. During this migration all persistent state, for example subscriptions, are exported to the WebSphere MQ Integrator broker as a series of messages sent to queue <insert_3>. A migration message could not be written to this queue for reason <insert_1>.",
                  "Use the MQPUT failure code <insert_1> to determine why the message cannot be written to the queue. The reason code could indicate that the queue manager is terminating in which case the migmqbrk command will need to be re-issued after the queue manager has restarted. Alternatively there may be a problem with the queue which may need to be rectified before migration can be attempted again."));
      amqMN_Ht.put("5891",new AMQMessageNumber(5891,
                  "WebSphere MQ Publish/Subscribe broker has failed to receive a reply while exporting its state to WebSphere MQ Integrator.",
                  "10 : Warning",
                  "The WebSphere MQ Publish/Subscribe broker is being migrated. During this migration all persistent state, for example subscriptions, are exported to the WebSphere MQ Integrator broker as a series of messages. A reply message for one of the migration messages could not be retrieved from queue <insert_3> for reason <insert_1>. The migration of the WebSphere MQ Publish/Subscribe broker has failed.",
                  "Use the MQGET failure code <insert_3> to determine why the reply message could not be received from the reply queue. The reason code could indicate that the queue manager is terminating in which the migmqbrk command will need to be re-issued after the queue manager has restarted. A reason code of 2033 indicates that no reply message was received within a 30 second wait interval. In this case the problem is more likely to have occurred at the WebSphere MQ Integrator broker. Check for error messages issued at the WebSphere MQ Integrator broker."));
      amqMN_Ht.put("5892",new AMQMessageNumber(5892,
                  "Migration of stream <insert_3> has failed for reason <insert_1>:<insert_4>.",
                  "0 : Information",
                  "Migration of stream <insert_3> has failed.",
                  "Use reason code <insert_1> to investigate the reason for the failure. Once the problem has been resolved, re-issue the migmqbrk command to retry migration."));
      amqMN_Ht.put("5892",new AMQMessageNumber(5892,
                  "(iSeries) Migration of stream <insert_3> has failed.",
                  "0 : Information",
                  "Migration of stream <insert_3> has failed for reason <insert_1>:<insert_4>.",
                  "Use reason code <insert_1> to investigate the reason for the failure. Once the problem has been resolved, re-issue the migmqbrk command to retry migration."));
      amqMN_Ht.put("5893",new AMQMessageNumber(5893,
                  "WebSphere MQ Publish/Subscribe broker (<insert_3>) cannot be migrated for reason <insert_1>:<insert_5>.",
                  "20 : Error",
                  "An attempt has been made to migrate the WebSphere MQ Publish/Subscribe broker (<insert_3>) but the request has failed for reason <insert_1>:<insert_5>.",
                  "Determine why the migmqbrk command cannot complete successfully. The message logs for the queue manager might contain more detailed information outlining why the broker cannot be migrated. Resolve the problem that is preventing the command from completing and reissue the migmqbrk command."));
      amqMN_Ht.put("5893",new AMQMessageNumber(5893,
                  "(iSeries) WebSphere MQ Publish/Subscribe broker cannot be migrated.",
                  "20 : Error",
                  "An attempt has been made to migrate the broker (<insert_3>) but the request has failed for reason <insert_1>:<insert_5>.",
                  "Determine why the migmqbrk command cannot complete successfully. The message logs for the queue manager might contain more detailed information outlining why the broker cannot be migrated. Resolve the problem that is preventing the command from completing and reissue the migmqbrk command."));
      amqMN_Ht.put("5894",new AMQMessageNumber(5894,
                  "WebSphere MQ Publish/Subscribe broker cannot be migrated.",
                  "10 : Warning",
                  "The WebSphere MQ Publish/Subscribe broker cannot be migrated yet because the state of stream <insert_3> is not consistent with respect to related broker <insert_4>. While an WebSphere MQ Publish/Subscribe broker is being migrated a check is made to ensure that the state of each stream is consistent with respect to all of the broker's relations. This check has failed because an inconsistency has been detected in the state of stream <insert_3> with respect to broker <insert_4>. The problem will most likely be of a transient nature, caused because the WebSphere MQ Publish/Subscribe broker has yet to complete processing a recent change to the topology of the broker network. For example, the stream in question may have recently been created or deleted at related broker <insert_4> and this broker has yet to complete its processing for this change. Another cause maybe that either this broker, or broker <insert_4>, have just been added into the broker network and subscriptions have yet to be exchanged the two brokers. If this is the case then the brokers will be inconsistent with respect to all streams. If no recent topology changes have been made then there maybe a current failure with the propagation of subscriptions to broker <insert_4>.",
                  "In all cases migration of the WebSphere MQ Publish/Subscribe broker will need to be suspended until the inconsistency has been resolved. You will need to restart the broker using the strmqbrk command so that it can resolve the problem. After a short while, the broker can be ended and migration reattempted. If repeated attempts to migrate the broker all fail with this message then try to resolve the underlying problem. Look for earlier occurrences of message AMQ5826 and follow the guidance given there. In all cases ensure that the channels between the two brokers are running."));
      amqMN_Ht.put("5895",new AMQMessageNumber(5895,
                  "WebSphere MQ Publish/Subscribe broker cannot be migrated.",
                  "10 : Warning",
                  "A topic has been detected which cannot be exported to the WebSphere MQ Integrator broker. The topic <insert_3> cannot be migrated because it contains wildcard characters recognised by the WebSphere MQ Integrator broker. The wildcard characters used by WebSphere MQ Integrator are the '+' and the '#' characters. The state associated with the topic isn't migrated and migration of the WebSphere MQ Publish/Subscribe broker fails.",
                  "The WebSphere MQ Publish/Subscribe broker cannot be migrated while topic <insert_3> is in use. All applications using topics which contain either the '+' or '#' characters will need to be redesigned to use different topic strings. Note that the amqspsd sample can be used to dump the state of the WebSphere MQ Publish/Subscribe broker. Within the dump produced by this program locate topic <insert_3> to determine information about the publishing or subscribing applications concerned. Until the problem has been resolved the WebSphere MQ Publish/Subscribe broker can be restarted as normal using the strmqbrk command."));
      amqMN_Ht.put("5896",new AMQMessageNumber(5896,
                  "Unknown attribute for WebSphere MQ Publish/Subscribe broker configuration parameter GroupId.",
                  "20 : Error",
                  "The broker has attempted to create stream <insert_4> belonging to group <insert_3>, this group is unknown.",
                  "Modify the attribute for broker configuration parameter GroupId, to a group that exists, or create the group <insert_3>."));
   }
   
   /**
    * Initialize the AMQ 6000's message number. 
    */
   public void init6s()
   {
      amqMN_Ht.put("6004",new AMQMessageNumber(6004,
            "An error occurred during WebSphere MQ initialization or ending.",
            "30 : Severe error",
            "An error was detected during initialization or ending of MQ. The MQ error recording routine has been called.",
            "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6005",new AMQMessageNumber(6005,
                  "(iSeries) An error occurred during WebSphere MQ startup.",
                  "30 : Severe error",
                  "An attempt to start the storage monitor process (job QMQM in subsystem QSYSWRK) was unsuccessful.",
                  "Check the joblog for this job and for the QMQM job for possible reasons for failure, correct the error and try the command again. If the problem is not resolved, a problem may have been logged. Use WRKPRB to record the problem identifier, and to save the QPSRVDMP, QPJOBLOG, and QPDSPJOB files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6015",new AMQMessageNumber(6015,
                  "The operating system is either too busy or has insufficient resources to complete a system request.",
                  "30 : Severe error",
                  "A system request <insert_3> was rejected by the operating system with return code <insert_1>. WebSphere MQ retried the request, but it continued to fail. This failure may indicate that the operating system is either too busy or has insufficient resources to complete the request.",
                  "Investigate whether the system is constrained by the workload on this system or by the workload on a server that it is using, and reduce the workload."));
      amqMN_Ht.put("6025",new AMQMessageNumber(6025,
                  "Program not found.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to start program <insert_3> because it was not found.",
                  "Check the program name is correctly specified and rerun the program."));
      amqMN_Ht.put("6026",new AMQMessageNumber(6026,
                  "A resource shortage prevented the creation of a WebSphere MQ process.",
                  "30 : Severe error",
                  "An attempt to create an MQ process was rejected by the operating system due to a process limit (either the number of processes for each user or the total number of processes running system wide), or because the system does not have the resources necessary to create another process.",
                  "Investigate whether a process limit is preventing the creation of the process and if so why the system is constrained in this way. Consider raising this limit or reducing the workload on the system."));
      amqMN_Ht.put("6035",new AMQMessageNumber(6035,
                  "WebSphere MQ failed, no storage available.",
                  "30 : Severe error",
                  "An internal function of the product attempted to obtain storage, but there was none available.",
                  "Stop the product and restart it. If this does not resolve the problem, save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("6037",new AMQMessageNumber(6037,
                  "WebSphere MQ was unable to obtain enough storage.",
                  "20 : Error",
                  "The product is unable to obtain enough storage. The product's error recording routine may have been called.",
                  "Stop the product and restart it. If this does not resolve the problem see if a problem has been recorded. If a problem has been recorded, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6047",new AMQMessageNumber(6047,
                  "Conversion not supported.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data tagged in CCSID <insert_1> to data in CCSID <insert_2>.",
                  "Check the WebSphere MQ Application Programming Reference Appendix and the appropriate National Language Support publications to see if the CCSIDs are supported by your system."));
      amqMN_Ht.put("6048",new AMQMessageNumber(6048,
                  "DBCS error",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data due to a DBCS error. Conversion is from CCSID <insert_1> to CCSID <insert_2>.",
                  "Check the WebSphere MQ Application Programming Reference Appendix and the appropriate National Language Support publications to see if the CCSIDs are supported by your system."));
      amqMN_Ht.put("6049",new AMQMessageNumber(6049,
                  "DBCS-only string not valid.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data in CCSID <insert_1> to data in CCSID <insert_2>. Message descriptor data must be in single-byte form. CCSID <insert_2> is a DBCS-only CCSID.",
                  "Check the CCSID of your job or system and change it to one supporting SBCS or mixed character sets. Refer to the WebSphere MQ Application Programming Reference Appendix and the appropriate National Language Support publications for character sets and CCSIDs supported."));
      amqMN_Ht.put("6050",new AMQMessageNumber(6050,
                  "CCSID error.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data in CCSID <insert_1> to data in CCSID <insert_2>.",
                  "Check the WebSphere MQ Application Programming Reference Appendix and the appropriate National Language Support publications to see if the CCSIDs are supported by your system."));
      amqMN_Ht.put("6051",new AMQMessageNumber(6051,
                  "Conversion length error.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data in CCSID <insert_1> to data in CCSID <insert_2>, due to an input length error.",
                  ""));
      amqMN_Ht.put("6052",new AMQMessageNumber(6052,
                  "Conversion length error.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data in CCSID <insert_1> to data in CCSID <insert_2>.",
                  ""));
      amqMN_Ht.put("6053",new AMQMessageNumber(6053,
                  "CCSID error",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data in CCSID <insert_1> to data in CCSID <insert_2>.",
                  "One of the CCSIDs is not supported by the system. Check the WebSphere MQ Application Programming Reference Appendix and the appropriate National Language Support publications to see if the CCSIDs are supported by your system."));
      amqMN_Ht.put("6064",new AMQMessageNumber(6064,
                  "An internal WebSphere MQ error has occurred.",
                  "30 : Severe error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6088",new AMQMessageNumber(6088,
                  "(iSeries) An internal WebSphere MQ error has occurred.",
                  "40 : Stop Error",
                  "An internal error occurred when API call <insert_3> was made.",
                  "Use WRKPRB to record the problem identifier, and to save the QPSRVDMP, QPJOBLOG, and QPDSPJOB files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6089",new AMQMessageNumber(6089,
                  "(iSeries) WebSphere MQ was unable to display an error message.",
                  "30 : Severe error",
                  "An attempt to display an error message was unsuccessful. This may be because the AMQMSG message file could not be found. The message identifier is <insert_3>.",
                  "Check that the library list is set up correctly to access the AMQMSG message file. If a change is necessary, rerun the failing application and record the error message. If you are unable to resolve the problem, contact your IBM support center."));
      amqMN_Ht.put("6090",new AMQMessageNumber(6090,
                  "WebSphere MQ was unable to display an error message <insert_6>.",
                  "0 : Information",
                  "MQ has attempted to display the message associated with return code hexadecimal <insert_6>. The return code indicates that there is no message text associated with the message. Associated with the request are inserts <insert_1> : <insert_2> : <insert_3> : <insert_4> : <insert_5>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6091",new AMQMessageNumber(6091,
                  "An internal WebSphere MQ error has occurred.",
                  "0 : Information",
                  "Private memory has detected an error, and is abending due to <insert_3>. The error data is <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6092",new AMQMessageNumber(6092,
                  " (Windows)Manual conversion required for CCSID: <insert_1>",
                  "0 : Information",
                  "CCSID <insert_1> exists in new format but could not be reconciled against your old format.",
                  "Manually edit CCSID entry <insert_1> in conv\\table\\ccsid.tbl if you wish to retain your old conversion. For assistance call your Service Representative."));
      amqMN_Ht.put("6100",new AMQMessageNumber(6100,
                  "An internal WebSphere MQ error has occurred.",
                  "0 : Information",
                  "MQ has detected an error, and is abending due to <insert_3>. The error data is <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6103",new AMQMessageNumber(6103,
                  "(iSeries) WebSphere MQ job submission error.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to submit job <insert_3>.",
                  ""));
      amqMN_Ht.put("6107",new AMQMessageNumber(6107,
                  "CCSID not supported.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data in CCSID <insert_1> to data in CCSID <insert_2>, because one of the CCSIDs is not recognized.",
                  "Check the WebSphere MQ Application Programming Reference Appendix and the appropriate National Language Support publications to see if the CCSIDs are supported by your system."));
      amqMN_Ht.put("6109",new AMQMessageNumber(6109,
                  "An internal WebSphere MQ error has occurred.",
                  "30 : Severe error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6110",new AMQMessageNumber(6110,
                  "An internal WebSphere MQ error has occurred.",
                  "30 : Severe error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6112",new AMQMessageNumber(6112,
                  "(iSeries) WebSphere MQ CCSID <insert_1> is using a default value.",
                  "10 : Warning",
                  "When initializing WebSphere MQ, no valid job CCSID was found, so the CCSID used is the default 37. This warning message will be issued until a valid CCSID has been set correctly.",
                  "Set the job CCSID."));
      amqMN_Ht.put("6114",new AMQMessageNumber(6114,
                  "(iSeries) An internal WebSphere MQ error has occurred.",
                  "30 : Severe error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "Use WRKPRB to record the problem identifier, and to save the QPSRVDMP, QPJOBLOG, and QPDSPJOB files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6115",new AMQMessageNumber(6115,
                  "An internal WebSphere MQ error has occurred.",
                  "10 : Warning",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6118",new AMQMessageNumber(6118,
                  "An internal WebSphere MQ error has occurred (<insert_1>)",
                  "40 : Stop Error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6119",new AMQMessageNumber(6119,
                  "An internal WebSphere MQ error has occurred (<insert_3>)",
                  "40 : Stop Error",
                  "MQ detected an unexpected error when calling the operating system. The MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6120",new AMQMessageNumber(6120,
                  "An internal WebSphere MQ error has occurred.",
                  "40 : Stop Error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6121",new AMQMessageNumber(6121,
                  "An internal WebSphere MQ error has occurred.",
                  "40 : Stop Error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "MQ has detected a parameter count of <insert_1> that is not valid. Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6122",new AMQMessageNumber(6122,
                  "An internal WebSphere MQ error has occurred.",
                  "40 : Stop Error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "MQ has detected parameter <insert_1> that is not valid, having value <insert_2><insert_3>. Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6125",new AMQMessageNumber(6125,
                  "An internal WebSphere MQ error has occurred.",
                  "40 : Stop Error",
                  "An internal error has occurred with identifier <insert_1>. This message is issued in association with other messages.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6134",new AMQMessageNumber(6134,
                  "(iSeries) Trace continues in buffer",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6135",new AMQMessageNumber(6135,
                  "(iSeries) Stopping early trace",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6136",new AMQMessageNumber(6136,
                  "(iSeries) Stopping early trace <insert_3> system time",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6137",new AMQMessageNumber(6137,
                  "(iSeries) Resuming MQI trace",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6138",new AMQMessageNumber(6138,
                  "(iSeries) Resuming MQI trace <insert_3> system time",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6139",new AMQMessageNumber(6139,
                  "(iSeries) Stopping MQI trace",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6140",new AMQMessageNumber(6140,
                  "(iSeries) Stopping MQI trace <insert_3> system time",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6141",new AMQMessageNumber(6141,
                  "(iSeries) Starting MQI trace",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6142",new AMQMessageNumber(6142,
                  "(iSeries) Starting MQI trace <insert_3> system time",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6143",new AMQMessageNumber(6143,
                  "(iSeries) WebSphere MQ function stack",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6144",new AMQMessageNumber(6144,
                  "(iSeries) No stack available",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6145",new AMQMessageNumber(6145,
                  "(iSeries) Terminating MQI trace",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6146",new AMQMessageNumber(6146,
                  "(iSeries) Entering end job processing",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6147",new AMQMessageNumber(6147,
                  "(iSeries) Terminating MQI trace <insert_3> system time",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6148",new AMQMessageNumber(6148,
                  "An internal WebSphere MQ error has occurred.",
                  "0 : Information",
                  "MQ has detected an error, and is abending due to <insert_3>. The error data is <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6150",new AMQMessageNumber(6150,
                  "(iSeries) WebSphere MQ resource <insert_3> busy.",
                  "30 : Severe error",
                  "MQ was unable to access an MQ object within the normal timeout period of <insert_1> minutes.",
                  "MQ will continue to wait for access. Ensure that all jobs using MQ are released. If the situation persists, quiesce the queue manager."));
      amqMN_Ht.put("6150",new AMQMessageNumber(6150,
                  " (Windows)WebSphere MQ semaphore is busy.",
                  "10 : Warning",
                  "WebSphere MQ was unable to acquire a semaphore within the normal timeout period of <insert_1> minutes.",
                  "MQ will continue to wait for access. If the situation does not resolve itself and you suspect that your system is locked then investigate the process which owns the semaphore. The PID of this process will be documented in the accompanying FFST."));
      amqMN_Ht.put("6151",new AMQMessageNumber(6151,
                  "(iSeries) WebSphere MQ resource <insert_3> released.",
                  "30 : Severe error",
                  "An MQ resource, for which another process has been waiting, for a period of over <insert_1> minutes has been released.",
                  "No recovery is needed."));
      amqMN_Ht.put("6152",new AMQMessageNumber(6152,
                  "(iSeries) WebSphere MQ failed to end commitment control while attempting to quiesce a queue manager.",
                  "30 : Severe error",
                  "WebSphere MQ failed to end commitment control whilst quiescing queue manager <insert_3>.",
                  "There are one or more active resources under commitment control. Use the Work with Job (WRKJOB) command with the OPTION(*CMTCTL) parameter to display the active resources under commitment control. Check the job log for previously issued messages."));
      amqMN_Ht.put("6153",new AMQMessageNumber(6153,
                  "(iSeries) The attempt to quiesce queue manager <insert_3> failed",
                  "30 : Severe error",
                  "The attempt to quiesce queue manager <insert_3> was unsuccessful",
                  "Check the job log for previously issued messages. If the quiesce was issued with the *CNTRLD option, re-issue the command with the *IMMED option. If a low TIMEOUT retry delay was used, re-issue the request with a higher value."));
      amqMN_Ht.put("6154",new AMQMessageNumber(6154,
                  "(iSeries) Queue manager <insert_3> has been quiesced.",
                  "0 : Information",
                  "The queue manager has been successfully quiesced.",
                  "None."));
      amqMN_Ht.put("6158",new AMQMessageNumber(6158,
                  "(iSeries) SBCS CCSID not found.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to find an SBCS CCSID which corresponds to mixed DBCS-SBCS CCSID <insert_1>.",
                  "Check the CCSID of your job or system and check it has a SBCS equivalent. Refer to the National Language Support Planning Guide for character sets and CCSIDs supported. If the CCSID used does have an SBCS equivalent, save the job log containing this message and contact your IBM support center."));
      amqMN_Ht.put("6159",new AMQMessageNumber(6159,
                  "(iSeries) WebSphere MQ job submission error.",
                  "30 : Severe error",
                  "WebSphere MQ for iSeries is unable to release job <insert_3>.",
                  "Contact you System Administrator to remove job <insert_3>. Ensure you have *JOBCTL authority and try again."));
      amqMN_Ht.put("6160",new AMQMessageNumber(6160,
                  "EXPLANATION:",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6161",new AMQMessageNumber(6161,
                  "ACTION:",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6162",new AMQMessageNumber(6162,
                  "An error has occurred reading an INI file.",
                  "20 : Error",
                  "An error has occurred when reading the MQS.INI file or a queue manager QM.INI file.",
                  "If you have been changing the INI file content check and correct the change. If you have not changed the INI file, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6162",new AMQMessageNumber(6162,
                  " (Windows)An error occurred when reading the configuration data.",
                  "20 : Error",
                  "An error has occurred when reading the configuration data.",
                  "If you have changed the configuration data, check and correct the change. If you have not changed the configuration data, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6163",new AMQMessageNumber(6163,
                  "An error has occurred locking an INI file.",
                  "10 : Warning",
                  "An error has occurred locking the MQS.INI file or a queue manager QM.INI file.",
                  "If you have been changing the INI file permissions check and correct the change. If you have not changed the INI file, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6163",new AMQMessageNumber(6163,
                  " (Windows)An error has occurred locking the configuration data.",
                  "10 : Warning",
                  "An error has occurred locking the configuration data.",
                  "If you have changed the the registry permissions, check and correct the change. If you have not changed the registry, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6164",new AMQMessageNumber(6164,
                  "An expected stanza in an INI file is missing or contains errors.",
                  "10 : Warning",
                  "An expected stanza is missing from the MQS.INI file or a queue manager QM.INI file or the stanza contains errors.",
                  "If you have been changing the INI file content check and correct the change."));
      amqMN_Ht.put("6164",new AMQMessageNumber(6164,
                  " (Windows)An expected stanza in the configuration data is missing or contains errors.",
                  "10 : Warning",
                  "An expected stanza is missing from the configuration data or the stanza contains errors.",
                  "If you have changed the configuration data, check and correct the change."));
      amqMN_Ht.put("6165",new AMQMessageNumber(6165,
                  "Unable to access an INI file.",
                  "10 : Warning",
                  "Access to the MQS.INI file or a queue manager QM.INI file is denied.",
                  "If you have been changing the INI file permissions check and correct the change."));
      amqMN_Ht.put("6165",new AMQMessageNumber(6165,
                  " (Windows)Unable to access the configuration data.",
                  "10 : Warning",
                  "Access to the configuration data is denied.",
                  "If you have changed the configuration data permissions, check and correct the changes."));
      amqMN_Ht.put("6166",new AMQMessageNumber(6166,
                  "An INI file is missing.",
                  "20 : Error",
                  "The MQS.INI file or a queue manager QM.INI file is missing.",
                  "If you have been changing the INI file recover the previous file and retry the operation."));
      amqMN_Ht.put("6166",new AMQMessageNumber(6166,
                  " (Windows)An entry in the configuration data is missing.",
                  "20 : Error",
                  "A required entry in the configuration data is missing.",
                  "If you have changed the configuration data, recover the previous configuration data and retry the operation."));
      amqMN_Ht.put("6172",new AMQMessageNumber(6172,
                  "No codeset found for current locale.",
                  "20 : Error",
                  "No codeset could be determined for the current locale. Check that the locale in use is supported.",
                  "None."));
      amqMN_Ht.put("6173",new AMQMessageNumber(6173,
                  "No CCSID found for codeset <insert_3>.",
                  "20 : Error",
                  "Codeset <insert_3>. has no supported CCSID. Check that the locale in use is supported. CCSIDs can be added by updating the file /var/mqm/conv/table/ccsid.tbl.",
                  "None."));
      amqMN_Ht.put("6174",new AMQMessageNumber(6174,
                  "The library <insert_3> was not found. The queue manager will continue without this module.",
                  "0 : Information",
                  "The dynamically loadable library <insert_3> was not found.",
                  "Check that the file exists and is either fully qualified or is in the appropriate directory."));
      amqMN_Ht.put("6174",new AMQMessageNumber(6174,
                  "(iSeries) The library was not found.",
                  "0 : Information",
                  "The dynamically loadable file <insert_3> was not found. The queue manager will continue without this module.",
                  "Check that the file exists and is either fully qualified or is in the appropriate directory."));
      amqMN_Ht.put("6174",new AMQMessageNumber(6174,
                  " (Unix)The dynamically loadable shared library <insert_3> was not found. The system returned error number <insert_2> and error message <insert_4>. The queue manager will continue without this module.",
                  "0 : Information",
                  "This message applies to UNIX systems. The shared library <insert_3> was not found.",
                  "Check that the file exists, and is either fully qualified or is in the appropriate director, also check the file access permissions."));
      amqMN_Ht.put("6175",new AMQMessageNumber(6175,
                  " (AIX)The system could not dynamically load the shared library <insert_3>. The system returned error number <insert_2> and error message <insert_4>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to AIX systems. The shared library <insert_3> failed to load correctly due to a problem with the library.",
                  "Check the file access permissions and that the file has not been corrupted."));
      amqMN_Ht.put("6175",new AMQMessageNumber(6175,
                  " (Unix)The system could not dynamically load the shared library <insert_3>. The system returned error message <insert_4>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. The shared library <insert_3> failed to load correctly due to a problem with the library.",
                  "Check the file access permissions and that the file has not been corrupted."));
      amqMN_Ht.put("6175",new AMQMessageNumber(6175,
                  " (Windows)The system could not dynamically load the library <insert_3>. The system return code was <insert_1>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to Windows NT and Windows 2000 systems only. The dynamically loadable file <insert_3> failed to load correctly due to an internal error. The MQ error recording routine has been called.",
                  "Check that the file has not been corrupted then use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6177",new AMQMessageNumber(6177,
                  " (Windows)An internal WebSphere MQ error has occurred.",
                  "40 : Stop Error",
                  "An error has been detected, and the MQ error recording routine has been called.",
                  "Details of the error have been stored at <insert_3>. A synopsis is given in the data section below. Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6179",new AMQMessageNumber(6179,
                  "The system could not find symbol <insert_5> in the dynamically loaded library <insert_3>. The system returned error number <insert_2> and error message <insert_4>.",
                  "30 : Severe error",
                  "The library <insert_3> does not contain symbol <insert_5> or it has not been exported.",
                  "Check that symbol name <insert_5> is correct and has been exported from the library."));
      amqMN_Ht.put("6179",new AMQMessageNumber(6179,
                  " (Unix)The system could not find the symbol <insert_5> in the dynamically loaded shared library <insert_3>. The system returned error message <insert_4>.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. The shared library <insert_3> does not contain symbol <insert_5> or it has not been exported.",
                  "Check that symbol name <insert_5> is correct and has been exported from the library."));
      amqMN_Ht.put("6180",new AMQMessageNumber(6180,
                  " (Windows)Default conversion not supported.",
                  "30 : Severe error",
                  "WebSphere MQ is unable to convert string data tagged in CCSID <insert_1> to data in CCSID <insert_2>.",
                  "Check the default CCSIDs specified in the ccsid.tbl file and make sure that conversion is supported between these CCSIDs."));
      amqMN_Ht.put("6182",new AMQMessageNumber(6182,
                  "Error found in line <insert_1> of ccsid.tbl",
                  "30 : Severe error",
                  "Line <insert_1> contains and error. The content of the line is <insert_3>. Processing continues but the line in error is ignored.",
                  "Correct the line and rerun the program or command giving this message."));
      amqMN_Ht.put("6183",new AMQMessageNumber(6183,
                  "An internal WebSphere MQ error has occurred.",
                  "10 : Warning",
                  "An error has been detected, and the WebSphere MQ error recording routine has been called. The failing process is process <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6184",new AMQMessageNumber(6184,
                  "An internal WebSphere MQ error has occurred on queue manager <insert_3>.",
                  "10 : Warning",
                  "An error has been detected, and the WebSphere MQ error recording routine has been called. The failing process is process <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6184",new AMQMessageNumber(6184,
                  "(iSeries) An internal WebSphere MQ error has occurred.",
                  "10 : Warning",
                  "An internal MQ error has occurred on queue manager <insert_3> and the MQ error recording routine has been called. The failing process is process <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6187",new AMQMessageNumber(6187,
                  "User is not authorized for RestrictedMode queue manager.",
                  "40 : Stop Error",
                  "All users must be in the RestrictedMode application_group.",
                  ""));
      amqMN_Ht.put("6188",new AMQMessageNumber(6188,
                  " (AIX)The system could not dynamically load the shared library <insert_3> as the entry point to the library, symbol 'MQStart', could not be located within the library. The queue manager will continue without this library.",
                  "30 : Severe error",
                  "This message applies to AIX systems. The shared library <insert_3> failed to load correctly due to a problem with the library.",
                  "Check that the entry point to the library, symbol 'MQStart', exists and has been exported from the library."));
      amqMN_Ht.put("6188",new AMQMessageNumber(6188,
                  " (Unix)The system could not dynamically load the shared library <insert_3> as the entry point to the library, symbol 'MQStart', could not be located within the library. The system returned error message <insert_4>. The queue manager will continue without this library.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. The shared library <insert_3> failed to load correctly due to a problem with the library.",
                  "Check that the entry point to the library, symbol 'MQStart', exists and has been exported from the library."));
      amqMN_Ht.put("6188",new AMQMessageNumber(6188,
                  " (Windows)The system could not dynamically load the library <insert_3> due to a problem with the dll. The errno was <insert_1>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to Windows NT and Windows 2000 systems only. The dynamically loadable file <insert_3> failed to load correctly due to a problem with the dll.",
                  "Check that the dll is in the correct place with the correct file permissions etc. and has not been corrupted."));
      amqMN_Ht.put("6190",new AMQMessageNumber(6190,
                  " (Windows)Program <insert_3> not found.",
                  "30 : Severe error",
                  "The program <insert_3> cannot be found.",
                  "Check that the program specified is available on your system. If the program name is not fully qualified, ensure that the PATH environment variable includes the directory where the program is located."));
      amqMN_Ht.put("6191",new AMQMessageNumber(6191,
                  " (Windows)Program <insert_3> failed to start, return code <insert_1>.",
                  "30 : Severe error",
                  "The program <insert_3> was invoked, but failed to start. The failure reason code is <insert_1>.",
                  "Check that the program specified is available on your system, and that sufficient system resources are available. Where applicable, verify that the user is authorized to run the program."));
      amqMN_Ht.put("6192",new AMQMessageNumber(6192,
                  " (Windows)IBM WebSphere MQ Utilities",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6193",new AMQMessageNumber(6193,
                  " (Windows)The registry entry <insert_3> was not found.",
                  "20 : Error",
                  "WebSphere MQ for Windows NT and Windows 2000 sets the registry entry <insert_3> when the product is installed, but the entry is now missing.",
                  "If the registry has been edited, restore the previous version. If the product is newly installed, check whether the installation was successful, and reinstall the product if necessary."));
      amqMN_Ht.put("6196",new AMQMessageNumber(6196,
                  "An error has occurred whilst processing a temporary INI file <insert_3>",
                  "20 : Error",
                  "An error has occurred when creating a backup of an INI file. The backup file <insert_4> already exists",
                  "You may have created a backup of the INI file with the name <insert_4>, or an earlier operation may have failed. Move or delete the file <insert_4> and reattempt the operation. If you have not changed the INI file, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("6207",new AMQMessageNumber(6207,
                  " (AIX)Failed to attach shared memory segment as Segment table is Full.",
                  "20 : Error",
                  "When running in native mode an application may attach only 10 shared memory segments. The application which issued this message attempted to exceed this number. By setting the environment variable EXTSHM=ON this limit can be removed. Further explanation on using this variable and other options available, may be found in the documentation.",
                  "Either reduce the number of segments to which your application needs to attach or set the EXTSHM=ON variable in your environment before starting the application."));
      amqMN_Ht.put("6208",new AMQMessageNumber(6208,
                  "<insert_3>",
                  "10 : Warning",
                  "<insert_4>",
                  "<insert_5>"));
      amqMN_Ht.put("6209",new AMQMessageNumber(6209,
                  "An unexpected asynchronous signal (<insert_1> : <insert_3>) has been received and ignored.",
                  "10 : Warning",
                  "Process <insert_2> received an unexpected asynchronous signal and ignored it. This has not caused an error but the source of the signal should be determined as it is likely that the signal has been generated externally to WebSphere MQ.",
                  "Determine the source of the signal and prevent it from re-occurring."));
      amqMN_Ht.put("6212",new AMQMessageNumber(6212,
                  "Failed to load Library <insert_3> as C++ environment is not initialised.",
                  "10 : Warning",
                  "An attempt was made to load the identified C++ shared library. However, the attempt failed because the C++ environment has not been initialized for the current process.",
                  "Ensure the application is linked with the appropriate C++ runtime environment."));
      amqMN_Ht.put("6218",new AMQMessageNumber(6218,
                  " (AIX)EXTSHM variable detected with unrecognised value <insert_3> and has been reset to <insert_4>.",
                  "20 : Error",
                  "Processes that access the internal queue manager control blocks must use the AIX Extended Shared Memory model, and while one such process was starting, WebSphere MQ detected that the EXTSHM variable was set but did not contain an appropriate value. This value has been reset and the process will continue with the new setting.",
                  "No further action is required. To prevent this message being issued in future, correct the value of the EXTSHM variable in your environment."));
      amqMN_Ht.put("6230",new AMQMessageNumber(6230,
                  "Message <insert_3> suppressed <insert_1> times in the last <insert_4> seconds.",
                  "20 : Error",
                  "Message <insert_3> was issued <insert_2> times in the last <insert_4> seconds but only the first instance of the message was written to the log. The suppressed messages may have included differing message arguments.",
                  "If you wish to see all occurrences of this message you should alter the definition of the SuppressMessage attribute in the Queue Manager configuration."));
      amqMN_Ht.put("6231",new AMQMessageNumber(6231,
                  " (Unix)Usage: mqchkdir <insert_-a | -m QMgrName> [-f]",
                  "20 : Error",
                  "An incorrect option was specified on the command line for the command.",
                  "Reissue the command specifying the correct parameters."));
      amqMN_Ht.put("6232",new AMQMessageNumber(6232,
                  " (Unix)Operating System userid <insert_3> not found.",
                  "20 : Error",
                  "A request was made to the operating system to lookup the details of the identified userid but the request failed.",
                  "Using the operating system supplied tools check for the existence of the identified userid, and if missing then recreate it."));
      amqMN_Ht.put("6233",new AMQMessageNumber(6233,
                  " (Unix)Operating System authorisation group <insert_3> not found.",
                  "20 : Error",
                  "A request was made to the operating system to lookup the details of the identified group but the request failed.",
                  "Using the operating system supplied tools check for the existence of the identified group, and if missing then recreate it."));
      amqMN_Ht.put("6234",new AMQMessageNumber(6234,
                  " (Unix)Unknown Queue Manager name specified.",
                  "20 : Error",
                  "An invalid Queue Manager name <insert_3> was specified in the parameters to the command.",
                  "Reissue the command specifying a valid Queue Manager name."));
      amqMN_Ht.put("6235",new AMQMessageNumber(6235,
                  " (Unix)Directory <insert_3> missing.",
                  "20 : Error",
                  "The identified directory is missing.",
                  "Reissue the command selecting the option to create missing directories."));
      amqMN_Ht.put("6236",new AMQMessageNumber(6236,
                  " (Unix)Missing directory <insert_3> has been created.",
                  "20 : Error",
                  "The identified directory was missing but has been created.",
                  "None"));
      amqMN_Ht.put("6237",new AMQMessageNumber(6237,
                  " (Unix)File <insert_3> missing.",
                  "20 : Error",
                  "The identified file is missing.",
                  "Reissue the command selecting the option to create missing files."));
      amqMN_Ht.put("6238",new AMQMessageNumber(6238,
                  " (Unix)Missing file <insert_3> has been created.",
                  "20 : Error",
                  "The identified file was missing but has been created.",
                  "None"));
      amqMN_Ht.put("6239",new AMQMessageNumber(6239,
                  " (Unix)Permission denied attempting to access filesystem location <insert_3>.",
                  "20 : Error",
                  "An attempt to query the filesystem object identified failed because the command issued did not have authority to access the object.",
                  "Check the authority on the object and of the user executing the command and reissue the command."));
      amqMN_Ht.put("6240",new AMQMessageNumber(6240,
                  " (Unix)You must be an operating system superuser to run this command.",
                  "20 : Error",
                  "In irder to run this command you must be logged on as a user with superuser privelages.",
                  "Log in as an appropriate user and reissue the command."));
      amqMN_Ht.put("6241",new AMQMessageNumber(6241,
                  " (Unix)The filesystem object <insert_3> is a symbolic link.",
                  "20 : Error",
                  "While checking the filesystem, an object was found which is a symbolic link.",
                  "This is not an error however you should verify that the symbolic link is expected and that the destination of the symbolic link is correct."));
      amqMN_Ht.put("6242",new AMQMessageNumber(6242,
                  " (Unix)Incorrect ownership for <insert_3>. Current(<insert_1>) Expected(<insert_2>)",
                  "20 : Error",
                  "The filesystem object <insert_3> is owned by the user with uid <insert_1> when it was expected to be owned by the user with uid <insert_2>.",
                  "Correct the ownership using operating system commands or reissue the command selecting the option to fix the incorrect owership."));
      amqMN_Ht.put("6243",new AMQMessageNumber(6243,
                  " (Unix)Incorrect group ownership for <insert_3>. Current(<insert_1>) Expected(<insert_2>)",
                  "20 : Error",
                  "The filesystem object <insert_3> is owned by the group with gid <insert_1> when it was expected to be owned by the group with gid <insert_2>.",
                  "Correct the ownership using operating system commands or reissue the command selecting the option to fix the incorrect owership."));
      amqMN_Ht.put("6244",new AMQMessageNumber(6244,
                  " (Unix)Incorrect permissions on object <insert_3>. Current(<insert_4>) Expected(<insert_5>)",
                  "20 : Error",
                  "The filesystem object <insert_3> has the wrong file permissions.",
                  "Correct the ownership using operating system commands or reissue the command selecting the option to fix the incorrect owership."));
      amqMN_Ht.put("6245",new AMQMessageNumber(6245,
                  " (Unix)Error executing system call <insert_3> on file <insert_4> error <insert_2>.",
                  "20 : Error",
                  "The execution of the system call <insert_3> on file <insert_4> failed and the error code <insert_2> was returned.",
                  "Investigate the cause of the failure using the operating system error code <insert_1> and reissue the command."));
      amqMN_Ht.put("6251",new AMQMessageNumber(6251,
                  " (Unix)The system could not dynamically load the shared library <insert_3>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. The shared library <insert_3> failed to load as it is probably a <insert_1>-bit library, a <insert_2>-bit library is required. Note that MQ tried to find a <insert_2>-bit library named either <insert_4> or <insert_5>, but failed. The following message gives details of the original failure.",
                  "Supply the name of a <insert_2>-bit library."));
      amqMN_Ht.put("6252",new AMQMessageNumber(6252,
                  " (Unix)The system could not dynamically load the shared library <insert_3>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. The shared library <insert_3> failed to load as it is probably a <insert_1>-bit library, a <insert_2>-bit library is required. Note that MQ found and loaded a <insert_2>-bit library named <insert_4> however this also failed to load with the system returning error message <insert_5>. The following message gives details of the original failure.",
                  "Supply the name of a <insert_2>-bit library."));
      amqMN_Ht.put("6253",new AMQMessageNumber(6253,
                  " (Unix)The system could not dynamically load the shared library <insert_3>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. The shared library <insert_3> failed to load as it is probably a <insert_1>-bit library, a <insert_2>-bit library is required. Note that MQ attempted to locate and load a <insert_2>-bit library named either of these: <insert_4>. The first library failed to load as it also is probably a <insert_1>-bit library, the second library is a <insert_2>-bit library, however this also failed to load with the system returning error message <insert_5>. The following message gives details of the original failure.",
                  "Supply the name of a <insert_2>-bit library."));
      amqMN_Ht.put("6254",new AMQMessageNumber(6254,
                  " (Unix)The system could not dynamically load the shared library <insert_3>, library <insert_4> has been used instead.",
                  "0 : Information",
                  "This message applies to UNIX systems. The shared library <insert_3> failed to load as it is probably a <insert_1>-bit library, a <insert_2>-bit library is required. Note that MQ has sucessfully located and loaded a <insert_2>-bit library named <insert_4>.",
                  "Supply the name of a <insert_2>-bit library or put the library (alternatively a symbolic link can be used) in the appropriate place: 32-bit libraries in /var/mqm/exits; 64-bit libraries in /var/mqm/exits64."));
      amqMN_Ht.put("6255",new AMQMessageNumber(6255,
                  " (Unix)The system could not dynamically load the shared library <insert_3>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. The shared library <insert_3> failed to load as it is probably a <insert_1>-bit library, a <insert_2>-bit library is required. The following message gives details of the original failure.",
                  "Supply the name of a <insert_2>-bit library."));
      amqMN_Ht.put("6256",new AMQMessageNumber(6256,
                  " (Unix)The system could not dynamically load the shared library <insert_3>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. The shared library <insert_3> failed to load as it is probably a <insert_1>-bit library, a <insert_2>-bit library is required. Note that MQ tried to find a <insert_2>-bit library named <insert_4>, but failed. The following message gives details of the original failure.",
                  "Supply the name of a <insert_2>-bit library."));
      amqMN_Ht.put("6257",new AMQMessageNumber(6257,
                  "Message suppression enabled for message numbers (<insert_3>).",
                  "0 : Information",
                  "The message contain's a list of message id's for which entries repeated within the <insert_1> suppression interval will be suppressed.",
                  "If you wish to see all occurrences of these messages you should alter the definition of the SuppressMessage attribute in the Queue Manager configuration."));
      amqMN_Ht.put("6258",new AMQMessageNumber(6258,
                  "Message exclusion enabled for message numbers (<insert_3>).",
                  "0 : Information",
                  "The message contain's a list of message id's which have been excluded. Requests to write these messages to the error log will be discarded.",
                  "If you wish to see instances of these messages you should alter the definition of the ExcludeMessage attribute in the Queue Manager configuration."));
      amqMN_Ht.put("6259",new AMQMessageNumber(6259,
                  "Message <insert_3> cannot be <insert_4>.",
                  "10 : Warning",
                  "Message <insert_3> cannot be excluded or suppressed but was specified in the ExcludeMessage or SuppressMessage configuration for the Queue Manager. The Queue Manager will continue however the request to suppress or exclude this message will be ignored.",
                  "Update the Queue Manager configuration to remove the specified message identifier."));
      amqMN_Ht.put("6260",new AMQMessageNumber(6260,
                  "Help Topic not found",
                  "10 : Warning",
                  "The requested help topic could not be located. For further assistance, please refer to the WebSphere MQ manuals.",
                  "Ensure that the WebSphere MQ InfoCenter is installed."));
      amqMN_Ht.put("6261",new AMQMessageNumber(6261,
                  " (Unix)An exception occurred trying to dynamically load shared library <insert_3>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to UNIX systems. Exception number <insert_1> name <insert_4>, occurred trying to dynamically load shared library <insert_3>.",
                  "Check the shared library has not been corrupted. If the shared library contains any initializer functions, ensure these are not causing the problem and that they conform to the expected function prototype."));
      amqMN_Ht.put("6261",new AMQMessageNumber(6261,
                  " (Windows)An exception occurred trying to load DLL <insert_3>. The queue manager will continue without this module.",
                  "30 : Severe error",
                  "This message applies to Windows systems only. Exception number <insert_1> error <insert_4>, occurred trying to load DLL <insert_3>.",
                  "Check the DLL has not been corrupted. If the DLL contains any initializer functions, ensure these are not causing the problem and that they conform to the expected function prototype."));
      amqMN_Ht.put("6666",new AMQMessageNumber(6666,
                  "(iSeries) Required WebSphere MQ system profile(s) can not be accessed.",
                  "40 : Stop Error",
                  "The required MQ system profile(s) QMQM and/or QMQMADM are not found or have been disabled. MQ can not continue processing the command without the profiles existing and enabled on the system. The major error code is <insert_3>, the minor error code is <insert_4>. The major error codes and their meanings are as follows: *DISABLED - The user profile has been disabled. *PWDEXP - The password for the user profile has expired . *EXIST - The user profile does not exist. If none of these error codes are shown the major error code contains the exception identifier. The minor error code identifies the user profile which cannot be accessed.",
                  "Check that both QMQM and QMQMADM profiles exist and are both enabled using the DSPUSRPRF command, or contact the WebSphere MQ system administrator."));
      amqMN_Ht.put("6708",new AMQMessageNumber(6708,
                  "A disk full condition was encountered when formatting a new log file in location <insert_3>.",
                  "20 : Error",
                  "The queue manager attempted to format a new log file in directory <insert_3>. The drive or file system containing this directory did not have sufficient free space to contain the new log file.",
                  "Increase the amount of space available for log files and retry the request."));
      amqMN_Ht.put("6708",new AMQMessageNumber(6708,
                  "(iSeries) A disk full condition was encountered when formatting a new log file.",
                  "20 : Error",
                  "The queue manager attempted to format a new log file in directory <insert_3>. The drive or file system containing this directory did not have sufficient free space to contain the new log file.",
                  "Increase the amount of space available for log files and retry the request."));
      amqMN_Ht.put("6709",new AMQMessageNumber(6709,
                  "The log for the Queue manager is full.",
                  "20 : Error",
                  "This message is issued when an attempt to write a log record is rejected because the log is full. The queue manager will attempt to resolve the problem.",
                  "This situation may be encountered during a period of unusually high message traffic. However, if you persistently fill the log, you may have to consider enlarging the size of the log. You can either increase the number of log files by changing the values in the queue manager configuration file. You will then have to stop and restart the queue manager. Alternatively, if you need to make the log files themselves bigger, you will have to delete and recreate the queue manager."));
      amqMN_Ht.put("6710",new AMQMessageNumber(6710,
                  "Queue manager unable to access directory <insert_3>.",
                  "20 : Error",
                  "The queue manager was unable to access directory <insert_3> for the log. This could be because the directory does not exist, or because the queue manager does not have sufficient authority.",
                  "Ensure that the directory exists and that the queue manager has authority to read and write to it. Ensure that the LogPath attribute in the queue manager's configuration file matches the intended log path."));
      amqMN_Ht.put("6767",new AMQMessageNumber(6767,
                  "Log file <insert_3> could not be opened for use.",
                  "20 : Error",
                  "Log file <insert_3> could not be opened for use. Possible reasons include the file being missing, the queue manager being denied permission to open the file or the contents of the file being incorrect.",
                  "If the log file was required to start the queue manager, ensure that the log file exists and that the queue manager is able to read from and write to it. If the log file was required to recreate an object from its media image and you do not have a copy of the required log file, delete the object instead of recreating it."));
      amqMN_Ht.put("6774",new AMQMessageNumber(6774,
                  "Log file <insert_3> did not contain the requested log record.",
                  "20 : Error",
                  "Log file <insert_3> does not contain the log record whose LSN is <insert_4>. This is because the log file numbers have wrapped and the log file name <insert_3> has been reused by a newer file. Once a log file name has been reused, it is not possible to access the data in the previous versions of the file to use this name. The operation which requested this log record cannot be completed.",
                  ""));
      amqMN_Ht.put("6782",new AMQMessageNumber(6782,
                  "The log file numbers have wrapped.",
                  "0 : Information",
                  "Each log file formatted is assigned a number which makes up part of its file name. The numbers are allocated sequentially and consist of seven digits giving a maximum of 10 million different log file names. Once all available numbers have been allocated, the queue manager again starts allocating numbers starting from zero. Once a file number has been re-allocated, you can no longer access data in the previous log files allocated the same number. The file numbers wrapped at log sequence number <insert_3>.",
                  "You should periodically take media images of all WebSphere MQ objects. You must ensure that media images of all objects which you may need to recreate do not span more than 10 million log files."));
      amqMN_Ht.put("6901",new AMQMessageNumber(6901,
                  "(iSeries) WebSphere MQ for iSeries",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6902",new AMQMessageNumber(6902,
                  "(iSeries) WebSphere MQ for iSeries - Samples",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6903",new AMQMessageNumber(6903,
                  "(iSeries) Installation failed, WebSphere MQ resources are still active.",
                  "30 : Severe error",
                  "An attempt to install WebSphere MQ was unsuccessful because MQ resources from a previous installation of MQ are still active. This failure may indicate that a queue manager from a previous installation of MQ is still running or has active jobs.",
                  "Ensure that all queue managers from previous installations of WebSphere MQ have been quiesced, and that the QMQM subsystem is not active using the WRKSBS and ENDSBS commands. Refer to the installation section in the WebSphere MQ for iSeries Quick Beginnings publication for further details."));
      amqMN_Ht.put("6904",new AMQMessageNumber(6904,
                  "(iSeries) Installation of WebSphere MQ for iSeries failed due to previous release installed.",
                  "30 : Severe error",
                  "Some releases of WebSphere MQ for iSeries require migration before a later release can be installed.",
                  "\n If you wish to retain your current MQ information you must step through the migration process - see the Quick Beginnings Manual. If you do not wish to retain your current MQ information remove the current version of MQ before retrying the install."));
      amqMN_Ht.put("6905",new AMQMessageNumber(6905,
                  "(iSeries) Found <insert_3> new MQ jobs to end, and <insert_4> MQ jobs currently ending.",
                  "0 : Information",
                  "Jobs with locks on library QMQM are ended so that WebSphere MQ may be deleted or updated.",
                  "None."));
      amqMN_Ht.put("6906",new AMQMessageNumber(6906,
                  "(iSeries) <insert_3> jobs still ending.",
                  "40 : Stop Error",
                  "Jobs report state of 'already being deleted' after timeout.",
                  "If system is heavily loaded wait and reissue the command CALL QMQM/AMQIQES4 to try to delete jobs using WebSphere MQ resources. If this message is issued again, issue the command WRKOBJLCK for library QMQM to see which jobs have not been deleted, and end them manually."));
      amqMN_Ht.put("6907",new AMQMessageNumber(6907,
                  "(iSeries) All WebSphere MQ pre-requisite PTFs on OS/400 programs are installed.",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("6908",new AMQMessageNumber(6908,
                  "(iSeries) WebSphere MQ pre-requisite PTF <insert_4> for program <insert_3> is not installed.",
                  "40 : Stop Error",
                  "PTF <insert_3>-<insert_4> is not installed on system in state 'Permanently applied' 'Temporarily applied' or 'Superseded'. WebSphere MQ installation will proceed, but you must install the PTF before starting WebSphere MQ.",
                  "Use the command GO CMDPTF to display commands to order and apply the required PTF <insert_3>-<insert_4>.."));
      amqMN_Ht.put("6909",new AMQMessageNumber(6909,
                  "(iSeries) User space recovery failed, WebSphere MQ is running.",
                  "30 : Severe error",
                  "An attempt to recover user space was unsuccessful because MQ was running.",
                  "Quiesce WebSphere MQ for iSeries and try again. See the section on \"Quiescing WebSphere MQ\" in the WebSphere MQ for iSeries Quick Beginnings."));
      amqMN_Ht.put("6910",new AMQMessageNumber(6910,
                  "(iSeries) The attempt to quiesce the queue manager failed.",
                  "30 : Severe error",
                  "The attempt to quiesce the queue manager was unsuccessful because the current job has locks on library QMQM.",
                  "Sign off the current job, sign on and attempt to quiesce the queue manager again. See the section on \"Quiescing WebSphere MQ\" in the WebSphere MQ for iSeries Quick Beginnings."));
      amqMN_Ht.put("6911",new AMQMessageNumber(6911,
                  "(iSeries) WebSphere MQ quiesce is performing a RCDMQMIMG, please wait.",
                  "0 : Information",
                  "WebSphere MQ quiesce is performing a Record Object Image (RCDMQMIMG) for all objects. Please wait, there may be some delay until this is completed.",
                  "None."));
      amqMN_Ht.put("6912",new AMQMessageNumber(6912,
                  "(iSeries) WebSphere MQ for iSeries - Java",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6913",new AMQMessageNumber(6913,
                  "(iSeries) WebSphere MQ for iSeries - Java samples",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6988",new AMQMessageNumber(6988,
                  "yes",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6988",new AMQMessageNumber(6988,
                  "(iSeries) Yes",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6989",new AMQMessageNumber(6989,
                  "no",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6989",new AMQMessageNumber(6989,
                  "(iSeries) No",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6992",new AMQMessageNumber(6992,
                  "(iSeries) Program <insert_3> parameter error.",
                  "40 : Stop Error",
                  "WebSphere MQ for iSeries program <insert_3> has an incorrect number of parameters, or an error in the parameter value.",
                  "Display the job log, using the DSPJOBLOG command, for more information on the problem."));
      amqMN_Ht.put("6993",new AMQMessageNumber(6993,
                  "(iSeries) Program <insert_3> ended abnormally.",
                  "40 : Stop Error",
                  "A WebSphere MQ for iSeries program, <insert_3>, is ending abnormally.",
                  "Display the job log, using the DSPJOBLOG command, for information why the job or subsystem ended abnormally. Correct the error and retry the request."));
      amqMN_Ht.put("6994",new AMQMessageNumber(6994,
                  " (Windows)5724-H72 (C) Copyright IBM Corp. 1994, 2004. ALL RIGHTS RESERVED.",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("6995",new AMQMessageNumber(6995,
                  "(iSeries) xcsFFST has been called; take a look at the job log.",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("6998",new AMQMessageNumber(6998,
                  "(iSeries) An internal WebSphere MQ error has occurred.",
                  "40 : Stop Error",
                  "WebSphere MQ for iSeries is diagnosing an unexpected error.",
                  "Save the job log, and contact your IBM support center."));
      amqMN_Ht.put("6999",new AMQMessageNumber(6999,
                  "(iSeries) An internal WebSphere MQ error has occurred.",
                  "0 : Information",
                  "WebSphere MQ has experienced an internal failure, from which it could not recover.",
                  "Use WRKPRB to check if a problem has been created. If one has, record the problem identifier, and save the QPSRVDMP, QPJOBLOG, and QPDSPJOB files. If a problem has not been created, save the job log. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
   }
   
   /**
    * Initialize the AMQ 7000's message number. 
    */
   public void init7s()
   {
      amqMN_Ht.put("7001",new AMQMessageNumber(7001,
            "The location specified for creation of the queue manager is not valid.",
            "40 : Stop Error",
            "The directory under which queue managers are to be created is not valid. It may not exist, or there may be a problem with authorization.",
            "The location is specified in the machine-wide ini file. Correct the file and submit the request again."));
      amqMN_Ht.put("7001",new AMQMessageNumber(7001,
                  " (Windows)The location specified for the creation of the queue manager is not valid.",
                  "40 : Stop Error",
                  "The directory under which the queue managers are to be created is not valid. It may not exist, or there may be a problem with authorization.",
                  "The location is specified in the configuration data. Correct the configuration data and submit the request again."));
      amqMN_Ht.put("7002",new AMQMessageNumber(7002,
                  "An error occurred manipulating a file.",
                  "40 : Stop Error",
                  "An internal error occurred while trying to create or delete a queue manager file. It is likely that the error was caused by a disk having insufficient space, or by problems with authorization to the underlying filesystem.",
                  "Identify the file that caused the error, using problem determination techniques. For example check if there are any FFST files, which may identify the queue manager file causing the error. This error may also be caused if users have created, renamed or deleted that file. Correct the error in the filesystem and submit the request again."));
      amqMN_Ht.put("7002",new AMQMessageNumber(7002,
                  " (Windows)An error occurred manipulating a file.",
                  "40 : Stop Error",
                  "An internal error occurred while trying to create or delete a queue manager file. In the case of a failure to delete a file a common reason for this error is that a non MQ process, such as the windows explorer or a virus checker, is accessing the file. In the case where the object that cannot be deleted is a directory then a non MQ process may be accessing a file within the directory or one of its subdirectories. It is also possible that the error was caused by a disk having insufficient space, or by problems with authorization to the underlying filesystem.",
                  "Identify the file that caused the error, using problem determination techniques. For example check if there are any FFST files, which may identify the queue manager file causing the error. This error may also be caused if users have created, renamed or deleted that file. Correct the error in the filesystem and submit the request again."));
      amqMN_Ht.put("7005",new AMQMessageNumber(7005,
                  "The queue manager is running.",
                  "40 : Stop Error",
                  "You tried to perform an action that requires the queue manager stopped, however, it is currently running. You probably tried to delete or start a queue manager that is currently running.",
                  "If the queue manager should be stopped, stop the queue manager and submit the failed command again."));
      amqMN_Ht.put("7006",new AMQMessageNumber(7006,
                  "Missing attribute <insert_5> on stanza starting on line <insert_1> of ini file <insert_3>.",
                  "20 : Error",
                  "The <insert_4> stanza starting on line <insert_1> of configuration file <insert_3> is missing the required <insert_5> attribute.",
                  "Check the contents of the file and retry the operation."));
      amqMN_Ht.put("7006",new AMQMessageNumber(7006,
                  " (Windows)Missing attribute <insert_5> from configuration data.",
                  "20 : Error",
                  "The <insert_4> stanza in the configuration data is missing the required <insert_5> attribute.",
                  "Check the contents of the configuration data and retry the operation."));
      amqMN_Ht.put("7008",new AMQMessageNumber(7008,
                  "The queue manager already exists.",
                  "40 : Stop Error",
                  "You tried to create a queue manager that already exists.",
                  "If you specified the wrong queue manager name, correct the name and submit the request again."));
      amqMN_Ht.put("7010",new AMQMessageNumber(7010,
                  "The queue manager does not exist.",
                  "40 : Stop Error",
                  "You tried to perform an action against a queue manager that does not exist. You may have specified the wrong queue manager name.",
                  "If you specified the wrong name, correct it and submit the command again. If the queue manager should exist, create it, and then submit the command again."));
      amqMN_Ht.put("7011",new AMQMessageNumber(7011,
                  "The queue manager files have not been completely deleted.",
                  "40 : Stop Error",
                  "While deleting the queue manager, an error occurred deleting a file or directory. The queue manager may not have been completely deleted.",
                  "Follow problem determination procedures to identify the file or directory and to complete deletion of the queue manager."));
      amqMN_Ht.put("7012",new AMQMessageNumber(7012,
                  "The specified trigger interval is not valid.",
                  "40 : Stop Error",
                  "You specified a value for the trigger interval that is not valid. The value must be not less than zero and not greater than 999 999 999.",
                  "Correct the value and resubmit the request."));
      amqMN_Ht.put("7013",new AMQMessageNumber(7013,
                  "There is an error in the name of the specified dead-letter queue.",
                  "40 : Stop Error",
                  "You specified a name for the dead-letter queue that is not valid.",
                  "Correct the name and resubmit the request."));
      amqMN_Ht.put("7014",new AMQMessageNumber(7014,
                  "There is an error in the name of the specified default transmission queue.",
                  "40 : Stop Error",
                  "You specified a name for the default transmission queue that is not valid.",
                  "Correct the name and submit the command again."));
      amqMN_Ht.put("7015",new AMQMessageNumber(7015,
                  "There is an error in the maximum number of open object handles specified.",
                  "40 : Stop Error",
                  "You specified a value for the maximum number of open object handles to be allowed that is not valid. The value must be not less than zero and not greater than 999 999 999.",
                  "Correct the value and submit the command again."));
      amqMN_Ht.put("7016",new AMQMessageNumber(7016,
                  "There is an error in the maximum number of uncommitted messages specified.",
                  "40 : Stop Error",
                  "You specified a value for the maximum number of uncommitted messages to be allowed that is not valid. The value must be not less than 1 and not greater than 999 999 999.",
                  "Correct the value and submit the command again."));
      amqMN_Ht.put("7017",new AMQMessageNumber(7017,
                  "Log not available.",
                  "40 : Stop Error",
                  "The queue manager was unable to use the log. This could be due to a log file being missing or damaged, or the log path to the queue manager being inaccessible.",
                  "Ensure that the LogPath attribute in the queue manager configuration file is correct. If a log file is missing or otherwise unusable, restore a backup copy of the file, or the entire queue manager."));
      amqMN_Ht.put("7018",new AMQMessageNumber(7018,
                  "The queue manager operation cannot be completed.",
                  "20 : Error",
                  "An attempt has been made to perform an operation on a queue manager. Resources required to perform the operation are not available.",
                  ""));
      amqMN_Ht.put("7019",new AMQMessageNumber(7019,
                  "An error occurred while creating the directory structure for the new queue manager.",
                  "40 : Stop Error",
                  "During creation of the queue manager an error occurred while trying to create a file or directory.",
                  "Identify why the queue manager files cannot be created. It is probable that there is insufficient space on the specified disk, or that there is a problem with access control. Correct the problem and submit the command again."));
      amqMN_Ht.put("7020",new AMQMessageNumber(7020,
                  "The operation was carried out, but one or more transactions remain in-doubt.",
                  "10 : Warning",
                  "The queue manager tried to resolve all internally coordinated transactions which are in-doubt. In-doubt transactions still remain after the queue manager has attempted to deliver the outcome of these transactions to the resource managers concerned. Transactions remain in-doubt when the queue manager cannot deliver the outcome of the transaction to each of the participating resource managers. For example, a resource manager may not be available at this time.",
                  "Use the DSPMQTRN command to display the remaining in-doubt transactions."));
      amqMN_Ht.put("7020",new AMQMessageNumber(7020,
                  "(iSeries) The operation was carried out, but one or more transactions remain in-doubt.",
                  "10 : Warning",
                  "The queue manager tried to resolve all internally coordinated transactions which are in-doubt. In-doubt transactions still remain after the queue manager has attempted to deliver the outcome of these transactions to the resource managers concerned. Transactions remain in-doubt when the queue manager cannot deliver the outcome of the transaction to each of the participating resource managers. For example, a resource manager may not be available at this time.",
                  "Use the Work with Transactions (WRKMQMTRN) command to display the remaining in-doubt transactions."));
      amqMN_Ht.put("7021",new AMQMessageNumber(7021,
                  "An error occurred while deleting the directory structure for the queue manager.",
                  "40 : Stop Error",
                  "While deleting the queue manager, an error occurred deleting a file or directory. The queue manager may not have been completely deleted.",
                  "Follow problem determination procedures to identify the file or directory and to complete deletion of the queue manager."));
      amqMN_Ht.put("7022",new AMQMessageNumber(7022,
                  "The resource manager identification number is not recognized.",
                  "20 : Error",
                  "The identification number of the resource manager you supplied was not recognized.",
                  "Ensure that you entered a valid resource manager identification number. Use the DSPMQTRN command to display a list of resource managers and their identification numbers."));
      amqMN_Ht.put("7023",new AMQMessageNumber(7023,
                  "The resource manager was in an invalid state.",
                  "20 : Error",
                  "The resource manager, the identification number of which you supplied, was in an invalid state.",
                  "Ensure that you entered the correct resource manager identification number. Use the DSPMQTRN command to display a list of resource managers and their identification numbers. A resource manager is in an invalid state, if it is still available to resolve the transaction, use the -a optional flag to resolve this and all other internally coordinated in-doubt transactions."));
      amqMN_Ht.put("7024",new AMQMessageNumber(7024,
                  "Arguments supplied to a command are not valid.",
                  "20 : Error",
                  "You supplied arguments to a command that it could not interpret. It is probable that you specified a flag not accepted by the command, or that you included extra flags.",
                  "Correct the command and submit it again. Additional information on the arguments causing the error may be found in the error logs for the queue, or queue manager, referenced in the command."));
      amqMN_Ht.put("7025",new AMQMessageNumber(7025,
                  "Error in the descriptive text argument (-c parameter) of the crtmqm command.",
                  "40 : Stop Error",
                  "The descriptive text you supplied to the crtmqm command was in error.",
                  "Correct the descriptive text argument and submit the command again."));
      amqMN_Ht.put("7026",new AMQMessageNumber(7026,
                  "A principal or group name was invalid.",
                  "40 : Stop Error",
                  "You specified the name of a principal or group which does not exist.",
                  "Correct the name and resubmit the request."));
      amqMN_Ht.put("7027",new AMQMessageNumber(7027,
                  "Argument <insert_3> supplied to command <insert_4> is invalid.",
                  "20 : Error",
                  "The argument <insert_3> was supplied to the command <insert_4> which could not be interpreted. This argument is either not accepted by the command, or an extra flag has been included.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7028",new AMQMessageNumber(7028,
                  "The queue manager is not available for use.",
                  "40 : Stop Error",
                  "You have requested an action that requires the queue manager running, however, the queue manager is not currently running.",
                  "Start the required queue manager and submit the command again."));
      amqMN_Ht.put("7030",new AMQMessageNumber(7030,
                  "Quiesce request accepted. The queue manager will stop when all outstanding work is complete.",
                  "0 : Information",
                  "You have requested that the queue manager end when there is no more work for it. In the meantime, it will refuse new applications that attempt to start, although it allows those already running to complete their work.",
                  "None."));
      amqMN_Ht.put("7031",new AMQMessageNumber(7031,
                  "The queue manager is stopping.",
                  "40 : Stop Error",
                  "You issued a command that requires the queue manager running, however, it is currently in the process of stopping. The command cannot be run.",
                  "None"));
      amqMN_Ht.put("7041",new AMQMessageNumber(7041,
                  "Object already exists.",
                  "40 : Stop Error",
                  "A Define Object operation was performed, but the name selected for the object is already in use by an object that is unknown to WebSphere MQ. The object name selected by MQ was <insert_3>, in directory <insert_4>, of object type <insert_5>.",
                  "Remove the conflicting object from the MQ system, then try the operation again."));
      amqMN_Ht.put("7042",new AMQMessageNumber(7042,
                  "Media image not available for object <insert_3> of type <insert_4>.",
                  "20 : Error",
                  "The media image for object <insert_3>, type <insert_4>, is not available for media recovery. A log file containing part of the media image cannot be accessed.",
                  "A previous message indicates which log file could not be accessed. Restore a copy of the log file and all subsequent log files from backup. If this is not possible, you must delete the object instead."));
      amqMN_Ht.put("7042",new AMQMessageNumber(7042,
                  "(iSeries) Media image not available for object <insert_3>.",
                  "20 : Error",
                  "The media image for object <insert_3>, type <insert_4>, is not available for media recovery. A log file containing part of the media image cannot be accessed.",
                  "A previous message indicates which log file could not be accessed. Restore a copy of the log file and all subsequent log files from backup. If this is not possible, you must delete the object instead."));
      amqMN_Ht.put("7044",new AMQMessageNumber(7044,
                  "Media recovery not allowed.",
                  "20 : Error",
                  "Media recovery is not possible on a queue manager using a circular log. Damaged objects must be deleted on such a queue manager.",
                  "None."));
      amqMN_Ht.put("7047",new AMQMessageNumber(7047,
                  "An unexpected error was encountered by a command.",
                  "40 : Stop Error",
                  "An internal error occurred during the processing of a command.",
                  "Follow problem determination procedures to identify the cause of the error."));
      amqMN_Ht.put("7048",new AMQMessageNumber(7048,
                  "The queue manager name is either not valid or not known",
                  "40 : Stop Error",
                  "Either the specified queue manager name does not conform to the rules required by WebSphere MQ or the queue manager does not exist. The rules for naming MQ objects are detailed in the WebSphere MQ Command Reference.",
                  "Correct the name and submit the command again."));
      amqMN_Ht.put("7053",new AMQMessageNumber(7053,
                  "The transaction has been committed.",
                  "0 : Information",
                  "The prepared transaction has been committed.",
                  "None."));
      amqMN_Ht.put("7054",new AMQMessageNumber(7054,
                  "The transaction has been backed out.",
                  "0 : Information",
                  "The prepared transaction has been backed out.",
                  "None."));
      amqMN_Ht.put("7055",new AMQMessageNumber(7055,
                  "The transaction number is not recognized.",
                  "20 : Error",
                  "The number of the transaction you supplied was not recognized as belonging to an in-doubt transaction.",
                  "Ensure that you entered a valid transaction number. It is possible that the transaction number you entered corresponds to a transaction which was committed or backed out before you issued the command to resolve it."));
      amqMN_Ht.put("7056",new AMQMessageNumber(7056,
                  "Transaction number <insert_1>,<insert_2>.",
                  "0 : Information",
                  "This message is used to report the number of an in-doubt transaction.",
                  "None."));
      amqMN_Ht.put("7059",new AMQMessageNumber(7059,
                  "An error has occurred reading an INI file.",
                  "20 : Error",
                  "An error has occurred when reading the MQS.INI file or a queue manager QM.INI file.",
                  "If you have been changing the INI file content check and correct the change. If you have not changed the INI file, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7059",new AMQMessageNumber(7059,
                  " (Windows)An error occurred when reading the configuration data.",
                  "20 : Error",
                  "An error has occurred when reading the configuration data.",
                  "If you have changed the configuration data, check and correct the change. If you have not changed the configuration data, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7060",new AMQMessageNumber(7060,
                  "An error has occurred locking an INI file.",
                  "20 : Error",
                  "An error has occurred locking the MQS.INI file or a queue manager QM.INI file.",
                  "If you have been changing the INI file permissions check and correct the change. If you have not changed the INI file, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7060",new AMQMessageNumber(7060,
                  " (Windows)An error has occurred locking the configuration data.",
                  "20 : Error",
                  "An error has occurred locking the configuration data.",
                  "If you have changed the configuration data permissions, check and correct the change. If you have not changed the configuration data, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7061",new AMQMessageNumber(7061,
                  "An expected stanza in an INI file is missing or contains errors.",
                  "20 : Error",
                  "An expected stanza is missing from the MQS.INI file or a queue manager QM.INI file or the stanza contains errors.",
                  "If you have been changing the INI file content check and correct the change."));
      amqMN_Ht.put("7061",new AMQMessageNumber(7061,
                  " (Windows)An expected stanza in the configuration data is missing or contains errors.",
                  "20 : Error",
                  "An expected stanza is missing from the configuration data or the stanza contains errors.",
                  "If you have changed the configuration data, check and correct the change."));
      amqMN_Ht.put("7062",new AMQMessageNumber(7062,
                  "Unable to access an INI file.",
                  "20 : Error",
                  "Access to the MQS.INI file or a queue manager QM.INI file is denied.",
                  "If you have been changing the INI file permissions check and correct the change."));
      amqMN_Ht.put("7062",new AMQMessageNumber(7062,
                  " (Windows)Unable to access the configuration data.",
                  "20 : Error",
                  "Access to the configuration data is denied.",
                  "If you have changed the configuration data permissions, check and correct the change."));
      amqMN_Ht.put("7063",new AMQMessageNumber(7063,
                  "An INI file is missing.",
                  "20 : Error",
                  "The MQS.INI file or a queue manager QM.INI file is missing.",
                  "If you have been changing the INI file recover the previous file and retry the operation."));
      amqMN_Ht.put("7063",new AMQMessageNumber(7063,
                  " (Windows)Configuration data is missing.",
                  "20 : Error",
                  "The configuration data for WebSphere MQ is missing.",
                  "If you have changed the configuration data, recover the previous configuration data and retry the operation."));
      amqMN_Ht.put("7064",new AMQMessageNumber(7064,
                  "Log path not valid or inaccessible.",
                  "40 : Stop Error",
                  "The supplied log path could not be used by the queue manager. Possible reasons for this include the path not existing, the queue manager not being able to write to the path, or the path residing on a remote device.",
                  "Ensure that the log path exists and that the queue manager has authority to read and write to it. If the queue manager already exists, ensure that the LogPath attribute in the queue manager's configuration file matches the intended log path."));
      amqMN_Ht.put("7064",new AMQMessageNumber(7064,
                  "(iSeries) Auxiliary storage pool identifier not found.",
                  "40 : Stop Error",
                  "The auxiliary storage pool identifier supplied does not exist on the system and could not be used by the queue manager to create a journal receiver.",
                  "Specify *SYSTEM, or the identifier of an existing auxiliary storage pool and try the request again. You can use WRKDSKSTS to check the assignment of disk units to auxiliary storage pools."));
      amqMN_Ht.put("7065",new AMQMessageNumber(7065,
                  "Insufficient space on disk.",
                  "40 : Stop Error",
                  "The operation cannot be completed due to shortage of disk space.",
                  "Either make more disk space available, or reduce the disk requirements of the command you issued."));
      amqMN_Ht.put("7066",new AMQMessageNumber(7066,
                  "There are no prepared transactions.",
                  "10 : Warning",
                  "There are no prepared transactions to be resolved.",
                  "None."));
      amqMN_Ht.put("7068",new AMQMessageNumber(7068,
                  "Authority file contains an authority stanza that is not valid.",
                  "40 : Stop Error",
                  "A syntax error has been found in one of the files containing authorization information for the queue manager.",
                  "Correct the contents of the incorrect authorization file by editing it."));
      amqMN_Ht.put("7069",new AMQMessageNumber(7069,
                  "The queue manager was created successfully, but cannot be made the default.",
                  "40 : Stop Error",
                  "The queue manager was defined to be the default queue manager for the machine when it was created. However, although the queue manager has been created, an error occurred trying to make it the default. There may not be a default queue manager defined for the machine at present.",
                  "There is probably a problem with the machine-wide ini file. Verify the existence of the file, its access permissions, and its contents. If its backup file exists, reconcile the contents of the two files and then delete the backup. Finally, either update the machine-wide ini file by hand to specify the desired default queue manager, or delete and recreate the queue manager."));
      amqMN_Ht.put("7069",new AMQMessageNumber(7069,
                  " (Windows)The queue manager was created successfully, but cannot be made the default.",
                  "40 : Stop Error",
                  "The queue manager was defined to be the default queue manager for the machine when it was created. However, although the queue manager has been created, an error occurred trying to make it the default. There may not be a default queue manager defined for the machine at present.",
                  "There is probably a problem with the configuration data. Update the configuration data to specify the desired default queue manager, or delete and recreate the queue manager."));
      amqMN_Ht.put("7072",new AMQMessageNumber(7072,
                  "Invalid QM.INI file stanza. Refer to the error log for more information.",
                  "40 : Stop Error",
                  "An invalid QM.INI file stanza was found. Refer to the error log for more information.",
                  "Correct the error and then retry the operation."));
      amqMN_Ht.put("7072",new AMQMessageNumber(7072,
                  " (Windows)Stanza not valid. Refer to the error log for more information.",
                  "40 : Stop Error",
                  "A stanza that is not valid was found. Refer to the error log for more information.",
                  "Correct the error and retry the operation."));
      amqMN_Ht.put("7073",new AMQMessageNumber(7073,
                  "Log size not valid.",
                  "40 : Stop Error",
                  "Either the number of log files or the size of the log files was outside the accepted values.",
                  "Make sure that the log parameters you enter lie within the valid range."));
      amqMN_Ht.put("7074",new AMQMessageNumber(7074,
                  "Unknown stanza key <insert_4> on line <insert_1> of ini file <insert_3>.",
                  "10 : Warning",
                  "Line <insert_1> of the configuration file <insert_3> contained a stanza called <insert_3>. This stanza is not recognized.",
                  "Check the contents of the file and retry the operation."));
      amqMN_Ht.put("7074",new AMQMessageNumber(7074,
                  "(iSeries) Unknown stanza key.",
                  "10 : Warning",
                  "Line <insert_1> of the configuration file <insert_3> contained a stanza key <insert_4>. This stanza is not recognized.",
                  "Check the contents of the file and retry the operation."));
      amqMN_Ht.put("7074",new AMQMessageNumber(7074,
                  " (Windows)Unknown stanza key <insert_4> at <insert_3> in the configuration data.",
                  "10 : Warning",
                  "Key <insert_3> contained a stanza called <insert_4>. This stanza is not recognized.",
                  "Check the contents of the configuration data and retry the operation."));
      amqMN_Ht.put("7075",new AMQMessageNumber(7075,
                  "Unknown attribute in ini file.",
                  "10 : Warning",
                  "Line <insert_1> of the configuration file <insert_3> contained an attribute called <insert_4> that is not valid. This attribute is not recognized in this context.",
                  "Check the contents of the file and retry the operation."));
      amqMN_Ht.put("7075",new AMQMessageNumber(7075,
                  " (Windows)Unknown attribute <insert_4> at <insert_3> in the configuration data.",
                  "10 : Warning",
                  "Key <insert_3> in the configuration data contained an attribute called <insert_4> that is not valid. This attribute is not recognized in this context.",
                  "Check the contents of the configuration data and retry the operation."));
      amqMN_Ht.put("7076",new AMQMessageNumber(7076,
                  "Invalid value for attribute in ini file.",
                  "10 : Warning",
                  "Line <insert_1> of the configuration file <insert_3> contained value <insert_5> that is not valid for the attribute <insert_4>.",
                  "Check the contents of the file and retry the operation."));
      amqMN_Ht.put("7076",new AMQMessageNumber(7076,
                  " (Windows)Value <insert_5> not valid for attribute <insert_4> at <insert_3> in the configuration data.",
                  "10 : Warning",
                  "Key <insert_3> in the configuration data contained value <insert_5> that is not valid for the attribute <insert_4>.",
                  "Check the contents of the configuration data and retry the operation."));
      amqMN_Ht.put("7077",new AMQMessageNumber(7077,
                  "You are not authorized to perform the requested operation.",
                  "40 : Stop Error",
                  "You tried to issue a command for the queue manager. You are not authorized to perform the command.",
                  "Contact your system administrator to perform the command for you. Alternatively, request authority to perform the command from your system administrator."));
      amqMN_Ht.put("7078",new AMQMessageNumber(7078,
                  "You entered an object type that is invalid with a generic profile name.",
                  "40 : Stop Error",
                  "You entered an object type of *ALL or *MQM and an object name that contains generic characters, this is an invalid combination.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7080",new AMQMessageNumber(7080,
                  "No objects processed.",
                  "10 : Warning",
                  "No objects were processed, either because no objects matched the criteria given, or because the objects found did not require processing.",
                  "None."));
      amqMN_Ht.put("7081",new AMQMessageNumber(7081,
                  "Object <insert_3>, type <insert_4> recreated.",
                  "0 : Information",
                  "The object <insert_3>, type <insert_4> was recreated from its media image.",
                  "None."));
      amqMN_Ht.put("7082",new AMQMessageNumber(7082,
                  "Object <insert_3>, type <insert_4> is not damaged.",
                  "10 : Warning",
                  "Object <insert_3>, type <insert_4> cannot be recreated since it is not damaged.",
                  "None"));
      amqMN_Ht.put("7083",new AMQMessageNumber(7083,
                  "A resource problem was encountered by a command.",
                  "20 : Error",
                  "The command failed due to a resource problem. Possible causes include the log being full or the command running out of memory.",
                  "Look at the previous messages to diagnose the problem. Rectify the problem and retry the operation."));
      amqMN_Ht.put("7084",new AMQMessageNumber(7084,
                  "Object <insert_3>, type <insert_4> damaged.",
                  "20 : Error",
                  "The object <insert_3>, type <insert_4> was damaged. The object must be deleted or, if the queue manager supports media recovery, recreated from its media image.",
                  "Delete the object or recreate it from its media image."));
      amqMN_Ht.put("7085",new AMQMessageNumber(7085,
                  "Object <insert_3>, type <insert_4> not found.",
                  "20 : Error",
                  "Object <insert_3>, type <insert_4> cannot be found.",
                  "None."));
      amqMN_Ht.put("7086",new AMQMessageNumber(7086,
                  "Media image for object <insert_3>, type <insert_4> recorded.",
                  "0 : Information",
                  "The media image for object <insert_3>, type <insert_4>, defined in Queue Manager <insert_5>, has been recorded.",
                  "None."));
      amqMN_Ht.put("7087",new AMQMessageNumber(7087,
                  "Object <insert_3>, type <insert_4> is a temporary object",
                  "20 : Error",
                  "Object <insert_3>, type <insert_4> is a temporary object. Media recovery operations are not permitted on temporary objects.",
                  "None."));
      amqMN_Ht.put("7088",new AMQMessageNumber(7088,
                  "Object <insert_3>, type <insert_4> in use.",
                  "20 : Error",
                  "Object <insert_3>, type <insert_4> is in use. Either an application has it open or, if it is a local queue, there are uncommitted messages on it.",
                  "Ensure that the object is not opened by any applications, and that there are no uncommitted messages on the object, if it is a local queue. Then, retry the operation."));
      amqMN_Ht.put("7089",new AMQMessageNumber(7089,
                  "Media recovery already in progress.",
                  "20 : Error",
                  "Another media recovery operation is already in progress. Only one media recovery operation is permitted at a time.",
                  "Wait for the existing media recovery operation to complete and retry the operation."));
      amqMN_Ht.put("7090",new AMQMessageNumber(7090,
                  "(iSeries) The queue manager CCSID is not valid.",
                  "40 : Stop Error",
                  "The CCSID to be used by the QMGR is not valid for the iSeries platform. The CCSID encoding must be a valid EBCDIC value.",
                  "Check that the CCSID that you have entered is a valid EBCDIC value."));
      amqMN_Ht.put("7090",new AMQMessageNumber(7090,
                  " (Windows)The queue manager CCSID is not valid.",
                  "40 : Stop Error",
                  "The CCSID to be used by the QMGR is not valid, because: 1) It is a DBCS CCSID. 2) The CCSID encoding is not ASCII or ASCII related. EBCDIC or UCS2 encodings are not valid on this machine. 3) The CCSID encoding is unknown.",
                  "Check the CCSID is valid for the machine on which you are working."));
      amqMN_Ht.put("7091",new AMQMessageNumber(7091,
                  "You are performing authorization for the queue manager, but you specified an object name.",
                  "40 : Stop Error",
                  "Modification of authorizations for a queue manager can be performed only from that queue manager. You must not specify an object name.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7092",new AMQMessageNumber(7092,
                  "An object name is required but you did not specify one.",
                  "40 : Stop Error",
                  "The command needs the name of an object, but you did not specify one.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7093",new AMQMessageNumber(7093,
                  "An object type is required but you did not specify one.",
                  "40 : Stop Error",
                  "The command needs the type of the object, but you did not specify one.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7094",new AMQMessageNumber(7094,
                  "You specified an object type that is not valid, or more than one object type.",
                  "40 : Stop Error",
                  "Either the type of object you specified was not valid, or you specified multiple object types on a command which supports only one.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7095",new AMQMessageNumber(7095,
                  "An entity name is required but you did not specify one.",
                  "40 : Stop Error",
                  "The command needs one or more entity names, but you did not specify any. Entities can be principals or groups.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7096",new AMQMessageNumber(7096,
                  "An authorization specification is required but you did not provide one.",
                  "40 : Stop Error",
                  "The command sets the authorizations on WebSphere MQ objects. However you did not specify which authorizations are to be set.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7097",new AMQMessageNumber(7097,
                  "You gave an authorization specification that is not valid.",
                  "40 : Stop Error",
                  "The authorization specification you provided to the command contained one or more items that could not be interpreted.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7098",new AMQMessageNumber(7098,
                  "The command accepts only one entity name. You specified more than one.",
                  "40 : Stop Error",
                  "The command can accept only one principal or group name. You specified more than one.",
                  "Correct the command and submit it again."));
      amqMN_Ht.put("7099",new AMQMessageNumber(7099,
                  "Entity <insert_3> has the following authorizations for object <insert_4>:",
                  "0 : Information",
                  "Informational message. The list of authorizations follows.",
                  "None."));
      amqMN_Ht.put("7104",new AMQMessageNumber(7104,
                  "Resource manager <insert_1> has prepared.",
                  "0 : Information",
                  "This message reports the state of a resource manager with respect to an in-doubt transaction.",
                  "None."));
      amqMN_Ht.put("7105",new AMQMessageNumber(7105,
                  "Resource manager <insert_1> has committed.",
                  "0 : Information",
                  "This message reports the state of a resource manager with respect to an in-doubt transaction.",
                  "None."));
      amqMN_Ht.put("7106",new AMQMessageNumber(7106,
                  "Resource manager <insert_1> has rolled back.",
                  "0 : Information",
                  "This message reports the state of a resource manager with respect to an in-doubt transaction.",
                  "None."));
      amqMN_Ht.put("7107",new AMQMessageNumber(7107,
                  "Resource manager <insert_1> is <insert_3>.",
                  "0 : Information",
                  "This message reports the identification number and name of a resource manager.",
                  "None."));
      amqMN_Ht.put("7108",new AMQMessageNumber(7108,
                  "Any in-doubt transactions have been resolved.",
                  "0 : Information",
                  "All, if there were any, of the internally coordinated transactions which were in-doubt, have now been resolved. This message reports successful completion of the RSVMQTRN command when the -a option is used.",
                  "None."));
      amqMN_Ht.put("7108",new AMQMessageNumber(7108,
                  "(iSeries) Any in-doubt transactions have been resolved.",
                  "0 : Information",
                  "All, if there were any, of the internally coordinated transactions which were in-doubt, have now been resolved.",
                  "None."));
      amqMN_Ht.put("7109",new AMQMessageNumber(7109,
                  "A decision on behalf of the unavailable resource manager has been delivered.",
                  "0 : Information",
                  "A decision for an internally coordinated transaction which was in-doubt, has now been delivered on behalf of the unavailable resource manager. This message reports successful completion of the RSVMQTRN command when the -r option is used.",
                  "None."));
      amqMN_Ht.put("7110",new AMQMessageNumber(7110,
                  "Media image for the syncfile recorded.",
                  "0 : Information",
                  "The media image for the syncfile has been recorded.",
                  "None."));
      amqMN_Ht.put("7111",new AMQMessageNumber(7111,
                  "Resource manager <insert_1> has participated.",
                  "0 : Information",
                  "This message reports the state of a resource manager with respect to an in-doubt transaction.",
                  "None."));
      amqMN_Ht.put("7112",new AMQMessageNumber(7112,
                  "Transaction number <insert_1>,<insert_2> has encountered an error.",
                  "0 : Information",
                  "This message is used to report the number of an in-doubt transaction which has encountered an error with one or more resource managers.",
                  "Refer to the queue manager error log for more information about which resource managers are in error. Ensure that the resource managers that were in error, are working correctly, restart the queue manager. If the problem persists, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7113",new AMQMessageNumber(7113,
                  "The Database Name argument, -rn, is missing from the command crtmqm",
                  "20 : Error",
                  "The required flag, -rn, was omitted from the command crtmqm",
                  "Add the flag and associated database name and submit it again."));
      amqMN_Ht.put("7114",new AMQMessageNumber(7114,
                  "The Database Password argument, -rp, is missing from the command crtmqm",
                  "20 : Error",
                  "The required flag, -rp, was omitted from the command crtmqm",
                  "Add the flag and associated database password and submit it again."));
      amqMN_Ht.put("7115",new AMQMessageNumber(7115,
                  "The Database Type argument, -rt, is missing from the command crtmqm",
                  "20 : Error",
                  "The required flag, -rt, was omitted from the command crtmqm",
                  "Add the flag and associated database type and submit it again"));
      amqMN_Ht.put("7116",new AMQMessageNumber(7116,
                  "The Database Type argument, -rt, is greater than 8 characters long",
                  "20 : Error",
                  "The argument supplied with the flag -rt, is greater than 8 characters long",
                  "Reduce the length of the database type argument and submit it again"));
      amqMN_Ht.put("7117",new AMQMessageNumber(7117,
                  "The MSD shared library failed to load.",
                  "20 : Error",
                  "The MSD shared library was either not located or failed to load correctly.",
                  "Ensure that the database type is specified correctly when creating a queue manager since this is used to form the name of the shared library to be loaded. Further information on the failure may be found in the FFST logs. Also, ensure that ensure that the MSD shared library is installed correctly."));
      amqMN_Ht.put("7120",new AMQMessageNumber(7120,
                  "The Trial Period license for this copy of WebSphere MQ has expired.",
                  "20 : Error",
                  "This copy of WebSphere MQ was licensed to be used in trial mode for a limited period only. This period has expired.",
                  "Install a Production license for this copy of WebSphere MQ."));
      amqMN_Ht.put("7121",new AMQMessageNumber(7121,
                  "The trial period for this copy of WebSphere MQ has now expired.",
                  "20 : Error",
                  "This copy of WebSphere MQ was licensed for a limited period only. This period has now expired.",
                  "Install a Production license for this copy of WebSphere MQ."));
      amqMN_Ht.put("7122",new AMQMessageNumber(7122,
                  "The Trial Period License Agreement was not accepted.",
                  "10 : Warning",
                  "When the Trial Period License Agreement is displayed, the user must accept it before this copy of WebSphere MQ can be used.",
                  "Submit the command again and accept the agreement."));
      amqMN_Ht.put("7123",new AMQMessageNumber(7123,
                  "There is one day left in the trial period for this copy of WebSphere MQ.",
                  "0 : Information",
                  "This copy of WebSphere MQ is licensed for a limited period only.",
                  "None."));
      amqMN_Ht.put("7124",new AMQMessageNumber(7124,
                  "This is the final day of the trial period for this copy of WebSphere MQ.",
                  "10 : Warning",
                  "This copy of WebSphere MQ is licensed for a limited period only.",
                  "Install a Production license for this copy of WebSphere MQ."));
      amqMN_Ht.put("7125",new AMQMessageNumber(7125,
                  "There are <insert_1> days left in the trial period for this copy of WebSphere MQ.",
                  "0 : Information",
                  "This copy of WebSphere MQ is licensed for a limited period only.",
                  "None."));
      amqMN_Ht.put("7126",new AMQMessageNumber(7126,
                  "This copy of WebSphere MQ is now running in Production mode.",
                  "0 : Information",
                  "A Production license has been installed for this copy of WebSphere MQ.",
                  "None."));
      amqMN_Ht.put("7127",new AMQMessageNumber(7127,
                  "",
                  "0 : Information",
                  "One or more messages have been displayed. They will disappear when the user presses the Enter key.",
                  "Press the Enter key when the messages are no longer required."));
      amqMN_Ht.put("7128",new AMQMessageNumber(7128,
                  "No license installed for this copy of WebSphere MQ.",
                  "20 : Error",
                  "The installation of WebSphere MQ is invalid since no Production, Beta, or Trial Period license has been installed.",
                  "Check that the installation steps described in the Quick Beginnings book have been followed, and if the problem persists contact your IBM service representative."));
      amqMN_Ht.put("7129",new AMQMessageNumber(7129,
                  "The trial period for this copy of WebSphere MQ has already been started.",
                  "0 : Information",
                  "This copy of WebSphere MQ is licensed for a limited period only and the trial period has been started previously.",
                  "None."));
      amqMN_Ht.put("7130",new AMQMessageNumber(7130,
                  "This copy of WebSphere MQ is running in Production mode.",
                  "0 : Information",
                  "A Production license has been installed for this copy of WebSphere MQ. A beta or trial period cannot be started.",
                  "None."));
      amqMN_Ht.put("7131",new AMQMessageNumber(7131,
                  "International License Agreement for Evaluation of Programs",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7132",new AMQMessageNumber(7132,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7133",new AMQMessageNumber(7133,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7134",new AMQMessageNumber(7134,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7135",new AMQMessageNumber(7135,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7136",new AMQMessageNumber(7136,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7137",new AMQMessageNumber(7137,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7138",new AMQMessageNumber(7138,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7139",new AMQMessageNumber(7139,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7140",new AMQMessageNumber(7140,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7141",new AMQMessageNumber(7141,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7142",new AMQMessageNumber(7142,
                  "Reply 'yes' to accept the Agreement. Reply 'no' if you do not agree to the terms of the Agreement. Reply 'no' and submit the command again, if you want to read the Agreement again.",
                  "0 : Information",
                  "The Trial Period License Agreement has been displayed to the user and the user should now accept or reject the Agreement.",
                  "Reply 'yes' or 'no' and press 'Enter'."));
      amqMN_Ht.put("7143",new AMQMessageNumber(7143,
                  "",
                  "0 : Information",
                  "Part of the Trial Period License Agreement has been displayed to the user. The user should press the Enter key to indicate that they are ready for the next part of the Agreement to be displayed.",
                  "Press the Enter key when ready for the next part of the Agreement to be displayed."));
      amqMN_Ht.put("7144",new AMQMessageNumber(7144,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7145",new AMQMessageNumber(7145,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7146",new AMQMessageNumber(7146,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7147",new AMQMessageNumber(7147,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7148",new AMQMessageNumber(7148,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7149",new AMQMessageNumber(7149,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7150",new AMQMessageNumber(7150,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7151",new AMQMessageNumber(7151,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7152",new AMQMessageNumber(7152,
                  "",
                  "0 : Information",
                  "This is part of the Trial Period License Agreement which must be accepted before a trial period can be started. A trial period allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7153",new AMQMessageNumber(7153,
                  "A license could not be installed for this copy of WebSphere MQ.",
                  "20 : Error",
                  "A Production, Beta or Trial Period license could not be installed for this copy of WebSphere MQ. This is because the 'nodelock' file in the 'qmgrs/@SYSTEM' directory could not be created or updated.",
                  "Check the ownership and permissions of the 'qmgrs/@SYSTEM' directory."));
      amqMN_Ht.put("7154",new AMQMessageNumber(7154,
                  "The Production license for this copy of WebSphere MQ has expired.",
                  "20 : Error",
                  "The production license for this copy of WebSphere MQ has an expiry date. This date has been passed.",
                  "Contact your IBM support center."));
      amqMN_Ht.put("7155",new AMQMessageNumber(7155,
                  "License file <insert_3> not found or not valid.",
                  "20 : Error",
                  "The program requires that the file <insert_3> is present, available and is a valid license file.",
                  "Check that the installation steps described in the Quick Beginnings book have been followed, and if the problem persists contact your IBM service representative."));
      amqMN_Ht.put("7156",new AMQMessageNumber(7156,
                  "This copy of WebSphere MQ is already running in Production mode.",
                  "0 : Information",
                  "A Production license has previously been installed for this copy of WebSphere MQ.",
                  "None."));
      amqMN_Ht.put("7157",new AMQMessageNumber(7157,
                  "The Production license is not valid for this copy of WebSphere MQ.",
                  "20 : Error",
                  "The license <insert_3> has been installed but it is not a valid production license for this copy of WebSphere MQ.",
                  "Submit the SETMQPRD command again specifying the name of a valid production license."));
      amqMN_Ht.put("7158",new AMQMessageNumber(7158,
                  "The Trial Period license is not valid for this copy of WebSphere MQ.",
                  "20 : Error",
                  "The license <insert_3> has been installed but it is not a valid trial period license for this copy of WebSphere MQ.",
                  "Check that the correct version of the file is available."));
      amqMN_Ht.put("7159",new AMQMessageNumber(7159,
                  "A FASTPATH application has ended unexpectedly.",
                  "10 : Warning",
                  "A FASTPATH application has ended in a way which did not allow the queue manager to clean up the resources owned by that application. Any resources held by the application can only be released by stopping and restarting the queue manager.",
                  "Investigate why the application ended unexpectedly. Avoid ending FASTPATH applications in a way which prevents WebSphere MQ from releasing resources held by the application."));
      amqMN_Ht.put("7160",new AMQMessageNumber(7160,
                  "Queue Manager Object",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7161",new AMQMessageNumber(7161,
                  "Object Catalogue",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7162",new AMQMessageNumber(7162,
                  "The setmqaut command completed successfully.",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7163",new AMQMessageNumber(7163,
                  "(iSeries) WebSphere MQ job <insert_2> started for <insert_3>.",
                  "0 : Information",
                  "The job's PID is <insert_2> the CCSID is <insert_1>. The job name is <insert_4>.",
                  "None"));
      amqMN_Ht.put("7164",new AMQMessageNumber(7164,
                  "(iSeries) WebSphere MQ is waiting for a job to start.",
                  "0 : Information",
                  "WebSphere MQ has been waiting <insert_1> seconds to start job <insert_3> for Queue Manager: <insert_5>",
                  "Check that the job queue that is associated with job description <insert_4> is not held, and that the subsystem that is associated with the job queue is active."));
      amqMN_Ht.put("7165",new AMQMessageNumber(7165,
                  "The Beta license for this copy of WebSphere MQ has expired.",
                  "20 : Error",
                  "This copy of WebSphere MQ was licensed to be used for Beta testing for a limited period only. This period has expired.",
                  "Install a Production license for this copy of WebSphere MQ."));
      amqMN_Ht.put("7166",new AMQMessageNumber(7166,
                  "The Beta period for this copy of WebSphere MQ has now expired.",
                  "20 : Error",
                  "This copy of WebSphere MQ was licensed for a limited period only. This period has now expired.",
                  "Install a Production license for this copy of WebSphere MQ."));
      amqMN_Ht.put("7167",new AMQMessageNumber(7167,
                  "The 'Early Release of Programs License Agreement' was not accepted.",
                  "10 : Warning",
                  "When the IBM International License Agreement for Early Release of Programs is displayed, the user must accept it before this copy of WebSphere MQ can be used.",
                  "Submit the command again and accept the agreement."));
      amqMN_Ht.put("7168",new AMQMessageNumber(7168,
                  "There is one day left in the Beta test period for this copy of WebSphere MQ.",
                  "0 : Information",
                  "This copy of WebSphere MQ is licensed for a limited period only.",
                  "None."));
      amqMN_Ht.put("7169",new AMQMessageNumber(7169,
                  "This is the final day of the Beta test period for this copy of WebSphere MQ.",
                  "10 : Warning",
                  "This copy of WebSphere MQ is licensed for a limited period only.",
                  "Install a Production license for this copy of WebSphere MQ."));
      amqMN_Ht.put("7170",new AMQMessageNumber(7170,
                  "(iSeries) Option is not valid for this transaction.",
                  "20 : Error",
                  "The Resolve option is not valid for external transactions. The Commit and Backout options are not valid for internal transactions.",
                  "Select a different option for this transaction."));
      amqMN_Ht.put("7171",new AMQMessageNumber(7171,
                  "IBM International License Agreement for Early Release of Programs",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7172",new AMQMessageNumber(7172,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7173",new AMQMessageNumber(7173,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7174",new AMQMessageNumber(7174,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7175",new AMQMessageNumber(7175,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7176",new AMQMessageNumber(7176,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7177",new AMQMessageNumber(7177,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7178",new AMQMessageNumber(7178,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7179",new AMQMessageNumber(7179,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7180",new AMQMessageNumber(7180,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7181",new AMQMessageNumber(7181,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7182",new AMQMessageNumber(7182,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7183",new AMQMessageNumber(7183,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7184",new AMQMessageNumber(7184,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7185",new AMQMessageNumber(7185,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7186",new AMQMessageNumber(7186,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7187",new AMQMessageNumber(7187,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7188",new AMQMessageNumber(7188,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7189",new AMQMessageNumber(7189,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7190",new AMQMessageNumber(7190,
                  "",
                  "0 : Information",
                  "This is part of the Early Release of Programs License Agreement (VZ125-5544-01 10/97 (MK002))which must be accepted before a Beta test period can be started. A Beta test version allows a copy of WebSphere MQ to be used for a limited period only.",
                  "None."));
      amqMN_Ht.put("7191",new AMQMessageNumber(7191,
                  "There are <insert_1> days left in the beta test period for this copy of WebSphere MQ.",
                  "0 : Information",
                  "This copy of WebSphere MQ is licensed for a limited period only.",
                  "None."));
      amqMN_Ht.put("7192",new AMQMessageNumber(7192,
                  "The Beta test period for this copy of WebSphere MQ has already been started.",
                  "0 : Information",
                  "This copy of WebSphere MQ is licensed for a limited period only and the Beta test period has been started previously.",
                  "None."));
      amqMN_Ht.put("7193",new AMQMessageNumber(7193,
                  "Reply 'yes' to accept the Agreement. Reply 'no' if you do not agree to the terms of the Agreement. Reply 'no' and submit the command again, if you want to read the Agreement again.",
                  "0 : Information",
                  "The IBM International License Agreement for Early Release of Programs has been displayed to the user and the user should now accept or reject the Agreement.",
                  "Reply 'yes' or 'no' and press 'Enter'."));
      amqMN_Ht.put("7194",new AMQMessageNumber(7194,
                  "",
                  "0 : Information",
                  "Part of the IBM International License Agreement for Early Release of Programs has been displayed to the user. The user should press the Enter key to indicate that they are ready for the next part of the Agreement to be displayed.",
                  "Press the Enter key when ready for the next part of the Agreement to be displayed."));
      amqMN_Ht.put("7195",new AMQMessageNumber(7195,
                  "The Beta test license is not valid for this copy of WebSphere MQ.",
                  "20 : Error",
                  "The license <insert_3> has been installed but it is not a valid trial period license for this copy of WebSphere MQ.",
                  "Check that the correct version of the file is available."));
      amqMN_Ht.put("7196",new AMQMessageNumber(7196,
                  "By installing this product, you accept the terms of the International Program License Agreement and the License Information supplied with the product.",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("7197",new AMQMessageNumber(7197,
                  "A production or trial license could not be installed for this copy of WebSphere MQ.",
                  "20 : Error",
                  "This copy of WebSphere MQ is a beta version and cannot be used with a production or trial license.",
                  "Uninstall the beta version of WebSphere MQ and install the production or trial version."));
      amqMN_Ht.put("7198",new AMQMessageNumber(7198,
                  "Insufficient license units.",
                  "10 : Warning",
                  "The purchased processor allowance (<insert_1>) is less than the number of processors (<insert_2>) in this machine.",
                  "Ensure sufficient license units have been purchased and use the MQ setmqcap command to set the purchased processor allowance for this installation. Refer to the Quick Beginnings book for more information."));
      amqMN_Ht.put("7198",new AMQMessageNumber(7198,
                  "(iSeries) Insufficient license units.",
                  "10 : Warning",
                  "The purchased processor allowance for this installation is zero.",
                  "Ensure sufficient license units have been purchased and use the MQ CHGMQMCAP command to set the purchased processor allowance for this installation. Refer to the Quick Beginnings book for more information."));
      amqMN_Ht.put("7199",new AMQMessageNumber(7199,
                  "The purchased processor allowance is set to <insert_1>.",
                  "0 : Information",
                  "The purchased processor allowance for this installation has been set to <insert_1> using the MQ setmqcap command.",
                  "None."));
      amqMN_Ht.put("7199",new AMQMessageNumber(7199,
                  "(iSeries) The purchased processor allowance is set to <insert_1>.",
                  "0 : Information",
                  "The purchased processor allowance for this installation has been set to <insert_1> using the MQ CHGMQMCAP command.",
                  "None."));
      amqMN_Ht.put("7200",new AMQMessageNumber(7200,
                  "The purchased processor allowance is <insert_1>",
                  "0 : Information",
                  "The purchased processor allowance is currently set to <insert_1>",
                  "Ensure sufficient license units have been purchased and, if necessary, use the MQ setmqcap command to change the purchased processor allowance for this installation. Refer to the Quick Beginnings book for more information."));
      amqMN_Ht.put("7200",new AMQMessageNumber(7200,
                  "(iSeries) The purchased processor allowance is <insert_1>",
                  "0 : Information",
                  "The purchased processor allowance is currently set to <insert_1>",
                  "Ensure sufficient license units have been purchased and, if necessary, use the MQ CHGMQMCAP command to change the purchased processor allowance for this installation. Refer to the Quick Beginnings book for more information."));
      amqMN_Ht.put("7201",new AMQMessageNumber(7201,
                  "The number of processors in this machine is <insert_1>",
                  "0 : Information",
                  "The operating system reports that the number of processors in this machine is <insert_1>",
                  "None."));
      amqMN_Ht.put("7202",new AMQMessageNumber(7202,
                  "The number of license units is sufficient for all future possible upgrades to this machine.",
                  "0 : Information",
                  "The purchased processor allowance for this installation has been set to -1, which allows any permitted processor configuration.",
                  "None."));
      amqMN_Ht.put("7203",new AMQMessageNumber(7203,
                  "Purchased processor allowance not set (use setmqcap).",
                  "10 : Warning",
                  "The purchased processor allowance for this installation has not been set.",
                  "Ensure sufficient license units have been purchased and use the MQ setmqcap command to set the purchased processor allowance for this installation. Refer to the Quick Beginnings book for more information."));
      amqMN_Ht.put("7203",new AMQMessageNumber(7203,
                  "(iSeries) Purchased processor allowance not set (use CHGMQMCAP).",
                  "10 : Warning",
                  "The purchased processor allowance for this installation has not been set.",
                  "Ensure sufficient license units have been purchased and use the MQ CHGMQMCAP command to set the purchased processor allowance for this installation. Refer to the Quick Beginnings book for more information."));
      amqMN_Ht.put("7204",new AMQMessageNumber(7204,
                  "Intel Hyper-Threading support enabled with Logical Processor Mask <insert_1>.",
                  "0 : Information",
                  "Install has detected Intel Hyper=Threading support enabled for Logical Processors. The current Processor Mask is set to <insert_1>.",
                  "None."));
      amqMN_Ht.put("7205",new AMQMessageNumber(7205,
                  "System call SetProcessAffinityMask failed for Logical Processor Mask <insert_1>. Process Affinity Mask is <insert_2>, process is continuing.",
                  "0 : Information",
                  "Microsoft Windows system call SetProcessAffinityMask failed. If this problem persists contact your IBM service representative.",
                  "None."));
      amqMN_Ht.put("7206",new AMQMessageNumber(7206,
                  " (Windows)Group name has been truncated.",
                  "40 : Stop Error",
                  "WebSphere MQ only supports group names up to 12 characters long. The operating system is attempting to return a group longer than this.",
                  "Reduce the group name to 12 characters or less."));
      amqMN_Ht.put("7207",new AMQMessageNumber(7207,
                  " (Windows)User ID longer than 12 characters.",
                  "40 : Stop Error",
                  "WebSphere MQ only supports user names up to 12 characters long. This operation is being attempted from a user name longer than this.",
                  "Reduce the user name to 12 characters or less."));
      amqMN_Ht.put("7208",new AMQMessageNumber(7208,
                  "The queue manager failed to pass a PCF message to another queue manager.",
                  "10 : Warning",
                  "The queue manager attempted to put a PCF message to <insert_3> to start the channel <insert_4> to cluster queue manager <insert_5>. The put failed with reason <insert_1>. When the queue manager resolves a cluster queue to a remote cluster queue manager, the message is put to the SYSTEM.CLUS.TRANSMIT.QUEUE. If the channel to the remote cluster queue manager is not running, the queue manager attempts to start the channel by sending a PCF message to <insert_3>.",
                  "Resolve the problem with <insert_3> and if necessary start the channel manually."));
      amqMN_Ht.put("7209",new AMQMessageNumber(7209,
                  "The queue manager attempted to open SYSTEM.CHANNEL.INITQ which failed with reason <insert_3>",
                  "10 : Warning",
                  "When the queue manager resolves a cluster queue to a remote cluster queue manager, the message is put to the SYSTEM.CLUS.TRANSMIT.QUEUE. If the channel to the remote cluster queue manager is not running, the queue manager attempts to start the channel by sending a PCF message to the SYSTEM.CHANNEL.INITQ",
                  "Resolve the problem with the SYSTEM.CHANNEL.INITQ and if necessary start the channels manually."));
      amqMN_Ht.put("7210",new AMQMessageNumber(7210,
                  "The Cluster Workload exit module could not be loaded.",
                  "10 : Warning",
                  "The Cluster Workload exit module <insert_3> could not be loaded for reason <insert_4>.",
                  "Correct the problem with the Cluster Workload exit module <insert_3>"));
      amqMN_Ht.put("7211",new AMQMessageNumber(7211,
                  "The Queue Manager is still waiting for a reply from the Cluster Workload Exit server process.",
                  "10 : Warning",
                  "The Queue Manager is configured to run the Cluster Workload Exit in SAFE mode. This means that the Cluster Workload Exit is run by a server process (amqzlw0). The Queue Manager has been waiting <insert_1> seconds for this server process to reply to a request to run the Cluster Workload Exit. It is possible that the exit is hung or is looping.",
                  "End the Queue Manager, resolve the problem with the Cluster Workload Exit and restart the Queue Manager"));
      amqMN_Ht.put("7212",new AMQMessageNumber(7212,
                  "The address of the Cluster exit function could not be found.",
                  "10 : Warning",
                  "The address of the Cluster exit function <insert_4> could not be found in module <insert_3> for reason <insert_1> <insert_5>.",
                  "Correct the problem with the Cluster exit function <insert_4> in the module <insert_3>"));
      amqMN_Ht.put("7214",new AMQMessageNumber(7214,
                  "The module for API Exit <insert_3> could not be loaded.",
                  "40 : Stop Error",
                  "The module <insert_4> for API Exit <insert_3> could not be loaded for reason <insert_5>.",
                  "Correct the problem with the API Exit module <insert_3>."));
      amqMN_Ht.put("7215",new AMQMessageNumber(7215,
                  "The API Exit <insert_3> function <insert_4> could not be found in the module <insert_5>.",
                  "40 : Stop Error",
                  "The API Exit <insert_3> function <insert_4> could not be found in the module <insert_5>. The internal return code was <insert_1>.",
                  "Correct the problem with the API Exit <insert_3>."));
      amqMN_Ht.put("7215",new AMQMessageNumber(7215,
                  "(iSeries) Could not find a function in API Exit <insert_3>.",
                  "40 : Stop Error",
                  "The API Exit <insert_3> function <insert_4> could not be found in the module <insert_5>. The internal return code was <insert_1>.",
                  "Correct the problem with the API Exit <insert_3>."));
      amqMN_Ht.put("7216",new AMQMessageNumber(7216,
                  "An API Exit initialization function returned an error.",
                  "10 : Warning",
                  "The API Exit <insert_3> function <insert_4> in the module <insert_5> returned CompCode <insert_1> and ReasonCode <insert_2>.",
                  "Correct the problem with the API Exit <insert_3>"));
      amqMN_Ht.put("7217",new AMQMessageNumber(7217,
                  "The response set by the exit is not valid.",
                  "10 : Warning",
                  "The API Exit <insert_3> module <insert_4> function <insert_5> returned a response code <insert_1> that is not valid in the ExitResponse field of the API Exit parameters (MQAXP).",
                  "Investigate why the API Exit <insert_3> set a response code that is not valid."));
      amqMN_Ht.put("7219",new AMQMessageNumber(7219,
                  "profile: <insert_3>",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7220",new AMQMessageNumber(7220,
                  "object type: <insert_3>",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7221",new AMQMessageNumber(7221,
                  "entity: <insert_3>",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7222",new AMQMessageNumber(7222,
                  "entity type: <insert_3>",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7223",new AMQMessageNumber(7223,
                  "authority: <insert_3>",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7224",new AMQMessageNumber(7224,
                  "profile: <insert_3>, object type: <insert_4>",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7225",new AMQMessageNumber(7225,
                  "No matching authority records.",
                  "0 : Information",
                  "No authority records match the specified parameters.",
                  ""));
      amqMN_Ht.put("7226",new AMQMessageNumber(7226,
                  "The profile name is invalid.",
                  "20 : Error",
                  "The profile name contains invalid characters, contains an invalid wildcard specification, or is of invalid length.",
                  "Correct the profile name and submit it again."));
      amqMN_Ht.put("7227",new AMQMessageNumber(7227,
                  "WebSphere MQ encountered the following network error: <insert_3>",
                  "10 : Warning",
                  "MQ failed to successfully complete a network operation due to the specified error.",
                  "Ensure that your network is functioning correctly."));
      amqMN_Ht.put("7228",new AMQMessageNumber(7228,
                  "(iSeries) Display MQ Authority Records for <insert_3>",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7229",new AMQMessageNumber(7229,
                  "<insert_1> log records accessed on queue manager <insert_3> during the log replay phase.",
                  "0 : Information",
                  "<insert_1> log records have been accessed so far on queue manager <insert_3> during the log replay phase in order to bring the queue manager back to a previously known state.",
                  "None."));
      amqMN_Ht.put("7230",new AMQMessageNumber(7230,
                  "Log replay for queue manager <insert_3> complete.",
                  "0 : Information",
                  "The log replay phase of the queue manager restart process has been completed for queue manager <insert_3>.",
                  "None."));
      amqMN_Ht.put("7231",new AMQMessageNumber(7231,
                  "<insert_1> log records accessed on queue manager <insert_3> during the recovery phase.",
                  "0 : Information",
                  "<insert_1> log records have been accessed so far on queue manager <insert_3> during the recovery phase of the transactions manager state.",
                  "None."));
      amqMN_Ht.put("7232",new AMQMessageNumber(7232,
                  "Transaction manager state recovered for queue manager <insert_3>.",
                  "0 : Information",
                  "The state of transactions at the time the queue manager ended has been recovered for queue manager <insert_3>.",
                  "None."));
      amqMN_Ht.put("7233",new AMQMessageNumber(7233,
                  "<insert_1> out of <insert_2> in-flight transactions resolved for queue manager <insert_3>.",
                  "0 : Information",
                  "<insert_1> transactions out of <insert_2> in-flight at the time queue manager <insert_3> ended have been resolved.",
                  "None."));
      amqMN_Ht.put("7234",new AMQMessageNumber(7234,
                  "<insert_1> messages from queue <insert_4> loaded on queue manager <insert_3>.",
                  "0 : Information",
                  "<insert_1> messages from queue <insert_4> have been loaded on queue manager <insert_3>.",
                  "None."));
      amqMN_Ht.put("7235",new AMQMessageNumber(7235,
                  "(iSeries) Queue manager library <insert_3> already exists.",
                  "40 : Stop Error",
                  "The library <insert_3> already exists.",
                  "Specify a library which does not already exist."));
      amqMN_Ht.put("7236",new AMQMessageNumber(7236,
                  "WebSphere MQ queue manager <insert_3> activated.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_3> has been activated.",
                  "None."));
      amqMN_Ht.put("7237",new AMQMessageNumber(7237,
                  "WebSphere MQ queue manager <insert_3> is not a backup queue manager.",
                  "10 : Warning",
                  "WebSphere MQ queue manager <insert_3> is not a backup queue manager and so cannot be activated. A possible reason might be that the queue manager is configured for circular logging.",
                  "Re-try the command without the '-a' option."));
      amqMN_Ht.put("7238",new AMQMessageNumber(7238,
                  "WebSphere MQ queue manager <insert_3> replay completed.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_3> replay has completed.",
                  "None."));
      amqMN_Ht.put("7249",new AMQMessageNumber(7249,
                  "WebSphere MQ queue manager <insert_3> cannot be started for replay.",
                  "20 : Error",
                  "WebSphere MQ queue manager <insert_3> cannot be started for replay. A possible reason might be that the queue manager is configured for circular logging.",
                  "Re-try the command without the '-r' option."));
      amqMN_Ht.put("7250",new AMQMessageNumber(7250,
                  "WebSphere MQ queue manager <insert_3> has not been activated.",
                  "20 : Error",
                  "WebSphere MQ queue manager <insert_3> cannot be started because it has previously been started for replay but has not been activated.",
                  "Activate the queue manager and try starting the queue manager again."));
      amqMN_Ht.put("7253",new AMQMessageNumber(7253,
                  "The command <insert_3> requires one of the following arguments: <insert_4>.",
                  "20 : Error",
                  "The command <insert_3> required at least one of the following arguments, none of which you supplied: <insert_4>.",
                  "Check the WebSphere MQ System Administration documentation for details on the usage of the command, correct the command and then retry."));
      amqMN_Ht.put("7305",new AMQMessageNumber(7305,
                  "Trigger message could not be put on an initiation queue.",
                  "10 : Warning",
                  "The attempt to put a trigger message on queue <insert_4> on queue manager <insert_5> failed with reason code <insert_1>. The message will be put on the dead-letter queue.",
                  "Ensure that the initiation queue is available, and operational."));
      amqMN_Ht.put("7306",new AMQMessageNumber(7306,
                  "The dead-letter queue must be a local queue.",
                  "10 : Warning",
                  "An undelivered message has not been put on the dead-letter queue <insert_4> on queue manager <insert_5>, because the queue is not a local queue. The message will be discarded.",
                  "Inform your system administrator."));
      amqMN_Ht.put("7307",new AMQMessageNumber(7307,
                  "A message could not be put on the dead-letter queue.",
                  "10 : Warning",
                  "The attempt to put a message on the dead-letter queue <insert_4> on queue manager <insert_5> failed with reason code <insert_1>. The message will be discarded.",
                  "Ensure that the dead-letter queue is available and operational."));
      amqMN_Ht.put("7308",new AMQMessageNumber(7308,
                  "Trigger condition <insert_1> was not satisfied.",
                  "0 : Information",
                  "At least one of the conditions required for generating a trigger message was not satisfied, so a trigger message was not generated. If you were expecting a trigger message, consult the WebSphere MQ Application Programming Guide for a list of the conditions required. (Note that arranging for condition <insert_1> to be satisfied might not be sufficient because the conditions are checked in an arbitrary order, and checking stops when the first unsatisfied condition is discovered.)",
                  "If a trigger message is required, ensure that all the conditions for generating one are satisfied."));
      amqMN_Ht.put("7310",new AMQMessageNumber(7310,
                  "Report message could not be put on a reply-to queue.",
                  "10 : Warning",
                  "The attempt to put a report message on queue <insert_4> on queue manager <insert_5> failed with reason code <insert_1>. The message will be put on the dead-letter queue.",
                  "Ensure that the reply-to queue is available and operational."));
      amqMN_Ht.put("7315",new AMQMessageNumber(7315,
                  "Failed to put message to accounting queue. Reason(<insert_1>)",
                  "20 : Error",
                  "The attempt to put a messsage containing accounting data to the queue <insert_3> failed with reason code <insert_1>. The message data has been discarded. This error message will be written only once for attempts to put a message to the queue as part of the same operation which fail for the same reason.",
                  "Ensure that the queue <insert_3> is available and operational."));
      amqMN_Ht.put("7316",new AMQMessageNumber(7316,
                  "Failed to put message to statistics queue. Reason(<insert_1>)",
                  "20 : Error",
                  "The attempt to put a messsage containing statistics data to the queue <insert_3> failed with reason code <insert_1>. The message data has been discarded. This error message will be written only once for attempts to put a message to the queue as part of the same operation which fail for the same reason.",
                  "Ensure that the queue <insert_3> is available and operational."));
      amqMN_Ht.put("7432",new AMQMessageNumber(7432,
                  "(iSeries) WebSphere MQ journal entry not available for replay.",
                  "40 : Stop Error",
                  "A journal replay operation was attempted, but the operation required journal entries from journal receivers that are not currently present on the system.",
                  "Restore the required journal receivers from backup. Then try the operation again."));
      amqMN_Ht.put("7433",new AMQMessageNumber(7433,
                  "(iSeries) An Error occured while performing a journal replay.",
                  "40 : Stop Error",
                  "WebSphere MQ encountered a problem reading one or more journal entries while performing a journal replay operation.",
                  "If you have previously created a journal receiver for a queue manager or are performing a cold restart of a queue manager, delete the QMQMCHKPT file from the queue manager subdirectory in /QIBM/UserData/mqm/qmgrs/ and attempt to restart the queue manager. If the problem persists, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7434",new AMQMessageNumber(7434,
                  "(iSeries) The MQ commitment control exit program was called incorrectly. Code <insert_1>.",
                  "40 : Stop Error",
                  "The WebSphere MQ commitment control exit program was called with incorrect parameters.",
                  "If the program was called by OS/400 as part of a commit or rollback, save the job log, and contact your IBM support center."));
      amqMN_Ht.put("7435",new AMQMessageNumber(7435,
                  "(iSeries) The MQ commitment control exit program failed. Code <insert_1>.",
                  "40 : Stop Error",
                  "The WebSphere MQ commitment control exit program failed due to an unexpected error.",
                  "Save the job log, and contact your IBM support center."));
      amqMN_Ht.put("7459",new AMQMessageNumber(7459,
                  "(iSeries) WebSphere MQ journal receiver <insert_3> is the oldest in the chain",
                  "0 : Information",
                  "The oldest journal receiver in the receiver chain is <insert_3> in library <insert_4>.",
                  "None"));
      amqMN_Ht.put("7460",new AMQMessageNumber(7460,
                  "(iSeries) WebSphere MQ startup journal information.",
                  "0 : Information",
                  "This message is issued periodically by WebSphere MQ to help you identify which journal receivers can be removed from the system because they are no longer required for startup recovery.",
                  "None"));
      amqMN_Ht.put("7461",new AMQMessageNumber(7461,
                  "(iSeries) WebSphere MQ object re-created - reapply authorities.",
                  "0 : Information",
                  "A previously damaged object has been re-created, either automatically, or by explicit use of the Recreate Object (RCRMQMOBJ) command. The authorities that applied to this object have not been re-created.",
                  "Use the Grant Authority (GRTMQMAUT) command, as appropriate, to re-create the required authorities to this MQ object."));
      amqMN_Ht.put("7462",new AMQMessageNumber(7462,
                  "(iSeries) WebSphere MQ media recovery journal information.",
                  "0 : Information",
                  "This message is issued periodically by WebSphere MQ to help you identify which journal receivers can be removed from the system because they are no longer required for media recovery.",
                  "None"));
      amqMN_Ht.put("7463",new AMQMessageNumber(7463,
                  "The log for queue manager <insert_3> is full.",
                  "20 : Error",
                  "This message is issued when an attempt to write a log record is rejected because the log is full. The queue manager will attempt to resolve the problem.",
                  "This situation may be encountered during a period of unusually high message traffic. However, if you persistently fill the log, you may have to consider enlarging the size of the log. You can either increase the number of log files by changing the values in the queue manager configuration file. You will then have to stop and restart the queue manager. Alternatively, if you need to make the log files themselves bigger, you will have to delete and recreate the queue manager."));
      amqMN_Ht.put("7464",new AMQMessageNumber(7464,
                  "The log for queue manager <insert_3> is no longer full.",
                  "0 : Information",
                  "This message is issued when a log was previously full, but an attempt to write a log record has now been accepted. The log full situation has been resolved.",
                  "None"));
      amqMN_Ht.put("7465",new AMQMessageNumber(7465,
                  "The log for queue manager <insert_3> is full.",
                  "20 : Error",
                  "An attempt to resolve a log full situation has failed. This is due to the presence of a long-running transaction.",
                  "Try to ensure that the duration of your transactions is not excessive. Commit or roll back any old transactions to release log space for further log records."));
      amqMN_Ht.put("7466",new AMQMessageNumber(7466,
                  "There is a problem with the size of the logfile.",
                  "10 : Warning",
                  "The log for queue manager <insert_3> is too small to support the current data rate. This message is issued when the monitoring tasks maintaining the log cannot keep up with the current rate of data being written.",
                  "The number of primary log files configured should be increased to prevent possible log full situations."));
      amqMN_Ht.put("7467",new AMQMessageNumber(7467,
                  "The oldest log file required to start queue manager <insert_3> is <insert_4>.",
                  "0 : Information",
                  "The log file <insert_4> contains the oldest log record required to restart the queue manager. Log records older than this may be required for media recovery.",
                  "You can move log files older than <insert_4> to an archive medium to release space in the log directory. If you move any of the log files required to recreate objects from their media images, you will have to restore them to recreate the objects."));
      amqMN_Ht.put("7468",new AMQMessageNumber(7468,
                  "The oldest log file required to perform media recovery of queue manager <insert_3> is <insert_4>.",
                  "0 : Information",
                  "The log file <insert_4> contains the oldest log record required to recreate any of the objects from their media images. Any log files prior to this will not be accessed by media recovery operations.",
                  "You can move log files older than <insert_4> to an archive medium to release space in the log directory."));
      amqMN_Ht.put("7469",new AMQMessageNumber(7469,
                  "Transactions rolled back to release log space.",
                  "0 : Information",
                  "The log space for the queue manager is becoming full. One or more long-running transactions have been rolled back to release log space so that the queue manager can continue to process requests.",
                  "Try to ensure that the duration of your transactions is not excessive. Consider increasing the size of the log to allow transactions to last longer before the log starts to become full."));
      amqMN_Ht.put("7472",new AMQMessageNumber(7472,
                  "Object <insert_3>, type <insert_4> damaged.",
                  "10 : Warning",
                  "Object <insert_3>, type <insert_4> has been marked as damaged. This indicates that the queue manager was either unable to access the object in the file system, or that some kind of inconsistency with the data in the object was detected.",
                  "If a damaged object is detected, the action performed depends on whether the queue manager supports media recovery and when the damage was detected. If the queue manager does not support media recovery, you must delete the object as no recovery is possible. If the queue manager does support media recovery and the damage is detected during the processing performed when the queue manager is being started, the queue manager will automatically initiate media recovery of the object. If the queue manager supports media recovery and the damage is detected once the queue manager has started, it may be recovered from a media image using the rcrmqmobj command or it may be deleted."));
      amqMN_Ht.put("7472",new AMQMessageNumber(7472,
                  "(iSeries) Object <insert_3>, type <insert_4> damaged.",
                  "10 : Warning",
                  "Object <insert_3>, type <insert_4> has been marked as damaged. This indicates that the queue manager was either unable to access the object in the file system, or that some kind of inconsistency with the data in the object was detected.",
                  "If a damaged object is detected, the action performed depends on whether the queue manager supports media recovery and when the damage was detected. If the queue manager does not support media recovery, you must delete the object as no recovery is possible. If the queue manager does support media recovery and the damage is detected during the processing performed when the queue manager is being started, the queue manager will automatically initiate media recovery of the object. If the queue manager supports media recovery and the damage is detected once the queue manager has started, it may be recovered from a media image using the RCRMQMOBJ command or it may be deleted."));
      amqMN_Ht.put("7477",new AMQMessageNumber(7477,
                  "(iSeries) WebSphere MQ session no longer active.",
                  "10 : Warning",
                  "The commitment control exit program was called during a commit or rollback operation. The queue manager was stopped while the program was registered. This might have resulted in the rolling back of some uncommitted message operations.",
                  "Inform your system administrator that uncommitted message operations might have been rolled back when the queue manager was stopped."));
      amqMN_Ht.put("7484",new AMQMessageNumber(7484,
                  "Failed to put message to logger event queue. Reason(<insert_2>)",
                  "20 : Error",
                  "The attempt to put a logger event messsage to the queue <insert_3> failed with reason code <insert_2>. The message data has been discarded.",
                  "Ensure that the queue <insert_3> is available and operational. Current logger status information can be displayed with the DISPLAY QMSTATUS runmqsc command."));
      amqMN_Ht.put("7601",new AMQMessageNumber(7601,
                  "Duplicate XA resource manager is not valid.",
                  "40 : Stop Error",
                  "Line <insert_1> of the configuration file <insert_3> contained a duplicate XA resource manager <insert_5>. This is not valid for attribute <insert_4>. Each XA resource manager must be given a unique name.",
                  "Check the contents of the file and retry the operation."));
      amqMN_Ht.put("7601",new AMQMessageNumber(7601,
                  " (Windows)Duplicate XA resource manager <insert_5> not valid for attribute <insert_4> at <insert_3> in the configuration data.",
                  "40 : Stop Error",
                  "Key <insert_3> in the configuration data contained a duplicate XA resource manager <insert_5>. This is not valid for attribute <insert_4>. Each XA resource manager must be given a unique name.",
                  "Check the contents of the configuration data and retry the operation."));
      amqMN_Ht.put("7602",new AMQMessageNumber(7602,
                  "(iSeries) The MQ commitment control exit program was called incorrectly.",
                  "40 : Stop Error",
                  "The WebSphere MQ commitment control exit program was called with incorrect parameters.",
                  "If the program was called by OS/400 as part of a commit or rollback, save the job log, and contact your IBM support center."));
      amqMN_Ht.put("7603",new AMQMessageNumber(7603,
                  "WebSphere MQ has been configured with invalid resource manager <insert_3>.",
                  "20 : Error",
                  "The XA switch file <insert_4> for resource manager <insert_3> indicates that an attempt has been made to configure another queue manager as an external resource manager. This is not allowed so the queue manager will terminate.",
                  "Remove the offending XAResourceManager stanza from the qm.ini configuration file and restart the queue manager."));
      amqMN_Ht.put("7603",new AMQMessageNumber(7603,
                  " (Windows)WebSphere MQ has been configured with resource manager <insert_3> that is not valid.",
                  "20 : Error",
                  "The XA switch file <insert_4> for resource manager <insert_3> indicates that an attempt has been made to configure another queue manager as an external resource manager. This is not allowed, so the queue manager will terminate.",
                  "Remove the offending XAResourceManager stanza from the configuration data and restart the queue manager."));
      amqMN_Ht.put("7604",new AMQMessageNumber(7604,
                  "The XA resource manager <insert_3> was not available when called for <insert_4>. The queue manager is continuing without this resource manager.",
                  "10 : Warning",
                  "The XA resource manager <insert_3> has indicated that it is not available, by returning XAER_RMERR on an xa_open request or XAER_RMFAIL when called for something else. Normally this indicates that the resource manager has been shut down. In this case the resource manager cannot participate in any new transactions. Any in-flight transactions in which it was involved will be backed out, and any transactions in which it is in-doubt will only be resolved when contact with the resource manager is re-established. A further message will be issued when the queue manager has been able to do this. If the problem occurred on an xa_open request, and the resource manager should be available, then there may be a configuration problem.",
                  "Try to establish the reason why the resource manager is unavailable. It may be that an invalid XAOpenString has been defined for the resource manager in the 'qm.ini' configuration file. If this is the case, stop and then restart the queue manager so that any change will be picked up. Alternatively, the queue manager may be reaching a resource constraint with this resource manager. For example, the resource manager may not be able to accommodate all of the queue manager processes being connected at one time, you may need to alter one of its tuning parameters."));
      amqMN_Ht.put("7604",new AMQMessageNumber(7604,
                  "(iSeries) The XA resource manager was not available when called.",
                  "10 : Warning",
                  "The XA resource manager <insert_3> has indicated that it is not available, by returning XAER_RMERR on an xa_open request or XAER_RMFAIL when called for <insert_4>. The queue manager is continuing without this resource manager. Normally this indicates that the resource manager has been shut down. In this case the resource manager cannot participate in any new transactions. Any in-flight transactions in which it was involved will be backed out, and any transactions in which it is in-doubt will only be resolved when contact with the resource manager is re-established. A further message will be issued when the queue manager has been able to do this. If the problem occurred on an xa_open request, and the resource manager should be available, then there may be a configuration problem.",
                  "Try to establish the reason why the resource manager is unavailable. It may be that an invalid XAOpenString has been defined for the resource manager in the 'qm.ini' configuration file. If this is the case, stop and then restart the queue manager so that any change will be picked up. Alternatively, the queue manager may be reaching a resource constraint with this resource manager. For example, the resource manager may not be able to accommodate all of the queue manager processes being connected at one time, you may need to alter one of its tuning parameters."));
      amqMN_Ht.put("7605",new AMQMessageNumber(7605,
                  "The XA resource manager <insert_3> has returned an unexpected return code <insert_1>, when called for <insert_4>.",
                  "20 : Error",
                  "WebSphere MQ received an unexpected return code when calling XA resource manager <insert_3> at its <insert_4> entry point. This indicates an internal error, either within MQ or the resource manager.",
                  "Try to determine the source of the error. A trace of the failure could be used to look at the XA flows between MQ and the resource manager. MQ has allocated an RMId of <insert_2> to this resource manager. This will be useful when isolating the flows associated with the resource manager concerned. If the error occurs on an xa_commit or xa_rollback request, the queue manager will not attempt to redeliver the commit or rollback instruction for this transaction, until after the queue manager has been restarted. The transaction indoubt is identified by the following XID of X<insert_5>. If you think that the error lies within the queue manager, contact your IBM support center. Do not discard any information describing the problem until after the problem has been resolved."));
      amqMN_Ht.put("7605",new AMQMessageNumber(7605,
                  "(iSeries) The XA resource manager has returned an unexpected return code.",
                  "20 : Error",
                  "WebSphere MQ received unexpected return code <insert_1> when calling XA resource manager <insert_3> at its <insert_4> entry point. This indicates an internal error, either within MQ or the resource manager.",
                  "Try to determine the source of the error. A trace of the failure could be used to look at the XA flows between MQ and the resource manager. MQ has allocated an RMId of <insert_2> to this resource manager. This will be useful when isolating the flows associated with the resource manager concerned. If the error occurs on an xa_commit or xa_rollback request, the queue manager will not attempt to redeliver the commit or rollback instruction for this transaction, until after the queue manager has been restarted. The transaction indoubt is identified by the following XID of X<insert_5>. If you think that the error lies within the queue manager, contact your IBM support center. Do not discard any information describing the problem until after the problem has been resolved."));
      amqMN_Ht.put("7606",new AMQMessageNumber(7606,
                  "A transaction has been committed but one or more resource managers have backed out.",
                  "20 : Error",
                  "WebSphere MQ was processing the commit operation for a transaction involving external resource managers. One or more of these resource managers failed to obey the commit request and instead rolled back their updates. The outcome of the transaction is now mixed and the resources owned by these resource managers may now be out of synchronization. MQ will issue further messages to indicate which resource managers failed to commit their updates.",
                  "The transaction with the mixed outcome is identified by the following XID of X<insert_3>. The messages which identify the failing resource managers will also contain this same XID. If the transaction has completed it won't be displayed by the dspmqtrn command and all other transaction participants will have committed their updates. If the transaction is displayed by the dspmqtrn command then there are some participants still in prepared state. In order to preserve data integrity you will need to perform recovery steps local to the failing resource managers."));
      amqMN_Ht.put("7607",new AMQMessageNumber(7607,
                  "A transaction has been rolled back but one or more resource managers have committed.",
                  "20 : Error",
                  "WebSphere MQ was rolling back a transaction involving external resource managers. One or more of these resource managers failed to obey the rollback request and instead committed their updates. The outcome of the transaction is now mixed and the resources owned by these resource managers may now be out of synchronization. MQ will issue further messages to indicate which resource managers failed to roll back their updates.",
                  "The transaction with the mixed outcome is identified by the following XID of X<insert_3>. The messages which identify the failing resource managers will also contain this same XID. If the transaction has completed it won't be displayed by the dspmqtrn command and all other transaction participants will have rolled back their updates. If the transaction is displayed by the dspmqtrn command then there are some participants still in prepared state. In order to preserve data integrity you will need to perform recovery steps local to the failing resource managers."));
      amqMN_Ht.put("7608",new AMQMessageNumber(7608,
                  "XA resource manager returned a heuristic return code.",
                  "20 : Error",
                  "This message is associated with an earlier AMQ7606 message reporting a mixed transaction outcome. It identifies one of the resource managers (<insert_4>) that failed to commit its updates. The transaction associated with this failure is identified by the following XID of X<insert_3>.",
                  "Use the return code <insert_1> returned by the resource manager to determine the effects of the failure. The return code indicates that the resource manager made a heuristic decision about the outcome of the transaction which disagrees with the commit decision of the queue manager. In order to preserve data integrity you will need to perform recovery steps local to this resource manager."));
      amqMN_Ht.put("7609",new AMQMessageNumber(7609,
                  "XA resource manager returned a heuristic return code.",
                  "20 : Error",
                  "This message is associated with an earlier AMQ7607 message reporting a mixed transaction outcome. It identifies one of the resource managers (<insert_4>) that failed to rollback its updates. The transaction associated with this failure is identified by the following XID of X<insert_3>.",
                  "Use the return code <insert_1> returned by the resource manager to determine the effects of the failure. The return code indicates that the resource manager made a heuristic decision about the outcome of the transaction which disagrees with the rollback decision of the queue manager. In order to preserve data integrity you will need to perform recovery steps local to this resource manager."));
      amqMN_Ht.put("7612",new AMQMessageNumber(7612,
                  "Switch call exception",
                  "20 : Error",
                  "Exception number <insert_1> occurred when calling resource manager switch <insert_3>.",
                  "Check the resource manager switch has not been corrupted."));
      amqMN_Ht.put("7622",new AMQMessageNumber(7622,
                  "WebSphere MQ could not load the XA switch load file for resource manager <insert_3>.",
                  "20 : Error",
                  "An error has occurred loading XA switch file <insert_4>. If the error occurred during startup then the queue manager will terminate. At all other times the queue manager will continue without this resource manager meaning that it will no longer be able to participate in global transactions. The queue manager will also retry the load of the switch file at regular intervals so that the resource manager will be able to participate again should the load problem be resolved.",
                  "Look for a previous message outlining the reason for the load failure. Message AMQ6175 is issued if the load failed because of a system error. If this is the case then follow the guidance given in message AMQ6175 to resolve the problem. In the absence of prior messages or FFST information related to this problem check that the name of the switch load file is correct and that it is present in a directory from which it can be dynamically loaded by the queue manager. The easiest method of doing this is to define the switch load file as a fully-qualified name. Note that if the queue manager is still running it will need to be restarted in order that any changes made to its configuration data can be picked up."));
      amqMN_Ht.put("7623",new AMQMessageNumber(7623,
                  "WebSphere MQ has not been configured with XA resource manager.",
                  "10 : Warning",
                  "The queue manager has noticed that XA resource manager <insert_3> was removed from the qm.ini file of the queue manager. However, it was logged as being involved in <insert_1> transactions that are still in-doubt. The queue manager cannot resolve these transactions.The queue manager is continuing without this resource manager.",
                  "First check that the qm.ini configuration file of the queue manager concerned hasn't been mistakenly altered resulting in an 'XAResourceManager' stanza being removed, or the 'Name' of any the resource managers being changed. If the qm.ini file was changed by mistake then you will need to reinstate resource manager <insert_3> in the qm.ini file before stopping and then restarting the queue manager in order that the change will be picked up. If you have intentionally removed a resource manager from the qm.ini file, consider the integrity implications of your action since the resource manager concerned may be in an in-doubt state. If you are sure that is not the case then you can use the 'rsvmqtrn' command to deliver an outcome on behalf of the resource manager in order that the queue manager can forget about the transactions concerned. If you cannot be sure that such an action will not cause an integrity problem then you should consider re-instating the resource manager in the qm.ini file so that the queue manager can contact the resource manager and automatically resolve the transactions concerned next time the queue manager is restarted."));
      amqMN_Ht.put("7623",new AMQMessageNumber(7623,
                  " (Windows)WebSphere MQ has not been configured with XA resource manager <insert_3> which may be involved in in-doubt transactions. The queue manager is continuing without this resource manager.",
                  "10 : Warning",
                  "The queue manager has recognized that XA resource manager <insert_3> was removed from the registry entry of the queue manager. However, it was logged as being involved in <insert_1> transactions that are still in-doubt. The queue manager cannot resolve these transactions.",
                  "Check that the configuration data entry of the queue manager concerned has not been altered by mistake, resulting in an 'XAResourceManager' stanza being removed, or the 'Name' of any the resource managers being changed. If the configuration data entry was changed by mistake, you need to reinstate resource manager <insert_3> in the configuration data before stopping, and then restarting the queue manager to access the change. If you have intentionally removed a resource manager from the configuration data, consider the integrity implications of your action because the resource manager concerned may be in an in-doubt state. If you are sure that this is not the case, you can use the 'rsvmqtrn' command to instruct the resource manager to inform the queue manager that it can forget about the transactions concerned. If using the 'rsvmqtrn' command could result in an integrity problem, you should consider reinstating the resource manager in the configuration data, so that the queue manager can contact the resource manager and automatically resolve the transactions concerned next time the queue manager is restarted."));
      amqMN_Ht.put("7624",new AMQMessageNumber(7624,
                  "An exception occurred during an <insert_4> call to XA resource manager <insert_3>.",
                  "20 : Error",
                  "An exception has been detected during a call to an XA resource manager. The queue manager will continue after assuming a return code of XAER_RMERR from the call.",
                  "An FFST should have been produced which documents the exception. Use this and any further FFSTs to try and determine the reason for the failure. A trace of the problem will be useful to identify the XA flows between the queue manager and the resource manager concerned. MQ has allocated an RMId of <insert_1> to this resource manager. Use this to isolate the flows concerned. First contact the supplier of the resource manager for problem resolution. If however you think that the problem lies within the queue manager then contact your IBM support center. Do not discard any information describing the problem until after it has been resolved."));
      amqMN_Ht.put("7625",new AMQMessageNumber(7625,
                  "The XA resource manager <insert_3> has become available again.",
                  "0 : Information",
                  "WebSphere MQ has managed to regain contact with a resource manager that had become unavailable. Any in-doubt transactions involving this resource manager will be resolved. The resource manager will now be able to participate in new transactions.",
                  "None."));
      amqMN_Ht.put("7626",new AMQMessageNumber(7626,
                  "XA resource manager initialization failure. Refer to the error log for more information.",
                  "20 : Error",
                  "The queue manager has failed to initialize one or more of the XA resource managers defined in the qm.ini configuration file.",
                  "Correct the error and restart the queue manager."));
      amqMN_Ht.put("7626",new AMQMessageNumber(7626,
                  " (Windows)XA resource manager initialization failure. Refer to the error log for more information.",
                  "20 : Error",
                  "The queue manager has failed to initialize one or more of the XA resource managers defined in the configuration data.",
                  "Correct the error and restart the queue manager."));
      amqMN_Ht.put("7701",new AMQMessageNumber(7701,
                  "DMPMQLOG command is starting.",
                  "0 : Information",
                  "You have started the DMPMQLOG command and it is processing your request.",
                  "None."));
      amqMN_Ht.put("7702",new AMQMessageNumber(7702,
                  "DMPMQLOG command has finished successfully.",
                  "0 : Information",
                  "The DMPMQLOG command has finished processing your request and no errors were detected.",
                  "None."));
      amqMN_Ht.put("7703",new AMQMessageNumber(7703,
                  "DMPMQLOG command has used option <insert_3> with an invalid value <insert_4>.",
                  "20 : Error",
                  "You started the DMPMQLOG command specifying an invalid option value. The <insert_4> value for option <insert_3> is either missing or of an incorrect format.",
                  "Refer to the command syntax, and then try the command again."));
      amqMN_Ht.put("7704",new AMQMessageNumber(7704,
                  "DMPMQLOG command has used an invalid option <insert_3>.",
                  "20 : Error",
                  "You started the DMPMQLOG command specifying an invalid option of <insert_3>.",
                  "Refer to the command syntax and then try the command again."));
      amqMN_Ht.put("7705",new AMQMessageNumber(7705,
                  "Usage: dmpmqlog [-b | -s StartLSN | -n ExtentNumber] [-e EndLSN] [-f LogFilePath] [-m QMgrName]",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("7706",new AMQMessageNumber(7706,
                  "DMPMQLOG command has used an incorrect queue manager name <insert_3> or path <insert_4>.",
                  "20 : Error",
                  "The DMPMQLOG command has used <insert_3> as the queue manager name and, if shown, <insert_4> as the directory path for <insert_3>. Either <insert_3> and/or <insert_4> is incorrect; if <insert_4> is not shown then it is <insert_3> which is incorrect. Possible reasons for the error include: that <insert_3> is not an existing queue manager name; the entries for <insert_3> in the MQ system initialization (INI) file are incorrect; <insert_4> is not a correct path for <insert_3>. If you started the command specifying option -m (queue manager name option) with a value then this value will have been used as the queue manager name, otherwise the default queue manager name will have been used.",
                  "Check that <insert_3> is an existing queue manager name. Check your MQ system's initialization (INI) file to ensure that <insert_3> and its associated entries are correct. If <insert_4> is shown, check that it is a correct MQ system directory path for <insert_3>."));
      amqMN_Ht.put("7706",new AMQMessageNumber(7706,
                  "(iSeries) DMPMQLOG command has used an incorrect queue manager name or path.",
                  "20 : Error",
                  "The DMPMQLOG command has used <insert_3> as the queue manager name and, if shown, <insert_4> as the directory path for <insert_3>. Either <insert_3> and/or <insert_4> is incorrect; if <insert_4> is not shown then it is <insert_3> which is incorrect. Possible reasons for the error include: that <insert_3> is not an existing queue manager name; the entries for <insert_3> in the MQ system initialization (INI) file are incorrect; <insert_4> is not a correct path for <insert_3>. If you started the command specifying option -m (queue manager name option) with a value then this value will have been used as the queue manager name, otherwise the default queue manager name will have been used.",
                  "Check that <insert_3> is an existing queue manager name. Check your MQ system's initialization (INI) file to ensure that <insert_3> and its associated entries are correct. If <insert_4> is shown, check that it is a correct MQ system directory path for <insert_3>."));
      amqMN_Ht.put("7706",new AMQMessageNumber(7706,
                  " (Windows)DMPMQLOG command has used an incorrect queue manager name <insert_3> or path <insert_4>.",
                  "20 : Error",
                  "The DMPMQLOG command has used <insert_3> as the queue manager name and, if shown, <insert_4> as the directory path for <insert_3>. Either <insert_3> and/or <insert_4> is incorrect; if <insert_4> is not shown then it is <insert_3> which is incorrect. Possible reasons for the error include: that <insert_3> is not an existing queue manager name; the entries for <insert_3> in the MQ configuration data are incorrect; <insert_4> is not a correct path for <insert_3>. If you started the command specifying option -m (queue manager name option) with a value then this value will have been used as the queue manager name, otherwise the default queue manager name will have been used.",
                  "Check that <insert_3> is an existing queue manager name. Check your MQ configuration data to ensure that <insert_3> and its associated entries are correct. If <insert_4> is shown, check that it is a correct MQ system directory path for <insert_3>."));
      amqMN_Ht.put("7707",new AMQMessageNumber(7707,
                  "DMPMQLOG command has failed: CompCode = 0x<insert_1>.",
                  "20 : Error",
                  "The DMPMQLOG command has detected an error and the MQ recording routine has been called. Possible reasons for this include a damaged log file, a problem during initialization for the queue manager or an internal MQ failure.",
                  "Check that the queue manager being used by DMPMQLOG, as specified by you using the -m command option or defaulted, exists and is not currently running. If it does not exist, try the command again specifying an existing queue manager. If it is running, stop the queue manager and then try the command again. Otherwise, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Note the completion code (CompCode) and then contact your IBM support center."));
      amqMN_Ht.put("7708",new AMQMessageNumber(7708,
                  "DMPMQLOG command has used an invalid default queue manager name.",
                  "20 : Error",
                  "You started the DMPMQLOG command without specifying option -m (queue manager name option) and so your MQ default queue manager name has been used. However, this default name either could not be found or is invalid.",
                  "Check that the default queue manager name exists and is valid, and then try the command again."));
      amqMN_Ht.put("7709",new AMQMessageNumber(7709,
                  "DMPMQLOG command has used an invalid combination of options.",
                  "20 : Error",
                  "You started the DMPMQLOG command specifying an invalid combination of the options -b (base LSN option), -s (start LSN option) and -n (extent number option). Only 1 or none of these options may be specified.",
                  "Refer to the command syntax and then try the command again."));
      amqMN_Ht.put("7710",new AMQMessageNumber(7710,
                  "DMPMQLOG command has used option -n which is invalid for circular logging.",
                  "20 : Error",
                  "You started the DMPMQLOG command specifying option -n (extent number option) but this is not valid when your MQ log is defined as circular.",
                  "Use a different option and then try the command again."));
      amqMN_Ht.put("7711",new AMQMessageNumber(7711,
                  "DMPMQLOG command has used option -m with a value that is too long.",
                  "20 : Error",
                  "You started the DMPMQLOG command specifying option -m (queue manager name option) with a value that is more than <insert_1> characters.",
                  "Specify a shorter queue manager name and then try the command again."));
      amqMN_Ht.put("7712",new AMQMessageNumber(7712,
                  "DMPMQLOG command has used option -f with a value which is too long.",
                  "20 : Error",
                  "You started the DMPMQLOG command specifying option -f (log file path option) with a value which is more than <insert_1> characters.",
                  "Specify a shorter log file path name and then try the command again."));
      amqMN_Ht.put("7713",new AMQMessageNumber(7713,
                  "DMPMQLOG command was unable to allocate sufficient storage.",
                  "20 : Error",
                  "The DMPMQLOG command has been unable to allocate some storage.",
                  "Free some storage and then try the command again."));
      amqMN_Ht.put("7714",new AMQMessageNumber(7714,
                  "DMPMQLOG command has reached the end of the log.",
                  "0 : Information",
                  "The DMPMQLOG command has processed any log data and has now reached the end of the log.",
                  "None."));
      amqMN_Ht.put("7715",new AMQMessageNumber(7715,
                  "DMPMQLOG command cannot open file <insert_3>.",
                  "20 : Error",
                  "The DMPMQLOG command was unable to open file <insert_3> for reading.",
                  "Check that the file exists, can be opened for reading, and that you have authority to access it, and then try the command again."));
      amqMN_Ht.put("7716",new AMQMessageNumber(7716,
                  "DMPMQLOG command has finished unsuccessfully.",
                  "0 : Information",
                  "The DMPMQLOG command has finished with your request but an error has been detected. The previous message issued by the command can be used to identify the error.",
                  "Refer to the previous message issued by the command."));
      amqMN_Ht.put("7717",new AMQMessageNumber(7717,
                  "DMPMQLOG command has failed to initialize: CompCode = 0x<insert_1>.",
                  "20 : Error",
                  "The DMPMQLOG command has failed during its initialization and the MQ recording routine has been called. Possible reasons for this include that your queue manager is already running. The completion code can be used to identify the error.",
                  "Check that the queue manager being used by DMPMQLOG, as specified by you using the -m command option or defaulted, exists and is not currently running. If it is running, stop the queue manager and then try the command again. Otherwise, use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7718",new AMQMessageNumber(7718,
                  "DMPMQLOG command is using a default of <insert_3> for the queue manager name.",
                  "0 : Information",
                  "You have started the DMPMQLOG command without specifying option -m (queue manager name option) and so a default value of <insert_3> is being used. This value is obtained from your default queue manager name.",
                  "None."));
      amqMN_Ht.put("7718",new AMQMessageNumber(7718,
                  "(iSeries) DMPMQLOG command is using a the default queue manager name.",
                  "0 : Information",
                  "You have started the DMPMQLOG command without specifying option -m (queue manager name option) and so a default value of <insert_3> is being used. This value is obtained from your MQ default queue manager name.",
                  "None."));
      amqMN_Ht.put("7719",new AMQMessageNumber(7719,
                  "DMPMQLOG command is using a default of <insert_3> for the starting dump location.",
                  "0 : Information",
                  "You have started the DMPMQLOG command without specifying option -b (base LSN option), option -s (start LSN option) or option -n (extent number option), and so a default value of <insert_3> is being used. This value is the Log Sequence Number (LSN) of the first record in the active part of the log, and will be used as the location from which to start dumping.",
                  "None."));
      amqMN_Ht.put("7719",new AMQMessageNumber(7719,
                  "(iSeries) DMPMQLOG command is using the default starting dump location.",
                  "0 : Information",
                  "You have started the DMPMQLOG command without specifying option -b (base LSN option), option -s (start LSN option) or option -n (extent number option), and so a default value of <insert_3> is being used. This value is the Log Sequence Number (LSN) of the first record in the active part of the log, and will be used as the location from which to start dumping.",
                  "None."));
      amqMN_Ht.put("7720",new AMQMessageNumber(7720,
                  "DMPMQLOG command is using extent <insert_1> but the current extent is <insert_2>.",
                  "20 : Error",
                  "You have started the DMPMQLOG command specifying option -n (extent number option) with a value of <insert_1> but this value is greater than <insert_2>, which represents the extent currently being used.",
                  "When using option -n, specify its value as being less than or equal to the extent number currently being used."));
      amqMN_Ht.put("7721",new AMQMessageNumber(7721,
                  "DMPMQLOG command has not found any log records in extent number <insert_1>.",
                  "0 : Information",
                  "During its normal processing, the DMPMQLOG command did not find any log records in this extent.",
                  "None."));
      amqMN_Ht.put("7722",new AMQMessageNumber(7722,
                  "DMPMQLOG command cannot find the object catalogue for queue manager <insert_3>.",
                  "20 : Error",
                  "The DMPMQLOG command is using the queue manager named <insert_3> but cannot find the manager's object catalogue file. This file should have been created at the time the queue manager was created.",
                  "Refer to the \"System Management Guide\" for a description of the location and name of the object catalogue file. Check that the file exists and is available for use by this command. If it does not exist then you will need to re-create the queue manager."));
      amqMN_Ht.put("7722",new AMQMessageNumber(7722,
                  "(iSeries) DMPMQLOG command cannot find the object catalogue for the queue manager.",
                  "20 : Error",
                  "The DMPMQLOG command is using the queue manager named <insert_3> but cannot find the manager's object catalogue file. This file should have been created at the time the queue manager was created.",
                  "Refer to the \"System Management Guide\" for a description of the location and name of the object catalogue file. Check that the file exists and is available for use by this command. If it does not exist then you will need to re-create the queue manager."));
      amqMN_Ht.put("7723",new AMQMessageNumber(7723,
                  "DMPMQLOG command cannot find the requested Log Sequence Number (LSN).",
                  "20 : Error",
                  "The DMPMQLOG command has been started with an LSN but it cannot be found in the log.",
                  "Check for an existing LSN and then try the command again."));
      amqMN_Ht.put("7724",new AMQMessageNumber(7724,
                  "DMPMQLOG command cannot use the requested extent number.",
                  "20 : Error",
                  "The DMPMQLOG command has been started with an extent number but it is beyond the end of the log.",
                  "Check for an existing extent number and then try the command again."));
      amqMN_Ht.put("7725",new AMQMessageNumber(7725,
                  "DMPMQLOG command cannot find an old Log Sequence Number (LSN).",
                  "20 : Error",
                  "The DMPMQLOG command has been started specifying an LSN which is older than the log's base LSN. However, the specified LSN could not be found.",
                  "Check for an existing LSN and then try the command again."));
      amqMN_Ht.put("7726",new AMQMessageNumber(7726,
                  "DMPMQLOG command has used option -s with an incorrect value for circular logging.",
                  "20 : Error",
                  "You started the DMPMQLOG command specifying option -s (start LSN option) with a value which is less than the base LSN of a log which is defined as circular. LSN values less than the base LSN can only be specified when using a linear log.",
                  "When using option -s with a circular log, specify an option value which is equal or greater to the log's base LSN, and then try the command again."));
      amqMN_Ht.put("7751",new AMQMessageNumber(7751,
                  "(iSeries) MIGRATEMQM program is starting.",
                  "0 : Information",
                  "You have started the MIGRATEMQM program.",
                  "None."));
      amqMN_Ht.put("7752",new AMQMessageNumber(7752,
                  "(iSeries) MIGRATEMQM has completed successfully.",
                  "0 : Information",
                  "The MIGRATEMQM program has completed migration of your queue manager and no errors were detected.",
                  "None."));
      amqMN_Ht.put("7753",new AMQMessageNumber(7753,
                  "(iSeries) MIGRATEMQM has failed due to errors.",
                  "20 : Error",
                  "See the previously listed messages in the job log. Correct the errors and then restart the MIGRATEMQM program.",
                  "None."));
      amqMN_Ht.put("7754",new AMQMessageNumber(7754,
                  "(iSeries) MIGRATEMQM has detected an error and is unable to continue.",
                  "20 : Error",
                  "See the previously listed messages in this job log, or in associated job logs. Correct the errors and then restart the MIGRATEMQM program.",
                  "None."));
      amqMN_Ht.put("7755",new AMQMessageNumber(7755,
                  "(iSeries) Unable to locate a required journal receiver.",
                  "20 : Error",
                  "The MIGRATEMQM program attempted to locate the journal receivers to use for migration, but the operation required access to a journal or journal receiver that is not currently present on the system.",
                  "Restore the required journal or journal receiver from backup. Then restart the MIGRATEMQM program."));
      amqMN_Ht.put("7756",new AMQMessageNumber(7756,
                  "(iSeries) Unable to locate a required journal entry.",
                  "20 : Error",
                  "The MIGRATEMQM program was unable to retrieve a journal entry required for migration. The operation may have failed because a required journal receiver is not currently present on the system.",
                  "Restore the required journal receiver from backup. Then restart the MIGRATEMQM program."));
      amqMN_Ht.put("7757",new AMQMessageNumber(7757,
                  "(iSeries) Queue manager <insert_3> already exists.",
                  "20 : Error",
                  "The MIGRATEMQM program is unable to create a queue manager with the same name as used in the previous release because a queue manager of this name has already been created.",
                  "Delete the queue manager. Then restart the MIGRATEMQM program."));
      amqMN_Ht.put("7758",new AMQMessageNumber(7758,
                  "(iSeries) Queue manager starting.",
                  "0 : Information",
                  "The queue manager \"<insert_3>\" is starting.",
                  "None."));
      amqMN_Ht.put("7759",new AMQMessageNumber(7759,
                  "(iSeries) Recreating WebSphere MQ objects.",
                  "0 : Information",
                  "WebSphere MQ objects are being recreated from their media images contained in the log.",
                  "None."));
      amqMN_Ht.put("7760",new AMQMessageNumber(7760,
                  "(iSeries) Recreating WebSphere MQ channels.",
                  "0 : Information",
                  "WebSphere MQ channels are being recreated from the previous channel definition file.",
                  "None."));
      amqMN_Ht.put("7761",new AMQMessageNumber(7761,
                  "(iSeries) Unexpected return code from command <insert_3>.",
                  "20 : Error",
                  "An unexpected return code, <insert_1>, was returned by command <insert_3>.",
                  "See the previously listed messages in this job log, or in associated job logs."));
      amqMN_Ht.put("7762",new AMQMessageNumber(7762,
                  "(iSeries) Unexpected error from channel migration.",
                  "20 : Error",
                  "The migration of channel definitions or channel synchronization data encountered an unexpected error.",
                  "See the previously listed messages in this job log, or in associated job logs."));
      amqMN_Ht.put("7770",new AMQMessageNumber(7770,
                  "Sent file <insert_3>",
                  "40 : Stop Error",
                  "The file was successfully sent.",
                  "None."));
      amqMN_Ht.put("7771",new AMQMessageNumber(7771,
                  "Received file.",
                  "40 : Stop Error",
                  "The file was successfully received.",
                  "None."));
      amqMN_Ht.put("7772",new AMQMessageNumber(7772,
                  "Complete file list",
                  "40 : Stop Error",
                  "Displays a list of complete files.",
                  "None."));
      amqMN_Ht.put("7773",new AMQMessageNumber(7773,
                  "Incomplete file list",
                  "40 : Stop Error",
                  "Displays a list of incomplete files.",
                  "None."));
      amqMN_Ht.put("7774",new AMQMessageNumber(7774,
                  "Other message list",
                  "40 : Stop Error",
                  "Displays a list of other messages.",
                  "None."));
      amqMN_Ht.put("7775",new AMQMessageNumber(7775,
                  "Nothing to list.",
                  "40 : Stop Error",
                  "Nothing to list.",
                  "None."));
      amqMN_Ht.put("7776",new AMQMessageNumber(7776,
                  "Deleted.",
                  "40 : Stop Error",
                  "File deleted.",
                  "None."));
      amqMN_Ht.put("7777",new AMQMessageNumber(7777,
                  "Nothing to delete.",
                  "40 : Stop Error",
                  "Nothing to delete.",
                  "None."));
      amqMN_Ht.put("7778",new AMQMessageNumber(7778,
                  "Syntax error. The correct syntax is:",
                  "40 : Stop Error",
                  "Invalid arguments supplied.",
                  "One or more options were incorrectly specified when issuing the send or receive command. Check the options used and reissue the command."));
      amqMN_Ht.put("7779",new AMQMessageNumber(7779,
                  "Cannot connect to default queue manager.",
                  "40 : Stop Error",
                  "Queue manager not available.",
                  "Check that the queue manager exists and that the listener is running."));
      amqMN_Ht.put("7780",new AMQMessageNumber(7780,
                  "Cannot connect to queue manager <insert_3>",
                  "40 : Stop Error",
                  "Queue manager not available.",
                  "Check that the queue manager exists and that the listener is running."));
      amqMN_Ht.put("7781",new AMQMessageNumber(7781,
                  "Application memory unavailable.",
                  "40 : Stop Error",
                  "There is insufficient memory to perform the requested action.",
                  "\n 1) Check the message size is not excessive 2) Close other applications and try the command again"));
      amqMN_Ht.put("7783",new AMQMessageNumber(7783,
                  "Queue name required.",
                  "40 : Stop Error",
                  "A queue name was not specified when issuing a send or receive command.",
                  "Reissue the command with the QueueName option."));
      amqMN_Ht.put("7784",new AMQMessageNumber(7784,
                  "Cannot open queue <insert_3>",
                  "40 : Stop Error",
                  "Cannot open queue <insert_3>",
                  "Check that the queue exists."));
      amqMN_Ht.put("7785",new AMQMessageNumber(7785,
                  "Cannot open file <insert_3>",
                  "40 : Stop Error",
                  "Cannot open file <insert_3>",
                  "Check that the file exists, that it is in the correct location and has the appropriate file permissions."));
      amqMN_Ht.put("7786",new AMQMessageNumber(7786,
                  "Cannot put to queue <insert_3>",
                  "40 : Stop Error",
                  "Cannot put to queue <insert_3>",
                  "\n 1) Check the Queue Manager has sufficient log space for sending large messages 2) Check the queue does not have put inhibited 3) Check the queue is not full 4) Check the message size of the queue is greater than the message size 5) Check the user has sufficient authority to put messages on the queue"));
      amqMN_Ht.put("7787",new AMQMessageNumber(7787,
                  "No file name specified.",
                  "40 : Stop Error",
                  "No file name specified.",
                  "A file name was not specified when issuing a send command. Reissue the command with the FileName option."));
      amqMN_Ht.put("7788",new AMQMessageNumber(7788,
                  "Message length is too small to send data.",
                  "40 : Stop Error",
                  "Message length is too small to send data.",
                  "Increase the message size and resend with a send command, using the -l MessageSize option to specify a larger message size."));
      amqMN_Ht.put("7789",new AMQMessageNumber(7789,
                  "Sending file has changed.",
                  "40 : Stop Error",
                  "The file being sent has been changed before the complete file has been sent.",
                  "Check the file for integrity and reissue the send command."));
      amqMN_Ht.put("7790",new AMQMessageNumber(7790,
                  "Cannot get from queue <insert_3>",
                  "40 : Stop Error",
                  "The list, get, delete or extract request has failed.",
                  "\n 1) Check the queue does have get inhibited 2) Check the user has sufficient WMQ authority to get messages from the queue"));
      amqMN_Ht.put("7791",new AMQMessageNumber(7791,
                  "Cannot write to file.",
                  "40 : Stop Error",
                  "The get or extract request has failed.",
                  "\n 1) Check that the file is not write-protected. In Windows Explorer, right-click the file name and select Properties. Check the user has sufficient authority to write to the destination file system. 2) Check the destination file system exists 3) Check the destination file system is not full"));
      amqMN_Ht.put("7792",new AMQMessageNumber(7792,
                  "CorrelId is invalid.",
                  "40 : Stop Error",
                  "CorrelId is invalid.",
                  "\n 1) Check that a valid correlation ID has been specified when receiving a file with the -c option. 2) It must be 48 characters in length. 3) Use the -v option of the receive command to display the correlation ID."));
      amqMN_Ht.put("7793",new AMQMessageNumber(7793,
                  "MsgId is invalid.",
                  "40 : Stop Error",
                  "MsgId is invalid.",
                  "\n 1) Check that a valid message ID has been specified when receiving an 'other' message with the -u option. 2) It must be 48 characters in length."));
      amqMN_Ht.put("7794",new AMQMessageNumber(7794,
                  "No messages to receive.",
                  "40 : Stop Error",
                  "There are no FTA files on the specified queue.",
                  "Check with the sender that the file was actually sent."));
      amqMN_Ht.put("7795",new AMQMessageNumber(7795,
                  "Cannot delete the file because it's not unique.",
                  "40 : Stop Error",
                  "Cannot delete the file because it's not unique.",
                  "None."));
      amqMN_Ht.put("7796",new AMQMessageNumber(7796,
                  "Cannot replace an existing file.",
                  "40 : Stop Error",
                  "Cannot replace an existing file.",
                  "Reissue the command with the -y option."));
      amqMN_Ht.put("7797",new AMQMessageNumber(7797,
                  "Unable to load the WebSphere MQ library.",
                  "40 : Stop Error",
                  "Unable to load the WebSphere MQ library.",
                  "None."));
      amqMN_Ht.put("7798",new AMQMessageNumber(7798,
                  "Unable to locate <insert_3>.",
                  "40 : Stop Error",
                  "This application requires <insert_3>.",
                  "Check that <insert_3> is available and installed correctly."));
      amqMN_Ht.put("7799",new AMQMessageNumber(7799,
                  "Unable to start <insert_3>.",
                  "40 : Stop Error",
                  "This application cannot start <insert_3>.",
                  "Check that <insert_3> is available and installed correctly."));
      amqMN_Ht.put("7800",new AMQMessageNumber(7800,
                  "CorrelId <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("7801",new AMQMessageNumber(7801,
                  "Dir <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("7802",new AMQMessageNumber(7802,
                  "UserData <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("7803",new AMQMessageNumber(7803,
                  "FileName <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("7804",new AMQMessageNumber(7804,
                  "Length <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("7805",new AMQMessageNumber(7805,
                  "MsgId <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("7806",new AMQMessageNumber(7806,
                  "Could not start WebSphere MQ web administration server: <insert_1>.",
                  "0 : Information",
                  "An unsuccessful attempt was made to start the web administration server on port <insert_1>.",
                  "Check the product is installed correctly; the required registry keys and values are correct and the web server port is not already in use. If the problem persists contact your service representative."));
      amqMN_Ht.put("7807",new AMQMessageNumber(7807,
                  "WebSphere MQ web administration server running.",
                  "0 : Information",
                  "WebSphere MQ web administration server running. Listening on port <insert_4>, root directory is <insert_5>.",
                  "No action is required."));
      amqMN_Ht.put("7808",new AMQMessageNumber(7808,
                  "Internal run-time error in WebSphere MQ web administration: <insert_4>.",
                  "0 : Information",
                  "WebSphere MQ web administration had the following internal run-time error: <insert_4>.",
                  "Check that: the product is installed correctly and that the required registry keys and values are correct. If the problem persists contact your service representative."));
      amqMN_Ht.put("7809",new AMQMessageNumber(7809,
                  "WebSphere MQ Publish/Subscribe web administration user limit reached.",
                  "10 : Warning",
                  "The maximum number of concurrent web administration users has been reached (<insert_4>).",
                  "Use the 'Web Administration Server' properties page in the Microsoft Management Console to increase the value of the web administration 'MaxClients' parameter."));
      amqMN_Ht.put("7810",new AMQMessageNumber(7810,
                  " (Windows)Failed to create class, reason code: <insert_1>.",
                  "20 : Error",
                  "While trying to create class <insert_3> on <insert_4> error code <insert_1> was encountered. The associated error message generated by the operating system is: <insert_5>",
                  "Check the system documentation to determine the course of action required to rectify the problem."));
      amqMN_Ht.put("7880",new AMQMessageNumber(7880,
                  " (Windows)Error code <insert_1> starting <insert_4>/<insert_3> WebSphere MQ service.",
                  "0 : Information",
                  "The service was unable to start <insert_4>/<insert_3>. The error message reported was as follows: <insert_5>",
                  "Use WebSphere MQ Explorer to investigate why the service could not begin. If recovery for this service is active, MQ will attempt to recover."));
      amqMN_Ht.put("7881",new AMQMessageNumber(7881,
                  " (Windows)Unable to stop <insert_4>/<insert_3> WebSphere MQ service, return code <insert_1>.",
                  "10 : Warning",
                  "The WebSphere MQ service was unable to stop <insert_4>/<insert_3>. The error message reported was as follows: <insert_5>",
                  "Use WebSphere MQ Explorer to investigate why the service could not be stopped."));
      amqMN_Ht.put("7882",new AMQMessageNumber(7882,
                  " (Windows)Attempting to recover <insert_4>/<insert_3> WebSphere MQ service.",
                  "0 : Information",
                  "The WebSphere MQ service has detected that <insert_4>/<insert_3> has failed, and is attempting to restart it.",
                  "No Action Required."));
      amqMN_Ht.put("7883",new AMQMessageNumber(7883,
                  " (Windows)<insert_4>/<insert_3> WebSphere MQ service started from recovery.",
                  "0 : Information",
                  "The WebSphere MQ service has successfully recovered <insert_4>/<insert_3>.",
                  "No Action Required."));
      amqMN_Ht.put("7884",new AMQMessageNumber(7884,
                  " (Windows)Unable to recover <insert_4>/<insert_3> WebSphere MQ service.",
                  "10 : Warning",
                  "The WebSphere MQ service has attempted to recover <insert_4>/<insert_3>, but all attempts have failed. There will be no more attempts to recover this service.",
                  "Use WebSphere MQ Explorer to investigate why the service failed and could not be restarted."));
      amqMN_Ht.put("7885",new AMQMessageNumber(7885,
                  " (Windows)Unable to delete queue manager <insert_4>, error <insert_1>.",
                  "10 : Warning",
                  "An attempt to delete queue manager <insert_4> failed. WebSphere MQ returned error code <insert_1>: <insert_5>",
                  "Ensure that the queue manager name has been specified correctly, and try again."));
      amqMN_Ht.put("7886",new AMQMessageNumber(7886,
                  " (Windows)Unable to create queue manager <insert_4>.",
                  "10 : Warning",
                  "Queue manager <insert_4> could not be created. WebSphere MQ returned error <insert_1>: <insert_5>",
                  "Check the error and application event logs to investigate the reason for the the returned error and suggested responses to take to rectify the fault. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7890",new AMQMessageNumber(7890,
                  " (Windows)Unable to open mapped file containing WebSphere MQ performance data.",
                  "20 : Error",
                  "The WebSphere MQ extensible counter dll was unable to open a mapped file used to collect queue performance data. Your system may be running short on virtual memory.",
                  "No action required. Performance statistics for MQ queues will not be displayed."));
      amqMN_Ht.put("7891",new AMQMessageNumber(7891,
                  " (Windows)Unable to create a mutex to access WebSphere MQ performance data.",
                  "20 : Error",
                  "The WebSphere MQ extensible counter dll was unable to create a mutex required to synchronise collection of queue performance data",
                  "No action required. Performance statistics for MQ queues will not be displayed."));
      amqMN_Ht.put("7892",new AMQMessageNumber(7892,
                  " (Windows)Unable to map to shared memory file containing WebSphere MQ performance data.",
                  "20 : Error",
                  "The WebSphere MQ extensible counter dll was unable to map the shared memory file required for collection of queue performance data.",
                  "No action required. Performance statistics for MQ queues will not be displayed."));
      amqMN_Ht.put("7893",new AMQMessageNumber(7893,
                  " (Windows)Unable to open \"Performance\" key for WebSphere MQ services. Status code: <insert_1>.",
                  "20 : Error",
                  "The WebSphere MQ extensible counter dll was unable to obtain performance counter values from the \"Performance\" key for WebSphere MQ services. Status code is the return value from the Windows registry call RegOpenKeyEx.",
                  "No action required. Performance statistics for MQ queues will not be displayed."));
      amqMN_Ht.put("7894",new AMQMessageNumber(7894,
                  " (Windows)Unable to read the \"Performance\\First Counter\" value for WebSphere MQ services. Status code: <insert_1>.",
                  "20 : Error",
                  "The WebSphere MQ extensible counter dll was unable to obtain performance counter values from the \"Performance\\First Counter\" key for WebSphere MQ services. Status code is the return value from the Windows registry call RegOpenKeyEx.",
                  "No action required. Performance statistics for MQ queues will not be displayed."));
      amqMN_Ht.put("7895",new AMQMessageNumber(7895,
                  " (Windows)Unable to read the \"Performance\\First Help\" value for WebSphere MQ services. Status code: <insert_1>.",
                  "20 : Error",
                  "The WebSphere MQ extensible counter dll was unable to obtain performance counter values from the \"Performance\\First Help\" key for WebSphere MQ services. Status code is the return value from the Windows registry call RegOpenKeyEx.",
                  "No action required. Performance statistics for MQ queues will not be displayed."));
      amqMN_Ht.put("7901",new AMQMessageNumber(7901,
                  "The data-conversion exit <insert_3> has not loaded.",
                  "30 : Severe error",
                  "The data-conversion exit program, <insert_3>, failed to load. The internal function gave exception <insert_4>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7903",new AMQMessageNumber(7903,
                  "The data-conversion exit <insert_3> cannot be found.",
                  "30 : Severe error",
                  "Message data conversion has been requested for a WebSphere MQ message with a user-defined format, but the necessary data-conversion exit program, <insert_3>, cannot be found. The internal function gave exception <insert_4>.",
                  "Check that the necessary data-conversion exit <insert_3> exists."));
      amqMN_Ht.put("7904",new AMQMessageNumber(7904,
                  "The data-conversion exit <insert_3> cannot be found, or loaded.",
                  "30 : Severe error",
                  "Message data conversion was requested for a WebSphere MQ message with a user-defined format, but the necessary data conversion exit program, <insert_3>, was not found, or loaded. The <insert_4> function call gave a return code of <insert_1>.",
                  "Check that the necessary data conversion exit routine exists in one of the standard directories for dynamically loaded modules. If necessary, inspect the generated output to examine the message descriptor (MQMD structure) of the MQ message for the conversion which was requested. This may help you to determine where the message originated."));
      amqMN_Ht.put("7905",new AMQMessageNumber(7905,
                  "Unexpected exception <insert_4> in data-conversion exit.",
                  "30 : Severe error",
                  "The data-conversion exit program, <insert_3>, ended with an unexpected exception <insert_4>. The message has not been converted.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7907",new AMQMessageNumber(7907,
                  "Unexpected exception in data-conversion exit.",
                  "30 : Severe error",
                  "The data-conversion exit routine, <insert_3>, ended with an unexpected exception. The message has not been converted.",
                  "Correct the error in the data-conversion exit routine."));
      amqMN_Ht.put("7908",new AMQMessageNumber(7908,
                  " (Windows)Display active directory CRL server details.",
                  "0 : Information",
                  "Display active directory CRL server details.",
                  "None."));
      amqMN_Ht.put("7909",new AMQMessageNumber(7909,
                  " (Windows)There are no active directory CRL server details to display.",
                  "0 : Information",
                  "No active directory CRL server definitions could be found.",
                  "None."));
      amqMN_Ht.put("7910",new AMQMessageNumber(7910,
                  " (Windows)Usage: setmqscp [-m QmgrName | * ] [-a] [-d] [-r]",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7911",new AMQMessageNumber(7911,
                  " (Windows)The default Active Directory could not be located on your domain.",
                  "20 : Error",
                  "No domain controllers with Active Directories could be found on the domain that your computer is a member of.",
                  "Active Directory support for MQ client connections cannot be used without a default Active Directory available on your domain."));
      amqMN_Ht.put("7912",new AMQMessageNumber(7912,
                  " (Windows)The Active Directory support library failed to initialize.",
                  "20 : Error",
                  "WebSphere MQ support libraries for Active Directory client connections could not be initialized.",
                  "Check that the Active Directory client pre-requisite software has been installed on your machine before attempting to use this feature."));
      amqMN_Ht.put("7913",new AMQMessageNumber(7913,
                  " (Windows)The WebSphere MQ Active Directory container could not be created.",
                  "20 : Error",
                  "WebSphere MQ has failed to create an IBM-MQClientConnections container as a child of your domain's system container in the Active Directory.",
                  "Ensure that you have permission to create sub-containers of the system container, and modify the otherWellKnownObjects property of the system container."));
      amqMN_Ht.put("7914",new AMQMessageNumber(7914,
                  " (Windows)Migration of the client connection table for Queue Manager <insert_3> failed with reason code <insert_1><insert_4>.",
                  "10 : Warning",
                  "The client connection table for this Queue Manager could not be migrated at this time.",
                  "Ensure that the client connection table exists and is not corrupted, and that you have authority to create new objects in the Active Directory on your domain."));
      amqMN_Ht.put("7915",new AMQMessageNumber(7915,
                  " (Windows)Created service connection point for connection <insert_3>.",
                  "0 : Information",
                  "The service connection point was successfully created for this client connection.",
                  "None."));
      amqMN_Ht.put("7916",new AMQMessageNumber(7916,
                  " (Windows)The Active Directory channel definition table could not be opened.",
                  "20 : Error",
                  "The IBM-MQClientConnections Active Directory container could not be located in the Global Catalog.",
                  "Ensure that setmqscp has been used to create the container object and that you have permission to read the container and its child objects."));
      amqMN_Ht.put("7917",new AMQMessageNumber(7917,
                  " (Windows)Display active directory channel details.",
                  "0 : Information",
                  "Display active directory channel details.",
                  "None."));
      amqMN_Ht.put("7918",new AMQMessageNumber(7918,
                  " (Windows)The WebSphere MQ Active Directory container could not be deleted.",
                  "20 : Error",
                  "There was a problem when attempting to delete the MQ Active Directory container. The container must be empty before it can be deleted from the directory.",
                  "None."));
      amqMN_Ht.put("7919",new AMQMessageNumber(7919,
                  " (Windows)There are no active directory client channel details to display.",
                  "0 : Information",
                  "No active directory client channel definitions could be found.",
                  "None."));
      amqMN_Ht.put("7920",new AMQMessageNumber(7920,
                  " (Windows)Usage: setmqcrl [-m QmgrName] [-a] [-d] [-r]",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("7921",new AMQMessageNumber(7921,
                  "An incorrect eye-catcher field in an MQDXP structure has been detected.",
                  "30 : Severe error",
                  "The MQDXP structure passed to the Internal Formats Conversion routine contains an incorrect eye-catcher field.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("7922",new AMQMessageNumber(7922,
                  "A PCF message is incomplete.",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message in Programmable Command Format (PCF) because the message is only <insert_1> bytes long and does not contain a PCF header. The message has either been truncated, or it contains data that is not valid.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7923",new AMQMessageNumber(7923,
                  "A message had an unrecognized integer encoding - <insert_1>.",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message because the integer encoding value of the message, <insert_1>, was not recognized.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7924",new AMQMessageNumber(7924,
                  "Bad length in the PCF header (length = <insert_1>).",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message in Programmable Command Format (PCF) because the PCF header structure contains an incorrect length field. Either the message has been truncated, or it contains data that is not valid.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7925",new AMQMessageNumber(7925,
                  "Message version <insert_1> is not supported.",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message because the Version field of the message contains an incorrect value.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7926",new AMQMessageNumber(7926,
                  "A PCF message has an incorrect parameter count value <insert_1>.",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message in Programmable Command Format (PCF) because the parameter count field of the PCF header is incorrect.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7927",new AMQMessageNumber(7927,
                  "Bad type in PCF structure number <insert_1> (type = <insert_2>).",
                  "30 : Severe error",
                  "A Programmable Command Format (PCF) structure passed to the Internal Formats Converter contained an incorrect type field.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7928",new AMQMessageNumber(7928,
                  "Bad length in PCF structure number <insert_1> (length = <insert_2>).",
                  "30 : Severe error",
                  "A Programmable Command Format (PCF) structure passed to the Internal Formats Converter contained an incorrect length field.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7929",new AMQMessageNumber(7929,
                  "A PCF structure is incomplete.",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message in Programmable Command Format (PCF) because structure number <insert_1>, of Type value <insert_2>, within the message is incomplete. The message has either been truncated, or it contains data that is not valid.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7930",new AMQMessageNumber(7930,
                  "Bad CCSID in PCF structure number <insert_1> (CCSID = <insert_2>).",
                  "30 : Severe error",
                  "A Programmable Command Format (PCF) structure passed to the Internal Formats Converter contains an incorrect CCSID.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7931",new AMQMessageNumber(7931,
                  "Bad length in PCF structure number <insert_1> (length = <insert_2>).",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message in Programmable Command Format (PCF) because one of the structures of the message contains an incorrect length field.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7932",new AMQMessageNumber(7932,
                  "Bad count in PCF structure number <insert_1> (count = <insert_2>).",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message in Programmable Command Format (PCF) because a StringList structure of the message contains an incorrect count field.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor, the headers of the message, and the incorrect structure to determine the source of the message, and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7933",new AMQMessageNumber(7933,
                  "Bad string length in PCF structure.",
                  "30 : Severe error",
                  "Message data conversion cannot convert a message in Programmable Command Format (PCF) because structure number <insert_1> of the message contains an incorrect string length value <insert_2>.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor, the headers of the message, and the incorrect structure to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7934",new AMQMessageNumber(7934,
                  "Wrong combination of MQCCSI_DEFAULT with MQCCSI_EMBEDDED or MQEPH_CCSID_EMBEDDED.",
                  "30 : Severe error",
                  "Message data conversion could not convert a message in Programmable Command Format (PCF) because structure <insert_1> of the message contained a CodedCharSetId field of MQCCSI_DEFAULT while the message itself had a CodedCharSetId of MQCCSI_EMBEDDED, or the Flags field of the MQEPH structure containing the PCF specified flag MQEPH_CCSID_EMBEDDED. These are incorrect combinations.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor, the headers of the message and the incorrect structure to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7935",new AMQMessageNumber(7935,
                  "Bad CCSID in message header (CCSID = <insert_1>).",
                  "30 : Severe error",
                  "Message data conversion could not convert a message because the Message Descriptor of the message contained an incorrect CodedCharSetId field.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Do not discard these files until the problem has been resolved. Use the file containing the Message Descriptor of the message to determine the source of the message and to see how data that is not valid became included in the message."));
      amqMN_Ht.put("7936",new AMQMessageNumber(7936,
                  "The file <insert_3> already exists.",
                  "30 : Severe error",
                  "The output file already exists, but REPLACE has not been specified.",
                  "Specify REPLACE to over-write the existing file, or select a different output file name."));
      amqMN_Ht.put("7937",new AMQMessageNumber(7937,
                  "Structure length <insert_1> in MQFMT_IMS_VAR_STRING format message is not valid.",
                  "30 : Severe error",
                  "This error is detected when attempting data conversion. The valid range for the length is 4 (with no string data) to 32767. The message is returned unconverted with a reason code of MQRC_CONVERTED_STRING_TOO_BIG.",
                  "Check the content of the message before data conversion and correct the message format. When converting data using two or more bytes per character, remember that the number of bytes in each character can change during data conversion. This causes the message lengths to change."));
      amqMN_Ht.put("7943",new AMQMessageNumber(7943,
                  "Usage: crtmqcvx SourceFile TargetFile",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("7953",new AMQMessageNumber(7953,
                  "One structure has been parsed.",
                  "0 : Information",
                  "The crtmqcvx command has parsed one structure.",
                  "None."));
      amqMN_Ht.put("7954",new AMQMessageNumber(7954,
                  "<insert_1> structures have been parsed.",
                  "0 : Information",
                  "The crtmqcvx command has parsed <insert_1> structures.",
                  "None."));
      amqMN_Ht.put("7955",new AMQMessageNumber(7955,
                  "Unexpected field: <insert_1>.",
                  "0 : Information",
                  "The field within the structure is of a type that is not recognized.",
                  "Correct the field and retry the command."));
      amqMN_Ht.put("7956",new AMQMessageNumber(7956,
                  "Bad array dimension.",
                  "0 : Information",
                  "An array field of the structure has an incorrect dimension value.",
                  "Correct the field and retry the command."));
      amqMN_Ht.put("7957",new AMQMessageNumber(7957,
                  "Warning at line <insert_1>.",
                  "20 : Error",
                  "The structure contains another field after a variable length field. A variable length field must be the last field of the structure.",
                  "Correct the structure and retry the command."));
      amqMN_Ht.put("7958",new AMQMessageNumber(7958,
                  "Error at line <insert_1> in field <insert_3>.",
                  "30 : Severe error",
                  "Field name <insert_3> is a field of type 'float'. Fields of type float are not supported by this command.",
                  "Either correct the structure to eliminate fields of type float, or write your own routine to support conversion of these fields."));
      amqMN_Ht.put("7959",new AMQMessageNumber(7959,
                  "Error at line <insert_1> in field <insert_3>.",
                  "30 : Severe error",
                  "Field name <insert_3> is a field of type 'double'. Fields of type double are not supported by this command.",
                  "Either correct the structure to eliminate fields of type double, or write your own routine to support conversion of these fields."));
      amqMN_Ht.put("7960",new AMQMessageNumber(7960,
                  "Error at line <insert_1> in field <insert_3>.",
                  "30 : Severe error",
                  "Field name <insert_3> is a 'pointer' field. Fields of type pointer are not supported by this command.",
                  "Either correct the structure to eliminate fields of type pointer, or write your own routine to support conversion of these fields."));
      amqMN_Ht.put("7961",new AMQMessageNumber(7961,
                  "Error at line <insert_1> in field <insert_3>.",
                  "30 : Severe error",
                  "Field name <insert_3> is a 'bit' field. Bit fields are not supported by this command.",
                  "Either correct the structure to eliminate bit fields, or write your own routine to support conversion of these fields."));
      amqMN_Ht.put("7962",new AMQMessageNumber(7962,
                  "No input file specified.",
                  "30 : Severe error",
                  "This command requires that an input file is specified.",
                  "Specify the name of the input file and retry the command."));
      amqMN_Ht.put("7963",new AMQMessageNumber(7963,
                  "No output file specified.",
                  "30 : Severe error",
                  "This command requires that an output file name is specified.",
                  "Specify the name of the output file and retry the command."));
      amqMN_Ht.put("7964",new AMQMessageNumber(7964,
                  "Unexpected option <insert_3>.",
                  "30 : Severe error",
                  "The option specified is not valid for this command.",
                  "Retry the command with a valid option."));
      amqMN_Ht.put("7965",new AMQMessageNumber(7965,
                  "Incorrect number of arguments.",
                  "30 : Severe error",
                  "The command was passed an incorrect number of arguments.",
                  "Retry the command, passing it the correct number of arguments."));
      amqMN_Ht.put("7968",new AMQMessageNumber(7968,
                  "Cannot open file <insert_3>.",
                  "30 : Severe error",
                  "You cannot open the file <insert_3>.",
                  "Check that you have the correct authorization to the file and retry the command."));
      amqMN_Ht.put("7969",new AMQMessageNumber(7969,
                  "Syntax error.",
                  "30 : Severe error",
                  "This line of the input file contains a language syntax error.",
                  "Correct the syntax error and retry the command."));
      amqMN_Ht.put("7970",new AMQMessageNumber(7970,
                  "Syntax error on line <insert_1>.",
                  "30 : Severe error",
                  "This message identifies where, in the input file, a previously reported error was detected.",
                  "Correct the error and retry the command."));
   }

   /**
    * Initialize the AMQ 8000's message number. 
    */
   public void init8s()
   {
      amqMN_Ht.put("8001",new AMQMessageNumber(8001,
            "WebSphere MQ queue manager created.",
            "0 : Information",
            "WebSphere MQ queue manager <insert_5> created.",
            "None."));
      amqMN_Ht.put("8002",new AMQMessageNumber(8002,
                  "WebSphere MQ queue manager <insert_5> deleted.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_5> deleted.",
                  "None."));
      amqMN_Ht.put("8003",new AMQMessageNumber(8003,
                  "WebSphere MQ queue manager <insert_5> started.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_5> started.",
                  "None."));
      amqMN_Ht.put("8004",new AMQMessageNumber(8004,
                  "WebSphere MQ queue manager <insert_5> ended.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_5> ended.",
                  "None."));
      amqMN_Ht.put("8005",new AMQMessageNumber(8005,
                  "WebSphere MQ queue manager changed.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_5> changed.",
                  "None."));
      amqMN_Ht.put("8006",new AMQMessageNumber(8006,
                  "WebSphere MQ queue created.",
                  "0 : Information",
                  "WebSphere MQ queue <insert_5> created.",
                  "None."));
      amqMN_Ht.put("8007",new AMQMessageNumber(8007,
                  "WebSphere MQ queue deleted.",
                  "0 : Information",
                  "WebSphere MQ queue <insert_5> deleted.",
                  "None."));
      amqMN_Ht.put("8008",new AMQMessageNumber(8008,
                  "WebSphere MQ queue changed.",
                  "0 : Information",
                  "WebSphere MQ queue <insert_5> changed.",
                  "None."));
      amqMN_Ht.put("8009",new AMQMessageNumber(8009,
                  "(iSeries) WebSphere MQ queue created by copying.",
                  "0 : Information",
                  "Queue <insert_5> created in library <insert_3> by copying.",
                  "None."));
      amqMN_Ht.put("8010",new AMQMessageNumber(8010,
                  "WebSphere MQ process created.",
                  "0 : Information",
                  "WebSphere MQ process <insert_5> created.",
                  "None."));
      amqMN_Ht.put("8011",new AMQMessageNumber(8011,
                  "WebSphere MQ process deleted.",
                  "0 : Information",
                  "WebSphere MQ process <insert_5> deleted.",
                  "None."));
      amqMN_Ht.put("8012",new AMQMessageNumber(8012,
                  "WebSphere MQ process changed.",
                  "0 : Information",
                  "WebSphere MQ process <insert_5> changed.",
                  "None."));
      amqMN_Ht.put("8013",new AMQMessageNumber(8013,
                  "(iSeries) WebSphere MQ process copied.",
                  "0 : Information",
                  "Process <insert_5> created in library <insert_3> by copying.",
                  "None."));
      amqMN_Ht.put("8014",new AMQMessageNumber(8014,
                  "WebSphere MQ channel created.",
                  "0 : Information",
                  "WebSphere MQ channel <insert_5> created.",
                  "None."));
      amqMN_Ht.put("8015",new AMQMessageNumber(8015,
                  "WebSphere MQ channel deleted.",
                  "0 : Information",
                  "WebSphere MQ channel <insert_5> deleted.",
                  "None."));
      amqMN_Ht.put("8016",new AMQMessageNumber(8016,
                  "WebSphere MQ channel changed.",
                  "0 : Information",
                  "WebSphere MQ channel <insert_5> changed.",
                  "None."));
      amqMN_Ht.put("8017",new AMQMessageNumber(8017,
                  "(iSeries) WebSphere MQ channel copied.",
                  "0 : Information",
                  "Channel <insert_5> created by copying.",
                  "None."));
      amqMN_Ht.put("8018",new AMQMessageNumber(8018,
                  "Start WebSphere MQ channel accepted.",
                  "0 : Information",
                  "The channel <insert_5> is being started. The start channel function has been initiated. This involves a series of operations across the network before the channel is actually started. The channel status displays \"BINDING\" for a short period while communication protocols are negotiated with the channel with whom communication is being initiated.",
                  "None."));
      amqMN_Ht.put("8019",new AMQMessageNumber(8019,
                  "Stop WebSphere MQ channel accepted.",
                  "0 : Information",
                  "The channel <insert_5> has been requested to stop.",
                  "None."));
      amqMN_Ht.put("8020",new AMQMessageNumber(8020,
                  "Ping WebSphere MQ channel complete.",
                  "0 : Information",
                  "Ping channel <insert_5> complete.",
                  "None."));
      amqMN_Ht.put("8021",new AMQMessageNumber(8021,
                  "Request to start WebSphere MQ Listener accepted.",
                  "0 : Information",
                  "The Request to start the Listener has been accepted and is being processed.",
                  "Should the request to start the listener be unsuccessful then information related to the error will be available in the queue manager error log. Once started the status of the listener may be monitored using the MQSC command 'DISPLAY LSSTATUS'. On iSeries the status of the listener may also be monitored using the 'WRKMQMLSR OPTION(*STATUS)' command."));
      amqMN_Ht.put("8022",new AMQMessageNumber(8022,
                  "WebSphere MQ queue cleared.",
                  "0 : Information",
                  "All messages on queue <insert_5> have been deleted.",
                  "None."));
      amqMN_Ht.put("8023",new AMQMessageNumber(8023,
                  "WebSphere MQ channel reset.",
                  "0 : Information",
                  "Channel <insert_5> has been reset.",
                  "None."));
      amqMN_Ht.put("8024",new AMQMessageNumber(8024,
                  "WebSphere MQ channel initiator started.",
                  "0 : Information",
                  "The channel initiator for queue <insert_5> has been started.",
                  "None."));
      amqMN_Ht.put("8025",new AMQMessageNumber(8025,
                  "WebSphere MQ channel resolved.",
                  "0 : Information",
                  "In doubt messages for WebSphere MQ channel <insert_5> have been resolved.",
                  "None."));
      amqMN_Ht.put("8026",new AMQMessageNumber(8026,
                  "End WebSphere MQ queue manager accepted.",
                  "0 : Information",
                  "A controlled stop request has been initiated for queue manager <insert_5>.",
                  "None."));
      amqMN_Ht.put("8027",new AMQMessageNumber(8027,
                  "WebSphere MQ command server started.",
                  "0 : Information",
                  "The command server has been started.",
                  "None."));
      amqMN_Ht.put("8028",new AMQMessageNumber(8028,
                  "WebSphere MQ command server ended.",
                  "0 : Information",
                  "The command server has been stopped.",
                  "None."));
      amqMN_Ht.put("8029",new AMQMessageNumber(8029,
                  "WebSphere MQ authority granted.",
                  "0 : Information",
                  "Authority for object <insert_5> granted.",
                  "None."));
      amqMN_Ht.put("8030",new AMQMessageNumber(8030,
                  "WebSphere MQ authority revoked.",
                  "0 : Information",
                  "Authority for object <insert_5> revoked.",
                  "None."));
      amqMN_Ht.put("8031",new AMQMessageNumber(8031,
                  "(iSeries) Message Queue Manager connected.",
                  "0 : Information",
                  "The message queue manager has been connected.",
                  "None."));
      amqMN_Ht.put("8032",new AMQMessageNumber(8032,
                  "(iSeries) Message Queue Manager disconnected.",
                  "0 : Information",
                  "The message queue manager has been disconnected.",
                  "None."));
      amqMN_Ht.put("8033",new AMQMessageNumber(8033,
                  "WebSphere MQ object recreated.",
                  "0 : Information",
                  "MQ object <insert_5> has been recreated from image.",
                  "None."));
      amqMN_Ht.put("8034",new AMQMessageNumber(8034,
                  "WebSphere MQ object image recorded.",
                  "0 : Information",
                  "Image of MQ object <insert_5> has been recorded.",
                  "None."));
      amqMN_Ht.put("8035",new AMQMessageNumber(8035,
                  "WebSphere MQ Command Server Status . . : Running",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8036",new AMQMessageNumber(8036,
                  "WebSphere MQ command server status . . : Stopping",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8037",new AMQMessageNumber(8037,
                  "WebSphere MQ command server status . . : Starting",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8038",new AMQMessageNumber(8038,
                  "WebSphere MQ command server status . . : Running with queue disabled",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8039",new AMQMessageNumber(8039,
                  "WebSphere MQ command server status . . : Stopped",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8040",new AMQMessageNumber(8040,
                  "WebSphere MQ command server ending.",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8041",new AMQMessageNumber(8041,
                  "The queue manager cannot be restarted or deleted because processes, that were previously connected, are still running.",
                  "40 : Stop Error",
                  "Processes, that were connected to the queue manager the last time it was running, are still active. The queue manager cannot be restarted.",
                  "Stop the processes and try to start the queue manager."));
      amqMN_Ht.put("8041",new AMQMessageNumber(8041,
                  "(iSeries) The queue manager cannot be restarted or deleted.",
                  "40 : Stop Error",
                  "Jobs that were connected to the queue manager the last time it was running, are still active. The queue manager cannot be restarted or deleted.",
                  "Use option 22 from WRKMQM to identify which jobs are connected to the queue manager. End the connected jobs and then retry the command."));
      amqMN_Ht.put("8042",new AMQMessageNumber(8042,
                  "Process <insert_1> is still running.",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8043",new AMQMessageNumber(8043,
                  "Non runtime application attempted to connect to runtime only queue manager.",
                  "0 : Information",
                  "A non runtime application attempted to connect to a queue manager on a node where support for non runtime applications has not been installed. The connect attempt will be rejected with a reason of MQRC_ENVIRONMENT_ERROR.",
                  "If the node is intended to support only runtime applications, investigate why a non runtime application has attempted to connect to the queue manager. If the node is intended to support non runtime only applications, investigate if the base option has been installed. The base option must be installed if non runtime applications are to run on this node."));
      amqMN_Ht.put("8044",new AMQMessageNumber(8044,
                  " (Windows)An error occurred while removing the queue manager from the Active Directory.",
                  "0 : Information",
                  "The attempt to remove the queue manager from the Windows Active Directory failed. This may be because the appropriate entry could not be opened or modified, or the Service Control Point has already been removed.",
                  "Check that your account has the authority to delete objects from the Active Directory, and that the entry has not already been deleted."));
      amqMN_Ht.put("8045",new AMQMessageNumber(8045,
                  " (Windows)An error occurred while removing the queue manager from the Service Control Manager.",
                  "0 : Information",
                  "The attempt to remove the queue manager from the Windows registry failed. This may be because the appropriate entry could not be opened or modified.",
                  "Check that the registry has sufficient space on your disk to grow and that your account has the authority to modify it."));
      amqMN_Ht.put("8046",new AMQMessageNumber(8046,
                  "Migrating objects for <insert_3>.",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("8047",new AMQMessageNumber(8047,
                  "Channel migration statistics : <insert_1> migrated. <insert_2> failed.",
                  "0 : Information",
                  "Information on the number of channel objects migrated from previous versions of WebSphere MQ channel definitions as well as any failures that occurred.",
                  "None."));
      amqMN_Ht.put("8048",new AMQMessageNumber(8048,
                  "Default objects statistics : <insert_1> created. <insert_2> replaced. <insert_3> failed.",
                  "0 : Information",
                  "Information on the number of objects created or replaced successfully as well as any failures that occurred while creating the default objects.",
                  "None."));
      amqMN_Ht.put("8049",new AMQMessageNumber(8049,
                  "Object <insert_4>. Unable to create or replace.",
                  "20 : Error",
                  "While creating or replacing the default object <insert_4> for WebSphere MQ queue manager <insert_5> an error occurred. The error was due to improper authorization. The reason code is <insert_1>.",
                  "Check this log for more details of what the problem may be. Make sure there are sufficient resources such as disk space and storage. For damaged or corrupted objects, replace these from backup objects. If all else fails, delete the queue manager <insert_5> using dltmqm and create it again using crtmqm."));
      amqMN_Ht.put("8050",new AMQMessageNumber(8050,
                  "Creating or replacing default objects for <insert_3>.",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("8051",new AMQMessageNumber(8051,
                  "For details of the failures that occurred, please check AMQERR01.LOG.",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("8052",new AMQMessageNumber(8052,
                  "Completing setup.",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("8053",new AMQMessageNumber(8053,
                  "Object <insert_4>. Unable to create or replace.",
                  "20 : Error",
                  "While creating or replacing the default object <insert_4> for WebSphere MQ queue manager <insert_5> an error occurred. The error was due to a broken connection. The reason code is <insert_1>.",
                  "Check this log for more details of what the problem may be. Make sure there is sufficient resources such as disk space and storage. For damaged or corrupted objects, replace these from backup objects. If all else fails, delete the queue manager <insert_5> using dltmqm and create it again using crtmqm."));
      amqMN_Ht.put("8054",new AMQMessageNumber(8054,
                  "Object <insert_4>. Unable to create or replace.",
                  "20 : Error",
                  "While creating or replacing the default object <insert_4> for WebSphere MQ queue manager <insert_5> an error occurred. The error was due to unavailable storage. The reason code is <insert_1>.",
                  "Check this log for more details of what the problem may be. Make sure there is sufficient resources such as disk space and storage. For damaged or corrupted objects, replace these from backup objects. If all else fails, delete the queue manager <insert_5> using dltmqm and create it again using crtmqm."));
      amqMN_Ht.put("8055",new AMQMessageNumber(8055,
                  "Object <insert_4>. Unable to create or replace.",
                  "20 : Error",
                  "While creating or replacing the default object <insert_4> for WebSphere MQ queue manager <insert_5> an error occurred. The error was due to a damaged object. The reason code is <insert_1>.",
                  "Check this log for more details of what the problem may be. Make sure there is sufficient resources such as disk space and storage. For damaged or corrupted objects, replace these from backup objects. If all else fails, delete the queue manager <insert_5> using dltmqm and create it again using crtmqm."));
      amqMN_Ht.put("8056",new AMQMessageNumber(8056,
                  "Object <insert_4>. Unable to create or replace.",
                  "20 : Error",
                  "While creating or replacing the default object <insert_4> for WebSphere MQ queue manager <insert_5> an error occurred. The error was due to a channel definition error. The error code is <insert_1> (X<insert_2>).",
                  "Check this log for more details of what the problem may be. Make sure there is sufficient resources such as disk space and storage. For damaged or corrupted objects, replace these from backup objects. If all else fails, delete the queue manager <insert_5> using dltmqm and create it again using crtmqm."));
      amqMN_Ht.put("8057",new AMQMessageNumber(8057,
                  "Object <insert_4>. Unable to create or replace.",
                  "20 : Error",
                  "While creating or replacing the default object <insert_4> for WebSphere MQ queue manager <insert_5> an error occurred. The error was due to invalid records in the channel definition file. The error code is <insert_1> (X<insert_2>).",
                  "Check this log for more details of what the problem may be. Make sure there is sufficient resources such as disk space and storage. For damaged or corrupted objects, replace these from backup objects. If all else fails, delete the queue manager <insert_5> using dltmqm and create it again using crtmqm."));
      amqMN_Ht.put("8058",new AMQMessageNumber(8058,
                  "Object <insert_4>. Unable to create or replace.",
                  "20 : Error",
                  "While creating or replacing the default object <insert_4> for WebSphere MQ queue manager <insert_5> an error occurred. The error was due to not finding the channel definition file. The error code is <insert_1> (X<insert_2>).",
                  "Check this log for more details of what the problem may be. Make sure there is sufficient resources such as disk space and storage. For damaged or corrupted objects, replace these from backup objects. If all else fails, delete the queue manager <insert_5> using dltmqm and create it again using crtmqm."));
      amqMN_Ht.put("8059",new AMQMessageNumber(8059,
                  "Object <insert_4>. Unable to create or replace.",
                  "20 : Error",
                  "While creating or replacing the default object <insert_4> for WebSphere MQ queue manager <insert_5> an error occurred. The error was due to an unexpected error, error code <insert_1> (X<insert_2>).",
                  "Check this log for more details of what the problem may be. Make sure there is sufficient resources such as disk space and storage. For damaged or corrupted objects, replace these from backup objects. If all else fails, delete the queue manager <insert_5> using dltmqm and create it again using crtmqm."));
      amqMN_Ht.put("8061",new AMQMessageNumber(8061,
                  " (Windows)Command <insert_4> is not valid.",
                  "10 : Warning",
                  "The command <insert_4> at line <insert_1> in the WebSphere MQ service command file <insert_3> for queue manager <insert_5> is not valid for use in the service command file. The line is ignored.",
                  "Check the contents of the file and retry the operation."));
      amqMN_Ht.put("8062",new AMQMessageNumber(8062,
                  " (Windows)Unexpected return code, <insert_1>, from command <insert_3>.",
                  "10 : Warning",
                  "An unexpected return code, <insert_1>, was returned by command <insert_3>. This command was issued by the WebSphere MQ service for queue manager <insert_4>.",
                  "Verify that the command and parameters are correct."));
      amqMN_Ht.put("8063",new AMQMessageNumber(8063,
                  " (Windows)Not authorized to issue command <insert_3>.",
                  "20 : Error",
                  "The current user <insert_5> is not authorized to issue the command <insert_3>. The command is ignored.",
                  "Add the user to the local 'mqm' security group and retry the operation."));
      amqMN_Ht.put("8064",new AMQMessageNumber(8064,
                  " (Windows)Not authorized to start trusted application.",
                  "20 : Error",
                  "The user <insert_5> is not authorized to start the trusted application <insert_3>. The application has not started.",
                  "Add the user to the local 'mqm' security group and restart the application."));
      amqMN_Ht.put("8065",new AMQMessageNumber(8065,
                  " (Windows)Local group <insert_3> not found.",
                  "20 : Error",
                  "The local group <insert_3> is unavailable. It is not possible to verify that the user is authorized. The function cannot continue.",
                  "Create the required local group and retry the operation."));
      amqMN_Ht.put("8066",new AMQMessageNumber(8066,
                  " (Windows)Local mqm group not found.",
                  "20 : Error",
                  "The local mqm group is unavailable. It is not possible to verify that the user is authorized. The function cannot continue.",
                  "Create the local mqm group and retry the operation."));
      amqMN_Ht.put("8067",new AMQMessageNumber(8067,
                  "WebSphere MQ channel auto-defined.",
                  "0 : Information",
                  "Channel <insert_5> auto-defined.",
                  "None."));
      amqMN_Ht.put("8068",new AMQMessageNumber(8068,
                  "Setup completed.",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("8069",new AMQMessageNumber(8069,
                  "ApplicationGroup for the crtmqm command does not contain the mqm userid.",
                  "40 : Stop Error",
                  "WebSphere MQ queue manager <insert_5> not created. The ApplicationGroup specified for the crtmqm command must contain the mqm userid when the RestrictedMode option (-g) is specified.",
                  "None."));
      amqMN_Ht.put("8070",new AMQMessageNumber(8070,
                  "ApplicationGroup for crtmqm command is not defined.",
                  "40 : Stop Error",
                  "WebSphere MQ queue manager <insert_5> not created. RestrictedMode option (-g) specified, but the ApplicationGroup does not exist.",
                  "None."));
      amqMN_Ht.put("8071",new AMQMessageNumber(8071,
                  "RestrictedMode option not supported on this platform.",
                  "40 : Stop Error",
                  "WebSphere MQ queue manager <insert_5> not created. The RestrictedMode option was specified but is not supported on this platform.",
                  "None."));
      amqMN_Ht.put("8072",new AMQMessageNumber(8072,
                  " (Windows)Not authorized to administer channels.",
                  "10 : Warning",
                  "The command server for queue manager <insert_3> received an administration command for channels. The user <insert_5> is not authorized to administer WebSphere MQ channels. The command server has not processed the command.",
                  "Add the user to the local 'mqm' security group, and ensure that the security policy is set as required."));
      amqMN_Ht.put("8073",new AMQMessageNumber(8073,
                  " (Windows)Authorization failed because SID: (<insert_3>) could not be resolved.",
                  "10 : Warning",
                  "The Object Authority Manager was unable to resolve the specified SID into entity and domain information.",
                  "Ensure that the application provides a SID that is recognized on this system, that all necessary domain controllers are available, and that the security policy is set as you required."));
      amqMN_Ht.put("8074",new AMQMessageNumber(8074,
                  " (Windows)Authorization failed as the SID <insert_3> does not match the entity <insert_4>.",
                  "10 : Warning",
                  "The Object Authority Manager received inconsistent data - the supplied SID does not match that of the supplied entity information.",
                  "Ensure that the application is supplying valid entity and SID information."));
      amqMN_Ht.put("8075",new AMQMessageNumber(8075,
                  " (Windows)Authorization failed because the SID for entity <insert_3> cannot be obtained.",
                  "10 : Warning",
                  "The Object Authority Manager was unable to obtain a SID for the specified entity.",
                  "Ensure that the entity is valid, and that all necessary domain controllers are available."));
      amqMN_Ht.put("8076",new AMQMessageNumber(8076,
                  " (Windows)Authorization failed because no SID was supplied for entity <insert_3>.",
                  "10 : Warning",
                  "The Object Authority Manager was not supplied with SID information for the specified entity, and the security policy is set to 'NTSIDsRequired'.",
                  "Ensure that the application is supplying a valid SID, and that the security policy is set as you require."));
      amqMN_Ht.put("8077",new AMQMessageNumber(8077,
                  " (Windows)Entity <insert_3> has insufficient authority to access object <insert_4>.",
                  "10 : Warning",
                  "The specified entity is not authorized to access the required object. The following requested permissions are unauthorized: <insert_5>",
                  "Ensure that the correct level of authority has been set for this entity against the required object, or ensure that the entity is a member of a privileged group."));
      amqMN_Ht.put("8078",new AMQMessageNumber(8078,
                  "Waiting for queue manager <insert_3> to end.",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("8079",new AMQMessageNumber(8079,
                  "(iSeries) WebSphere MQ trigger monitor job started.",
                  "0 : Information",
                  "The message queue manager trigger monitor job has been started for queue manager <insert_5> to process messages on the selected initiation queue. See previously issued messages for job details.'",
                  "None."));
      amqMN_Ht.put("8079",new AMQMessageNumber(8079,
                  " (Windows)Access was denied when attempting to retrieve group membership information for user <insert_3>.",
                  "10 : Warning",
                  "WebSphere MQ, running with the authority of user <insert_4>, was unable to retrieve group membership information for the specified user.",
                  "Ensure Active Directory access permissions allow user <insert_4> to read group memberships for user <insert_3>. To retrieve group membership information for a domain user, MQ must run with the authority of a domain user."));
      amqMN_Ht.put("8080",new AMQMessageNumber(8080,
                  "(iSeries) WebSphere MQ trigger monitor job start failed.",
                  "40 : Stop Error",
                  "Message queue manager trigger job failed to start for manager <insert_5>. Failure reason code is <insert_2>. See previously issued messages for more information.'",
                  "None."));
      amqMN_Ht.put("8081",new AMQMessageNumber(8081,
                  " (Windows)Not authorized to administer queue managers.",
                  "10 : Warning",
                  "The command server for queue manager <insert_3> received an administration command for a queue manager. The user <insert_5> is not authorized to administer WebSphere MQ queue managers. The command server has not processed the command.",
                  "Add the user to the local 'mqm' security group, and ensure that the security policy is set as required."));
      amqMN_Ht.put("8082",new AMQMessageNumber(8082,
                  " (Windows)Not authorized to administer clusters.",
                  "10 : Warning",
                  "The command server for queue manager <insert_3> received an administration command for clusters. The user <insert_5> is not authorized to administer WebSphere MQ clusters. The command server has not processed the command.",
                  "Add the user to the local 'mqm' security group, and ensure that the security policy is set as required."));
      amqMN_Ht.put("8083",new AMQMessageNumber(8083,
                  "WebSphere MQ queue manager <insert_3> starting.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_3> starting.",
                  "None."));
      amqMN_Ht.put("8084",new AMQMessageNumber(8084,
                  "WebSphere MQ connection not found.",
                  "40 : Stop Error",
                  "The connection specified does not exist.",
                  "Correct the connection name and then try the command again."));
      amqMN_Ht.put("8085",new AMQMessageNumber(8085,
                  "WebSphere MQ queue manager <insert_3> is being started for replay.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_3> is being started for replay. The strmqm command has been issued with the '-r' option. see the WebSphere MQ System Administration documentation for details.",
                  "None."));
      amqMN_Ht.put("8086",new AMQMessageNumber(8086,
                  "WebSphere MQ queue manager <insert_3> is being activated.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_3> is being activated. The strmqm command has been issued with the '-a' option. see the WebSphere MQ System Administration documentation for details.",
                  "None."));
      amqMN_Ht.put("8086",new AMQMessageNumber(8086,
                  "(iSeries) WebSphere MQ queue manager <insert_3> is being activated.",
                  "0 : Information",
                  "WebSphere MQ queue manager <insert_3> is being activated. The STRMQM command has been issued with the ACTIVATE(*YES) option. see the WebSphere MQ System Administration documentation for further details.",
                  "None."));
      amqMN_Ht.put("8087",new AMQMessageNumber(8087,
                  "Attempt to migrate listener <insert_3> to a QM object failed with <insert_1>.",
                  "20 : Error",
                  "Whilst processing legacy services, listener <insert_3> could not be migrated to an MQ object named <insert_4>, the object creation failed with <insert_1>.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8088",new AMQMessageNumber(8088,
                  "Attempt to migrate trigger monitor <insert_3> to a QM object failed with <insert_1>.",
                  "20 : Error",
                  "Whilst processing legacy services, trigger monitor <insert_3> could not be migrated to an MQ object named <insert_4>, the object creation failed with <insert_1>.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8089",new AMQMessageNumber(8089,
                  "Attempt to migrate channel service <insert_3> to a QM object failed with <insert_1>.",
                  "20 : Error",
                  "Whilst processing legacy services, channel service <insert_3> could not be migrated to an MQ object named <insert_4>, the object creation failed with <insert_1>.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8090",new AMQMessageNumber(8090,
                  "Attempt to migrate channel initiator <insert_3> to a QM object failed with <insert_1>.",
                  "20 : Error",
                  "Whilst processing legacy services, channel initiator <insert_3> could not be migrated to an MQ object named <insert_4>, the object creation failed with <insert_1>.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8091",new AMQMessageNumber(8091,
                  "Attempt to migrate custom service <insert_3> to a QM object failed with <insert_1>.",
                  "20 : Error",
                  "Whilst processing legacy services, custom service <insert_3> could not be migrated to an MQ object named <insert_4>, the object creation failed with <insert_1>.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8092",new AMQMessageNumber(8092,
                  "Service migration statistics : <insert_1> migrated. <insert_2> failed.",
                  "0 : Information",
                  "Information on the number of service objects migrated from previous versions of WebSphere MQ for Windows services as well as any failures that occurred.",
                  "None."));
      amqMN_Ht.put("8101",new AMQMessageNumber(8101,
                  "WebSphere MQ error (<insert_1>) has occurred.",
                  "40 : Stop Error",
                  "An unexpected reason code with hexadecimal value <insert_1> was received from the WebSphere MQ queue manager during command processing. (Note that hexadecimal values in the range X'07D1'-X'0BB7' correspond to MQI reason codes 2001-2999.) More information might be available in the log. If the reason code value indicates that the error was associated with a particular parameter, the parameter concerned is <insert_4>.",
                  "Correct the error and then try the command again."));
      amqMN_Ht.put("8102",new AMQMessageNumber(8102,
                  "WebSphere MQ object name specified in <insert_4> not valid.",
                  "30 : Severe error",
                  "The object name <insert_5> specified in <insert_4> is not valid. The length of the name must not exceed 48 characters, or 20 characters if it is a channel name. The name should contain the following characters only: lowercase a-z, uppercase A-Z, numeric 0-9, period (.), forward slash (/), underscore (_) and percent sign (%).",
                  "Change the length of the parameter value or change the parameter value to contain a valid combination of characters, then try the command again."));
      amqMN_Ht.put("8103",new AMQMessageNumber(8103,
                  "Insufficient storage available.",
                  "40 : Stop Error",
                  "There was insufficient storage available to perform the requested operation.",
                  "Free some storage and then try the command again."));
      amqMN_Ht.put("8104",new AMQMessageNumber(8104,
                  "WebSphere MQ directory <insert_3> not found.",
                  "40 : Stop Error",
                  "Directory <insert_3> was not found. This directory is created when WebSphere MQ is installed successfully. Refer to the log for more information.",
                  "Verify that installation of WebSphere MQ was successful. Correct the error and then try the command again."));
      amqMN_Ht.put("8105",new AMQMessageNumber(8105,
                  "Object error.",
                  "40 : Stop Error",
                  "An object error occurred. Refer to the log for more information.",
                  "Correct the error and then try the command again."));
      amqMN_Ht.put("8106",new AMQMessageNumber(8106,
                  "WebSphere MQ queue manager being created.",
                  "0 : Information",
                  "The queue manager is being created.",
                  "Wait for the creation process to complete and then try the command again."));
      amqMN_Ht.put("8107",new AMQMessageNumber(8107,
                  "WebSphere MQ queue manager running.",
                  "10 : Warning",
                  "The queue manager is running.",
                  "None."));
      amqMN_Ht.put("8108",new AMQMessageNumber(8108,
                  "WebSphere MQ queue manager <insert_3> ending.",
                  "10 : Warning",
                  "The queue manager <insert_3> is ending.",
                  "Wait for the queue manager to end and then try the command again."));
      amqMN_Ht.put("8109",new AMQMessageNumber(8109,
                  "WebSphere MQ queue manager being deleted.",
                  "40 : Stop Error",
                  "The queue manager is being deleted.",
                  "Wait for the deletion process to complete."));
      amqMN_Ht.put("8110",new AMQMessageNumber(8110,
                  "WebSphere MQ queue manager already exists.",
                  "40 : Stop Error",
                  "The queue manager <insert_5> already exists.",
                  "None."));
      amqMN_Ht.put("8111",new AMQMessageNumber(8111,
                  "(iSeries) Message Queue Manager exists under a different name.",
                  "30 : Severe error",
                  "A message queue manager exists with a name different from the value <insert_5> specified in <insert_2>.",
                  "Change the parameter value to the name of the existing message queue manager and then try the command again."));
      amqMN_Ht.put("8112",new AMQMessageNumber(8112,
                  "(iSeries) PRCNAME not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The PRCNAME parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the PRCNAME parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8113",new AMQMessageNumber(8113,
                  "(iSeries) TRGENBL not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The TRGENBL parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the TRGENBL parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8114",new AMQMessageNumber(8114,
                  "(iSeries) GETENBL not allowed with queue type *RMT.",
                  "40 : Stop Error",
                  "The GETENBL parameter may not be specified for a queue of type *RMT.",
                  "Remove the GETENBL parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8115",new AMQMessageNumber(8115,
                  "(iSeries) SHARE not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The SHARE parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the SHARE parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8116",new AMQMessageNumber(8116,
                  "(iSeries) MSGDLYSEQ not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The MSGDLYSEQ parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the MSGDLYSEQ parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8117",new AMQMessageNumber(8117,
                  "WebSphere MQ queue manager deletion incomplete.",
                  "40 : Stop Error",
                  "Deletion of queue manager <insert_5> was only partially successful. An object was not found, or could not be deleted. Refer to the log for more information.",
                  "Delete any remaining queue manager objects."));
      amqMN_Ht.put("8118",new AMQMessageNumber(8118,
                  "WebSphere MQ queue manager does not exist.",
                  "40 : Stop Error",
                  "The queue manager <insert_5> does not exist.",
                  "Either create the queue manager (crtmqm command) or correct the queue manager name used in the command and then try the command again."));
      amqMN_Ht.put("8119",new AMQMessageNumber(8119,
                  "(iSeries) TRGTYPE not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The TRGTYPE parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the TRGTYPE parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8120",new AMQMessageNumber(8120,
                  "(iSeries) TRGDEPTH not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The TRGDEPTH parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the TRGDEPTH parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8121",new AMQMessageNumber(8121,
                  "(iSeries) TRGMSGPTY not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The TRGMSGPTY parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the TRGMSGPTY parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8122",new AMQMessageNumber(8122,
                  "(iSeries) TRGDATA not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The TRGDATA parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the TRGDATA parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8123",new AMQMessageNumber(8123,
                  "(iSeries) RTNITV not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The RTNITV parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the RTNITV parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8124",new AMQMessageNumber(8124,
                  "(iSeries) MAXMSGLEN not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The MAXMSGLEN parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the MAXMSGLEN parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8125",new AMQMessageNumber(8125,
                  "(iSeries) BKTTHLD not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The BKTTHLD parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the BKTTHLD parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8126",new AMQMessageNumber(8126,
                  "(iSeries) BKTQNAME not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The BKTQNAME parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the BKTQNAME parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8127",new AMQMessageNumber(8127,
                  "(iSeries) INITQNAME not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The INITQNAME parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the INITQNAME parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8128",new AMQMessageNumber(8128,
                  "(iSeries) USAGE not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The USAGE parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the USAGE parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8129",new AMQMessageNumber(8129,
                  "(iSeries) DFNTYPE only allowed with queue type *MDL.",
                  "40 : Stop Error",
                  "The DFNTYPE parameter may only be specified for a queue of type *MDL.",
                  "Remove the DFNTYPE parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8130",new AMQMessageNumber(8130,
                  "(iSeries) TGTQNAME only allowed with queue type *ALS.",
                  "40 : Stop Error",
                  "The TGTQNAME parameter may only be specified for a queue of type *ALS.",
                  "Remove the TGTQNAME parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8131",new AMQMessageNumber(8131,
                  "(iSeries) RMTQNAME only allowed with queue type *RMT.",
                  "40 : Stop Error",
                  "The RMTQNAME parameter may only be specified for a queue of type *RMT.",
                  "Remove the RMTQNAME parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8132",new AMQMessageNumber(8132,
                  "(iSeries) RMTMQMNAME only allowed with queue type *RMT.",
                  "40 : Stop Error",
                  "The RMTMQMNAME parameter may only be specified for a queue of type *RMT.",
                  "Remove the RMTMQMNAME parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8133",new AMQMessageNumber(8133,
                  "(iSeries) TMQNAME only allowed with queue type *RMT.",
                  "40 : Stop Error",
                  "The TMQNAME parameter may only be specified for a queue of type *RMT.",
                  "Remove the TMQNAME parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8134",new AMQMessageNumber(8134,
                  "(iSeries) HDNBKTCNT not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The HDNBKTCNT parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the HDNBKTCNT parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8135",new AMQMessageNumber(8135,
                  "Not authorized.",
                  "40 : Stop Error",
                  "You are not authorized to perform the requested operation for the WebSphere MQ object <insert_5> specified in <insert_3>. Either you are not authorized to perform the requested operation, or you are not authorized to the specified MQ object. For a copy command, you may not be authorized to the specified source MQ object, or, for a create command, you may not be authorized to the system default MQ object of the specified type.",
                  "Obtain the necessary authority from your security officer or WebSphere MQ administrator. Then try the command again."));
      amqMN_Ht.put("8136",new AMQMessageNumber(8136,
                  "(iSeries) Error detected by prompt control program.",
                  "30 : Severe error",
                  "A prompt control program detected errors.",
                  "See the previously listed messages in the job log. Correct the errors and then prompt for the command again."));
      amqMN_Ht.put("8137",new AMQMessageNumber(8137,
                  "WebSphere MQ queue manager already starting.",
                  "40 : Stop Error",
                  "The strmqm command was unsuccessful because the queue manager <insert_5> is already starting.",
                  "Wait for the strmqm command to complete."));
      amqMN_Ht.put("8138",new AMQMessageNumber(8138,
                  "The WebSphere MQ queue has an incorrect type.",
                  "40 : Stop Error",
                  "The operation is not valid with queue <insert_5> because it is not a local queue.",
                  "Change the QNAME parameter to specify a queue of the correct type."));
      amqMN_Ht.put("8139",new AMQMessageNumber(8139,
                  "Already connected.",
                  "10 : Warning",
                  "A connection to the WebSphere MQ queue manager already exists.",
                  "None."));
      amqMN_Ht.put("8140",new AMQMessageNumber(8140,
                  "Resource timeout error.",
                  "40 : Stop Error",
                  "A timeout occurred in the communication between internal WebSphere MQ queue manager components. This is most likely to occur when the system is heavily loaded.",
                  "Wait until the system is less heavily loaded, then try the command again."));
      amqMN_Ht.put("8141",new AMQMessageNumber(8141,
                  "WebSphere MQ queue manager starting.",
                  "40 : Stop Error",
                  "The queue manager <insert_5> is starting.",
                  "Wait for the queue manager startup process to complete and then try the command again."));
      amqMN_Ht.put("8142",new AMQMessageNumber(8142,
                  "WebSphere MQ queue manager stopped.",
                  "40 : Stop Error",
                  "The queue manager <insert_5> is stopped.",
                  "Use the strmqm command to start the queue manager, and then try the command again."));
      amqMN_Ht.put("8143",new AMQMessageNumber(8143,
                  "WebSphere MQ queue not empty.",
                  "40 : Stop Error",
                  "The queue <insert_5> specified in <insert_2> is not empty or contains uncommitted updates.",
                  "Commit or roll back any uncommitted updates. If the command is DELETE QLOCAL, use the CLEAR QLOCAL command to clear the messages from the queue. Then try the command again."));
      amqMN_Ht.put("8144",new AMQMessageNumber(8144,
                  "Log not available.",
                  "40 : Stop Error",
                  "The WebSphere MQ logging resource is not available.",
                  "Use the dltmqm command to delete the queue manager and then the crtmqm command to create the queue manager. Then try the command again."));
      amqMN_Ht.put("8145",new AMQMessageNumber(8145,
                  "Connection broken.",
                  "40 : Stop Error",
                  "The connection to the WebSphere MQ queue manager failed during command processing. This may be caused by an endmqm command being issued by another user, or by a queue manager error.",
                  "Use the strmqm command to start the message queue manager, wait until the message queue manager has started, and try the command again."));
      amqMN_Ht.put("8146",new AMQMessageNumber(8146,
                  "WebSphere MQ queue manager not available.",
                  "40 : Stop Error",
                  "The queue manager is not available because it has been stopped or has not been created.",
                  "Use the crtmqm command to create the message queue manager, or the strmqm command to start the message queue manager as necessary. Then try the command again."));
      amqMN_Ht.put("8146",new AMQMessageNumber(8146,
                  "(iSeries) WebSphere MQ queue manager not available.",
                  "40 : Stop Error",
                  "The queue manager is not available because it has been stopped or has not been created.",
                  "Use the CRTMQM command to create the message queue manager or the STRMQM command to start the message queue manager as necessary, then retry the command. If a queue manager was not specified, ensure that a default queue manager has been created and is started using the WRKMQM command."));
      amqMN_Ht.put("8147",new AMQMessageNumber(8147,
                  "WebSphere MQ object <insert_3> not found.",
                  "40 : Stop Error",
                  "If the command entered was Change or Display, the object <insert_3> specified does not exist. If the command entered was Copy, the source object does not exist. If the command entered was Create, the system default MQ object of the specified type does not exist.",
                  "Correct the object name and then try the command again or, if you are creating a new queue or process object, either specify all parameters explicitly or ensure that the system default object of the required type exists. The system default queue names are SYSTEM.DEFAULT.LOCAL.QUEUE, SYSTEM.DEFAULT.ALIAS.QUEUE and SYSTEM.DEFAULT.REMOTE.QUEUE. The system default process name is SYSTEM.DEFAULT.PROCESS."));
      amqMN_Ht.put("8147",new AMQMessageNumber(8147,
                  "(iSeries) WebSphere MQ object <insert_5> not found.",
                  "40 : Stop Error",
                  "If the command entered was Change or Display, the MQ object <insert_5> specified does not exist. If the command entered was Copy, the source MQ object does not exist. If the command entered was Create, the system default MQ object of the specified type does not exist.",
                  "Correct the MQ object name and then try the command again or, if you are creating a new MQ queue or process object, either specify all parameters explicitly or ensure that the system default object of the required type exists. The system default queue names are SYSTEM.DEFAULT.LOCAL.QUEUE, SYSTEM.DEFAULT.ALIAS.QUEUE and SYSTEM.DEFAULT.REMOTE.QUEUE. The system default process name is SYSTEM.DEFAULT.PROCESS."));
      amqMN_Ht.put("8148",new AMQMessageNumber(8148,
                  "WebSphere MQ object in use.",
                  "40 : Stop Error",
                  "The object <insert_5> specified in <insert_3> is in use by an MQ application program.",
                  "Wait until the object is no longer in use and then try the command again. If the command is ALTER or CHANGE, specify FORCE to force the processing of the object regardless of any application program affected by the change. If the object is the dead-letter queue and the open input count is nonzero, it may be in use by an MQ channel. If the object is another queue object with a nonzero open output count, it may be in use by a MQ channel (of type RCVR or RQSTR). In either case, use the STOP CHANNEL and START CHANNEL commands to stop and restart the channel in order to solve the problem. To alter the queue USAGE the FORCE option must be used if the queue is not empty."));
      amqMN_Ht.put("8149",new AMQMessageNumber(8149,
                  "WebSphere MQ object damaged.",
                  "40 : Stop Error",
                  "The object <insert_5> specified in <insert_4> is damaged.",
                  "The object contents are not valid. Issue the DISPLAY CHANNEL, DISPLAY QUEUE, or DISPLAY PROCESS command, as required, to determine the name of the damaged object. Issue the DEFINE command, for the appropriate object type, to replace the damaged object, then try the command again."));
      amqMN_Ht.put("8150",new AMQMessageNumber(8150,
                  "WebSphere MQ object already exists.",
                  "40 : Stop Error",
                  "The object <insert_5> specified for <insert_3> could not be created because it already exists.",
                  "Check that the name is correct and try the command again specifying REPLACE, or delete the object. Then try the command again."));
      amqMN_Ht.put("8151",new AMQMessageNumber(8151,
                  "WebSphere MQ object has different type.",
                  "40 : Stop Error",
                  "The type specified for object <insert_5> is different from the type of the object being altered or defined.",
                  "Use the correct MQ command for the object type, and then try the command again."));
      amqMN_Ht.put("8152",new AMQMessageNumber(8152,
                  "Source WebSphere MQ object has different type.",
                  "40 : Stop Error",
                  "The type of the source object is different from that specified.",
                  "Correct the name of the command, or source object name, and then try the command again, or try the command using the REPLACE option."));
      amqMN_Ht.put("8153",new AMQMessageNumber(8153,
                  "Insufficient disk space for the specified queue.",
                  "40 : Stop Error",
                  "The command failed because there was insufficient disk space available for the specified queue.",
                  "Release some disk space and then try the command again."));
      amqMN_Ht.put("8154",new AMQMessageNumber(8154,
                  "API exit load error.",
                  "40 : Stop Error",
                  "The WebSphere MQ queue manager was unable to load the API crossing exit.",
                  "Ensure that the API crossing exit program is valid, and that its name and directory are correctly specified. Correct any error and then try the command again."));
      amqMN_Ht.put("8155",new AMQMessageNumber(8155,
                  "Connection limit exceeded.",
                  "40 : Stop Error",
                  "The queue manager connection limit has been exceeded.",
                  "The maximum limit on the number of WebSphere MQ application programs that may be connected to the queue manager has been exceeded. Try the command later."));
      amqMN_Ht.put("8156",new AMQMessageNumber(8156,
                  "WebSphere MQ queue manager quiescing.",
                  "40 : Stop Error",
                  "The queue manager is quiescing.",
                  "The queue manager was stopping with -c specified for endmqm. Wait until the queue manager has been restarted and then try the command again."));
      amqMN_Ht.put("8157",new AMQMessageNumber(8157,
                  "Security error.",
                  "40 : Stop Error",
                  "An error was reported by the security manager program.",
                  "Inform your systems administrator, wait until the problem has been corrected, and then try the command again."));
      amqMN_Ht.put("8158",new AMQMessageNumber(8158,
                  "(iSeries) API exit not found.",
                  "40 : Stop Error",
                  "The API crossing exit program was not found.",
                  "Ensure that the API crossing exit program for the MQI exists, and that its name and library are correctly specified. Correct any errors and then try the command again."));
      amqMN_Ht.put("8159",new AMQMessageNumber(8159,
                  "(iSeries) MAXDEPTH not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The MAXDEPTH parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the MAXDEPTH parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8160",new AMQMessageNumber(8160,
                  "(iSeries) DFTSHARE not allowed with queue type *ALS or *RMT.",
                  "40 : Stop Error",
                  "The DFTSHARE parameter may not be specified for a queue of type *ALS or *RMT.",
                  "Remove the DFTSHARE parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("8161",new AMQMessageNumber(8161,
                  "(iSeries) AUT(*MQMPASSID) only allowed with OBJTYPE(*ADM).",
                  "40 : Stop Error",
                  "AUT(*MQMPASSID) may only be specified with OBJTYPE(*ADM).",
                  "Change the AUT parameter to specify another value and then try the command again."));
      amqMN_Ht.put("8162",new AMQMessageNumber(8162,
                  "(iSeries) AUT(*MQMPASSALL) only allowed with OBJTYPE(*ADM).",
                  "40 : Stop Error",
                  "AUT(*MQMPASSALL) may only be specified with OBJTYPE(*ADM).",
                  "Change the AUT parameter to specify another value and then try the command again."));
      amqMN_Ht.put("8163",new AMQMessageNumber(8163,
                  "(iSeries) AUT(*MQMSETID) only allowed with OBJTYPE(*ADM).",
                  "40 : Stop Error",
                  "AUT(*MQMSETID) may only be specified with OBJTYPE(*ADM).",
                  "Change the AUT parameter to specify another value and then try the command again."));
      amqMN_Ht.put("8164",new AMQMessageNumber(8164,
                  "(iSeries) AUT(*MQMSETALL) only allowed with OBJTYPE(*ADM).",
                  "40 : Stop Error",
                  "AUT(*MQMSETALL) may only be specified with OBJTYPE(*ADM).",
                  "Change the AUT parameter to specify another value and then try the command again."));
      amqMN_Ht.put("8165",new AMQMessageNumber(8165,
                  "(iSeries) AUT(*MQMALTUSR) only allowed with OBJTYPE(*ADM).",
                  "40 : Stop Error",
                  "AUT(*MQMALTUSR) may only be specified with OBJTYPE(*ADM).",
                  "Change the AUT parameter to specify another value and then try the command again."));
      amqMN_Ht.put("8166",new AMQMessageNumber(8166,
                  "(iSeries) WebSphere MQ reference object not found.",
                  "40 : Stop Error",
                  "The object specified by the REFOBJ and REFOBJTYPE parameters does not exist.",
                  "Correct the reference object name and type, and then try the command again."));
      amqMN_Ht.put("8167",new AMQMessageNumber(8167,
                  "(iSeries) Referenced object name not valid.",
                  "30 : Severe error",
                  "The referenced object name specified in REFOBJ is not valid. The length of the name must not exceed 48 characters and the name should contain the following characters only: lowercase a-z, uppercase A-Z, numeric 0-9, period (.), forward slash (/), underscore (_) and percent sign (%).",
                  "Change the length of the parameter value or change the parameter value to contain a valid combination of characters. Then try the command again."));
      amqMN_Ht.put("8168",new AMQMessageNumber(8168,
                  "(iSeries) User profile name for parameter USER not found.",
                  "30 : Severe error",
                  "The user profile name specified for parameter USER could not be found on the system, and is not the special value *PUBLIC.",
                  "Correct the user profile name, or use the Create User Profile (CRTUSRPRF) command to create the user profile then try the request again."));
      amqMN_Ht.put("8169",new AMQMessageNumber(8169,
                  "(iSeries) Authorization list for parameter AUTL does not exist.",
                  "30 : Severe error",
                  "The authorization list specified for parameter AUTL does not exist. It may have been destroyed.",
                  "Either specify an authorization list that exists, or create the authorization list using the Create Authorization List (CRTAUTL) command. Try the request again."));
      amqMN_Ht.put("8170",new AMQMessageNumber(8170,
                  "(iSeries) REFOBJTYPE(*OBJTYPE) and OBJTYPE(*ALL) cannot be used together.",
                  "30 : Severe error",
                  "REFOBJTYPE(*OBJTYPE) can be specified only with a specific object type.",
                  "Change the REFOBJTYPE or OBJTYPE input value to a specific object type. Then try the Grant Authority (GRTMQMAUT) command again."));
      amqMN_Ht.put("8171",new AMQMessageNumber(8171,
                  "(iSeries) Authority of *AUTL is only allowed with USER(*PUBLIC).",
                  "30 : Severe error",
                  "AUT(*AUTL) was specified on either the Grant Authority (GRTMQMAUT) command or the Revoke Authority (RVKMQMAUT) command with the USER parameter not set to *PUBLIC. Only the authority for *PUBLIC can be deferred to the authorization list.",
                  "Change the AUT parameter to the authorities that are correct for the users or change the USER parameter to *PUBLIC. Then try the command again."));
      amqMN_Ht.put("8172",new AMQMessageNumber(8172,
                  "Already disconnected.",
                  "10 : Warning",
                  "The MQI reason code of 2018 was returned from the WebSphere MQ queue manager in response to an MQDISC request issued during command processing.",
                  "None."));
      amqMN_Ht.put("8173",new AMQMessageNumber(8173,
                  "No processes to display.",
                  "0 : Information",
                  "There are no matching processes defined on this system.",
                  "Using the DEFINE PROCESS command to create a process."));
      amqMN_Ht.put("8174",new AMQMessageNumber(8174,
                  "No queues to display.",
                  "0 : Information",
                  "There are no matching queues defined on this system.",
                  "Use the appropriate command to define a queue of the type that you require, that is, DEFINE QALIAS, DEFINE QLOCAL, DEFINE QMODEL, or DEFINE QREMOTE."));
      amqMN_Ht.put("8175",new AMQMessageNumber(8175,
                  "(iSeries) WebSphere MQ trace has started.",
                  "0 : Information",
                  "The trace has started successfully.",
                  "None."));
      amqMN_Ht.put("8176",new AMQMessageNumber(8176,
                  "(iSeries) WebSphere MQ trace has been written.",
                  "0 : Information",
                  "The trace has been written successfully.",
                  "None."));
      amqMN_Ht.put("8177",new AMQMessageNumber(8177,
                  "(iSeries) WebSphere MQ trace has stopped.",
                  "0 : Information",
                  "The trace has stopped.",
                  "None."));
      amqMN_Ht.put("8178",new AMQMessageNumber(8178,
                  "(iSeries) WebSphere MQ trace did not start.",
                  "40 : Stop Error",
                  "The trace did not start successfully.",
                  "None."));
      amqMN_Ht.put("8179",new AMQMessageNumber(8179,
                  "(iSeries) WebSphere MQ trace output error.",
                  "40 : Stop Error",
                  "The trace was not output successfully.",
                  "None."));
      amqMN_Ht.put("8180",new AMQMessageNumber(8180,
                  "(iSeries) WebSphere MQ trace end request failed.",
                  "40 : Stop Error",
                  "Your request to end the trace was not successful.",
                  "None."));
      amqMN_Ht.put("8181",new AMQMessageNumber(8181,
                  "(iSeries) No jobs to display.",
                  "10 : Warning",
                  "There are no matching jobs running on this system.",
                  "Specify another job name from the STRMQMSRV command."));
      amqMN_Ht.put("8182",new AMQMessageNumber(8182,
                  "(iSeries) WebSphere MQ trace already off.",
                  "10 : Warning",
                  "An attempt was made to set trace off, but the trace is not active.",
                  "None."));
      amqMN_Ht.put("8183",new AMQMessageNumber(8183,
                  "(iSeries) WebSphere MQ trace already running.",
                  "10 : Warning",
                  "An attempt was made to start trace, but trace is already running.",
                  "Either leave trace running as it is, or, if you want to change the trace settings, turn trace off and then turn it on again with appropriate settings."));
      amqMN_Ht.put("8184",new AMQMessageNumber(8184,
                  "(iSeries) Requested job cannot be found",
                  "10 : Warning",
                  "The job specified cannot be found in the table that controls WebSphere MQ for iSeries trace. As a result no trace action can be performed.",
                  "Specify an appropriate job name."));
      amqMN_Ht.put("8185",new AMQMessageNumber(8185,
                  "Operating system object already exists.",
                  "40 : Stop Error",
                  "The WebSphere MQ object cannot be created because an object that is not known to MQ already exists in the MQ directory with the name that should be used for the new object. Refer to the log for previous messages.",
                  "Remove the non-MQ object from the MQ library, and try the command again."));
      amqMN_Ht.put("8186",new AMQMessageNumber(8186,
                  "Image not available for WebSphere MQ object <insert_5>.",
                  "40 : Stop Error",
                  "The object <insert_5> type <insert_3> cannot be recreated because the image is not fully available in the logs that are currently online. Refer to earlier messages in the error log for information about the logs that need to be brought online for this object to be recreated.",
                  "Bring the relevant logs online, and try the command again."));
      amqMN_Ht.put("8187",new AMQMessageNumber(8187,
                  "WebSphere MQ object <insert_5> is currently open.",
                  "40 : Stop Error",
                  "The object <insert_5>, type <insert_3>, is currently in use, so the <insert_1> command cannot be issued against it. If a generic list was presented to the command, the command is still issued against the other objects in the list.",
                  "Wait until the object is no longer in use, and try the command again."));
      amqMN_Ht.put("8188",new AMQMessageNumber(8188,
                  "Insufficient authorization to WebSphere MQ object <insert_5>.",
                  "40 : Stop Error",
                  "You are not authorized to issue the <insert_1> command against the object <insert_5> type <insert_3>. If a generic list was presented to the command, the command is still issued against the other objects in the list.",
                  "Obtain sufficient authorization for the object, and retry the command."));
      amqMN_Ht.put("8189",new AMQMessageNumber(8189,
                  "WebSphere MQ object <insert_5> is damaged.",
                  "40 : Stop Error",
                  "The object <insert_5> type <insert_4> is damaged and the <insert_3> command cannot be issued against it. If a generic list was presented to the command then the command is still issued against the other objects in the list.",
                  "Issue the appropriate DEFINE command for the object, specifying REPLACE, and then try the command again."));
      amqMN_Ht.put("8190",new AMQMessageNumber(8190,
                  "<insert_3> succeeded on <insert_1> objects and failed on <insert_2> objects.",
                  "40 : Stop Error",
                  "An operation performed on a generic list of objects was not completely successful.",
                  "Examine the log for details of the errors encountered, and take appropriate action."));
      amqMN_Ht.put("8191",new AMQMessageNumber(8191,
                  "WebSphere MQ command server is starting.",
                  "40 : Stop Error",
                  "The command server is starting.",
                  "Wait for the strmqcsv command to complete and then try the operation again."));
      amqMN_Ht.put("8191",new AMQMessageNumber(8191,
                  "(iSeries) WebSphere MQ command server is starting.",
                  "40 : Stop Error",
                  "The command server is starting.",
                  "Wait for the STRMQMCSVR command to complete and then try the operation again."));
      amqMN_Ht.put("8192",new AMQMessageNumber(8192,
                  "WebSphere MQ command server already starting.",
                  "40 : Stop Error",
                  "The request to start the command server was unsuccessful because the command server is already starting.",
                  "Wait for the strmqcsv command to complete."));
      amqMN_Ht.put("8192",new AMQMessageNumber(8192,
                  "(iSeries) WebSphere MQ command server already starting.",
                  "40 : Stop Error",
                  "The request to start the command server was unsuccessful because the command server is already starting.",
                  "Wait for the STRMQMCSVR command to complete."));
      amqMN_Ht.put("8193",new AMQMessageNumber(8193,
                  "WebSphere MQ command server is ending.",
                  "40 : Stop Error",
                  "The command server is ending.",
                  "Wait for the endmqcsv command to complete and then try the command again."));
      amqMN_Ht.put("8193",new AMQMessageNumber(8193,
                  "(iSeries) WebSphere MQ command server is ending.",
                  "40 : Stop Error",
                  "The command server is ending.",
                  "Wait for the ENDMQMCSVR command to complete and then try the command again."));
      amqMN_Ht.put("8194",new AMQMessageNumber(8194,
                  "WebSphere MQ command server already ending.",
                  "40 : Stop Error",
                  "The end command server request was unsuccessful because the command server is already ending.",
                  "Wait for the endmqcsv command to complete."));
      amqMN_Ht.put("8194",new AMQMessageNumber(8194,
                  "(iSeries) WebSphere MQ command server already ending.",
                  "40 : Stop Error",
                  "The end command server request was unsuccessful because the command server is already ending.",
                  "Wait for the ENDMQMCSVR command to complete."));
      amqMN_Ht.put("8195",new AMQMessageNumber(8195,
                  "WebSphere MQ command server already running.",
                  "40 : Stop Error",
                  "The strmqcsv command was unsuccessful because the command server is already running.",
                  "None."));
      amqMN_Ht.put("8195",new AMQMessageNumber(8195,
                  "(iSeries) WebSphere MQ command server already running.",
                  "40 : Stop Error",
                  "The STRMQMCSVR command was unsuccessful because the command server is already running.",
                  "None."));
      amqMN_Ht.put("8196",new AMQMessageNumber(8196,
                  "WebSphere MQ command server already stopped.",
                  "40 : Stop Error",
                  "The request to end the command server was unsuccessful because the command server is already stopped.",
                  "None."));
      amqMN_Ht.put("8197",new AMQMessageNumber(8197,
                  "Deleted WebSphere MQ queue damaged.",
                  "20 : Error",
                  "The deleted MQ queue <insert_5> was damaged, and any messages it contained have been lost.",
                  "None."));
      amqMN_Ht.put("8198",new AMQMessageNumber(8198,
                  "(iSeries) Program <insert_3> called with incorrect number of parameters.",
                  "20 : Error",
                  "The number of parameters passed in the call to program <insert_3> is not correct.",
                  "Correct the calling program and then retry the operation."));
      amqMN_Ht.put("8199",new AMQMessageNumber(8199,
                  "(iSeries) Error in call identifier parameter passed to program QMQM.",
                  "20 : Error",
                  "The call identifier, the first parameter passed to program QMQM, is not in the required packed decimal format, or its value is not supported. Permitted values of the call identifier are contained in the RPG copy file CMQR.",
                  "Correct the calling program, and retry the call."));
      amqMN_Ht.put("8200",new AMQMessageNumber(8200,
                  "(iSeries) MODENAME only allowed with TRPTYPE(*LU62).",
                  "40 : Stop Error",
                  "The MODENAME parameter may only be specified with TRPTYPE(*LU62).",
                  "Remove the MODENAME parameter from the command or change the TRPTYPE parameter value to specify *LU62 and then try the command again."));
      amqMN_Ht.put("8201",new AMQMessageNumber(8201,
                  "(iSeries) TPGMNAME only allowed with TRPTYPE(*LU62).",
                  "40 : Stop Error",
                  "The TPGMNAME parameter may only be specified with TRPTYPE(*LU62).",
                  "Remove the TPGMNAME parameter from the command or change the TRPTYPE parameter value to specify *LU62. Then try the command again."));
      amqMN_Ht.put("8202",new AMQMessageNumber(8202,
                  "TMQNAME only allowed with channel type *SDR or *SVR.",
                  "40 : Stop Error",
                  "The TMQNAME parameter may only be specified with channel type *SDR or *SVR.",
                  "Remove the TMQNAME parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR or *SVR. Then try the command again."));
      amqMN_Ht.put("8203",new AMQMessageNumber(8203,
                  "(iSeries) CONNAME only allowed with channel type *SDR, *SVR, *RQSTR, *CLUSSDR, *CLTCN and *CLUSRCVR",
                  "40 : Stop Error",
                  "The CONNAME parameter may only be specified with channel type *SDR, *SVR, *RQSTR, *CLUSSDR, *CLTCN or *CLUSRCVR.",
                  "Remove the CONNAME parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *RQSTR, *CLUSSDR, *CLTCN or *CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8204",new AMQMessageNumber(8204,
                  "MCANAME only allowed with channel type *SDR, *SVR, or *RQSTR.",
                  "40 : Stop Error",
                  "The MCANAME parameter may only be specified with channel type *SDR, *SVR, or *RQSTR.",
                  "Remove the MCANAME parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, or *RQSTR. Then try the command again."));
      amqMN_Ht.put("8205",new AMQMessageNumber(8205,
                  "DSCITV only allowed with channel type *CLUSSDR, *CLUSRCVR, *SVRCN, *SDR or *SVR.",
                  "40 : Stop Error",
                  "The DSCITV parameter may only be specified with channel type *CLUSSDR, *CLUSRCVR, *SVRCN, *SDR or *SVR.",
                  "Remove the DSCITV parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSSDR, *CLUSRCVR, *SVRCN, *SDR or *SVR. Then try the command again."));
      amqMN_Ht.put("8206",new AMQMessageNumber(8206,
                  "SHORTRTY only allowed with channel type *CLUSSDR, CLUSRCVR, *SDR or *SVR.",
                  "40 : Stop Error",
                  "The SHORTRTY parameter may only be specified with channel type *CLUSSDR, *CLUSRCVR, *SDR or *SVR.",
                  "Remove the SHORTRTY parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSSDR, *CLUSRCVR, *SDR or *SVR. Then try the command again."));
      amqMN_Ht.put("8207",new AMQMessageNumber(8207,
                  "SHORTTMR only allowed with channel type *CLUSSDR, CLUSRCVR, *SDR or *SVR.",
                  "40 : Stop Error",
                  "The SHORTTMR parameter may only be specified with channel type *CLUSSDR, *CLUSRCVR, *SDR or *SVR.",
                  "Remove the SHORTTMR parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSSDR, CLUSRCVR, *SDR or *SVR. Then try the command again."));
      amqMN_Ht.put("8208",new AMQMessageNumber(8208,
                  "LONGRTY only allowed with channel type *CLUSSDR, *CLUSRCVR, *SDR or *SVR.",
                  "40 : Stop Error",
                  "The LONGRTY parameter may only be specified with channel type *CLUSSDR, *CLUSRCVR, *SDR or *SVR.",
                  "Remove the LONGRTY parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSSDR, CLUSRCVR, *SDR or *SVR. Then try the command again."));
      amqMN_Ht.put("8209",new AMQMessageNumber(8209,
                  "LONGTMR only allowed with channel type *CLUSSDR, *CLUSRCVR, *SDR or *SVR.",
                  "40 : Stop Error",
                  "The LONGTMR parameter may only be specified with channel type *CLUSSDR, *CLUSRCVR, *SDR or *SVR.",
                  "Remove the LONGTMR parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSSDR, *CLUSRCVR, *SDR or *SVR. Then try the command again."));
      amqMN_Ht.put("8210",new AMQMessageNumber(8210,
                  "PUTAUT only allowed with channel type *RCVR or RQSTR.",
                  "40 : Stop Error",
                  "The PUTAUT parameter may only be specified with channel type *RCVR or RQSTR.",
                  "Remove the PUTAUT parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *RCVR or RQSTR. Then try the command again."));
      amqMN_Ht.put("8211",new AMQMessageNumber(8211,
                  "BATCHINT only allowed with channel type *SDR or *SVR.",
                  "40 : Stop Error",
                  "The BATCHINT parameter may only be specified with channel type *SDR or *SVR.",
                  "Remove the BATCHINT parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR or *SVR. Then try the command again."));
      amqMN_Ht.put("8212",new AMQMessageNumber(8212,
                  "(iSeries) TPGMNAME parameter required with TRPTYPE(*LU62).",
                  "40 : Stop Error",
                  "A required parameter was not specified.",
                  "Enter a value for parameter TPGMNAME."));
      amqMN_Ht.put("8213",new AMQMessageNumber(8213,
                  "(iSeries) TMQNAME parameter required with channel type *SDR or *SVR.",
                  "40 : Stop Error",
                  "The TMQNAME parameter must be specified with channel type *SDR or *SVR.",
                  "Enter a value for parameter TMQNAME."));
      amqMN_Ht.put("8214",new AMQMessageNumber(8214,
                  "CONNAME parameter missing.",
                  "40 : Stop Error",
                  "The CONNAME parameter must be specified with channel types SDR, RQSTR, CLNTCONN, and CLUSSDR. It is also required with channel type CLUSRCVR if the TRPTYPE is not TCP.",
                  "Enter a value for parameter CONNAME."));
      amqMN_Ht.put("8214",new AMQMessageNumber(8214,
                  "(iSeries) CONNAME parameter missing.",
                  "40 : Stop Error",
                  "The CONNAME parameter must be specified with channel types *SDR, *RQSTR, and *CLUSSDR. It is also required with channel type *CLUSRCVR if the TRPTYPE is not *TCP.",
                  "Enter a value for parameter CONNAME."));
      amqMN_Ht.put("8215",new AMQMessageNumber(8215,
                  "(iSeries) CVTMSG only allowed with channel type *SDR, *SVR, *CLUSSDR or *CLUSRCVR.",
                  "40 : Stop Error",
                  "The CVTMSG parameter may only be specified with channel type *SDR, *SVR, *CLUSSDR or *CLUSRCVR.",
                  "Remove the CVTMSG parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *CLUSSDR or CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8216",new AMQMessageNumber(8216,
                  "(iSeries) MODENAME only allowed with TRPTYPE(*LU62).",
                  "40 : Stop Error",
                  "The MODENAME parameter may only be specified with TRPTYPE(*LU62).",
                  "Remove the MODENAME parameter from the command or change the TRPTYPE parameter value to specify *LU62. Then try the command again."));
      amqMN_Ht.put("8217",new AMQMessageNumber(8217,
                  "(iSeries) CONNAME only allowed with channel type *SDR, *SVR, *RQSTR, *CLUSSDR or CLUSRCVR.",
                  "40 : Stop Error",
                  "The CONNAME parameter may only be specified with channel type *SDR, *SVR, *RQSTR, CLUSSDR or CLUSRCVR.",
                  "Remove the CONNAME parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *RQSTR, CLUSSDR or CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8218",new AMQMessageNumber(8218,
                  "The system cannot accept the combination of parameters entered.",
                  "30 : Severe error",
                  "",
                  ""));
      amqMN_Ht.put("8219",new AMQMessageNumber(8219,
                  "Command server queue is open, retry later.",
                  "30 : Severe error",
                  "",
                  "Wait and try again later."));
      amqMN_Ht.put("8220",new AMQMessageNumber(8220,
                  "(iSeries) The PNGMQMCHL command has completed.",
                  "0 : Information",
                  "The PNGMQMCHL command sent <insert_1> bytes of data to <insert_3> and received the data back in <insert_4>.<insert_5> seconds. The number of bytes will be less than the amount requested on the command, when the length requested is greater than the allowed maximum, in one communications transmission, for the operating system and communications protocol.",
                  "None."));
      amqMN_Ht.put("8221",new AMQMessageNumber(8221,
                  "(iSeries) Ping data length truncated, specified length <insert_1>, actual length <insert_2>.",
                  "10 : Warning",
                  "The length of the ping data sent was reduced because of constraints in the current configuration.",
                  "None."));
      amqMN_Ht.put("8222",new AMQMessageNumber(8222,
                  "(iSeries) The data sent and received by the PNGMQMCHL command was not identical.",
                  "40 : Stop Error",
                  "Ping data compare failed at offset <insert_1>, data sent <insert_3>, data received <insert_4>.",
                  "This is probably due to a communications failure. Other messages may have been issued."));
      amqMN_Ht.put("8223",new AMQMessageNumber(8223,
                  "(iSeries) No channels to display.",
                  "0 : Information",
                  "There are no channels defined on this system.",
                  "Create a channel using the CRTMQMCHL command."));
      amqMN_Ht.put("8224",new AMQMessageNumber(8224,
                  "(iSeries) From channel <insert_3> not found.",
                  "30 : Severe error",
                  "The source WebSphere MQ channel does not exist.",
                  "Correct the MQ channel name and then try the command again."));
      amqMN_Ht.put("8225",new AMQMessageNumber(8225,
                  "(iSeries) From channel and to channel names are equal.",
                  "30 : Severe error",
                  "The same name has been specified for the from channel name and the to channel name.",
                  "Choose two different names, of which the from channel must exist."));
      amqMN_Ht.put("8226",new AMQMessageNumber(8226,
                  "WebSphere MQ channel already exists.",
                  "40 : Stop Error",
                  "The channel <insert_3> cannot be created because it already exists.",
                  "Check that the name is correct and try the command again specifying REPLACE, or delete the channel and then try the command again."));
      amqMN_Ht.put("8227",new AMQMessageNumber(8227,
                  "Channel <insert_3> not found.",
                  "30 : Severe error",
                  "The channel could not be found.",
                  "Correct the Channel Name if wrong and then try the command again. For DEFINE CHANNEL check that the Channel Name in error exists."));
      amqMN_Ht.put("8228",new AMQMessageNumber(8228,
                  "(iSeries) Unexpected return code <insert_1>.",
                  "30 : Severe error",
                  "The unexpected return code <insert_1> was returned to a channel command.",
                  "This message is associated with an internal error. Use WRKPRB to record the problem identifier, and to save the QPSRVDMP, QPJOBLOG, and QPDSPJOB files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8229",new AMQMessageNumber(8229,
                  "(iSeries) No message queue managers to display.",
                  "0 : Information",
                  "There are no message queue managers to administer.",
                  "Add a queue manager using PF6 or the ADMQMNAM command."));
      amqMN_Ht.put("8230",new AMQMessageNumber(8230,
                  "(iSeries) No queue manager objects to display.",
                  "0 : Information",
                  "Either the queue manager has no objects to display (this is unlikely), or the selection criteria resulted in zero objects to display.",
                  "Change or remove the selection criteria."));
      amqMN_Ht.put("8231",new AMQMessageNumber(8231,
                  "(iSeries) No responses to display.",
                  "0 : Information",
                  "There are no commands or command responses to display.",
                  "None."));
      amqMN_Ht.put("8232",new AMQMessageNumber(8232,
                  "(iSeries) No messages to display.",
                  "0 : Information",
                  "The queue is empty, or the queue does not exist.",
                  "None."));
      amqMN_Ht.put("8233",new AMQMessageNumber(8233,
                  "(iSeries) No message data to display.",
                  "0 : Information",
                  "The message contains no data.",
                  "None."));
      amqMN_Ht.put("8234",new AMQMessageNumber(8234,
                  "(iSeries) No response data to display.",
                  "0 : Information",
                  "There is no response data to display for this command. This is probably because the command has not yet completed.",
                  "None."));
      amqMN_Ht.put("8235",new AMQMessageNumber(8235,
                  "(iSeries) No command parameters to display.",
                  "0 : Information",
                  "Some commands have no required parameters.",
                  "None."));
      amqMN_Ht.put("8236",new AMQMessageNumber(8236,
                  "(iSeries) Channel <insert_3> not found.",
                  "30 : Severe error",
                  "CHGMQMCHL was issued for a non-existent channel.",
                  "Correct the WebSphere MQ channel name and then try the command again."));
      amqMN_Ht.put("8237",new AMQMessageNumber(8237,
                  "(iSeries) NPMSPEED only allowed with channel type *SDR, *SVR, *RCVR *RQSTR, CLUSSDR or CLUSRCVR.",
                  "40 : Stop Error",
                  "The NPMSPEED parameter may only be specified with channel type *SDR, *SVR, *RCVR *RQSTR, CLUSSDR or CLUSRCVR.",
                  "Remove the NPMSPEED parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *RCVR *RQSTR, CLUSSDR or CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8238",new AMQMessageNumber(8238,
                  "(iSeries) Queue manager connection already open.",
                  "30 : Severe error",
                  "An MQCONN call was issued, but the thread or process is already connected to a different queue manager. The thread or process can connect to only one queue manager at a time.",
                  "Use the MQDISC call to disconnect from the queue manager which is already connected, and then issue the MQCONN call to connect to the new queue manager. Disconnecting from the existing queue manager will close any queues which are currently open, it is recommended that any uncommitted units of work should be committed or backed out before the MQDISC call is used."));
      amqMN_Ht.put("8239",new AMQMessageNumber(8239,
                  "(iSeries) LOCLADDR not valid for channel type *RCVR or *SVRCN.",
                  "40 : Stop Error",
                  "The LOCLADDR parameter may only be specified with channel type *SDR, *SVR, *RQSTR, *CLUSSDR, *CLUSRCVR or *CLTCN.",
                  "Remove the CONNAME parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *RQSTR, *CLUSSDR, *CLUSRCVR or *CLTCN. Then try the command again."));
      amqMN_Ht.put("8240",new AMQMessageNumber(8240,
                  "(iSeries) Unexpected error <insert_1> in <insert_3>.",
                  "40 : Stop Error",
                  "The unexpected return code <insert_1> was returned during <insert_3> processing.",
                  "This message is associated with an internal error. Use WRKPRB to record the problem identifier, and to save the QPSRVDMP, QPJOBLOG, and QPDSPJOB files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8241",new AMQMessageNumber(8241,
                  "(iSeries) Unexpected message format <insert_3> received.",
                  "40 : Stop Error",
                  "The unexpected message format <insert_3> was received in message on the internal reply queue.",
                  "This message is probably a message sent erroneously to this queue. The message in error is written to the SYSTEM.ADMIN.EXCEPTION.QUEUE, where it may be viewed using the WRKMQMMSG command."));
      amqMN_Ht.put("8242",new AMQMessageNumber(8242,
                  "SSLCIPH definition wrong.",
                  "40 : Stop Error",
                  "The definition of the SSLCIPH parameter was wrong.",
                  "Correct the SSLCIPH definition and try the command again."));
      amqMN_Ht.put("8243",new AMQMessageNumber(8243,
                  "SSLPEER definition wrong.",
                  "40 : Stop Error",
                  "The definition of the SSLPEER parameter was wrong. Possible causes may be that the syntax was invalid or that it contained an invalid attribute type.",
                  "Correct the SSLPEER definition and try the command again."));
      amqMN_Ht.put("8266",new AMQMessageNumber(8266,
                  "(iSeries) No objects to display.",
                  "0 : Information",
                  "There are no objects with the specified name and type.",
                  "None."));
      amqMN_Ht.put("8276",new AMQMessageNumber(8276,
                  "Display Connection details.",
                  "0 : Information",
                  "The DISPLAY CONN command completed successfully. Details follow this message.",
                  ""));
      amqMN_Ht.put("8277",new AMQMessageNumber(8277,
                  "(iSeries) Object changed.",
                  "40 : Stop Error",
                  "The definition of WebSphere MQ object <insert_5> changed after it had been opened, thereby invalidating the open operation.",
                  "Try the command again."));
      amqMN_Ht.put("8278",new AMQMessageNumber(8278,
                  "(iSeries) Maximum handle limit reached.",
                  "40 : Stop Error",
                  "An attempt was made to exceed the maximum handle limit specified for the message queue manager.",
                  "Increase the maximum handle limit specified for the message queue manager using the CHGMQM command. Then try the command again."));
      amqMN_Ht.put("8279",new AMQMessageNumber(8279,
                  "(iSeries) Option not valid for type.",
                  "40 : Stop Error",
                  "The options specified when opening WebSphere MQ object <insert_5> were not valid for the object type.",
                  "Correct the definition of the failing object. Then try the command again."));
      amqMN_Ht.put("8280",new AMQMessageNumber(8280,
                  "(iSeries) Queue does not exist.",
                  "30 : Severe error",
                  "The queue being displayed does not exist on this queue manager.",
                  "Check the name of the queue and retry the operation. If you are attempting to display a queue of type *ALS, check the queue definition references an existing queue definition."));
      amqMN_Ht.put("8282",new AMQMessageNumber(8282,
                  "(iSeries) Queue manager <insert_3> is not defined on the connected queue manager.",
                  "30 : Severe error",
                  "Either the necessary queue manager name has been entered incorrectly on the add queue manager panel, or the queue manager has not been defined on the connected queue manager.",
                  "Correct the name, or define <insert_3> on the connected queue manager by creating a local queue with name <insert_3> and usage *TMQ (transmission queue), and then creating sender and receiver channels on both the connected queue manager and queue manager <insert_3>."));
      amqMN_Ht.put("8283",new AMQMessageNumber(8283,
                  "(iSeries) The administration application data store program failed to start.",
                  "40 : Stop Error",
                  "The program AMQMCPRA (data store program) was not started because of reason code <insert_1>.",
                  "Check the joblog for AMQMCPRA by issuing a WRKSPLF QMQM. Correct the problem and try to start the Administration application by invoking the STRMQMADM command. If the problem persists contact your systems programmer."));
      amqMN_Ht.put("8284",new AMQMessageNumber(8284,
                  "(iSeries) This user is not authorized to queue <insert_3>.",
                  "40 : Stop Error",
                  "Queue <insert_3> (queue manager <insert_4>) has not been authorized for your use.",
                  "Have queue <insert_3> authorized for your use. If queue manager <insert_4> is not the local queue manager, you might not be authorized to the transmission queue for this queue manager."));
      amqMN_Ht.put("8287",new AMQMessageNumber(8287,
                  "No channels with status to display.",
                  "0 : Information",
                  "There are no channels having status information to display. This indicates either, that the channel has not been started previously, or, that the channel has been started but has not yet completed a transmission sequence.",
                  "None."));
      amqMN_Ht.put("8288",new AMQMessageNumber(8288,
                  "(iSeries) Not authorized to command <insert_1>",
                  "40 : Stop Error",
                  "You are not authorized to perform the requested operation for WebSphere MQ command <insert_1>.",
                  "Obtain the necessary authority from your WebSphere MQ administrator. Then try the command again."));
      amqMN_Ht.put("8289",new AMQMessageNumber(8289,
                  "(iSeries) You are not authorized to the WebSphere MQ command.",
                  "40 : Stop Error",
                  "You are not authorized to the WebSphere MQ command because your user profile is not a member of the QMQMADM group.",
                  "Ask your MQ administrator to give your user profile *ALLOBJ authority, or add your user profile to the QMQMADM group (either as a primary or supplemental group)"));
      amqMN_Ht.put("8291",new AMQMessageNumber(8291,
                  "(iSeries) WebSphere MQ remote trace already running.",
                  "10 : Warning",
                  "An attempt was made to start remote trace, but it is already running.",
                  "Either leave remote trace running as it is, or, if you want to change the settings, turn remote trace off and then turn it on again with appropriate settings."));
      amqMN_Ht.put("8294",new AMQMessageNumber(8294,
                  "(iSeries) WebSphere MQ remote trace already off.",
                  "10 : Warning",
                  "An attempt was made to end remote trace, but it is already off.",
                  "Leave remote trace off."));
      amqMN_Ht.put("8295",new AMQMessageNumber(8295,
                  "(iSeries) WebSphere MQ object not secured by authorization list.",
                  "40 : Stop Error",
                  "The specified object is not secured by the authorization list to be revoked from it.",
                  "Use the display authority (DSPMQMAUT) command to determine what authorization list is securing the object, if any. Issue the RVKMQMAUT command again with the authorization list that is securing the the object to revoke the authorization list's authority."));
      amqMN_Ht.put("8296",new AMQMessageNumber(8296,
                  "<insert_1> MQSC commands completed successfully.",
                  "0 : Information",
                  "The <insert_3> command has completed successfully. The <insert_1> MQ commands from <insert_5> have been processed without error and a report written to the printer spool file.",
                  "None."));
      amqMN_Ht.put("8297",new AMQMessageNumber(8297,
                  "<insert_1> MQSC commands verified successfully.",
                  "0 : Information",
                  "The <insert_3> command completed successfully. The <insert_1> MQ commands from <insert_5> have been verified and a report written to the printer spool file.",
                  "None."));
      amqMN_Ht.put("8298",new AMQMessageNumber(8298,
                  "Error report generated for MQSC command process.",
                  "40 : Stop Error",
                  "The <insert_3> command attempted to process a sequence of MQ commands and encountered some errors, however, the operation may have partially completed.",
                  "If the STRMQMMQSC command was executed a report has been written to a printer spool file. Examine the spooled printer file for details of the errors encountered and correct the MQSC source <insert_5> and retry the operation."));
      amqMN_Ht.put("8299",new AMQMessageNumber(8299,
                  "Cannot open <insert_5> for MQSC process.",
                  "40 : Stop Error",
                  "The <insert_1> command failed to open <insert_5> for MQ command processing.",
                  "Check that the intended file exists, and has been specified correctly. Correct the specification or create the object, and try the operation again."));
      amqMN_Ht.put("8300",new AMQMessageNumber(8300,
                  "(iSeries) Too many exit programs/user data fields defined.",
                  "30 : Severe error",
                  "An attempt was made to create or change a channel which had more than the allowed maximum of a total of six exit programs and/or user data fields defined.",
                  "Re-define the channel so that a total of six exit programs and/or user data fields are defined."));
      amqMN_Ht.put("8301",new AMQMessageNumber(8301,
                  "(iSeries) WebSphere MQ storage monitor job could not be started.",
                  "50 : System Error",
                  "An attempt to start the storage monitor process (job QMQM in subsystem QSYSWRK) was unsuccessful.",
                  "Check the job log for the reason for the failure, and try the command again."));
      amqMN_Ht.put("8302",new AMQMessageNumber(8302,
                  "Internal failure initializing WebSphere MQ services.",
                  "50 : System Error",
                  "An error occurred while attempting to initialize WebSphere MQ services.",
                  "A call to xcsInitialize ended with the FAIL, STOP, or STOP_ALL return code. Refer to the log for messages diagnosing this problem."));
      amqMN_Ht.put("8303",new AMQMessageNumber(8303,
                  "Insufficient storage available to process request.",
                  "50 : System Error",
                  "",
                  ""));
      amqMN_Ht.put("8304",new AMQMessageNumber(8304,
                  "Tracing cannot be started. Too many traces are already running.",
                  "40 : Stop Error",
                  "A maximum of 9 traces may be running concurrently. This number is already running.",
                  "Stop one or more of the other traces and try the command again."));
      amqMN_Ht.put("8305",new AMQMessageNumber(8305,
                  "Tracing cannot be started. Too many traces are already running.",
                  "40 : Stop Error",
                  "A maximum of 9 traces can be running concurrently, and this number of traces is already running.",
                  "Stop one or more of the other traces and try the command again."));
      amqMN_Ht.put("8306",new AMQMessageNumber(8306,
                  "(iSeries) BATCHSIZE only allowed with channel type *SDR, *SVR, *RCVR, *RQSTR, CLUSSDR or CLUSRCVR.",
                  "40 : Stop Error",
                  "The BATCHSIZE parameter may only be specified with channel type *SDR, *SVR, *RCVR, *RQSTR, CLUSSDR or CLUSRCVR.",
                  "Remove the BATCHSIZE parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *RCVR *RQSTR, CLUSSDR or CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8307",new AMQMessageNumber(8307,
                  "(iSeries) SEQNUMWRAP only allowed with channel type *SDR, *SVR, *RCVR , *RQSTR, CLUSSDR or CLUSRCVR.",
                  "40 : Stop Error",
                  "The SEQNUMWRAP parameter may only be specified with channel type *SDR, *SVR, *RCVR, *RQSTR, CLUSSDR or CLUSRCVR.",
                  "Remove the SEQNUMWRAP parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *RCVR *RQSTR, CLUSSDR or CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8308",new AMQMessageNumber(8308,
                  "(iSeries) MSGRTYEXIT only allowed with channel type *CLUSRCVR, *RCVR or *RQSTR.",
                  "40 : Stop Error",
                  "The MSGRTYEXIT parameter may only be specified with channel type *CLUSRCVR, *RCVR or *RQSTR.",
                  "Remove the MSGRTYEXIT parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSRCVR, *RCVR or *RQSTR. Then try the command again."));
      amqMN_Ht.put("8309",new AMQMessageNumber(8309,
                  "(iSeries) MSGRTYDATA only allowed with channel type *CLUSRCVR, *RCVR or *RQSTR.",
                  "40 : Stop Error",
                  "The MSGRTYDATA parameter may only be specified with channel type *CLUSRCVR, *RCVR or *RQSTR.",
                  "Remove the MSGRTYDATA parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSRCVR, *RCVR or *RQSTR. Then try the command again."));
      amqMN_Ht.put("8310",new AMQMessageNumber(8310,
                  "(iSeries) MSGRTYNBR only allowed with channel type *CLUSRCVR, *RCVR or *RQSTR.",
                  "40 : Stop Error",
                  "The MSGRTYNBR parameter may only be specified with channel type *CLUSRCVR, *RCVR or *RQSTR.",
                  "Remove the MSGRTYNBR parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSRCVR, *RCVR or *RQSTR. Then try the command again."));
      amqMN_Ht.put("8311",new AMQMessageNumber(8311,
                  "(iSeries) MSGRTYITV only allowed with channel type *CLUSRCVR, *RCVR or *RQSTR.",
                  "40 : Stop Error",
                  "The MSGRTYITV parameter may only be specified with channel type *CLUSRCVR, *RCVR or *RQSTR.",
                  "Remove the MSGRTYITV parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSRCVR, *RCVR or *RQSTR. Then try the command again."));
      amqMN_Ht.put("8312",new AMQMessageNumber(8312,
                  "(iSeries) CLUSTER only allowed with queue type *ALS, *LCL and *RMT.",
                  "40 : Stop Error",
                  "The CLUSTER parameter may only be specified with queue type *ALS, *LCL and *RMT.",
                  "Remove the CLUSTER parameter from the command or, if the command is CRTMQMQ, change the QTYPE parameter value to specify *ALS, *LCL or *RMT. Then try the command again."));
      amqMN_Ht.put("8313",new AMQMessageNumber(8313,
                  "(iSeries) CLUSNL only allowed with queue type *ALS, *LCL and *RMT.",
                  "40 : Stop Error",
                  "The CLUSNL parameter may only be specified with queue type *ALS, *LCL and *RMT.",
                  "Remove the CLUSNL parameter from the command or, if the command is CRTMQMQ, change the QTYPE parameter value to specify *ALS, *LCL or *RMT. Then try the command again."));
      amqMN_Ht.put("8314",new AMQMessageNumber(8314,
                  "(iSeries) DEFBIND only allowed with queue type *ALS, *LCL and *RMT.",
                  "40 : Stop Error",
                  "The DEFBIND parameter may only be specified with queue type *ALS, *LCL and *RMT.",
                  "Remove the DEFBIND parameter from the command or, if the command is CRTMQMQ, change the QTYPE parameter value to specify *ALS, *LCL or *RMT. Then try the command again."));
      amqMN_Ht.put("8315",new AMQMessageNumber(8315,
                  "No namelists to display.",
                  "0 : Information",
                  "There are no matching namelists defined on this system.",
                  "Use the Create Namelist (CRTMQMNL) command to create a namelist."));
      amqMN_Ht.put("8316",new AMQMessageNumber(8316,
                  "No cluster queue managers to display.",
                  "0 : Information",
                  "There are no matching cluster queue managers defined on this system.",
                  "None."));
      amqMN_Ht.put("8317",new AMQMessageNumber(8317,
                  "(iSeries) CLUSTER only allowed with channel type *CLUSSDR and *CLUSRCVR.",
                  "40 : Stop Error",
                  "The CLUSTER parameter may only be specified with channel type *CLUSSDR and *CLUSRCVR.",
                  "Remove the CLUSTER parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *CLUSSDR or *CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8318",new AMQMessageNumber(8318,
                  "(iSeries) CLUSNL only allowed with channel type *CLUSSDR and *CLUSRCVR.",
                  "40 : Stop Error",
                  "The CLUSNL parameter may only be specified with channel type *CLUSSDR and *CLUSRCVR.",
                  "Remove the CLUSNL parameter from the command or, if the command is CRTMQMCHL, change the CHLQTYPE parameter value to specify *CLUSSDR or *CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8319",new AMQMessageNumber(8319,
                  "MSGEXIT only allowed with channel type *SDR, *SVR, *RCVR *RQSTR, *CLUSSDR or *CLUSRCVR.",
                  "40 : Stop Error",
                  "The MSGEXIT parameter may only be specified with channel type *SDR, *SVR, *RCVR, *RQSTR, *CLUSSDR, or *CLUSRCVR.",
                  "Remove the MSGEXIT parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR or *SVR or *RCVR or *RQSTR or *CLUSSDR or *CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8320",new AMQMessageNumber(8320,
                  "(iSeries) MSGUSRDATA only allowed with channel type *SDR, *SVR, *RCVR *RQSTR, or *CLUSSDR or *CLUSRCVR.",
                  "40 : Stop Error",
                  "The MSGUSRDATA parameter may only be specified with channel type *SDR, *SVR, *RCVR *RQSTR, *CLUSSDR or *CLUSRCVR.",
                  "Remove the MSGUSRDATA parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR or *SVR or *RCVR or *RQSTR or *CLUSSDR or *CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8321",new AMQMessageNumber(8321,
                  "(iSeries) Process <insert_3> is still running.",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8322",new AMQMessageNumber(8322,
                  "(iSeries) TIMEOUT only allowed with ENDCCTJOB(*YES).",
                  "40 : Stop Error",
                  "The TIMEOUT parameter may only be specified when connected jobs are being ended with the ENDCCTJOB option set to *YES.",
                  "Remove the TIMEOUT parameter from the command or, if you want to fully quiesce the queue manager, change the ENDCCTJOB parameter to *YES. Then try the command again."));
      amqMN_Ht.put("8323",new AMQMessageNumber(8323,
                  "(iSeries) OPTION(*PREEMPT) must not be used with ENDCCTJOB(*YES).",
                  "40 : Stop Error",
                  "When performing a pre-emptive shutdown of the queue manager the ENDCCTJOB(*YES) parameter is not allowed.",
                  "Change the ENDCCTJOB(*YES) parameter to ENDCCTJOB(*NO) or, if you want to fully quiesce the queue manager without doing a pre-emptive shutdown, change the OPTION(*PREEMPT) parameter to another value. Then try the command again."));
      amqMN_Ht.put("8324",new AMQMessageNumber(8324,
                  "(iSeries) OPTION(*WAIT) not allowed with MQMNAME(*ALL).",
                  "40 : Stop Error",
                  "The OPTION(*WAIT) parameter is not allowed when performing a shutdown of all queue managers.",
                  "Remove the OPTION(*WAIT) parameter from the command or, specify individual queue manager names to shut down the queue managers one-by-one with the OPTION(*WAIT) parameter. Then try the command again."));
      amqMN_Ht.put("8325",new AMQMessageNumber(8325,
                  "(iSeries) MQMNAME(*ALL) is not allowed with ENDCCTJOB(*NO).",
                  "40 : Stop Error",
                  "The MQMNAME(*ALL) parameter is only allowed when performing a full shutdown of the queue managers.",
                  "Specify individual queue manager names to shut the queue managers down one-by-one or change the ENDCCTJOB parameter to *YES. Then try the command again."));
      amqMN_Ht.put("8330",new AMQMessageNumber(8330,
                  "Running",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8331",new AMQMessageNumber(8331,
                  "Ended normally",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8332",new AMQMessageNumber(8332,
                  "Ended immediately",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8333",new AMQMessageNumber(8333,
                  "Ended preemptively",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8334",new AMQMessageNumber(8334,
                  "Ended unexpectedly",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8335",new AMQMessageNumber(8335,
                  "Starting",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8336",new AMQMessageNumber(8336,
                  "Quiescing",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8337",new AMQMessageNumber(8337,
                  "Ending immediately",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8338",new AMQMessageNumber(8338,
                  "Ending preemptively",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8339",new AMQMessageNumber(8339,
                  "Being deleted",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8340",new AMQMessageNumber(8340,
                  "Not available",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8341",new AMQMessageNumber(8341,
                  "SUBPOOL(<insert_3>)<insert_4>PID(<insert_1>)",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8342",new AMQMessageNumber(8342,
                  "No authorities to display.",
                  "0 : Information",
                  "There are no authority records defined on this system, satisfying the input parameters.",
                  "Use the appropriate input to list all the authorities defined on the system, or enter the command again with different input.."));
      amqMN_Ht.put("8343",new AMQMessageNumber(8343,
                  "(iSeries) The requested operation is not valid for user QMQMADM.",
                  "0 : Information",
                  "You are not allowed to completely delete the authorities assigned to user QMQMADM, for a valid WebSphere MQ object, with the authority *REMOVE or *NONE.",
                  "Remove QMQMADM from the list of users to this command."));
      amqMN_Ht.put("8344",new AMQMessageNumber(8344,
                  "(iSeries) The delete option is only valid for a generic profile name.",
                  "0 : Information",
                  "The delete option, which will delete this authority profile by removing all the users from this authority profile, is not valid for an object name or the special value &class.",
                  "To delete users from an object, work from the WRKMQMAUTD command."));
      amqMN_Ht.put("8345",new AMQMessageNumber(8345,
                  "(iSeries) BATCHHB not valid for channel type *RCVR, *RQSTR, *SVRCN or *CLTCN.",
                  "40 : Stop Error",
                  "The BATCHHB parameter may only be specified with channel type *SDR, *SVR, *CLUSSDR, or *CLUSRCVR.",
                  "Remove the BATCHHB parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *CLUSSDR or *CLUSRCVR. Then try the command again."));
      amqMN_Ht.put("8346",new AMQMessageNumber(8346,
                  "(iSeries) Parameter mismatch between QMNAME and QMID.",
                  "40 : Stop Error",
                  "The Queue Manager Name for Removal (QMNAME) parameter is not *QMID and there is a value for the Queue Manager Identifier for Removal (QMID) parameter.",
                  "A value for QMID is not allowed unless QMNAME is *QMID. Change the value specified on the QMNAME parameter or the value of the QMID parameter and then try the request again."));
      amqMN_Ht.put("8347",new AMQMessageNumber(8347,
                  "(iSeries) USERID not valid for channel type *RCVR, *SVRCN or *CLUSRCVR.",
                  "40 : Stop Error",
                  "The USERID parameter may only be specified with channel type *SDR, *SVR, *RQSTR, *CLUSSDR, or *CLTCN.",
                  "Remove the USERID parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *RQSTR, *CLUSSDR, or *CLTCN. Then try the command again."));
      amqMN_Ht.put("8348",new AMQMessageNumber(8348,
                  "(iSeries) PASSWORD not valid for channel type *RCVR, *SVRCN or *CLUSRCVR.",
                  "40 : Stop Error",
                  "The PASSWORD parameter may only be specified with channel type *SDR, *SVR, *RQSTR, *CLUSSDR, or *CLTCN.",
                  "Remove the PASSWORD parameter from the command or, if the command is CRTMQMCHL, change the CHLTYPE parameter value to specify *SDR, *SVR, *RQSTR, *CLUSSDR, or *CLTCN. Then try the command again."));
      amqMN_Ht.put("8349",new AMQMessageNumber(8349,
                  "(iSeries) Authority changes to <insert_5> failed.",
                  "40 : Stop Error",
                  "Authority changes to an object were requested but could not be made.",
                  "Check the authorities that you are granting are relevant to the object type of <insert_5>."));
      amqMN_Ht.put("8350",new AMQMessageNumber(8350,
                  "Usage: dspmqver [-p Components] [-f Fields] [-b] [-v]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("8359",new AMQMessageNumber(8359,
                  "QMNAME(<insert_3>)<insert_4>STATUS(Being deleted)",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8370",new AMQMessageNumber(8370,
                  "Usage: runmqdnm -q Queue -a Assembly",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("8371",new AMQMessageNumber(8371,
                  "<insert_3> is not a valid command line option.",
                  "40 : Stop Error",
                  "The option <insert_3> was specified on the command line to the application however this is not one of the valid set of command line options.",
                  "Check the usage information for the application and then retry."));
      amqMN_Ht.put("8372",new AMQMessageNumber(8372,
                  "The required command line option <insert_3> is missing.",
                  "40 : Stop Error",
                  "The application expects several mandatory command line options. One of these, <insert_3>, was not specified.",
                  "Check the usage information for the application and ensure that all required parameters are specified then retry."));
      amqMN_Ht.put("8373",new AMQMessageNumber(8373,
                  "Invalid value specified for command line option <insert_3> (<insert_4>).",
                  "40 : Stop Error",
                  "The value specified for command line option <insert_3> (<insert_4>) is invalid.",
                  "Check the usage information for the application and ensure that all options specify values in the valid range then retry."));
      amqMN_Ht.put("8374",new AMQMessageNumber(8374,
                  "WebSphere MQ queue manager <insert_3> does not exist.",
                  "40 : Stop Error",
                  "The WebSphere MQ queue manager <insert_3> does not exist.",
                  "Either create the queue manager (crtmqm command) or correct the queue manager name used in the command and then try the command again."));
      amqMN_Ht.put("8375",new AMQMessageNumber(8375,
                  "WebSphere MQ queue manager <insert_3> not available.",
                  "40 : Stop Error",
                  "The WebSphere MQ queue manager <insert_3> is not available because it has been stopped or is otherwise not contactable.",
                  "Use the strmqm command to start the message queue manager as necessary or correct any intermittent problems (eg. network connectivity) then try the command again."));
      amqMN_Ht.put("8376",new AMQMessageNumber(8376,
                  "WebSphere MQ queue <insert_3> not found.",
                  "40 : Stop Error",
                  "The queue <insert_3> could not be found, it may not have been created.",
                  "Ensure that the name of the queue specified is correct, queue names are case sensitive. If the queue is not created, use the runmqsc command to create it. Then try the command again."));
      amqMN_Ht.put("8377",new AMQMessageNumber(8377,
                  "Unexpected error <insert_1> was received by the application.",
                  "40 : Stop Error",
                  "The error <insert_1> was returned unexpectedly to the application.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8378",new AMQMessageNumber(8378,
                  "Unexpected exception received from .NET Framework",
                  "40 : Stop Error",
                  "The application received an exception from the underlying .NET framework, information about the exception follows: <insert_4>",
                  "Examine the information contained within the exception to determine if it is possible to resolve locally. If it is not possible to resolve the problem locally, save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8379",new AMQMessageNumber(8379,
                  "Assembly <insert_3> could not be loaded",
                  "40 : Stop Error",
                  "The WebSphere MQ .NET Monitor attempted to load assembly <insert_3> but received an exception from the underlying .NET framework indicating that it could not be found. <insert_4>",
                  "Check that the assembly does exist and is accessible to the user running the application then retry. If the assembly should be available, contact your IBM support center."));
      amqMN_Ht.put("8380",new AMQMessageNumber(8380,
                  "No classes implementing IMQObjectTrigger found in <insert_3>.",
                  "40 : Stop Error",
                  "The WebSphere MQ .NET monitor was unable to identify any classes in referenced assembly <insert_3> which implement the IMQObjectTrigger interface.",
                  "It is a requirement of the WebSphere MQ .NET monitor that either a single class implementing the IMQObjectTrigger interface exists in the referenced assembly or that a class is identified in that assembly to execute. Either modify the assembly to include a single class implementing IMQObjectTrigger or specify a class name on the command line and retry."));
      amqMN_Ht.put("8381",new AMQMessageNumber(8381,
                  "Too many classes implementing IMQObjectTrigger (<insert_1>) found in <insert_3>.",
                  "40 : Stop Error",
                  "The WebSphere MQ .NET monitor found <insert_1> classes in referenced assembly <insert_3> all of which implement the IMQObjectTrigger interface.",
                  "It is a requirement of the WebSphere MQ .NET monitor that either a single class implementing the IMQObjectTrigger interface exists in the referenced assembly or that a class is identified in that assembly to execute. Either modify the assembly to include a single class implementing IMQObjectTrigger or specify a class name on the command line and retry."));
      amqMN_Ht.put("8382",new AMQMessageNumber(8382,
                  "A Message breaking the backout threshold (<insert_1>) was moved to <insert_4>",
                  "10 : Warning",
                  "Whilst processing queue <insert_3> a message whose backout count exceeded the specified backout threshold (<insert_1>) was successfully moved to <insert_4>",
                  "The message moved to the backout queue has a backout count greater than the backout threshold specified (or picked up from the input queue BOTHRESH attribute). You should investigate the reason why this message was rolled back onto the input queue and resolve that issue. If backout processing is not required, modify the command line options and or queue definitions to achieve the required behaviour from the .NET monitor."));
      amqMN_Ht.put("8383",new AMQMessageNumber(8383,
                  "A Message breaking the backout threshold (<insert_1>) could not be moved.",
                  "40 : Stop Error",
                  "While processing queue <insert_3> a message whose backout count exceeded the specified backout threshold (<insert_1>) was encountered however, it was not possible to move it to either a backout queue or the dead-letter queue.",
                  "Because it was not possible to move the backed out message to another queue, it has been left on the input queue. As a result, the .NET monitor has ended. It is possible that the backout queue or dead-letter queue are full or disabled for put - in this case, resolve this problem first. If backout processing should have resulted in the message being placed on another queue, check the command line options, input queue definition and queue manager dead-letter queue attribute to ensure that they are correct, then retry."));
      amqMN_Ht.put("8390",new AMQMessageNumber(8390,
                  "Usage: endmqdnm -q Queue [-m QueueManager]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("8391",new AMQMessageNumber(8391,
                  "<insert_3> is not a valid command line option.",
                  "40 : Stop Error",
                  "The option <insert_3> was specified on the command line to the application however this is not one of the valid set of command line options.",
                  "Check the usage information for the application and then retry."));
      amqMN_Ht.put("8392",new AMQMessageNumber(8392,
                  "The required command line option <insert_3> is missing.",
                  "40 : Stop Error",
                  "The application expects mandatory command line options. One of these, <insert_3>, was not specified.",
                  "Check the usage information for the application and ensure that all required parameters are specified then retry."));
      amqMN_Ht.put("8393",new AMQMessageNumber(8393,
                  "Invalid value specified for command line option <insert_3> (<insert_4>).",
                  "40 : Stop Error",
                  "The value specified for command line option <insert_3> (<insert_4>) is invalid.",
                  "Check the usage information for the application and ensure that all options specify values in the valid range then retry."));
      amqMN_Ht.put("8394",new AMQMessageNumber(8394,
                  "WebSphere MQ queue manager <insert_3> does not exist.",
                  "40 : Stop Error",
                  "The WebSphere MQ queue manager <insert_3> does not exist.",
                  "Either create the queue manager (crtmqm command) or correct the queue manager name used in the command and then try the command again."));
      amqMN_Ht.put("8395",new AMQMessageNumber(8395,
                  "WebSphere MQ queue manager <insert_3> not available.",
                  "40 : Stop Error",
                  "The WebSphere MQ queue manager <insert_3> is not available because it has been stopped or is otherwise not contactable.",
                  "Use the strmqm command to start the message queue manager as necessary or correct any intermittent problems (eg. network connectivity) then try the command again."));
      amqMN_Ht.put("8396",new AMQMessageNumber(8396,
                  "WebSphere MQ queue <insert_3> not found.",
                  "40 : Stop Error",
                  "The queue <insert_3> could not be found, it may not have been created.",
                  "Ensure that the name of the queue specified is correct, queue names are case sensitive. If the queue is not created, use the runmqsc command to create it. Then try the command again."));
      amqMN_Ht.put("8397",new AMQMessageNumber(8397,
                  "Unexpected error <insert_1> was received by the application.",
                  "40 : Stop Error",
                  "The error <insert_1> was returned unexpectedly to the application.",
                  "Save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8398",new AMQMessageNumber(8398,
                  "Unexpected exception received from .NET Framework",
                  "40 : Stop Error",
                  "The application received an exception from the underlying .NET framework, information about the exception follows: <insert_4>",
                  "Examine the information contained within the exception to determine if it is possible to resolve locally. If it is not possible to resolve the problem locally, save the generated output files and contact your IBM support center."));
      amqMN_Ht.put("8401",new AMQMessageNumber(8401,
                  "<insert_1> MQSC commands read.",
                  "0 : Information",
                  "The MQSC script contains <insert_1> commands.",
                  "None."));
      amqMN_Ht.put("8402",new AMQMessageNumber(8402,
                  "<insert_1> commands have a syntax error.",
                  "0 : Information",
                  "The MQSC script contains <insert_1> commands having a syntax error.",
                  "None."));
      amqMN_Ht.put("8403",new AMQMessageNumber(8403,
                  "<insert_1> valid MQSC commands could not be processed.",
                  "0 : Information",
                  "The MQSC script contains <insert_1> commands that failed to process.",
                  "None."));
      amqMN_Ht.put("8404",new AMQMessageNumber(8404,
                  "Command failed.",
                  "0 : Information",
                  "An MQSC command has been recognized, but cannot be processed.",
                  "None."));
      amqMN_Ht.put("8405",new AMQMessageNumber(8405,
                  "Syntax error detected at or near end of command segment below:-",
                  "0 : Information",
                  "The MQSC script contains <insert_1> commands having a syntax error.",
                  "None."));
      amqMN_Ht.put("8406",new AMQMessageNumber(8406,
                  "Unexpected 'end of input' in MQSC.",
                  "0 : Information",
                  "An MQSC command contains a continuation character, but the 'end of input' has been reached without completing the command.",
                  "None."));
      amqMN_Ht.put("8407",new AMQMessageNumber(8407,
                  "Display Process details.",
                  "0 : Information",
                  "The MQSC DISPLAY PROCESS command completed successfully, and details follow this message.",
                  "None."));
      amqMN_Ht.put("8408",new AMQMessageNumber(8408,
                  "Display Queue Manager details.",
                  "0 : Information",
                  "The MQSC DISPLAY QMGR command completed successfully, and details follow this message.",
                  "None."));
      amqMN_Ht.put("8409",new AMQMessageNumber(8409,
                  "Display Queue details.",
                  "0 : Information",
                  "The MQSC DISPLAY QUEUE command completed successfully, and details follow this message.",
                  "None."));
      amqMN_Ht.put("8410",new AMQMessageNumber(8410,
                  "Parser Error.",
                  "0 : Information",
                  "The MQSC Parser has an internal error.",
                  "None."));
      amqMN_Ht.put("8411",new AMQMessageNumber(8411,
                  "Duplicate Keyword Error.",
                  "0 : Information",
                  "A command in the MQSC script contains duplicate keywords.",
                  "None."));
      amqMN_Ht.put("8412",new AMQMessageNumber(8412,
                  "Numeric Range Error.",
                  "0 : Information",
                  "The value assigned to an MQSC command keyword is out of the permitted range.",
                  "None."));
      amqMN_Ht.put("8413",new AMQMessageNumber(8413,
                  "String Length Error.",
                  "0 : Information",
                  "A string assigned to an MQSC keyword is either NULL, or longer than the maximum permitted for that keyword.",
                  "None."));
      amqMN_Ht.put("8414",new AMQMessageNumber(8414,
                  "Display Channel details.",
                  "0 : Information",
                  "The MQSC DISPLAY CHL command completed successfully, and details follow this message.",
                  "None."));
      amqMN_Ht.put("8415",new AMQMessageNumber(8415,
                  "Ping WebSphere MQ Queue Manager command complete.",
                  "0 : Information",
                  "The MQSC PING QMGR command completed successfully.",
                  "None."));
      amqMN_Ht.put("8416",new AMQMessageNumber(8416,
                  "MQSC timed out waiting for a response from the command server.",
                  "0 : Information",
                  "MQSC did not receive a response message from the remote command server in the time specified.",
                  "None."));
      amqMN_Ht.put("8417",new AMQMessageNumber(8417,
                  "Display Channel Status details.",
                  "0 : Information",
                  "The MQSC DISPLAY CHANNEL STATUS command completed successfully, and details follow this message.",
                  "None."));
      amqMN_Ht.put("8418",new AMQMessageNumber(8418,
                  "<insert_1> command responses received.",
                  "0 : Information",
                  "Running in queued mode, <insert_1> command responses were received from the remote command server.",
                  "None."));
      amqMN_Ht.put("8419",new AMQMessageNumber(8419,
                  "The Queue is already in the DCE cell.",
                  "0 : Information",
                  "The Queue is already in the cell, that is, its SCOPE attribute is already CELL.",
                  "None."));
      amqMN_Ht.put("8420",new AMQMessageNumber(8420,
                  "Channel Status not found.",
                  "0 : Information",
                  "No status was found for the specified channel(s).",
                  "None."));
      amqMN_Ht.put("8421",new AMQMessageNumber(8421,
                  "A required keyword was not specified.",
                  "0 : Information",
                  "A keyword required in this command was not specified.",
                  "None."));
      amqMN_Ht.put("8422",new AMQMessageNumber(8422,
                  "MQSC found the following response to a previous command on the reply q :-",
                  "0 : Information",
                  "MQSC found additional command responses on the reply q. They will fill follow this message.",
                  "None."));
      amqMN_Ht.put("8423",new AMQMessageNumber(8423,
                  "Cell Directory not available.",
                  "0 : Information",
                  "The DCE cell directory is not available, so the requested operation has failed.",
                  "None."));
      amqMN_Ht.put("8424",new AMQMessageNumber(8424,
                  "Error detected in a name keyword.",
                  "0 : Information",
                  "A keyword in an MQSC command contained a name string which was not valid. This may be because it contained characters which are not accepted in MQ names. Typical keywords which can produce this error are QLOCAL (and the other q types), CHANNEL, XMITQ, INITQ, MCANAME etc.",
                  "None."));
      amqMN_Ht.put("8425",new AMQMessageNumber(8425,
                  "Attribute value error.",
                  "0 : Information",
                  "A keyword in an MQSC command contained a value that was not valid.",
                  "None."));
      amqMN_Ht.put("8426",new AMQMessageNumber(8426,
                  "Valid MQSC commands are:",
                  "0 : Information",
                  "The text shows valid MQSC commands.",
                  "None."));
      amqMN_Ht.put("8427",new AMQMessageNumber(8427,
                  "Valid syntax for the MQSC command:",
                  "0 : Information",
                  "The text shown is the valid syntax for the MQSC command.",
                  "None."));
      amqMN_Ht.put("8428",new AMQMessageNumber(8428,
                  "TYPE Keyword has already been specified.",
                  "0 : Information",
                  "The TYPE has already been specified after the DISPLAY verb, for example DISPLAY QUEUE(*) type(QLOCAL) type(QALIAS).",
                  "Delete the second TYPE keyword and run the command again."));
      amqMN_Ht.put("8429",new AMQMessageNumber(8429,
                  "(iSeries) Error detected in a exit parameter.",
                  "0 : Information",
                  "A syntax error occurred an the exit parameter. This may be because it contained characters which are not accepted as exit names. Check the parameters in the MSGEXIT, RCVEXIT, SCYEXIT and SENDEXIT definitions.",
                  "None."));
      amqMN_Ht.put("8430",new AMQMessageNumber(8430,
                  "Remote queue manager name is unknown.",
                  "0 : Information",
                  "The Remote queue manager name is not known to this queue manager. Check that a transmission queue of the same name as the remote queue manager name exists.",
                  "Create a transmission queue of the same name as the remote queue manager if one does not exist."));
      amqMN_Ht.put("8431",new AMQMessageNumber(8431,
                  "Transmission queue does not exist",
                  "0 : Information",
                  "The transmission queue does not exist on this queue manager.",
                  "None."));
      amqMN_Ht.put("8432",new AMQMessageNumber(8432,
                  "You are not allowed to set both the REPOS and REPOSNL fields.",
                  "0 : Information",
                  "An attempt to set both the REPOS and REPOSNL fields has been made. Only one of these fields can have a value other than blank. Both of the fields may be blank.",
                  "None."));
      amqMN_Ht.put("8433",new AMQMessageNumber(8433,
                  "You are not allowed to set both the CLUSTER and CLUSNL fields.",
                  "0 : Information",
                  "An attempt to set both the CLUSTER and CLUSNL fields has been made. Only one of these fields can have a value other than blank. Both of the fields may be blank.",
                  "None."));
      amqMN_Ht.put("8434",new AMQMessageNumber(8434,
                  "The repository is unavailable.",
                  "0 : Information",
                  "The repository is unavailable and the data cannot be accessed. Stop and restart the queue manager.",
                  "None."));
      amqMN_Ht.put("8435",new AMQMessageNumber(8435,
                  "All valid MQSC commands were processed.",
                  "0 : Information",
                  "The MQSC script contains no commands that failed to process.",
                  "None."));
      amqMN_Ht.put("8436",new AMQMessageNumber(8436,
                  "One valid MQSC command could not be processed.",
                  "0 : Information",
                  "The MQSC script contains one command that failed to process.",
                  "None."));
      amqMN_Ht.put("8437",new AMQMessageNumber(8437,
                  "No MQSC commands read.",
                  "0 : Information",
                  "The MQSC script contains no commands.",
                  "None."));
      amqMN_Ht.put("8438",new AMQMessageNumber(8438,
                  "One MQSC command read.",
                  "0 : Information",
                  "The MQSC script contains one command.",
                  "None."));
      amqMN_Ht.put("8439",new AMQMessageNumber(8439,
                  "No commands have a syntax error.",
                  "0 : Information",
                  "The MQSC script contains no commands having a syntax error.",
                  "None."));
      amqMN_Ht.put("8440",new AMQMessageNumber(8440,
                  "One command has a syntax error.",
                  "0 : Information",
                  "The MQSC script contains one command which has a syntax error.",
                  "None."));
      amqMN_Ht.put("8441",new AMQMessageNumber(8441,
                  "Display Cluster Queue Manager details.",
                  "0 : Information",
                  "The MQSC DISPLAY CLUSQMG command completed successfully, and details follow this message.",
                  "None."));
      amqMN_Ht.put("8442",new AMQMessageNumber(8442,
                  "USAGE can not be set to XMITQ with either the CLUSTER or CLUSNL fields set.",
                  "0 : Information",
                  "An attempt has been made to set USAGE to XMITQ when the CLUSTER or CLUSNL field has a value. Change the value of USAGE, or set the CLUSTER and CLUSNL fields to blank, and try the command again.",
                  "None."));
      amqMN_Ht.put("8442",new AMQMessageNumber(8442,
                  "(iSeries) USAGE can not be set to *TMQ with either the CLUSTER or CLUSNL fields set.",
                  "0 : Information",
                  "An attempt has been made to set USAGE to *TMQ when the CLUSTER or CLUSNL field has a value. Change the value of USAGE, or set the CLUSTER and CLUSNL fields to blank, and try the command again.",
                  "None."));
      amqMN_Ht.put("8443",new AMQMessageNumber(8443,
                  "Only the CLUSTER or CLUSNL field may have a value.",
                  "0 : Information",
                  "An attempt has been made to set both CLUSTER and CLUSNL fields. One and only one of the fields may have a value, the other field must be blank. Change the value of one of the fields to blank and try the command again.",
                  "None."));
      amqMN_Ht.put("8444",new AMQMessageNumber(8444,
                  "The CLUSTER or CLUSNL fields must have a value.",
                  "0 : Information",
                  "Both the CLUSTER and CLUSNL fields are blank. One and only one of the fields may be blank, the other field must be a value. Change one of the fields from blank to a value and try the command again.",
                  "None."));
      amqMN_Ht.put("8445",new AMQMessageNumber(8445,
                  "Program cannot open queue manager object.",
                  "30 : Severe error",
                  "An attempt to open a queue manager object has failed.",
                  "See the previously listed messages in the job log."));
      amqMN_Ht.put("8446",new AMQMessageNumber(8446,
                  "Channel is currently active.",
                  "30 : Severe error",
                  "The requested operation failed because the channel is currently active.",
                  "See the previously listed messages in the job log."));
      amqMN_Ht.put("8447",new AMQMessageNumber(8447,
                  "Requested operation on channel <insert_5> not valid for this channel type.",
                  "30 : Severe error",
                  "The operation requested cannot be performed because channel <insert_5> is not of a suitable type. For example, only sender, server and cluster-sender channels can be resolved.",
                  "Check that the correct operation was requested. If it was, check that the correct channel name was specified."));
      amqMN_Ht.put("8448",new AMQMessageNumber(8448,
                  "Channel <insert_5> is not running.",
                  "30 : Severe error",
                  "A request to end channel <insert_5> has failed because the channel is not running.",
                  "Check that the correct operation was requested. If it was, check that the correct channel name was specified."));
      amqMN_Ht.put("8449",new AMQMessageNumber(8449,
                  "Queue <insert_5> inhibited for MQGET.",
                  "30 : Severe error",
                  "An MQGET failed because the queue <insert_5> had been previously inhibited for MQGET.",
                  "None."));
      amqMN_Ht.put("8450",new AMQMessageNumber(8450,
                  "Display queue status details.",
                  "0 : Information",
                  "The MQSC DISPLAY QSTATUS command completed successfully. Details follow this message.",
                  ""));
      amqMN_Ht.put("8451",new AMQMessageNumber(8451,
                  "(iSeries) STATUS(*STOPPED) not allowed with CONNAME specified.",
                  "0 : Information",
                  "The STATUS(*STOPPED) parameter is not allowed when specifying CONNAME on the ENDMQMCHL command.",
                  "Remove the CONNAME parameter from the command or, specify STATUS(*INACTIVE) to end the channel instance for the specified connection name."));
      amqMN_Ht.put("8452",new AMQMessageNumber(8452,
                  "(iSeries) STATUS(*STOPPED) not allowed with RQMNAME specified.",
                  "0 : Information",
                  "The STATUS(*STOPPED) parameter is not allowed when specifying RQMNAME on the ENDMQMCHL command.",
                  "Remove the RQMNAME parameter from the command or, specify STATUS(*INACTIVE) to end the channel instance for the specified remote queue manager."));
      amqMN_Ht.put("8453",new AMQMessageNumber(8453,
                  "The path <insert_3> is invalid",
                  "20 : Error",
                  "You typed a path which was not syntactically correct for the operating system you are running WebSphere MQ on.",
                  "Determine the correct syntax of a path name for the operating system you are running WebSphere MQ on and use this information to type in a valid path."));
      amqMN_Ht.put("8454",new AMQMessageNumber(8454,
                  "Syntax error found in parameter <insert_3>.",
                  "20 : Error",
                  "The data you entered for <insert_3> does not conform to the syntax rules laid down by WebSphere MQ for this parameter.",
                  "Carefully check the data entered for this parameter in conjunction with the WebSphere MQ Command Reference to determine the cause of error."));
      amqMN_Ht.put("8455",new AMQMessageNumber(8455,
                  "Password length error",
                  "20 : Error",
                  "The password string length is rounded up by WebSphere MQ to the nearest eight bytes. This rounding causes the total length of the SSLCRYP string to exceed its maximum.",
                  "Decrease the size of the password, or of earlier fields in the SSLCRYP string."));
      amqMN_Ht.put("8456",new AMQMessageNumber(8456,
                  "Conflicting parameters in command.",
                  "20 : Error",
                  "The command contains parameters that cannot be used together.",
                  "Refer to the WebSphere MQ Script (MQSC) Command Reference to determine an allowable combination of parameters for this command."));
      amqMN_Ht.put("8457",new AMQMessageNumber(8457,
                  "WebSphere MQ connection stopped.",
                  "0 : Information",
                  "The STOP CONN command successfully stopped the connection that was specified.",
                  "None."));
      amqMN_Ht.put("8458",new AMQMessageNumber(8458,
                  "WebSphere MQ connection not stopped.",
                  "0 : Information",
                  "The STOP CONN command could not stop the connection that was specified.",
                  "None."));
      amqMN_Ht.put("8459",new AMQMessageNumber(8459,
                  "Not Found.",
                  "0 : Information",
                  "You specified an identifier that was not found. Please try the command again and supply a valid identifier.",
                  "None."));
      amqMN_Ht.put("8460",new AMQMessageNumber(8460,
                  "Syntax error in connection identifier.",
                  "0 : Information",
                  "You specified an invalid connection identifier. A valid connection identifier contains 16 hex characters, where all of the characters in the connection identifier should lie within the range 0-9, a-z or A-Z.",
                  "Correct the connection identifier so that it conforms to the above specification."));
      amqMN_Ht.put("8461",new AMQMessageNumber(8461,
                  "Connection identifier not found.",
                  "0 : Information",
                  "You specified a connection identifier which is not associated with this queue manager.",
                  "Correct the connection identifier so that it describes a connection identifier which is associated with this queue manager. Use the command DISPLAY CONN to identify potential connection identifiers to use with this command."));
      amqMN_Ht.put("8498",new AMQMessageNumber(8498,
                  "Starting MQSC for queue manager <insert_3>.",
                  "0 : Information",
                  "The MQSC script contains <insert_1> commands.",
                  "None."));
      amqMN_Ht.put("8499",new AMQMessageNumber(8499,
                  "Usage: runmqsc [-e] [-v] [-w WaitTime [-x]] QMgrName",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8500",new AMQMessageNumber(8500,
                  "WebSphere MQ Display MQ Files",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8501",new AMQMessageNumber(8501,
                  "Common services initialization failed with return code <insert_1>.",
                  "20 : Error",
                  "A request by the command server to initialize common services failed with return code <insert_1>.",
                  "None."));
      amqMN_Ht.put("8502",new AMQMessageNumber(8502,
                  "Connect shared memory failed with return code <insert_1>.",
                  "20 : Error",
                  "A request by the command server to connect shared memory failed with return code <insert_1>.",
                  "None."));
      amqMN_Ht.put("8503",new AMQMessageNumber(8503,
                  "Post event semaphore failed with return code <insert_1>.",
                  "20 : Error",
                  "A request by the command server to post an event semaphore failed with return code <insert_1>.",
                  "None."));
      amqMN_Ht.put("8504",new AMQMessageNumber(8504,
                  "Command server MQINQ failed with reason code <insert_1>.",
                  "20 : Error",
                  "An MQINQ request by the command server, for the WebSphere MQ queue <insert_3>, failed with reason code <insert_1>.",
                  "None."));
      amqMN_Ht.put("8505",new AMQMessageNumber(8505,
                  "Reallocate memory failed with return code <insert_1>.",
                  "20 : Error",
                  "A request by the command server to reallocate memory failed with return code <insert_1>.",
                  "None."));
      amqMN_Ht.put("8506",new AMQMessageNumber(8506,
                  "Command server MQGET failed with reason code <insert_1>.",
                  "20 : Error",
                  "An MQGET request by the command server, for the WebSphere MQ queue <insert_3>, failed with reason code <insert_1>.",
                  "None."));
      amqMN_Ht.put("8507",new AMQMessageNumber(8507,
                  "Command server MQPUT1 request for an undelivered message failed with reason code <insert_1>.",
                  "20 : Error",
                  "An attempt by the command server to put a message to the dead-letter queue, using MQPUT1, failed with reason code <insert_1>. The MQDLH reason code was <insert_2>.",
                  "None."));
      amqMN_Ht.put("8508",new AMQMessageNumber(8508,
                  "Queue Manager Delete Object List failed with return code <insert_1>.",
                  "20 : Error",
                  "A request by the command server to delete a queue manager object list failed with return code <insert_1>.",
                  "None."));
      amqMN_Ht.put("8509",new AMQMessageNumber(8509,
                  "Command server MQCLOSE reply-to queue failed with reason code <insert_1>.",
                  "20 : Error",
                  "An MQCLOSE request by the command server for the reply-to queue failed with reason code <insert_1>.",
                  "None."));
      amqMN_Ht.put("8510",new AMQMessageNumber(8510,
                  "Command server queue is open, try again later.",
                  "30 : Severe error",
                  "",
                  ""));
      amqMN_Ht.put("8511",new AMQMessageNumber(8511,
                  "Usage: strmqcsv [QMgrName]",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8512",new AMQMessageNumber(8512,
                  "Usage: endmqcsv [-c | -i] QMgrName",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8513",new AMQMessageNumber(8513,
                  "Usage: dspmqcsv [QMgrName]",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8514",new AMQMessageNumber(8514,
                  "No response received after <insert_1> seconds.",
                  "20 : Error",
                  "The command server has not reported the status of running, to the start request, before the timeout of <insert_1> seconds was reached.",
                  "None."));
      amqMN_Ht.put("8549",new AMQMessageNumber(8549,
                  "Total string length exceeds the maximum value of 999 characters.",
                  "0 : Information",
                  "The total length of a channel exit string is 999 characters. The string list assigned to an MQSC keyword is longer than the maximum value of 999 characters permitted for that keyword.",
                  "None."));
      amqMN_Ht.put("8550",new AMQMessageNumber(8550,
                  "Display namelist details.",
                  "0 : Information",
                  "The MQSC DISPLAY NAMELIST command completed successfully, and details follow this message.",
                  "None."));
      amqMN_Ht.put("8551",new AMQMessageNumber(8551,
                  "WebSphere MQ namelist changed.",
                  "0 : Information",
                  "WebSphere MQ namelist <insert_5> changed.",
                  "None."));
      amqMN_Ht.put("8552",new AMQMessageNumber(8552,
                  "WebSphere MQ namelist created.",
                  "0 : Information",
                  "WebSphere MQ namelist <insert_5> created.",
                  "None."));
      amqMN_Ht.put("8553",new AMQMessageNumber(8553,
                  "WebSphere MQ namelist deleted.",
                  "0 : Information",
                  "WebSphere MQ namelist <insert_5> deleted.",
                  "None."));
      amqMN_Ht.put("8554",new AMQMessageNumber(8554,
                  "String List String Count Error.",
                  "0 : Information",
                  "The number of strings within the stringlist is greater than the maximum number allowed for the keyword. Reduce the number of strings within the list and try the command again.",
                  "None."));
      amqMN_Ht.put("8555",new AMQMessageNumber(8555,
                  "String List String Length Error.",
                  "0 : Information",
                  "A string in a string list assigned to a keyword is longer than the maximum permitted for that keyword.",
                  "None."));
      amqMN_Ht.put("8556",new AMQMessageNumber(8556,
                  "RESUME QUEUE MANAGER accepted.",
                  "0 : Information",
                  "The RESUME QUEUE MANAGER command has been accepted for processing. The command will be sent to the repository which will process the command and notify all other repositories that this queue manager is now back in the cluster.",
                  "None."));
      amqMN_Ht.put("8557",new AMQMessageNumber(8557,
                  "SUSPEND QUEUE MANAGER accepted.",
                  "0 : Information",
                  "The SUSPEND QUEUE MANAGER command has been accepted for processing. The command will be sent to the repository which will process the command and notify all other repositories that this queue manager is leaving the cluster.",
                  "None."));
      amqMN_Ht.put("8558",new AMQMessageNumber(8558,
                  "REFRESH CLUSTER accepted.",
                  "0 : Information",
                  "The REFRESH CLUSTER command has been accepted for processing. The command will be sent to the Repository which will process the command and notify all other repositories that the Cluster needs refreshing.",
                  "None."));
      amqMN_Ht.put("8559",new AMQMessageNumber(8559,
                  "RESET CLUSTER accepted.",
                  "0 : Information",
                  "The RESET CLUSTER command has been accepted for processing. The command will be sent to the Repository which will process the command and notify all other repositories that the Cluster needs resetting.",
                  "None."));
      amqMN_Ht.put("8560",new AMQMessageNumber(8560,
                  "WebSphere MQ security cache refreshed.",
                  "0 : Information",
                  "The Object Authority Manager security cache has been refreshed.",
                  "None."));
      amqMN_Ht.put("8561",new AMQMessageNumber(8561,
                  " (Windows)Domain controller unavailable.",
                  "10 : Warning",
                  "WebSphere MQ was unable to contact the domain controller to obtain information for user <insert_3>.",
                  "Ensure that a domain controller for the domain on which user <insert_3> is defined is available. Alternatively, if you are using a computer which is not currently connected to the network and have logged on using a domain user ID, you may wish to log on using a local user ID instead."));
      amqMN_Ht.put("8563",new AMQMessageNumber(8563,
                  "WebSphere MQ authentication information object created.",
                  "0 : Information",
                  "WebSphere MQ authentication information object <insert_5> created.",
                  "None."));
      amqMN_Ht.put("8564",new AMQMessageNumber(8564,
                  "WebSphere MQ authentication information object deleted.",
                  "0 : Information",
                  "WebSphere MQ authentication information object <insert_5> deleted.",
                  "None."));
      amqMN_Ht.put("8565",new AMQMessageNumber(8565,
                  "Queue Status not found.",
                  "0 : Information",
                  "Queue Status for the specified queue could not be found.",
                  "None."));
      amqMN_Ht.put("8566",new AMQMessageNumber(8566,
                  "Display authentication information details.",
                  "0 : Information",
                  "The MQSC DISPLAY AUTHINFO command completed successfully. Details follow this message.",
                  "None."));
      amqMN_Ht.put("8567",new AMQMessageNumber(8567,
                  "WebSphere MQ authentication information changed.",
                  "0 : Information",
                  "WebSphere MQ authentication information <insert_5> changed.",
                  "None."));
      amqMN_Ht.put("8568",new AMQMessageNumber(8568,
                  "(iSeries) No authinfo objects to display.",
                  "0 : Information",
                  "There are no matching authinfo objects defined on this system.",
                  "Using the DEFINE AUTHINFO command to create an authinfo object."));
      amqMN_Ht.put("8569",new AMQMessageNumber(8569,
                  "Error in filter specification",
                  "0 : Information",
                  "You specified an invalid filter. Check the WHERE statement and make sure that the operator is valid for the type of parameter, that the parameter can be filtered on, and that the value that you specified for the filter is valid for the type of attribute you are filtering on.",
                  "None."));
      amqMN_Ht.put("8570",new AMQMessageNumber(8570,
                  "Attribute value error in <insert_3>.",
                  "0 : Information",
                  "The keyword <insert_3> contained a value that was not valid for this configuration. Please check the MQSC Command Reference to determine valid values for <insert_3>.",
                  "None."));
      amqMN_Ht.put("8601",new AMQMessageNumber(8601,
                  "WebSphere MQ trigger monitor started.",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor has been started.",
                  "None."));
      amqMN_Ht.put("8601",new AMQMessageNumber(8601,
                  "(iSeries) WebSphere MQ trigger monitor started.",
                  "0 : Information",
                  "The trigger monitor has been started with initiation queue <insert_3>.",
                  "None."));
      amqMN_Ht.put("8602",new AMQMessageNumber(8602,
                  "WebSphere MQ trigger monitor ended.",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor has ended.",
                  "None."));
      amqMN_Ht.put("8603",new AMQMessageNumber(8603,
                  "Usage: runmqtrm [-m QMgrName] [-q InitQ]",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8604",new AMQMessageNumber(8604,
                  "Use of WebSphere MQ trigger monitor not authorized.",
                  "0 : Information",
                  "The trigger monitor cannot be run due to lack of authority to the requested queue manager or initiation queue.",
                  "Obtain the necessary authority from your security officer or WebSphere MQ administrator. Then try the command again."));
      amqMN_Ht.put("8605",new AMQMessageNumber(8605,
                  "Queue manager not available to the WebSphere MQ trigger monitor",
                  "0 : Information",
                  "The queue manager specified for the trigger monitor does not exist, or is not active.",
                  "Check that you named the correct queue manager. Ask your systems administrator to start it, if it is not active. Then try the command again."));
      amqMN_Ht.put("8606",new AMQMessageNumber(8606,
                  "Insufficient storage available for the WebSphere MQ trigger monitor.",
                  "0 : Information",
                  "There was insufficient storage available for the WebSphere MQ trigger monitor to run.",
                  "Free some storage and then try the command again."));
      amqMN_Ht.put("8607",new AMQMessageNumber(8607,
                  "WebSphere MQ trigger monitor connection failed.",
                  "0 : Information",
                  "The trigger monitor's connection to the requested queue manager failed because of MQI reason code <insert_1> from MQCONN.",
                  "Consult your systems administrator about the state of the queue manager."));
      amqMN_Ht.put("8608",new AMQMessageNumber(8608,
                  "WebSphere MQ trigger monitor connection broken.",
                  "0 : Information",
                  "The connection to the queue manager failed while the trigger monitor was running. This may be caused by an endmqm command being issued by another user, or by a queue manager error.",
                  "Consult your systems administrator about the state of the queue manager."));
      amqMN_Ht.put("8609",new AMQMessageNumber(8609,
                  "Initiation queue missing or wrong type",
                  "0 : Information",
                  "The named initiation queue could not be found; or the queue type is not correct for an initiation queue.",
                  "Check that the named queue exists, and is a local queue, or that the named queue is an alias for a local queue which exists."));
      amqMN_Ht.put("8610",new AMQMessageNumber(8610,
                  "Initiation queue in use",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor could not open the initiation queue because the queue is open for exclusive use by another application.",
                  "Wait until the queue is no longer in use, and try the command again."));
      amqMN_Ht.put("8611",new AMQMessageNumber(8611,
                  "Initiation queue could not be opened.",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor could not open the initiation queue; reason code <insert_1> was returned from MQOPEN.",
                  "Consult your systems administrator."));
      amqMN_Ht.put("8612",new AMQMessageNumber(8612,
                  "Waiting for a trigger message",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor is waiting for a message to arrive on the initiation queue.",
                  "None."));
      amqMN_Ht.put("8613",new AMQMessageNumber(8613,
                  "Initiation queue changed or deleted",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor is unable to continue because the initiation queue has been deleted or changed since it was opened.",
                  "Retry the command."));
      amqMN_Ht.put("8614",new AMQMessageNumber(8614,
                  "Initiation queue not enabled for input.",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor cannot read from the initiation queue because input is not enabled.",
                  "Ask your systems administrator to enable the queue for input."));
      amqMN_Ht.put("8615",new AMQMessageNumber(8615,
                  "WebSphere MQ trigger monitor failed to get message.",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor failed because of MQI reason code <insert_1> from MQGET.",
                  "Consult your systems administrator."));
      amqMN_Ht.put("8616",new AMQMessageNumber(8616,
                  "End of application trigger.",
                  "0 : Information",
                  "The action to trigger an application has been completed.",
                  "None."));
      amqMN_Ht.put("8617",new AMQMessageNumber(8617,
                  "Not a valid trigger message.",
                  "0 : Information",
                  "The WebSphere MQ trigger monitor received a message that is not recognized as a valid trigger message. It has been written to the undelivered message queue.",
                  "Consult your systems administrator."));
      amqMN_Ht.put("8618",new AMQMessageNumber(8618,
                  "Error starting triggered application.",
                  "0 : Information",
                  "An error was detected when trying to start the application identified in a trigger message.",
                  "Check that the application the trigger monitor was trying to start is available."));
      amqMN_Ht.put("8619",new AMQMessageNumber(8619,
                  "Application type <insert_1> not supported.",
                  "0 : Information",
                  "A trigger message was received which specifies application type <insert_1>; the trigger monitor does not support this type.",
                  "Use an alternative trigger monitor for this initiation queue."));
      amqMN_Ht.put("8620",new AMQMessageNumber(8620,
                  "Trigger message with warning <insert_1>",
                  "0 : Information",
                  "The trigger monitor received a message with a warning. For example, it may have been truncated or it could not be converted to the trigger monitor's data representation. The reason code for the warning is <insert_1>.",
                  "None."));
      amqMN_Ht.put("8621",new AMQMessageNumber(8621,
                  "Usage: runmqtmc [-m QMgrName] [-q InitQ]",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8622",new AMQMessageNumber(8622,
                  "Usage: CICS-Transaction-Name [MQTMC2 structure]",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8623",new AMQMessageNumber(8623,
                  "WebSphere MQ listener changed.",
                  "0 : Information",
                  "WebSphere MQ listener <insert_5> changed.",
                  "None."));
      amqMN_Ht.put("8624",new AMQMessageNumber(8624,
                  "WebSphere MQ service changed.",
                  "0 : Information",
                  "WebSphere MQ service <insert_5> changed.",
                  "None."));
      amqMN_Ht.put("8625",new AMQMessageNumber(8625,
                  "WebSphere MQ service created.",
                  "0 : Information",
                  "WebSphere MQ service <insert_5> created.",
                  "None."));
      amqMN_Ht.put("8626",new AMQMessageNumber(8626,
                  "WebSphere MQ listener created.",
                  "0 : Information",
                  "WebSphere MQ listener <insert_5> created.",
                  "None."));
      amqMN_Ht.put("8627",new AMQMessageNumber(8627,
                  "WebSphere MQ service object deleted.",
                  "0 : Information",
                  "WebSphere MQ service object <insert_5> deleted.",
                  "None."));
      amqMN_Ht.put("8628",new AMQMessageNumber(8628,
                  "WebSphere MQ listener object deleted.",
                  "0 : Information",
                  "WebSphere MQ listener object <insert_5> deleted.",
                  "None."));
      amqMN_Ht.put("8629",new AMQMessageNumber(8629,
                  "Display service information details.",
                  "0 : Information",
                  "The MQSC DISPLAY SERVICE command completed successfully. Details follow this message.",
                  "None."));
      amqMN_Ht.put("8630",new AMQMessageNumber(8630,
                  "Display listener information details.",
                  "0 : Information",
                  "The MQSC DISPLAY LISTENER command completed successfully. Details follow this message.",
                  "None."));
      amqMN_Ht.put("8631",new AMQMessageNumber(8631,
                  "Display listener status details.",
                  "0 : Information",
                  "The MQSC DISPLAY LSSTATUS command completed successfully. Details follow this message.",
                  ""));
      amqMN_Ht.put("8632",new AMQMessageNumber(8632,
                  "Display service status details.",
                  "0 : Information",
                  "The MQSC DISPLAY SVSTATUS command completed successfully. Details follow this message.",
                  ""));
      amqMN_Ht.put("8649",new AMQMessageNumber(8649,
                  "Reset WebSphere MQ Queue Manager accepted.",
                  "0 : Information",
                  "The MQSC RESET QMGR command completed successfully. Details follow this message.",
                  "None."));
      amqMN_Ht.put("8650",new AMQMessageNumber(8650,
                  "Activity information unavailable.",
                  "0 : Information",
                  "The DSPMQRTE command was expecting activity information but it was unavailable. This does not always constitute an error. Reasons why the activity information is unavailable include the following: 1) One of the queue managers on the route did not support trace-route messaging. 2) One of the queue managers on the route did not allow route information to be returned to the reply queue. See the documentation on the ActivityRecording and TraceRouteRecording queue manager attributes for more details. 3) The report could not a find route back to the reply queue.",
                  "Try and determine whether the activity information should have been available. Running the command with the 'outline' verbosity option (used with the -v flag) may be useful in determining where the message was when the activity information was generated."));
      amqMN_Ht.put("8650",new AMQMessageNumber(8650,
                  "(iSeries) Activity information unavailable.",
                  "0 : Information",
                  "The DSPMQMRTE command was expecting activity information but it was unavailable. This does not always constitute an error. Reasons why the activity information is unavailable include the following: 1) One of the queue managers on the route did not support trace-route messaging. 2) One of the queue managers on the route did not allow route information to be returned to the reply queue. See the documentation on the ActivityRecording and TraceRouteRecording queue manager attributes for more details. 3) The report could not a find route back to the reply queue.",
                  "Try and determine whether the activity information should have been available. Running the command with DSPINF(*ALL) may be useful in determining where the message was when the activity information was generated."));
      amqMN_Ht.put("8651",new AMQMessageNumber(8651,
                  "DSPMQRTE command has finished with errors.",
                  "0 : Information",
                  "The DSPMQRTE command has finished processing your request but an execution error was detected. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8651",new AMQMessageNumber(8651,
                  "(iSeries) DSPMQMRTE command has finished with errors.",
                  "0 : Information",
                  "The DSPMQMRTE command has finished processing your request but an execution error was detected. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8652",new AMQMessageNumber(8652,
                  "DSPMQRTE command has finished.",
                  "0 : Information",
                  "The DSPMQRTE command has finished processing your request and no execution errors were detected.",
                  "None."));
      amqMN_Ht.put("8652",new AMQMessageNumber(8652,
                  "(iSeries) DSPMQMRTE command has finished.",
                  "0 : Information",
                  "The DSPMQMRTE command has finished processing your request and no execution errors were detected.",
                  "None."));
      amqMN_Ht.put("8653",new AMQMessageNumber(8653,
                  "DSPMQRTE command started with options <insert_3>.",
                  "0 : Information",
                  "You have started the DSPMQRTE command with command line options <insert_3> and the command is now processing your request.",
                  "Wait for the command to finish processing your request. Any further messages that are issued can be used to determine the outcome of the request."));
      amqMN_Ht.put("8653",new AMQMessageNumber(8653,
                  "(iSeries) DSPMQMRTE command started.",
                  "0 : Information",
                  "You have started the DSPMQMRTE command and the command is now processing your request.",
                  "Wait for the command to finish processing your request. Any further messages that are issued can be used to determine the outcome of the request."));
      amqMN_Ht.put("8654",new AMQMessageNumber(8654,
                  "Trace-route message arrived on queue manager <insert_3>.",
                  "0 : Information",
                  "The DSPMQRTE command has received confirmation of the successful arrival of the trace-route message at its destination queue on queue manager <insert_3>.",
                  "None."));
      amqMN_Ht.put("8654",new AMQMessageNumber(8654,
                  "(iSeries) Trace-route message arrived on queue manager <insert_3>.",
                  "0 : Information",
                  "The DSPMQMRTE command has received confirmation of the successful arrival of the trace-route message at its destination queue on queue manager <insert_3>.",
                  "None."));
      amqMN_Ht.put("8655",new AMQMessageNumber(8655,
                  "Trace-route message expired.",
                  "0 : Information",
                  "The DSPMQRTE command has received confirmation that the trace-route message has expired.",
                  "The expiry interval of trace-route messages generated by the DSPMQRTE command can be altered using the -xs option if this is required."));
      amqMN_Ht.put("8655",new AMQMessageNumber(8655,
                  "(iSeries) Trace-route message expired.",
                  "0 : Information",
                  "The DSPMQMRTE command has received confirmation that the trace-route message has expired.",
                  "The expiry interval of trace-route messages generated by the DSPMQMRTE command can be altered using the EXPIRY parameter if this is required."));
      amqMN_Ht.put("8656",new AMQMessageNumber(8656,
                  "DSPMQRTE command received an exception report from queue manager <insert_4> with feedback <insert_1> <insert_3>.",
                  "0 : Information",
                  "The DSPMQRTE command trace-route message caused an exception on queue manager <insert_4>. The Feedback field in the report was <insert_1> or <insert_3>.",
                  "Use the feedback given to determine why the trace-route message caused the exception."));
      amqMN_Ht.put("8656",new AMQMessageNumber(8656,
                  "(iSeries) DSPMQMRTE command received an exception report from queue manager <insert_4> with feedback <insert_1> <insert_3>.",
                  "0 : Information",
                  "The DSPMQMRTE command trace-route message caused an exception on queue manager <insert_4>. The Feedback field in the report was <insert_1> or <insert_3>.",
                  "Use the feedback given to determine why the trace-route message caused the exception."));
      amqMN_Ht.put("8657",new AMQMessageNumber(8657,
                  "DSPMQRTE command used <insert_3> 0x<insert_4>.",
                  "0 : Information",
                  "You started the DSPMQRTE command specifying that it should generate a trace-route message. This took place and the trace-route message had <insert_3> X<insert_4>.",
                  "The <insert_3> can be used to retrieve responses to this trace-route request. Run the DSPMQRTE command again specifying this identifier with the -i flag and with the target queue specified as the queue where the responses are expected to return or where the trace-route message is expected to have arrived. This may be on another queue manager."));
      amqMN_Ht.put("8657",new AMQMessageNumber(8657,
                  "(iSeries) DSPMQMRTE command used <insert_3> 0x<insert_4>.",
                  "0 : Information",
                  "You started the DSPMQMRTE command specifying that it should generate a trace-route message. This took place and the trace-route message had <insert_3> X<insert_4>.",
                  "The <insert_3> can be used to retrieve responses to this trace-route request. Run the DSPMQMRTE command again specifying this identifier for CRLLID and with the target queue specified as the queue where the responses are expected to return or where the trace-route message is expected to have arrived. This may be on another queue manager."));
      amqMN_Ht.put("8658",new AMQMessageNumber(8658,
                  "DSPMQRTE command failed to put a message on the target queue.",
                  "0 : Information",
                  "The request for the DSPMQRTE command to put a trace-route message on the target queue was unsuccessful. Previous messages issued by the command can be used to identify why the message could not be put on the target queue.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8658",new AMQMessageNumber(8658,
                  "(iSeries) DSPMQMRTE command failed to put a message on the target queue.",
                  "0 : Information",
                  "The request for the DSPMQMRTE command to put a trace-route message on the target queue was unsuccessful. Previous messages issued by the command can be used to identify why the message could not be put on the target queue.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8659",new AMQMessageNumber(8659,
                  "DSPMQRTE command successfully put a message on queue <insert_3>, queue manager <insert_4>.",
                  "0 : Information",
                  "The request for the DSPMQRTE command to put a message on the target queue was successful. The target queue resolved to <insert_3> on queue manager <insert_4>.",
                  "None."));
      amqMN_Ht.put("8659",new AMQMessageNumber(8659,
                  "(iSeries) DSPMQMRTE command successfully put a message on queue <insert_3>, queue manager <insert_4>.",
                  "0 : Information",
                  "The request for the DSPMQMRTE command to put a message on the target queue was successful. The target queue resolved to <insert_3> on queue manager <insert_4>.",
                  "None."));
      amqMN_Ht.put("8660",new AMQMessageNumber(8660,
                  "DSPMQRTE command could not correctly order the following activities:",
                  "0 : Information",
                  "The DSPMQRTE command received the following activities, but they could not be printed in the correct order. This is commonly because an activity report has been received that does not contain a TraceRoute PCF group or is missing the RecordedActivities parameter which would allow it to be ordered correctly.",
                  "Find and correct the application that is generating activity reports without the necessary information for them to be ordered correctly."));
      amqMN_Ht.put("8660",new AMQMessageNumber(8660,
                  "(iSeries) DSPMQMRTE command could not correctly order the following activities:",
                  "0 : Information",
                  "The DSPMQMRTE command received the following activities, but they could not be printed in the correct order. This is commonly because an activity report has been received that does not contain a TraceRoute PCF group or is missing the RecordedActivities parameter which would allow it to be ordered correctly.",
                  "Find and correct the application that is generating activity reports without the necessary information for them to be ordered correctly."));
      amqMN_Ht.put("8661",new AMQMessageNumber(8661,
                  "DSPMQRTE command will not put to queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying that the trace-route message should not be delivered to a local queue (-d yes was not specified). However, it has been determined that the target queue does not resolve to a transmission queue. Therefore the DSPMQRTE command has chosen not to put the trace-route message to the target queue <insert_3> on queue manager <insert_4>.",
                  "Determine whether it was expected that the target queue would resolve to a local queue."));
      amqMN_Ht.put("8661",new AMQMessageNumber(8661,
                  "(iSeries) DSPMQMRTE command will not put to queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQMRTE command specifying that the trace-route message should not be delivered to a local queue (DLVRMSG(*NO) was specified). However, it has been determined that the target queue does not resolve to a transmission queue. Therefore the DSPMQMRTE command has chosen not to put the trace-route message to the target queue <insert_3> on queue manager <insert_4>.",
                  "Determine whether it was expected that the target queue would resolve to a local queue."));
      amqMN_Ht.put("8662",new AMQMessageNumber(8662,
                  "Trace-route message delivered on queue manager <insert_3>.",
                  "0 : Information",
                  "The DSPMQRTE command has received confirmation of the successful delivery of the trace-route message on queue manager <insert_3> to a requesting application.",
                  "None."));
      amqMN_Ht.put("8662",new AMQMessageNumber(8662,
                  "(iSeries) Trace-route message delivered on queue manager <insert_3>.",
                  "0 : Information",
                  "The DSPMQMRTE command has received confirmation of the successful delivery of the trace-route message on queue manager <insert_3> to a requesting application.",
                  "None."));
      amqMN_Ht.put("8663",new AMQMessageNumber(8663,
                  "Client connection not supported in this environment.",
                  "20 : Error",
                  "An attempt was made to connect to a queue manager using a client connection. However, client connections are not supported in your environment.",
                  "Connect to the queue manager using a server connection."));
      amqMN_Ht.put("8664",new AMQMessageNumber(8664,
                  "DSPMQRTE command could not connect to queue manager <insert_3>.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying that it should connect to queue manager <insert_3>. The command could not connect to that queue manager. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8664",new AMQMessageNumber(8664,
                  "(iSeries) DSPMQMRTE command could not connect to queue manager <insert_3>.",
                  "20 : Error",
                  "You started the DSPMQMRTE command specifying that it should connect to queue manager <insert_3>. The command could not connect to that queue manager. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8665",new AMQMessageNumber(8665,
                  "DSPMQRTE command was supplied an invalid CorrelId <insert_3>.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying option -i with a CorrelId <insert_3> that was invalid. The CorrelId was either too long or not in the correct format.",
                  "Refer to the command syntax, and then try the command again."));
      amqMN_Ht.put("8665",new AMQMessageNumber(8665,
                  "(iSeries) DSPMQMRTE command was supplied an invalid CorrelId <insert_3>.",
                  "20 : Error",
                  "You started the DSPMQMRTE command specifying CRLLID with a CorrelId <insert_3> that was invalid.",
                  "Refer to the command syntax, and then try the command again."));
      amqMN_Ht.put("8666",new AMQMessageNumber(8666,
                  "Queue <insert_3> on queue manager <insert_4>.",
                  "0 : Information",
                  "The DSPMQRTE command trace-route message has been confirmed as having taken a route involving queue <insert_3> on queue manager <insert_4> in an attempt to reach the destination queue.",
                  "Wait for subsequent messages which may indicate another queue which the message has been routed through."));
      amqMN_Ht.put("8666",new AMQMessageNumber(8666,
                  "(iSeries) Queue <insert_3> on queue manager <insert_4>.",
                  "0 : Information",
                  "The DSPMQMRTE command trace-route message has been confirmed as having taken a route involving queue <insert_3> on queue manager <insert_4> in an attempt to reach the destination queue.",
                  "Wait for subsequent messages which may indicate another queue which the message has been routed through."));
      amqMN_Ht.put("8667",new AMQMessageNumber(8667,
                  "DSPMQRTE command could not open reply queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying reply queue <insert_3>. However the DSPMQRTE command could not successfully open a queue of that name on queue manager <insert_4>. Previous messages issued by the command can be used to identify the error. If the -rq option was not specified then the reply queue will be a temporary dynamic queue modelled on SYSTEM.DEFAULT.MODEL.QUEUE.",
                  "Refer to previous messages issued by the command. Specify a reply queue that can be opened and then retry the command."));
      amqMN_Ht.put("8667",new AMQMessageNumber(8667,
                  "(iSeries) DSPMQMRTE command could not open reply queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQMRTE command specifying reply queue <insert_3>. However the DSPMQMRTE command could not successfully open a queue of that name on queue manager <insert_4>. Previous messages issued by the command can be used to identify the error. If the RPLYQ parameter was not specified then the reply queue will be a temporary dynamic queue modelled on SYSTEM.DEFAULT.MODEL.QUEUE.",
                  "Refer to previous messages issued by the command. Specify a reply queue that can be opened and then retry the command."));
      amqMN_Ht.put("8668",new AMQMessageNumber(8668,
                  "DSPMQRTE command could not open queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying queue <insert_3>, using the -q option. However the DSPMQRTE command could not successfully open a queue of that name on queue manager <insert_4>. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command. Specify a queue, using the -q option, that can be opened and then retry the command."));
      amqMN_Ht.put("8668",new AMQMessageNumber(8668,
                  "(iSeries) DSPMQMRTE command could not open queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQMRTE command specifying queue <insert_3> for the QNAME parameter. However the DSPMQMRTE command could not successfully open a queue of that name on queue manager <insert_4>. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command. Specify a queue, using the QNAME parameter, that can be opened and then retry the command."));
      amqMN_Ht.put("8669",new AMQMessageNumber(8669,
                  "DSPMQRTE command failed to resolve queue manager <insert_3> on queue manager <insert_4>.",
                  "20 : Error",
                  "The DSPMQRTE command attempted to resolve queue manager <insert_3> (supplied by the -qm option) on queue manager <insert_4> but the attempt failed. The queue specified by the -q option could not be opened.",
                  "Ensure that queue manager <insert_3> can be resolved on queue manager <insert_4> or specify a different queue manager with the -qm option. Retry the command."));
      amqMN_Ht.put("8669",new AMQMessageNumber(8669,
                  "(iSeries) DSPMQMRTE command failed to resolve queue manager <insert_3> on queue manager <insert_4>.",
                  "20 : Error",
                  "The DSPMQMRTE command attempted to resolve queue manager <insert_3> (supplied by the TGTMQM parameter) on queue manager <insert_4> but the attempt failed. The queue specified by the QNAME parameter could not be opened.",
                  "Ensure that queue manager <insert_3> can be resolved on queue manager <insert_4> or specify a different queue manager with the TGTMQM parameter. Retry the command."));
      amqMN_Ht.put("8670",new AMQMessageNumber(8670,
                  "Loading of server module <insert_3> failed.",
                  "20 : Error",
                  "An attempt to dynamically load the server module <insert_3> failed. Typically this is because only the client modules are installed.",
                  "Check which modules are installed and retry the command with the -c option specified if applicable."));
      amqMN_Ht.put("8671",new AMQMessageNumber(8671,
                  "DSPMQRTE command was not supplied a reply queue when one was required.",
                  "20 : Error",
                  "The DSPMQRTE command was expecting a reply queue specified by the -rq option but no reply queue was specified. Specifying a reply queue is mandatory if both the -n (no display) option and a response generating option (-ar or -ro [activity|coa|cod|exception|expiration]) is specified.",
                  "Specify a reply queue and retry the command."));
      amqMN_Ht.put("8672",new AMQMessageNumber(8672,
                  "DSPMQRTE command failed to get a message from queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "The DSPMQRTE command attempted to get a message from queue <insert_3>, queue manager <insert_4>, but the attempt failed. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8672",new AMQMessageNumber(8672,
                  "(iSeries) DSPMQMRTE command failed to get a message from queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "The DSPMQMRTE command attempted to get a message from queue <insert_3>, queue manager <insert_4>, but the attempt failed. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8673",new AMQMessageNumber(8673,
                  "DSPMQRTE command was supplied option <insert_3> with an invalid object name <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying option <insert_3> with an object name <insert_4> that is invalid. In general, the names of WebSphere MQ objects can have up to 48 characters. An object name can contain the following characters: 1) Uppercase alphabetic characters (A through Z). 2) Lowercase alphabetic characters (a through z). 3) Numeric digits (0 through 9). 4) Period (.), forward slash (/), underscore (_), percent (%). See the WebSphere MQ System Administration documentation for further details and restrictions.",
                  "Specify a valid object name and then try the command again."));
      amqMN_Ht.put("8673",new AMQMessageNumber(8673,
                  "(iSeries) DSPMQMRTE command was supplied with an invalid object name <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQMRTE command specifying an object name <insert_4> that is invalid. In general, the names of WebSphere MQ objects can have up to 48 characters. An object name can contain the following characters: 1) Uppercase alphabetic characters (A through Z). 2) Lowercase alphabetic characters (a through z). 3) Numeric digits (0 through 9). 4) Period (.), forward slash (/), underscore (_), percent (%). See the WebSphere MQ System Administration documentation for further details and restrictions.",
                  "Specify a valid object name and then try the command again."));
      amqMN_Ht.put("8674",new AMQMessageNumber(8674,
                  "DSPMQRTE command is now waiting for information to display.",
                  "0 : Information",
                  "The DSPMQRTE command has successfully generated and put the trace-route message and is now waiting for responses to be returned to the reply queue to indicate the route that the trace-route message took to its destination.",
                  "Wait for responses to be returned to the reply queue and for the information about the route to be displayed."));
      amqMN_Ht.put("8674",new AMQMessageNumber(8674,
                  "(iSeries) DSPMQMRTE command is now waiting for information to display.",
                  "0 : Information",
                  "The DSPMQMRTE command has successfully generated and put the trace-route message and is now waiting for responses to be returned to the reply queue to indicate the route that the trace-route message took to its destination.",
                  "Wait for responses to be returned to the reply queue and for the information about the route to be displayed."));
      amqMN_Ht.put("8675",new AMQMessageNumber(8675,
                  "DSPMQRTE command was supplied an invalid option <insert_3>.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying an option of <insert_3> that was not recognized. The command will end.",
                  "Refer to the command syntax and retry the command."));
      amqMN_Ht.put("8676",new AMQMessageNumber(8676,
                  "DSPMQRTE command was supplied an invalid combination of options.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying a combination of the options that is not valid. The -i option cannot be specified with one or more of the following options: -ac, -ar, -d, -f, -l, -n, -o, -p, -qm, -ro, -rq, -rqm, -s, -t, -xs, -xp. The -n option cannot be specified with one or more of the following options: -b, -i, -v, -w. The -ar option can only be specified if the -ac option has also been specified. The -rqm option can only be specified if the -rq option has also been specified.",
                  "Refer to the command documentation and then try the command again."));
      amqMN_Ht.put("8677",new AMQMessageNumber(8677,
                  "DSPMQRTE command was supplied an option <insert_3> with conflicting values.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying values for option <insert_3> that conflict. At least two values were specified for the same option but they conflict with each other. The DSPMQRTE command will end.",
                  "Refer to the command syntax and then try the command again."));
      amqMN_Ht.put("8677",new AMQMessageNumber(8677,
                  "(iSeries) DSPMQMRTE command was supplied a parameter with conflicting values.",
                  "20 : Error",
                  "You started the DSPMQMRTE command specifying values that conflict. At least two values were specified for the same parameter but they conflict with each other. The DSPMQMRTE command will end.",
                  "Refer to the command syntax and then try the command again."));
      amqMN_Ht.put("8678",new AMQMessageNumber(8678,
                  "DSPMQRTE command was supplied option <insert_3> with an invalid value <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQRTE command specifying an invalid option value. The <insert_4> value for option <insert_3> is either not recognized or of an incorrect format.",
                  "Refer to the command syntax, and then try the command again."));
      amqMN_Ht.put("8678",new AMQMessageNumber(8678,
                  "(iSeries) DSPMQMRTE command was supplied an invalid value <insert_4>.",
                  "20 : Error",
                  "You started the DSPMQMRTE command specifying an invalid parameter value. Value <insert_4> is either not recognized or of an incorrect format.",
                  "Refer to the command syntax, and then try the command again."));
      amqMN_Ht.put("8679",new AMQMessageNumber(8679,
                  "Persistent messages not allowed on reply queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "It was specified that the DSPMQRTE command should put a persistent trace-route message on the target queue (see the documentation for the -l option). However, persistent messages are not allowed on the reply queue because it is a temporary dynamic queue and persistent responses were expected to return to it. The trace-route message was not put on the target queue.",
                  "Ensure that the reply queue is not a temporary dynamic queue. Use the -rq option to specify the reply queue."));
      amqMN_Ht.put("8679",new AMQMessageNumber(8679,
                  "(iSeries) Persistent messages not allowed on reply queue <insert_3>, queue manager <insert_4>.",
                  "20 : Error",
                  "It was specified that the DSPMQMRTE command should put a persistent trace-route message on the target queue (see the documentation for the MSGPST parameter). However, persistent messages are not allowed on the reply queue because it is a temporary dynamic queue and persistent responses were expected to return to it. The trace-route message was not put on the target queue.",
                  "Ensure that the reply queue is not a temporary dynamic queue. Use the RPLYQ parameter to specify the reply queue."));
      amqMN_Ht.put("8680",new AMQMessageNumber(8680,
                  "DSPMQRTE command failed to open queue manager <insert_3>.",
                  "20 : Error",
                  "The DSPMQRTE command tried to open queue manager <insert_3> for inquire but the open failed. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8680",new AMQMessageNumber(8680,
                  "(iSeries) DSPMQMRTE command failed to open queue manager <insert_3>.",
                  "20 : Error",
                  "The DSPMQMRTE command tried to open queue manager <insert_3> for inquire but the open failed. Previous messages issued by the command can be used to identify the error.",
                  "Refer to previous messages issued by the command."));
      amqMN_Ht.put("8681",new AMQMessageNumber(8681,
                  "DSPMQRTE command has detected an error, reason <insert_1> <insert_3>.",
                  "20 : Error",
                  "The DSPMQRTE command has detected an error from an MQI call during the execution of your request. The reason for failure is <insert_1> or <insert_3>.",
                  "See the WebSphere MQ Messages documentation for an explanation of the reason for failure. Follow any correction action and retry the command."));
      amqMN_Ht.put("8681",new AMQMessageNumber(8681,
                  "(iSeries) DSPMQMRTE command has detected an error, reason <insert_1> <insert_3>.",
                  "20 : Error",
                  "The DSPMQMRTE command has detected an error from an MQI call during the execution of your request. The reason for failure is <insert_1> or <insert_3>.",
                  "See the WebSphere MQ Messages documentation for an explanation of the reason for failure. Follow any correction action and retry the command."));
      amqMN_Ht.put("8682",new AMQMessageNumber(8682,
                  "Trace-route message processed by application <insert_3> on queue manager <insert_4>.",
                  "0 : Information",
                  "The DSPMQRTE command successfully put a trace-route message on the target queue and it was then delivered by queue manager <insert_4> to application <insert_3> which processed the message.",
                  "Determine if it was expected that this application would process the trace-route message."));
      amqMN_Ht.put("8682",new AMQMessageNumber(8682,
                  "(iSeries) Trace-route message processed by application <insert_3> on queue manager <insert_4>.",
                  "0 : Information",
                  "The DSPMQMRTE command successfully put a trace-route message on the target queue and it was then delivered by queue manager <insert_4> to application <insert_3> which processed the message.",
                  "Determine if it was expected that this application would process the trace-route message."));
      amqMN_Ht.put("8683",new AMQMessageNumber(8683,
                  "Trace-route message reached the maximum activities limit of <insert_1>.",
                  "0 : Information",
                  "The DSPMQRTE command trace-route message was rejected after the number of activities of which it was a participant reached the maximum activities limit. The limit was set to <insert_1>. The maximum activities limit is set using the -s option.",
                  "Using the output from the command determine whether it is expected that the trace-route message should have reached the maximum activities limit."));
      amqMN_Ht.put("8683",new AMQMessageNumber(8683,
                  "(iSeries) Trace-route message reached the maximum activities limit of <insert_1>.",
                  "0 : Information",
                  "The DSPMQMRTE command trace-route message was rejected after the number of activities of which it was a participant reached the maximum activities limit. The limit was set to <insert_1>. The maximum activities limit is set using the MAXACTS parameter.",
                  "Using the output from the command determine whether it is expected that the trace-route message should have reached the maximum activities limit."));
      amqMN_Ht.put("8684",new AMQMessageNumber(8684,
                  "Trace-route message reached trace-route incapable queue manager <insert_3>.",
                  "0 : Information",
                  "The DSPMQRTE command trace-route message was rejected because it was about to be sent to a queue manager which does not support trace-route messaging. This behaviour was requested because the forwarding options specified on the command only allowed the trace-route message to be forwarded to queue managers which support trace-route messaging. Sending a trace-route message to a queue manager which cannot process it in accordance with its specified options could cause undesirable results, including having the trace-route message be put to a local queue on the remote queue manager. If this is acceptable then the '-f all' option can be specified.",
                  "Retry the command with different forwarding options, if appropriate."));
      amqMN_Ht.put("8684",new AMQMessageNumber(8684,
                  "(iSeries) Trace-route message reached trace-route incapable queue manager <insert_3>.",
                  "0 : Information",
                  "The DSPMQMRTE command trace-route message was rejected because it was about to be sent to a queue manager which does not support trace-route messaging. This behaviour was requested because the forwarding options specified on the command only allowed the trace-route message to be forwarded to queue managers which support trace-route messaging. Sending a trace-route message to a queue manager which cannot process it in accordance with its specified options could cause undesirable results, including having the trace-route message be put to a local queue on the remote queue manager. If this is acceptable then FWDMSG(*ALL) can be specified.",
                  "Retry the command with different forwarding options, if appropriate."));
      amqMN_Ht.put("8685",new AMQMessageNumber(8685,
                  "Trace-route message rejected due to invalid forwarding options X<insert_1>.",
                  "20 : Error",
                  "The DSPMQRTE command trace-route message was rejected because one or more of the forwarding options was not recognized and it was in the MQROUTE_FORWARD_REJ_UNSUP_MASK bitmask. The forwarding options, when they were last observed, in hexadecimal were X<insert_1>.",
                  "Change the application that inserted the forwarding options that were not recognized to insert valid and supported forwarding options."));
      amqMN_Ht.put("8685",new AMQMessageNumber(8685,
                  "(iSeries) Trace-route message rejected due to invalid forwarding options X<insert_1>.",
                  "20 : Error",
                  "The DSPMQMRTE command trace-route message was rejected because one or more of the forwarding options was not recognized and it was in the MQROUTE_FORWARD_REJ_UNSUP_MASK bitmask. The forwarding options, when they were last observed, in hexadecimal were X<insert_1>.",
                  "Change the application that inserted the forwarding options that were not recognized to insert valid and supported forwarding options."));
      amqMN_Ht.put("8686",new AMQMessageNumber(8686,
                  "Trace-route message rejected due to invalid delivery options X<insert_1>.",
                  "20 : Error",
                  "The DSPMQRTE command trace-route message was rejected because one or more of the delivery options was not recognized and it was in the MQROUTE_DELIVER_REJ_UNSUP_MASK bitmask. The delivery options, when they were last observed, in hexadecimal were X<insert_1>.",
                  "Change the application that inserted the delivery options that were not recognized to insert valid and supported delivery options."));
      amqMN_Ht.put("8686",new AMQMessageNumber(8686,
                  "(iSeries) Trace-route message rejected due to invalid delivery options X<insert_1>.",
                  "20 : Error",
                  "The DSPMQMRTE command trace-route message was rejected because one or more of the delivery options was not recognized and it was in the MQROUTE_DELIVER_REJ_UNSUP_MASK bitmask. The delivery options, when they were last observed, in hexadecimal were X<insert_1>.",
                  "Change the application that inserted the delivery options that were not recognized to insert valid and supported delivery options."));
      amqMN_Ht.put("8687",new AMQMessageNumber(8687,
                  "Program ending.",
                  "0 : Information",
                  "The program operation was interrupted by a SIGINT signal on UNIX systems or a CTRL+c/CTRL+BREAK signal on Windows systems. The program is now ending.",
                  "Wait for the program to end."));
      amqMN_Ht.put("8688",new AMQMessageNumber(8688,
                  "DSPMQRTE command has detected an unexpected error, reason <insert_1> <insert_3>.",
                  "20 : Error",
                  "The DSPMQRTE command has detected an unexpected error during execution of your request. The reason for failure is <insert_1> or <insert_3>. The WebSphere MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8688",new AMQMessageNumber(8688,
                  "(iSeries) DSPMQMRTE command has detected an unexpected error, reason <insert_1> <insert_3>.",
                  "20 : Error",
                  "The DSPMQMRTE command has detected an unexpected error during execution of your request. The reason for failure is <insert_1> or <insert_3>. The WebSphere MQ error recording routine has been called.",
                  "Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8689",new AMQMessageNumber(8689,
                  "Loading of client module <insert_3> failed.",
                  "20 : Error",
                  "An attempt to dynamically load the client module <insert_3> failed. Typically this is because the client modules are not installed.",
                  "Check which modules are installed and retry the command without the -c option specified, if applicable."));
      amqMN_Ht.put("8701",new AMQMessageNumber(8701,
                  "Usage: rcdmqimg [-z] [-l] [-m QMgrName] -t ObjType [GenericObjName]",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8702",new AMQMessageNumber(8702,
                  "Usage: rcrmqobj [-z] [-m QMgrName] -t ObjType [GenericObjName]",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8703",new AMQMessageNumber(8703,
                  "Usage: dspmqfls [-m QMgrName] [-t ObjType] GenericObjName",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("8705",new AMQMessageNumber(8705,
                  "Display Queue Manager Status Details.",
                  "0 : Information",
                  "The MQSC DISPLAY QMSTATUS command completed successfully. Details follow this message.",
                  "None."));
      amqMN_Ht.put("8706",new AMQMessageNumber(8706,
                  "Request to stop WebSphere MQ Listener accepted.",
                  "0 : Information",
                  "The channel listener program has been requested to stop. This command executes asynchronously so may complete after this message has been displayed.",
                  "Further information on the progress of the request is available in the queue manager error log."));
      amqMN_Ht.put("8707",new AMQMessageNumber(8707,
                  "(iSeries) Start WebSphere MQ DLQ Handler",
                  "0 : Information",
                  "",
                  ""));
      amqMN_Ht.put("8708",new AMQMessageNumber(8708,
                  "Dead-letter queue handler started to process INPUTQ(<insert_3>).",
                  "0 : Information",
                  "The dead-letter queue handler (runmqdlq) has been started and has parsed the input file without detecting any errors and is about to start processing the queue identified in the message.",
                  "None."));
      amqMN_Ht.put("8708",new AMQMessageNumber(8708,
                  "(iSeries) Dead-letter queue handler started to process INPUTQ(<insert_3>).",
                  "0 : Information",
                  "The dead-letter queue handler (STRMQMDLQ) has been started and has parsed the input file without detecting any errors and is about to start processing the queue identified in the message.",
                  "None."));
      amqMN_Ht.put("8709",new AMQMessageNumber(8709,
                  "Dead-letter queue handler ending.",
                  "0 : Information",
                  "The dead-letter queue handler (runmqdlq) is ending because the WAIT interval has expired and there are no messages on the dead-letter queue, or because the queue manager is shutting down, or because the dead-letter queue handler has detected an error. If the dead-letter queue handler has detected an error, an earlier message will have identified the error.",
                  "None."));
      amqMN_Ht.put("8709",new AMQMessageNumber(8709,
                  "(iSeries) Dead-letter queue handler ending.",
                  "0 : Information",
                  "The dead-letter queue handler (STRMQMDLQ) is ending because the WAIT interval has expired and there are no messages on the dead-letter queue, or because the queue manager is shutting down, or because the dead-letter queue handler has detected an error. If the dead-letter queue handler has detected an error, an earlier message will have identified the error.",
                  "None."));
      amqMN_Ht.put("8710",new AMQMessageNumber(8710,
                  "Usage: runmqdlq [QName[QMgrName]].",
                  "0 : Information",
                  "Syntax for the usage of runmqdlq.",
                  "None."));
      amqMN_Ht.put("8711",new AMQMessageNumber(8711,
                  "(iSeries) Job <insert_5> has terminated unexpectedly.",
                  "10 : Warning",
                  "Execution of the command <insert_3> caused job <insert_5> to be started, but the job terminated unexpectedly.",
                  "Consult the log for job <insert_5> to determine why it was terminated."));
      amqMN_Ht.put("8721",new AMQMessageNumber(8721,
                  "Dead-letter queue message not prefixed by a valid MQDLH.",
                  "10 : Warning",
                  "The dead-letter queue handler (runmqdlq) retrieved a message from the nominated dead-letter queue, but the message was not prefixed by a recognizable MQDLH. This typically occurs because an application is writing directly to the dead-letter queue but is not prefixing messages with a valid MQDLH. The message is left on the dead-letter queue and the dead-letter queue handler continues to process the dead-letter queue. Each time the dead-letter queue handler repositions itself to a position before this message to process messages that could not be processed on a previous scan it will reprocess the failing message and will consequently re-issue this message.",
                  "Remove the invalid message from the dead-letter queue. Do not write messages to the dead-letter queue unless they have been prefixed by a valid MQDLH. If you require a dead-letter queue handler that can process messages not prefixed by a valid MQDLH, you must change the sample program called amqsdlq to cater for your needs."));
      amqMN_Ht.put("8721",new AMQMessageNumber(8721,
                  "(iSeries) Dead-letter queue message not prefixed by a valid MQDLH.",
                  "10 : Warning",
                  "The dead-letter queue handler (STRMQMDLQ) retrieved a message from the nominated dead-letter queue, but the message was not prefixed by a recognizable MQDLH. This typically occurs because an application is writing directly to the dead-letter queue but is not prefixing messages with a valid MQDLH. The message is left on the dead-letter queue and the dead-letter queue handler continues to process the dead-letter queue. Each time the dead-letter queue handler repositions itself to a position before this message to process messages that could not be processed on a previous scan it will reprocess the failing message and will consequently re-issue this message.",
                  "Remove the invalid message from the dead-letter queue. Do not write messages to the dead-letter queue unless they have been prefixed by a valid MQDLH. If you require a dead-letter queue handler that can process messages not prefixed by a valid MQDLH, you must change the sample program called amqsdlq to cater for your needs."));
      amqMN_Ht.put("8722",new AMQMessageNumber(8722,
                  "Dead-letter queue handler unable to put message: Rule <insert_1> Reason <insert_2>.",
                  "10 : Warning",
                  "This message is produced by the dead-letter queue handler when it is requested to redirect a message to another queue but is unable to do so. If the reason that the redirect fails is the same as the reason the message was put to the dead-letter queue then it is assumed that no new error has occurred and no message is produced. The retry count for the message will be incremented and the dead-letter queue handler will continue.",
                  "Investigate why the dead-letter queue handler was unable to put the message to the dead-letter queue. The line number of the rule used to determine the action for the message should be used to help identify to which queue the dead-letter queue handler attempted to PUT the message."));
      amqMN_Ht.put("8729",new AMQMessageNumber(8729,
                  "The listener could not be stopped at this time.",
                  "10 : Warning",
                  "A request was made to stop a listener, however the listener could not be stopped at this time. Reasons why a listener could not be stopped are: The listener has active channels and the communications protocol being used is LU 6.2, SPX or NETBIOS. The listener has active channels and the communications protocol being used is TCP/IP and channel threads are restricted to run within the listener process.",
                  "End the channels using the STOP CHANNEL command and reissue the request."));
      amqMN_Ht.put("8730",new AMQMessageNumber(8730,
                  "Listener already active.",
                  "10 : Warning",
                  "A request was made to start a listener, however the listener is already running and cannot be started.",
                  "If the listener should not be running then use the STOP LISTENER command to stop the listener before reissuing the command."));
      amqMN_Ht.put("8731",new AMQMessageNumber(8731,
                  "Listener not active.",
                  "10 : Warning",
                  "A request was made to stop a listener, however the listener is not running.",
                  "If the listener should be running then use the START LISTENER command to start the listener."));
      amqMN_Ht.put("8732",new AMQMessageNumber(8732,
                  "Request to stop Service accepted.",
                  "0 : Information",
                  "The Request to stop the Service has been accepted and is being processed.",
                  "None."));
      amqMN_Ht.put("8733",new AMQMessageNumber(8733,
                  "Request to start Service accepted.",
                  "0 : Information",
                  "The Request to start the Service has been accepted and is being processed.",
                  "None."));
      amqMN_Ht.put("8734",new AMQMessageNumber(8734,
                  "Command failed - Program could not be started.",
                  "20 : Error",
                  "The command requested was unsuccessful because the program which was defined to be run to complete the action could not be started. Reasons why the program could not be started are The program does not exist at the specified location. The WebSphere MQ user does not have sufficient access to execute the program. If STDOUT or STDERR are defined for the program, the WebSphere MQ user does not have sufficient access to the locations specified.",
                  "Check the Queue Manager error logs for further details on the cause of the failure and correct before reissuing the command."));
      amqMN_Ht.put("8735",new AMQMessageNumber(8735,
                  "Command failed - Access denied.",
                  "20 : Error",
                  "The command requested was unsuccessful because access was denied attempting to execuete the program defined to run.",
                  "Examine the definition of the object and ensure that the path to program file is correct. If the defined path is correct ensure that the program exists at the location specified and that the WebSphere MQ user has access to execute the program."));
      amqMN_Ht.put("8736",new AMQMessageNumber(8736,
                  "Command failed - Program start failed.",
                  "20 : Error",
                  "The command requested was unsuccessful because the attempt to execute the program defined to run was unsuccessful.",
                  "Examine the definition of the object and ensure that the path to program file is correct. If the defined path is correct ensure that the program exists at the location specified and that the WebSphere MQ user has access to execute the program. Further information on the failure may be available in the WebSphere MQ error logs."));
      amqMN_Ht.put("8737",new AMQMessageNumber(8737,
                  "Service already active.",
                  "10 : Warning",
                  "A request was made to start a service, however the service is already running and cannot be started.",
                  "If the service should not be running then use the STOP SERVICE command to stop the service before reissuing the command. If the intention is to allow more than one instance of s service to run, then the service definition may be altered to be of SERVTYPE(COMMAND) which allows more than one instance of the service to be executed concurrently, however status of services of type COMMAND is not available from the SVSTAUS command."));
      amqMN_Ht.put("8738",new AMQMessageNumber(8738,
                  "Service not active.",
                  "10 : Warning",
                  "A request was made to stop a service, however the service is not running.",
                  "If the service should be running then use the START SERVICE command to start the service."));
      amqMN_Ht.put("8739",new AMQMessageNumber(8739,
                  "Stop cannot be executed for service with blank STOPCMD.",
                  "20 : Error",
                  "A request was made to STOP a service, however the service has no Stop Command defined so no action could be taken.",
                  "Examine the definition of the service and if necessary update the definition of the service to include the command to run when STOP is issued. For services of type 'SERVER' the command to run when STOP is executed is stored when the service is started so any alteration to the service definition will have no effect until the service is restarted following the update."));
      amqMN_Ht.put("8740",new AMQMessageNumber(8740,
                  "Start cannot be executed for service with blank STARTCMD.",
                  "20 : Error",
                  "A request was made to START a service, however the service has no Start Command defined so no action could be taken.",
                  "Examine the definition of the service and if necessary update the definition of the service to include the command to run when START is issued."));
      amqMN_Ht.put("8741",new AMQMessageNumber(8741,
                  "Unable to connect to queue manager.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) could not connect to queue manager <insert_3>. This message is typically issued when the requested queue manager has not been started or is quiescing, or if the process does not have sufficient authority. The completion code (<insert_1>) and the reason (<insert_2>) can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8741",new AMQMessageNumber(8741,
                  "(iSeries) Unable to connect to queue manager.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) could not connect to queue manager <insert_3>. This message is typically issued when the requested queue manager has not been started or is quiescing, or if the process does not have sufficient authority. The completion code (<insert_1>) and the reason (<insert_2>) can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8742",new AMQMessageNumber(8742,
                  "Unable to open queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) could not open the queue manager object. This message is typically issued because of a resource shortage or because the process does not have sufficient authority. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8742",new AMQMessageNumber(8742,
                  "(iSeries) Unable to open queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) could not open the queue manager object. This message is typically issued because of a resource shortage or because the process does not have sufficient authority. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8743",new AMQMessageNumber(8743,
                  "Unable to inquire on queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) could not inquire on the queue manager. This message is typically issued because of a resource shortage or because the queue manager is ending. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8743",new AMQMessageNumber(8743,
                  "(iSeries) Unable to inquire on queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) could not inquire on the queue manager. This message is typically issued because of a resource shortage or because the queue manager is ending. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8744",new AMQMessageNumber(8744,
                  "Unable to close queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) could not close the queue manager. This message is typically issued because of a resource shortage or because the queue manager is ending. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8744",new AMQMessageNumber(8744,
                  "(iSeries) Unable to close queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) could not close the queue manager. This message is typically issued because of a resource shortage or because the queue manager is ending. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8745",new AMQMessageNumber(8745,
                  "Unable to open dead-letter queue for browse.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) could not open the dead-letter queue <insert_3> for browsing. This message is typically issued because another process has opened the dead-letter queue for exclusive access, or because an invalid dead-letter queue name was specified. Other possible reasons include resource shortages or insufficient authority. The completion code(<insert_1>) and the reason(<insert_2>) can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8745",new AMQMessageNumber(8745,
                  "(iSeries) Unable to open dead-letter queue for browse.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) could not open the dead-letter queue <insert_3> for browsing. This message is typically issued because another process has opened the dead-letter queue for exclusive access, or because an invalid dead-letter queue name was specified. Other possible reasons include resource shortages or insufficient authority. The completion code(<insert_1>) and the reason(<insert_2>) can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8746",new AMQMessageNumber(8746,
                  "Unable to close dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) could not close the dead-letter queue. This message is typically issued because of a resource shortage or because the queue manager is ending. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8746",new AMQMessageNumber(8746,
                  "(iSeries) Unable to close dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) could not close the dead-letter queue. This message is typically issued because of a resource shortage or because the queue manager is ending. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8747",new AMQMessageNumber(8747,
                  "Integer parameter outside permissible range.",
                  "20 : Error",
                  "The integer parameter (<insert_2>) supplied to the dead-letter handler was outside of the valid range for <insert_3> on line <insert_1>.",
                  "Correct the input data and restart the dead-letter queue handler."));
      amqMN_Ht.put("8748",new AMQMessageNumber(8748,
                  "Unable to get message from dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) could not get the next message from the dead-letter queue. This message is typically issued because of the queue manager ending, a resource problem, or another process having deleted the dead-letter queue. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8748",new AMQMessageNumber(8748,
                  "(iSeries) Unable to get message from dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) could not get the next message from the dead-letter queue. This message is typically issued because of the queue manager ending, a resource problem, or another process having deleted the dead-letter queue. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8749",new AMQMessageNumber(8749,
                  "Unable to commit/backout action on dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) was unable to commit or backout an update to the dead-letter queue. This message is typically issued because of the queue manager ending, or because of a resource shortage. If the queue manager has ended, the update to the dead-letter queue (and any associated updates) will be backed out when the queue manager restarts. If the problem was due to a resource problem then the updates will be backed out when the dead-letter queue handler terminates. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8749",new AMQMessageNumber(8749,
                  "(iSeries) Unable to commit/backout action on dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) was unable to commit or backout an update to the dead-letter queue. This message is typically issued because of the queue manager ending, or because of a resource shortage. If the queue manager has ended, the update to the dead-letter queue (and any associated updates) will be backed out when the queue manager restarts. If the problem was due to a resource problem then the updates will be backed out when the dead-letter queue handler terminates. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Take appropriate action based upon the completion code and reason."));
      amqMN_Ht.put("8750",new AMQMessageNumber(8750,
                  "No valid input provided to runmqdlq.",
                  "20 : Error",
                  "Either no input was provided to runmqdlq, or the input to runmqdlq contained no valid message templates. If input was provided to runmqdlq but was found to be invalid, earlier messages will have been produced explaining the cause of the error. The dead-letter queue handler will ends.",
                  "Correct the input data and restart the dead-letter queue handler."));
      amqMN_Ht.put("8750",new AMQMessageNumber(8750,
                  "(iSeries) No valid input provided to STRMQMDLQ.",
                  "20 : Error",
                  "Either no input was provided to STRMQMDLQ, or the input to STRMQMDLQ contained no valid message templates. If input was provided to STRMQMDLQ but was found to be invalid, earlier messages will have been produced explaining the cause of the error. The dead-letter queue handler will ends.",
                  "Correct the input data and restart the dead-letter queue handler."));
      amqMN_Ht.put("8751",new AMQMessageNumber(8751,
                  "Unable to obtain private storage.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) was unable to obtain private storage. This problem would typically arise as a result of some more global problem. For example if there is a persistent problem that is causing messages to be written to the DLQ and the same problem (for example queue full) is preventing the dead-letter queue handler from taking the requested action with the message, it is necessary for the dead-letter queue handler to maintain a large amount of state data to remember the retry counts associated with each message, or if the dead-letter queue contains a large number of messages and the rules table has directed the dead-letter queue handler to ignore the messages.",
                  "Investigate if some more global problem exists, and if the dead-letter queue contains a large number of messages. If the problem persists contact your support center."));
      amqMN_Ht.put("8751",new AMQMessageNumber(8751,
                  "(iSeries) Unable to obtain private storage.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) was unable to obtain private storage. This problem would typically arise as a result of some more global problem. For example if there is a persistent problem that is causing messages to be written to the DLQ and the same problem (for example queue full) is preventing the dead-letter queue handler from taking the requested action with the message, it is necessary for the dead-letter queue handler to maintain a large amount of state data to remember the retry counts associated with each message, or if the dead-letter queue contains a large number of messages and the rules table has directed the dead-letter queue handler to ignore the messages.",
                  "Investigate if some more global problem exists, and if the dead-letter queue contains a large number of messages. If the problem persists contact your support center."));
      amqMN_Ht.put("8752",new AMQMessageNumber(8752,
                  "Parameter(<insert_3>) exceeds maximum length on line <insert_1>.",
                  "20 : Error",
                  "A parameter supplied as input to the dead-letter handler exceeded the maximum length for parameters of that type.",
                  "Correct the input data and restart the dead-letter queue handler."));
      amqMN_Ht.put("8753",new AMQMessageNumber(8753,
                  "Duplicate parameter(<insert_3>) found on line <insert_1>.",
                  "20 : Error",
                  "Two or more parameters of the same type were supplied on a single input line to the dead-letter queue handler.",
                  "Correct the input and restart the dead-letter queue handler."));
      amqMN_Ht.put("8756",new AMQMessageNumber(8756,
                  "Error detected releasing private storage.",
                  "20 : Error",
                  "The dead-letter queue handler (runmqdlq) was informed of an error while attempting to release an area of private storage. The dead-letter queue handler ends.",
                  "This message should be preceded by a message or FFST information from the internal routine that detected the error. Take the action associated with the earlier error information."));
      amqMN_Ht.put("8756",new AMQMessageNumber(8756,
                  "(iSeries) Error detected releasing private storage.",
                  "20 : Error",
                  "The dead-letter queue handler (STRMQMDLQ) was informed of an error while attempting to release an area of private storage. The dead-letter queue handler ends.",
                  "This message should be preceded by a message or FFST information from the internal routine that detected the error. Take the action associated with the earlier error information."));
      amqMN_Ht.put("8757",new AMQMessageNumber(8757,
                  "Integer parameter(<insert_3>) outside permissible range on line <insert_1>.",
                  "20 : Error",
                  "An integer supplied as input to the dead-letter handler was outside of the valid range of integers supported by the dead-letter queue handler.",
                  "Correct the input data and restart the dead-letter queue handler."));
      amqMN_Ht.put("8758",new AMQMessageNumber(8758,
                  "<insert_1> errors detected in input to runmqdlq.",
                  "20 : Error",
                  "One or more errors have been detected in the input to the dead-letter queue handler(runmqdlq). Error messages will have been generated for each of these errors. The dead-letter queue handler ends.",
                  "Correct the input data and restart the dead-letter queue handler."));
      amqMN_Ht.put("8758",new AMQMessageNumber(8758,
                  "(iSeries) <insert_1> errors detected in input to STRMQMDLQ.",
                  "20 : Error",
                  "One or more errors have been detected in the input to the dead-letter queue handler(STRMQMDLQ). Error messages will have been generated for each of these errors. The dead-letter queue handler ends.",
                  "Correct the input data and restart the dead-letter queue handler."));
      amqMN_Ht.put("8759",new AMQMessageNumber(8759,
                  "Invalid combination of parameters to dead-letter queue handler on line <insert_1>.",
                  "20 : Error",
                  "An invalid combination of input parameters has been supplied to the dead-letter queue handler. Possible causes are: no ACTION specified, ACTION(FWD) but no FWDQ specified, HEADER(YES|NO) specified without ACTION(FWD).",
                  "Correct the input data and restart the dead-letter queue handler."));
      amqMN_Ht.put("8760",new AMQMessageNumber(8760,
                  "Unexpected failure while initializing process: Reason = <insert_1>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) could not perform basic initialization required to use MQ services because of an unforeseen error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8760",new AMQMessageNumber(8760,
                  "(iSeries) Unexpected failure while initializing process: Reason = <insert_1>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) could not perform basic initialization required to use MQ services because of an unforeseen error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8761",new AMQMessageNumber(8761,
                  "Unexpected failure while connecting to queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) could not connect to the requested queue manager because of an unforeseen error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8761",new AMQMessageNumber(8761,
                  "(iSeries) Unexpected failure while connecting to queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) could not connect to the requested queue manager because of an unforeseen error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8762",new AMQMessageNumber(8762,
                  "Unexpected error while attempting to open queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) could not open the queue manager because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8762",new AMQMessageNumber(8762,
                  "(iSeries) Unexpected error while attempting to open queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) could not open the queue manager because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8763",new AMQMessageNumber(8763,
                  "Unexpected error while inquiring on queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead letter queue handler (runmqdlq) could not inquire on the queue manager because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8763",new AMQMessageNumber(8763,
                  "(iSeries) Unexpected error while inquiring on queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead letter queue handler (STRMQMDLQ) could not inquire on the queue manager because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8764",new AMQMessageNumber(8764,
                  "Unexpected error while attempting to close queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) could not close the queue manager because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8764",new AMQMessageNumber(8764,
                  "(iSeries) Unexpected error while attempting to close queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) could not close the queue manager because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8765",new AMQMessageNumber(8765,
                  "Unexpected failure while opening dead-letter queue for browse: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) could not open the dead-letter queue for browsing because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8765",new AMQMessageNumber(8765,
                  "(iSeries) Unexpected failure while opening dead-letter queue for browse: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) could not open the dead-letter queue for browsing because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8766",new AMQMessageNumber(8766,
                  "Unexpected error while closing dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) could not close the dead-letter queue because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8766",new AMQMessageNumber(8766,
                  "(iSeries) Unexpected error while closing dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) could not close the dead-letter queue because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8767",new AMQMessageNumber(8767,
                  "Unexpected error while getting message from dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) could not get the next message from the dead-letter queue because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8767",new AMQMessageNumber(8767,
                  "(iSeries) Unexpected error while getting message from dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) could not get the next message from the dead-letter queue because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8768",new AMQMessageNumber(8768,
                  "Unexpected error committing/backing out action on dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) was unable to either commit or backout an update to the dead-letter queue because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8768",new AMQMessageNumber(8768,
                  "(iSeries) Unexpected error committing/backing out action on dead-letter queue: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) was unable to either commit or backout an update to the dead-letter queue because of an unforeseen error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8769",new AMQMessageNumber(8769,
                  "Unable to disconnect from queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (runmqdlq) was unable to disconnect from the queue manager because of an unexpected error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8769",new AMQMessageNumber(8769,
                  "(iSeries) Unable to disconnect from queue manager: CompCode = <insert_1> Reason = <insert_2>.",
                  "30 : Severe error",
                  "The dead-letter queue handler (STRMQMDLQ) was unable to disconnect from the queue manager because of an unexpected error. The completion code and the reason can be used to identify the error. The dead-letter queue handler ends.",
                  "Use the standard facilities supplied with your system to record the problem identifier and to save the generated output files. Contact your support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("8770",new AMQMessageNumber(8770,
                  "(iSeries) Cannot open <insert_5> for command <insert_3>.",
                  "40 : Stop Error",
                  "The <insert_3> command failed to open <insert_5> for WebSphere MQ processing.",
                  "Check that the intended file or member exists, and was specified correctly. Correct the specification or create the object and try the operation again."));
      amqMN_Ht.put("8822",new AMQMessageNumber(8822,
                  "Invalid response, please re-enter (y or n):",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("8919",new AMQMessageNumber(8919,
                  "There are no matching WebSphere MQ queue manager names.",
                  "30 : Severe error",
                  "",
                  ""));
   }

   /**
    * Initialize the AMQ 9000's message number. 
    */
   public void init9s()
   {
      amqMN_Ht.put("9001",new AMQMessageNumber(9001,
            "Channel <insert_3> ended normally.",
            "0 : Information",
            "Channel <insert_3> ended normally.",
            "None."));
      amqMN_Ht.put("9002",new AMQMessageNumber(9002,
                  "Channel <insert_3> is starting.",
                  "0 : Information",
                  "Channel <insert_3> is starting.",
                  "None."));
      amqMN_Ht.put("9003",new AMQMessageNumber(9003,
                  "(iSeries) Channel <insert_3> last message sequence number is <insert_1>.",
                  "0 : Information",
                  "Channel <insert_3> last message sequence number is <insert_1>.",
                  "None."));
      amqMN_Ht.put("9004",new AMQMessageNumber(9004,
                  "(iSeries) Channel <insert_3> status information.",
                  "0 : Information",
                  "Channel <insert_3> status information: Number of Messages in Doubt - <insert_1> In Doubt Sequence Number - <insert_2> In Doubt Logic Unit of Work ID - <insert_4>",
                  "None."));
      amqMN_Ht.put("9181",new AMQMessageNumber(9181,
                  "The response set by the exit is not valid.",
                  "30 : Severe error",
                  "The user exit <insert_3> returned a response code <insert_1> that is not valid in the ExitResponse field of the channel exit parameters (MQCXP). Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program set a response code that is not valid."));
      amqMN_Ht.put("9182",new AMQMessageNumber(9182,
                  "The secondary response set by the exit is not valid.",
                  "30 : Severe error",
                  "The user exit <insert_3> returned a secondary response code <insert_1> in the ExitResponse2 field of the channel exit parameters (MQCXP) that is not valid. Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program set a secondary response code that is not valid."));
      amqMN_Ht.put("9184",new AMQMessageNumber(9184,
                  "The exit buffer address set by the exit is not valid.",
                  "30 : Severe error",
                  "The user exit <insert_3> returned an address <insert_1> for the exit buffer that is not valid, when the secondary response code in the ExitResponse2 field of the channel exit parameters (MQCXP) is set to MQXR2_USE_EXIT_BUFFER. Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program set an exit buffer address that is not valid. The most likely cause is the failure to set a value, so that the value is 0."));
      amqMN_Ht.put("9185",new AMQMessageNumber(9185,
                  "The exit space set by the exit is not valid.",
                  "30 : Severe error",
                  "The user exit <insert_3> returned an exit space value <insert_1> that is not valid in the ExitSpace field of the channel exit parameters (MQCXP). Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program set an exit space value that is not valid. Correct the error."));
      amqMN_Ht.put("9186",new AMQMessageNumber(9186,
                  "Too much exit space reserved by send exits.",
                  "30 : Severe error",
                  "At exit initialization the send exits in the send exit chain for channel <insert_3> returned values in the ExitSpace field of the channel exit parameters (MQCXP). The total of these ExitSpace values is <insert_1>. The maximum number of bytes that can be sent in a single transmission is <insert_2>. Room must be left for at least 1024 bytes of message data in each transmission. So too much exit space has been reserved by the send exits. The channel stops.",
                  "Investigate why the send exit programs set exit space values that are too large. Correct the error."));
      amqMN_Ht.put("9187",new AMQMessageNumber(9187,
                  "The header compression value set by the exit is not valid.",
                  "30 : Severe error",
                  "The user exit <insert_3> returned a header compression value <insert_1> in the CurHdrCompression field of the channel exit parameters (MQCXP) that was not one of the negotiated supported values specified in the HdrCompList field of the channel description (MQCD). Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program specified a header compression value that was not one of the negotiated supported values."));
      amqMN_Ht.put("9188",new AMQMessageNumber(9188,
                  "The message compression value set by the exit is not valid.",
                  "30 : Severe error",
                  "The user exit <insert_3> returned a message compression value <insert_1> in the CurMsgCompression field of the channel exit parameters (MQCXP) that was not one of the negotiated supported values specified in the MsgCompList field of the channel description (MQCD). Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program specified a message compression value that was not one of the negotiated supported values."));
      amqMN_Ht.put("9189",new AMQMessageNumber(9189,
                  "The data length set by the exit is not valid.",
                  "30 : Severe error",
                  "The user exit <insert_3> returned a data length value <insert_1> that was not greater than zero. Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program set a data length that is not valid."));
      amqMN_Ht.put("9190",new AMQMessageNumber(9190,
                  "Channel stopping because of an error in the exit.",
                  "30 : Severe error",
                  "The user exit <insert_3>, invoked for channel <insert_4> with id <insert_1> and reason <insert_2>, returned values that are not valid, as reported in the preceding messages. The channel stops.",
                  "Investigate why the user exit program set values that are not valid."));
      amqMN_Ht.put("9195",new AMQMessageNumber(9195,
                  "Data length larger than maximum segment length.",
                  "30 : Severe error",
                  "The data length <insert_1> set by send exit <insert_3> is larger than the maximum segment length (<insert_2>). The maximum segment length is the maximum number of bytes that can be sent in a single transmission minus the user exit space required by all the send exits subsequent to the current one in the send exit chain. Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program set a data length that is not valid. Correct the error."));
      amqMN_Ht.put("9196",new AMQMessageNumber(9196,
                  "Data length is larger than the agent buffer length.",
                  "30 : Severe error",
                  "The data length <insert_1> set by exit <insert_3> is larger than the agent buffer length. The user exit returned data in the supplied agent buffer, but the length specified is greater than the length of the buffer. Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program set a data length that is not valid. Correct the error."));
      amqMN_Ht.put("9197",new AMQMessageNumber(9197,
                  "Data length is larger than the exit buffer length.",
                  "30 : Severe error",
                  "The data length <insert_1> set by exit <insert_3> is larger than the exit buffer length. The user exit returned data in the supplied exit buffer, but the length specified is greater than the length of the buffer. Message AMQ9190 is issued giving more details, and the channel stops.",
                  "Investigate why the user exit program set a data length that is not valid."));
      amqMN_Ht.put("9201",new AMQMessageNumber(9201,
                  "Allocate failed to host <insert_3>.",
                  "30 : Severe error",
                  "The attempt to allocate a conversation using <insert_4> to host <insert_3> was not successful.",
                  "The error may be due to an incorrect entry in the <insert_4> parameters contained in the channel definition to host <insert_3>. Correct the error and try again. If the error persists, record the error values and contact your systems administrator. The return code from the <insert_4><insert_5> call was <insert_1> (X<insert_2>). It may be possible that the listening program at host <insert_3> is not running. If this is the case, perform the relevant operations to start the listening program for protocol <insert_4> and try again."));
      amqMN_Ht.put("9202",new AMQMessageNumber(9202,
                  "Remote host <insert_3> not available, retry later.",
                  "30 : Severe error",
                  "The attempt to allocate a conversation using <insert_4> to host <insert_3> was not successful. However the error may be a transitory one and it may be possible to successfully allocate a <insert_4> conversation later.",
                  "Try the connection again later. If the failure persists, record the error values and contact your systems administrator. The return code from <insert_4> is <insert_1> (X<insert_2>). The reason for the failure may be that this host cannot reach the destination host. It may also be possible that the listening program at host <insert_3> was not running. If this is the case, perform the relevant operations to start the <insert_4> listening program, and try again."));
      amqMN_Ht.put("9203",new AMQMessageNumber(9203,
                  "A configuration error for <insert_4> occurred.",
                  "30 : Severe error",
                  "Error in configuration for communications to host <insert_3>. Allocation of a <insert_4> conversation to host <insert_3> was not possible.",
                  "The configuration error may be one of the following:\n 1.If the communications protocol is LU 6.2, it may be that one of the transmission parameters (Mode, or TP Name) is incorrect. Correct the error and try again. The mode name should be the same as the mode defined on host <insert_3>. The TP name on <insert_3> should be defined.\n 2.If the communications protocol is LU 6.2, it may be that an LU 6.2 session has not been established. Contact your systems administrator.\n 3.If the communications protocol is TCP/IP, it may be that the host name specified is incorrect. Correct the error and try again.\n 4.If the communications protocol is TCP/IP, it may be that the host name specified cannot be resolved to a network address. The host name may not be in the nameserver. The return code from the <insert_4><insert_5> call was <insert_1> (X<insert_2>). Record the error values and tell the system administrator."));
      amqMN_Ht.put("9204",new AMQMessageNumber(9204,
                  "Connection to host <insert_3> rejected.",
                  "30 : Severe error",
                  "Connection to host <insert_3> over <insert_4> was rejected.",
                  "The remote system may not be configured to allow connections from this host. Check the <insert_4> listener program has been started on host <insert_3>. If the conversation uses LU 6.2, it is possible that either the User ID or Password supplied to the remote host is incorrect. If the conversation uses TCP/IP, it is possible that the remote host does not recognize the local host as a valid host. The return code from the <insert_4><insert_5> call was <insert_1> X(<insert_2>). Record the error values and tell the systems administrator."));
      amqMN_Ht.put("9205",new AMQMessageNumber(9205,
                  "The host name supplied is not valid.",
                  "30 : Severe error",
                  "The supplied <insert_4> host name <insert_3> could not be resolved into a network address. Either the name server does not contain the host, or the name server was not available.",
                  "Check the <insert_4> configuration on your host."));
      amqMN_Ht.put("9206",new AMQMessageNumber(9206,
                  "Error sending data to host <insert_3>.",
                  "30 : Severe error",
                  "An error occurred sending data over <insert_4> to <insert_3>. This may be due to a communications failure.",
                  "The return code from the <insert_4><insert_5> call was <insert_1> X(<insert_2>). Record these values and tell your systems administrator."));
      amqMN_Ht.put("9207",new AMQMessageNumber(9207,
                  "The data received from host <insert_3> is not valid.",
                  "30 : Severe error",
                  "Incorrect data format received from host <insert_3> over <insert_4>. It may be that an unknown host is attempting to send data. An FFST file has been generated containing the invalid data received.",
                  "Tell the systems administrator."));
      amqMN_Ht.put("9208",new AMQMessageNumber(9208,
                  "Error on receive from host <insert_3>.",
                  "30 : Severe error",
                  "An error occurred receiving data from <insert_3> over <insert_4>. This may be due to a communications failure.",
                  "The return code from the <insert_4><insert_5> call was <insert_1> (X<insert_2>). Record these values and tell the systems administrator."));
      amqMN_Ht.put("9209",new AMQMessageNumber(9209,
                  "Connection to host <insert_3> closed.",
                  "30 : Severe error",
                  "An error occurred receiving data from <insert_3> over <insert_4>. The connection to the remote host has unexpectedly terminated.",
                  "Tell the systems administrator."));
      amqMN_Ht.put("9210",new AMQMessageNumber(9210,
                  "Remote attachment failed.",
                  "30 : Severe error",
                  "There was an incoming attachment from a remote host, but the local host could not complete the bind.",
                  "The return code from the <insert_4><insert_5> call was <insert_1> (X<insert_2>). Record these values and tell the systems administrator who should check the <insert_4> configuration."));
      amqMN_Ht.put("9211",new AMQMessageNumber(9211,
                  "Error allocating storage.",
                  "30 : Severe error",
                  "The program was unable to obtain enough storage.",
                  "Stop some programs which are using storage and retry the operation. If the problem persists contact your systems administrator."));
      amqMN_Ht.put("9212",new AMQMessageNumber(9212,
                  "A TCP/IP socket could not be allocated.",
                  "30 : Severe error",
                  "A TCP/IP socket could not be created, possibly because of a storage problem.",
                  "The return code from the <insert_4><insert_5> call was <insert_1> (X<insert_2>). Try the program again. If the failure persists, record the error values and tell the systems administrator."));
      amqMN_Ht.put("9213",new AMQMessageNumber(9213,
                  "A communications error for <insert_4> occurred.",
                  "30 : Severe error",
                  "An unexpected error occurred in communications.",
                  "The return code from the <insert_4><insert_5> call was <insert_1> (X<insert_2>). Record these values and tell the systems administrator."));
      amqMN_Ht.put("9214",new AMQMessageNumber(9214,
                  "Attempt to use an unsupported communications protocol.",
                  "30 : Severe error",
                  "An attempt was made to use an unsupported communications protocol type <insert_2>.",
                  "Check the channel definition file. It may be that the communications protocol entered is not a currently supported one."));
      amqMN_Ht.put("9215",new AMQMessageNumber(9215,
                  "Communications subsystem unavailable.",
                  "30 : Severe error",
                  "An attempt was made to use the communications subsystem, but it has not been started.",
                  "Start the communications subsystem, and rerun the program."));
      amqMN_Ht.put("9216",new AMQMessageNumber(9216,
                  "Usage: <insert_3> [-m QMgrName] [-n TPName]",
                  "20 : Error",
                  "Values passed to the responder channel program are not valid. The parameters that are not valid are as follows :- <insert_4> The responder channel program exits.",
                  "Correct the parameters passed to the channel program and retry the operation."));
      amqMN_Ht.put("9216",new AMQMessageNumber(9216,
                  " (AIX)Usage: <insert_3> [-m QMgrName]",
                  "20 : Error",
                  "Values passed to the responder channel program are not valid. The parameters that are not valid are as follows :- <insert_4> The responder channel program exits.",
                  "Correct the parameters passed to the channel program and retry the operation."));
      amqMN_Ht.put("9216",new AMQMessageNumber(9216,
                  " (HP-UX)Usage: <insert_3> [-m QMgrName]",
                  "20 : Error",
                  "Values passed to the responder channel program are not valid. The parameters that are not valid are as follows :- <insert_4> The responder channel program exits.",
                  "Correct the parameters passed to the channel program and retry the operation."));
      amqMN_Ht.put("9217",new AMQMessageNumber(9217,
                  "The TCP/IP listener program could not be started.",
                  "30 : Severe error",
                  "An attempt was made to start a new instance of the listener program, but the program was rejected.",
                  "The failure could be because either the subsystem has not been started (in this case you should start the subsystem), or there are too many programs waiting (in this case you should try to start the listener program later)."));
      amqMN_Ht.put("9218",new AMQMessageNumber(9218,
                  "The <insert_4> listener program could not bind to port number <insert_1>.",
                  "30 : Severe error",
                  "An attempt to bind the <insert_4> socket to the listener port was unsuccessful.",
                  "The failure could be due to another program using the same port number. The return code from the <insert_3> call for port <insert_5><insert_1> was <insert_2>. Record these values and tell the systems administrator."));
      amqMN_Ht.put("9219",new AMQMessageNumber(9219,
                  "The TCP/IP listener program could not create a new connection for the incoming conversation.",
                  "30 : Severe error",
                  "An attempt was made to create a new socket because an attach request was received, but an error occurred.",
                  "The failure may be transitory, try again later. If the problem persists, record the return code <insert_1> and tell the systems administrator. It may be necessary to free some jobs, or restart the communications system."));
      amqMN_Ht.put("9220",new AMQMessageNumber(9220,
                  "The <insert_4> communications program could not be loaded.",
                  "30 : Severe error",
                  "The attempt to load the <insert_4> library or procedure <insert_3> failed with error code <insert_1>.",
                  "Either the library must be installed on the system or the environment changed to allow the program to locate it."));
      amqMN_Ht.put("9221",new AMQMessageNumber(9221,
                  "Unsupported protocol was specified.",
                  "30 : Severe error",
                  "The specified value of <insert_3> was not recognized as one of the protocols supported.",
                  "Correct the parameter and retry the operation."));
      amqMN_Ht.put("9222",new AMQMessageNumber(9222,
                  "Cannot find the configuration file.",
                  "10 : Warning",
                  "The configuration file <insert_3> cannot be found. This file contains default definitions for communication parameters. Default values will be used.",
                  "None."));
      amqMN_Ht.put("9223",new AMQMessageNumber(9223,
                  "Enter a protocol type.",
                  "30 : Severe error",
                  "The operation you are performing requires that you enter the type of protocol.",
                  "Add the protocol parameter and retry the operation."));
      amqMN_Ht.put("9224",new AMQMessageNumber(9224,
                  "Unexpected token detected.",
                  "30 : Severe error",
                  "On line <insert_1> of the INI file, keyword <insert_3> was read when a keyword was expected.",
                  "Correct the file and retry the operation."));
      amqMN_Ht.put("9224",new AMQMessageNumber(9224,
                  " (Windows)Unexpected token detected.",
                  "30 : Severe error",
                  "Keyword <insert_3> was read when a keyword was expected.",
                  "Correct the configuration data and retry the operation."));
      amqMN_Ht.put("9225",new AMQMessageNumber(9225,
                  "File syntax error.",
                  "30 : Severe error",
                  "A syntax error was detected on line <insert_1> while processing the INI file.",
                  "Correct the problem and retry the operation."));
      amqMN_Ht.put("9225",new AMQMessageNumber(9225,
                  " (Windows)File syntax error.",
                  "30 : Severe error",
                  "A syntax error was detected while processing the configuration data.",
                  "Correct the problem and retry the operation."));
      amqMN_Ht.put("9226",new AMQMessageNumber(9226,
                  "Usage: <insert_3> [-m QMgrName] -t (TCP | LU62 | NETBIOS | SPX) [ProtocolOptions]",
                  "10 : Warning",
                  "Values passed to the listener program were invalid. The parameter string passed to this program is as follows: [-m QMgrName] ( -t TCP [-p Port] |\n-t LU62 [-n TPName] |\n-t NETBIOS [-l LocalName] [-e Names] [-s Sessions] [-o Commands] [-a Adaptor] |\n-t SPX [-x Socket]) Default values will be used for parameters not supplied.",
                  "Correct the parameters passed to the listener program and retry the operation."));
      amqMN_Ht.put("9226",new AMQMessageNumber(9226,
                  " (AIX)Usage: <insert_3> [-m QMgrName] -t TCP [ProtocolOptions]",
                  "10 : Warning",
                  "Values passed to the listener program were invalid. The parameter string passed to this program is as follows: [-m QMgrName] -t TCP [-p Port] Default values will be used for parameters not supplied.",
                  "Correct the parameters passed to the listener program and retry the operation."));
      amqMN_Ht.put("9226",new AMQMessageNumber(9226,
                  " (Unix)Usage: <insert_3> [-m QMgrName] -t TCP [ProtocolOptions]",
                  "10 : Warning",
                  "Values passed to the listener program were invalid. The parameter string passed to this program is as follows: [-m QMgrName] -t TCP [-p Port] Default values will be used for parameters not supplied.",
                  "Correct the parameters passed to the listener program and retry the operation."));
      amqMN_Ht.put("9227",new AMQMessageNumber(9227,
                  "<insert_3> local host name not provided.",
                  "30 : Severe error",
                  "A name is required for the <insert_3> process to register with the network.",
                  "Add a local name to the configuration file and retry the operation."));
      amqMN_Ht.put("9228",new AMQMessageNumber(9228,
                  "The <insert_4> responder program could not be started.",
                  "30 : Severe error",
                  "An attempt was made to start an instance of the responder program, but the program was rejected.",
                  "The failure could be because either the subsystem has not been started (in this case you should start the subsystem), or there are too many programs waiting (in this case you should try to start the responder program later). The <insert_5> reason code was <insert_1>."));
      amqMN_Ht.put("9229",new AMQMessageNumber(9229,
                  "The application has been ended.",
                  "30 : Severe error",
                  "You have issued a request to end the application.",
                  "None."));
      amqMN_Ht.put("9230",new AMQMessageNumber(9230,
                  "An unexpected <insert_4> event occurred.",
                  "30 : Severe error",
                  "During the processing of network events, an unexpected event <insert_1> occurred.",
                  "None."));
      amqMN_Ht.put("9231",new AMQMessageNumber(9231,
                  "The supplied parameter is not valid.",
                  "30 : Severe error",
                  "The value of the <insert_4> <insert_5> parameter has the value <insert_3>. This value has either not been specified or has been specified incorrectly.",
                  "Check value of the <insert_5> parameter and correct it if necessary. If the fault persists, record the return code (<insert_1>,<insert_2>) and <insert_4> and tell the systems administrator."));
      amqMN_Ht.put("9232",new AMQMessageNumber(9232,
                  "No <insert_3> specified",
                  "30 : Severe error",
                  "The operation requires the specification of the <insert_3> field.",
                  "Specify the <insert_3> and retry the operation."));
      amqMN_Ht.put("9233",new AMQMessageNumber(9233,
                  "Error creating <insert_3> thread.",
                  "30 : Severe error",
                  "The process attempted to create a new thread. The most likely cause of this problem is a shortage of an operating system resource (for example: memory). Use any previous FFSTs to determine the reason for the failure. The WebSphere MQ internal return code describing the reason for the failure is <insert_1>.",
                  "Contact the systems administrator. If the problem persists contact your IBM support center."));
      amqMN_Ht.put("9235",new AMQMessageNumber(9235,
                  "The supplied local communications address cannot be resolved.",
                  "30 : Severe error",
                  "The local communications address (LOCLADDR) value <insert_3> cannot be resolved into an IP address.",
                  "Enter a local communications address value which can be resolved into an IP address, and try again."));
      amqMN_Ht.put("9236",new AMQMessageNumber(9236,
                  "The supplied Partner LU was invalid.",
                  "30 : Severe error",
                  "The <insert_4> Partner LU name <insert_3> was invalid.",
                  "Either the Partner LU name was entered incorrectly or it was not in the <insert_4> communications configuration. Correct the error and try again."));
      amqMN_Ht.put("9237",new AMQMessageNumber(9237,
                  "A configuration error for <insert_4> occurred.",
                  "30 : Severe error",
                  "Allocation of a <insert_4> conversation to host <insert_3> was not possible. The configuration error may be one of the following: 1. It may be that one of the transmission parameters (Mode, or TP Name) was incorrect. Correct the error and try again. The mode name should be the same as the mode defined on host <insert_3>. The TP name on <insert_3> should be defined.\n 2. It may be that an LU 6.2 session has not been established. Contact your systems administrator. The return code from <insert_4> is <insert_1> with associated <insert_5> <insert_2>.",
                  "Record the error values and tell the system administrator."));
      amqMN_Ht.put("9238",new AMQMessageNumber(9238,
                  "A communications error for <insert_4> occurred.",
                  "30 : Severe error",
                  "An unexpected error occurred in communications.",
                  "The return code from the <insert_4><insert_3> call was <insert_1> with associated <insert_5> <insert_2>."));
      amqMN_Ht.put("9239",new AMQMessageNumber(9239,
                  "Usage: <insert_3> [-m QMgrName] -n TpName -g Gateway-name",
                  "10 : Warning",
                  "Values passed to the listener program were invalid. The parameter string passed to this program is as follows, default values being used for parameters not supplied: [-m QMgrName] -n TpName -g Gateway-name",
                  "Correct the parameters passed to the listener program and retry the operation."));
      amqMN_Ht.put("9240",new AMQMessageNumber(9240,
                  "An SPX socket was already in use.",
                  "30 : Severe error",
                  "The Listener received return code <insert_1> when attempting to open socket <insert_2>.",
                  "The specified socket is already in use by another process. To use another socket specify another socket on the command line to RUNMQLSR or update the default in the qm.ini file."));
      amqMN_Ht.put("9240",new AMQMessageNumber(9240,
                  "(iSeries) An SPX socket was already in use.",
                  "30 : Severe error",
                  "The Listener received return code <insert_1> when attempting to open socket <insert_2>.",
                  "The specified socket is already in use by another process. To use another socket specify another socket on the command line to STRMQMLSR or update the default in the qm.ini file."));
      amqMN_Ht.put("9240",new AMQMessageNumber(9240,
                  " (Windows)An SPX socket was already in use.",
                  "30 : Severe error",
                  "The listener received return code <insert_1> when attempting to open socket <insert_2>.",
                  "The specified socket is already in use by another process. To use another socket, specify a different socket on the command line to the runmqlsr command, or update the default in the configuration data."));
      amqMN_Ht.put("9241",new AMQMessageNumber(9241,
                  "SPX is not available.",
                  "30 : Severe error",
                  "WebSphere MQ received return code <insert_1> when attempting to start SPX communications.",
                  "Ensure that IPX/SPX support is installed on the machine and that it is started before trying to start a WebSphere MQ SPX channel."));
      amqMN_Ht.put("9242",new AMQMessageNumber(9242,
                  "SPX resource problem.",
                  "30 : Severe error",
                  "WebSphere MQ received return code <insert_1> when attempting to start SPX communications, indicating a resource problem.",
                  "Ensure that sufficient IPX/SPX resources are available before commencing communications over IPX/SPX."));
      amqMN_Ht.put("9243",new AMQMessageNumber(9243,
                  "The queue manager <insert_3> does not exist.",
                  "30 : Severe error",
                  "You tried to perform an action against a queue manager that does not exist. You may have specified the wrong queue manager name.",
                  "If you specified the wrong name, correct the name and submit the command again. If the queue manager does not exist, create the queue manager and submit the command again."));
      amqMN_Ht.put("9244",new AMQMessageNumber(9244,
                  "The default queue manager does not exist.",
                  "30 : Severe error",
                  "You tried to perform an action against a queue manager that does not exist.",
                  "Create the default queue manager and submit the command again."));
      amqMN_Ht.put("9245",new AMQMessageNumber(9245,
                  " (Windows)Unable to obtain account details for channel MCA user ID.",
                  "10 : Warning",
                  "WebSphere MQ was unable to obtain the account details for MCA user ID <insert_3>. This user ID was the MCA user ID for channel <insert_4> on queue manager <insert_5> and may have been defined in the channel definition, or supplied either by a channel exit or by a client.",
                  "Ensure that the user ID is correct and that it is defined on the Windows local system, the local domain or on a trusted domain. For a domain user ID, ensure that all necessary domain controllers are available."));
      amqMN_Ht.put("9246",new AMQMessageNumber(9246,
                  "The TCP/IP listener on port <insert_1> could not start a new channel.",
                  "30 : Severe error",
                  "An attempt has been made to connect to the queue manager by starting a new channel within the TCP/IP listener which is listening on port <insert_1>. The maximum socket number which can be used by a channel running on this listener is <insert_2>. A socket number beyond this maximum was allocated for the new channel. This connection attempt has been rejected, but the listener continues to listen for further connection requests. The socket number allocated for a new listener channel is related to the number of channels currently running within that listener process. The problem has arisen because too many channels are directed at the port on which this listener is listening.",
                  "An extra listener process should be started to listen on a different port. Some of the channels to the queue manager should be redirected from the port on which the existing listener is listening to the new port."));
      amqMN_Ht.put("9247",new AMQMessageNumber(9247,
                  "SSPI Security: bad return from SSPI call.",
                  "30 : Severe error",
                  "Channel <insert_3> has been closed because the SSPI channel exit received a bad return code from SSPI.",
                  "Consult the appropriate SSPI manuals to find out the meaning of status <insert_4> on call <insert_5>, and correct the error."));
      amqMN_Ht.put("9248",new AMQMessageNumber(9248,
                  "The program could not bind to a <insert_3> socket.",
                  "30 : Severe error",
                  "The attempt to bind to socket <insert_4> failed with return code <insert_1>. The failing <insert_3> call was <insert_5>. The most likely cause of this problem is incorrect configuration of the <insert_3> local address or incorrect start and end port parameters.",
                  "Contact the system administrator. If the problem persists contact your IBM support center."));
      amqMN_Ht.put("9255",new AMQMessageNumber(9255,
                  "Listener already running.",
                  "30 : Severe error",
                  "The request to start the WebSphere MQ listener failed because there is already a listener running against the specified network resources.",
                  "None."));
      amqMN_Ht.put("9259",new AMQMessageNumber(9259,
                  "Connection timed out from host <insert_3>.",
                  "30 : Severe error",
                  "A connection from host <insert_3> over <insert_4> timed out.",
                  "Check to see why data was not received in the expected time. Correct the problem. Reconnect the channel, or wait for a retrying channel to reconnect itself."));
      amqMN_Ht.put("9401",new AMQMessageNumber(9401,
                  "Channel <insert_3> autodefined.",
                  "0 : Information",
                  "Channel <insert_3> which did not previously exist has been autodefined.",
                  "None."));
      amqMN_Ht.put("9402",new AMQMessageNumber(9402,
                  "Autodefinition exit for Channel <insert_3> failed to load.",
                  "30 : Severe error",
                  "Autodefinition of Channel <insert_3> failed because <insert_4> would not load.",
                  "Ensure that the user exit is specified correctly in the queue manager definition, and that the user exit program is correct and available."));
      amqMN_Ht.put("9403",new AMQMessageNumber(9403,
                  "Autodefinition of Channel <insert_3> suppressed by user exit.",
                  "30 : Severe error",
                  "Autodefinition exit <insert_4> for Channel <insert_3> returned a failure code.",
                  "None."));
      amqMN_Ht.put("9404",new AMQMessageNumber(9404,
                  "REFRESH CLUSTER REPOS(YES) command processed, cluster <insert_4>, <insert_1> objects changed.",
                  "0 : Information",
                  "The queue manager successfully processed a REFRESH CLUSTER command with the REPOS(YES) option for the indicated cluster.",
                  "None."));
      amqMN_Ht.put("9405",new AMQMessageNumber(9405,
                  "FORCEREMOVE QUEUES(YES) command processed, cluster <insert_3> target <insert_4>.",
                  "0 : Information",
                  "The repository queue manager successfully processed a RESET ACTION(FORCEREMOVE) command with the QUEUES(YES) option for the indicated cluster and target queue manager.",
                  "None."));
      amqMN_Ht.put("9406",new AMQMessageNumber(9406,
                  "REFRESH CLUSTER REPOS(YES) command failed, this queue manager is a full repository for cluster <insert_4>.",
                  "30 : Severe error",
                  "The repository queue manager could not process a REFRESH CLUSTER command with the REPOS(YES) option for the indicated cluster, because the local queue manager provides full repository management services for the cluster. The command is ignored.",
                  "Either 1) Reissue the command without REPOS(YES), or 2) Issue the command on a queue manager which is not a full repository, or 3) Change this queue manager definition so that it is not a full repository."));
      amqMN_Ht.put("9407",new AMQMessageNumber(9407,
                  "Cluster queue <insert_3> is defined inconsistently.",
                  "10 : Warning",
                  "The definition of cluster queue <insert_3> on the queue manager with UUID <insert_4> has different DEFPRTY, DEFPSIST and DEFBIND values from the definition of the same cluster queue on the queue manager with UUID <insert_5>. Both definitions now exist in the local repository. All definitions of the same cluster queue should be identical. In particular, problems arise if your applications rely on a queue default value which is defined inconsistently to determine messaging behavior. This applies, for example, if the applications open a cluster queue with option MQOO_BIND_AS_Q_DEF. If different instances of the queue have different DEFBIND values the behavior of the message transfer differs depending on which instance of the queue is selected when it is opened. In general the instance selected varies across opens.",
                  "For each inconsistency decide which of the values is the correct one. Alter the definitions of cluster queue <insert_3> so that all definitions have correct DEFPRTY, DEFPSIST and DEFBIND values."));
      amqMN_Ht.put("9408",new AMQMessageNumber(9408,
                  "BIND_ON_OPEN messages for channel <insert_3> to dead-letter queue.",
                  "0 : Information",
                  "The remote CLUSRCVR for channel <insert_3> was deleted while undelivered BIND_ON_OPEN messages associated with that channel existed on the local SYSTEM.CLUSTER.TRANSMIT.QUEUE. These messages could not be allocated to another channel because they were put BIND_ON_OPEN, but were very unlikely to ever flow along the channel with which they were associated as this has now been deleted. An attempt has therefore been made to move them from the transmission queue to the local dead-letter queue. The MQDLH reason is MQFB_BIND_OPEN_CLUSRCVR_DEL. Note that any internal WebSphere MQ Clustering messages for the deleted channel will also have been removed from the SYSTEM.CLUSTER.TRANSMIT.QUEUE (these are discarded) so the current depth of the queue may have decreased by more than the number of user messages moved to the dead-letter queue.",
                  "Examine the contents of the dead-letter queue. Each message is contained in an MQDLH structure that includes the reason why it was written and where it was originally addressed. Also look at previous error messages to see if the attempt to put messages to the dead-letter queue failed."));
      amqMN_Ht.put("9409",new AMQMessageNumber(9409,
                  "Repository manager ended abnormally.",
                  "30 : Severe error",
                  "The repository manager ended abnormally.",
                  "Look at previous error messages for the repository manager in the error files to determine the cause of the failure."));
      amqMN_Ht.put("9410",new AMQMessageNumber(9410,
                  "Repository manager started",
                  "0 : Information",
                  "The repository manager started successfully.",
                  "None."));
      amqMN_Ht.put("9411",new AMQMessageNumber(9411,
                  "Repository manager ended normally.",
                  "0 : Information",
                  "The repository manager ended normally.",
                  "None."));
      amqMN_Ht.put("9412",new AMQMessageNumber(9412,
                  "Repository command received for <insert_3>.",
                  "30 : Severe error",
                  "The repository manager received a command intended for some other queue manager, whose identifier is <insert_3>. The command was sent by the queue manager with identifier <insert_4>.",
                  "Check the channel and cluster definitions of the sending queue manager."));
      amqMN_Ht.put("9413",new AMQMessageNumber(9413,
                  "Repository command format error, command code <insert_1>",
                  "30 : Severe error",
                  "An internal error has occurred.",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9415",new AMQMessageNumber(9415,
                  "Repository command unexpected, command code <insert_1>, cluster object <insert_3>, sender <insert_4>",
                  "30 : Severe error",
                  "An internal error has occurred.",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9415",new AMQMessageNumber(9415,
                  "(iSeries) An internal error has occurred.",
                  "30 : Severe error",
                  "Repository command unexpected, command code <insert_1>, cluster object <insert_3>, sender <insert_4>",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9416",new AMQMessageNumber(9416,
                  "Repository command processing error, RC=<insert_2>, command code <insert_1>, cluster object <insert_3>, sender <insert_4>.",
                  "30 : Severe error",
                  "An internal error has occurred.",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9416",new AMQMessageNumber(9416,
                  "(iSeries) An internal error has occurred.",
                  "30 : Severe error",
                  "Repository command processing error, RC=<insert_2>, command code <insert_1>, cluster object <insert_3>, sender <insert_4>.",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9417",new AMQMessageNumber(9417,
                  "Manually defined CLUSSDR channels have been forcibly removed.",
                  "0 : Information",
                  "The administrator has asked for the queue manager <insert_3> to be deleted, or forcibly removed, but has not yet deleted the manually defined CLUSSDR channels to <insert_3>. The auto-defined channels to <insert_3> have been deleted, but <insert_3> continues to receive updates until the manually defined CLUSSDR channels have been deleted.",
                  "Delete the manually defined CLUSSDR channels to <insert_3>."));
      amqMN_Ht.put("9418",new AMQMessageNumber(9418,
                  "Only one repository for cluster <insert_3>.",
                  "0 : Information",
                  "The queue manager has received information about a cluster for which it is the only repository.",
                  "Alter the REPOS or REPOSNL attribute of the queue manager, that is to have the second full repository for the cluster, to specify the cluster name."));
      amqMN_Ht.put("9419",new AMQMessageNumber(9419,
                  "No cluster-receiver channels for cluster <insert_3>",
                  "0 : Information",
                  "The repository manager has received information about a cluster for which no cluster-receiver channels are known.",
                  "Define cluster-receiver channels for the cluster on the local queue manager."));
      amqMN_Ht.put("9420",new AMQMessageNumber(9420,
                  "No repositories for cluster <insert_3>.",
                  "0 : Information",
                  "The queue manager has received information about a cluster for which no repositories are known.",
                  "Alter the REPOS or REPOSNL attribute of the queue manager, that is to have a full repository for the cluster, to specify the cluster name."));
      amqMN_Ht.put("9421",new AMQMessageNumber(9421,
                  "Invalid cluster record action code detected",
                  "30 : Severe error",
                  "An invalid record was read from the SYSTEM.CLUSTER.REPOSITORY.QUEUE. An FFST record has been generated containing the invalid record. was",
                  "Collect the items listed in the Problem Determination section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9422",new AMQMessageNumber(9422,
                  "Repository manager error, RC=<insert_1>",
                  "30 : Severe error",
                  "An internal error has occurred.",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9425",new AMQMessageNumber(9425,
                  "An internal error has occurred.",
                  "30 : Severe error",
                  "Repository command merge error, command code <insert_1>, cluster object <insert_3>, sender <insert_4>",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9426",new AMQMessageNumber(9426,
                  "Repository command recipient unknown.",
                  "30 : Severe error",
                  "The repository manager tried to send a command to another queue manager using channel <insert_4>. The recipient queue manager, whose identifier is <insert_3>, could not be found. Command code <insert_1>.",
                  "Check the channel and cluster definitions of the sending and receiving queue managers."));
      amqMN_Ht.put("9427",new AMQMessageNumber(9427,
                  "CLUSSDR channel does not point to a repository queue manager.",
                  "30 : Severe error",
                  "A CLUSSDR channel must point to a queue manager that hosts repositories for all clusters of which the channel is a member. In addition, the CLUSRCVR for the channel must be a member of all the same clusters as the CLUSSDR channel. The queue manager pointed to by CLUSSDR channel <insert_3> does not meet these criteria for cluster <insert_4>. The remote queue manager has a QMID of <insert_5>.",
                  "Check the definitions on the local and remote queue managers to ensure that the CLUSSDR channel points to a queue manager that hosts a repository for the cluster, and that the CLUSRCVR for the channel is a member of the cluster."));
      amqMN_Ht.put("9428",new AMQMessageNumber(9428,
                  "Unexpected publication of a cluster queue object received.",
                  "30 : Severe error",
                  "The local queue manager has received a publication of a cluster queue object from a remote queue manager on cluster <insert_3>. The local queue manager discards the request because it does not host a repository for cluster <insert_3> and has not subscribed to the published object. The remote CLUSSDR channel used to access the local queue manager has a channel name of <insert_4> and the remote queue manager has a QMID of <insert_5>.",
                  "Check the definitions on the local and remote queue managers to ensure that the CLUSSDR channel points to a repository queue manager for the cluster."));
      amqMN_Ht.put("9429",new AMQMessageNumber(9429,
                  "Unexpected publication of a cluster queue deletion received.",
                  "30 : Severe error",
                  "The local queue manager has received a publication of a cluster queue deletion from a remote queue manager on cluster <insert_3>. The local queue manager discards the request because it does not host a repository for cluster <insert_3> and has not subscribed to the published object. The remote CLUSSDR channel used to access the local queue manager has a channel name of <insert_4> and the remote queue manager has a QMID of <insert_5>.",
                  "Check the definitions on the local and remote queue managers to ensure that the CLUSSDR channel points to a repository queue manager for the cluster."));
      amqMN_Ht.put("9430",new AMQMessageNumber(9430,
                  "Unexpected cluster queue manager publication received.",
                  "30 : Severe error",
                  "The local queue manager has received a cluster queue manager publication on cluster <insert_3>. The local queue manager should not have received the publication because it does not host a repository for cluster <insert_3>, it has not subscribed to information concerning the published object, and the published object does not match any of its CLUSSDRs. The queue manager that sent the publication to the local queue manager has QMID <insert_4> (note that this is not necessarily the queue manager which originated the publication). CLUSSDR channel <insert_5> was used to send the publication.",
                  "Check the CLUSSDR definition on the sending queue manager to ensure that it points to a repository queue manager for the cluster."));
      amqMN_Ht.put("9431",new AMQMessageNumber(9431,
                  "Remote queue manager no longer hosts a repository for cluster",
                  "0 : Information",
                  "The local queue manager has received a message from remote queue manager QMID <insert_3> indicating that it no longer hosts a repository for cluster <insert_4>. CLUSSDR channel <insert_5> is altered so that it can no longer be used to access queue manager <insert_3> within cluster <insert_4>. If the local queue manager does not host a repository for cluster <insert_4> the relevant subscriptions and publications are remade if possible.",
                  "None."));
      amqMN_Ht.put("9432",new AMQMessageNumber(9432,
                  "Query received by a non-repository queue manager",
                  "30 : Severe error",
                  "The local queue manager has received a query from a remote queue manager on cluster <insert_3>. The local queue manager discards the query because it does not host a repository for cluster <insert_3>. The remote CLUSSDR channel used to access the local queue manager has a channel name of <insert_4> and the remote queue manager has a QMID of <insert_5>.",
                  "Check the definitions on the local and remote queue managers to ensure that the CLUSSDR channel points to a repository queue manager for the cluster."));
      amqMN_Ht.put("9433",new AMQMessageNumber(9433,
                  "CLUSRCVR must be in the same cluster as its matching CLUSSDR.",
                  "30 : Severe error",
                  "CLUSRCVR channel <insert_3> is not defined as a member of cluster <insert_4>. The local queue manager has received a command that indicates that CLUSSDR channel <insert_3> on the remote queue manager with QMID <insert_5> is defined as a member of cluster <insert_4>.",
                  "Alter the CLUSRCVR or CLUSSDR definitions for channel <insert_3>, so that they are both members of the same cluster."));
      amqMN_Ht.put("9434",new AMQMessageNumber(9434,
                  "Unrecognized message on <insert_3>.",
                  "30 : Severe error",
                  "The repository manager found a message on one of its queues having, either a format that could not be recognized, or that did not come from a queue manager or repository manager. The message was put on the dead-letter queue.",
                  "Examine the message on the dead-letter queue to determine the originator of the message."));
      amqMN_Ht.put("9435",new AMQMessageNumber(9435,
                  "Unable to put repository manager message.",
                  "30 : Severe error",
                  "The repository manager tried to send a message to the SYSTEM.CLUSTER.COMMAND.QUEUE on another queue manager whose identifier is <insert_3>, but the MQPUT call was unsuccessful. MQCC=<insert_1>, MQRC=<insert_2>. Processing continues, but the repository information may be out of date.",
                  "Refer to the Application Programming Reference manual for information about MQCC <insert_1> and MQRC <insert_2>. Check the channel and cluster definitions on the local and target queue managers, and ensure that the channels between them are running. When the problem is corrected, the repository information will normally be updated automatically. The REFRESH CLUSTER command can be used to ensure that the repository information is up to date."));
      amqMN_Ht.put("9436",new AMQMessageNumber(9436,
                  "Unable to send repository manager message.",
                  "30 : Severe error",
                  "The repository manager tried to send a message to the SYSTEM.CLUSTER.COMMAND.QUEUE on a queue manager that has the full repository for the specified cluster(<insert_3>), but the MQPUT call was unsuccessful. MQCC=<insert_1>, MQRC=<insert_2>. Processing continues, but repository information may be out of date.",
                  "Refer to the Application Programming Reference manual for information about MQCC <insert_1> and MQRC <insert_2>. Check the channel and cluster definitions on the local and target queue managers, and ensure that the channels between them are running. When the problem is corrected, the repository information will normally be updated automatically. The REFRESH CLUSTER command can be used to ensure that the repository information is up to date."));
      amqMN_Ht.put("9437",new AMQMessageNumber(9437,
                  "Unable to commit repository changes.",
                  "30 : Severe error",
                  "The repository manager tried to commit some updates to the repository but was unsuccessful. Processing continues, but repository information may be out of date.",
                  "If this occurs when the repository manager is stopping, this message can be ignored, because the repository information will normally be updated automatically when the repository manager is restarted. If there is an isolated occurrence at other times, use the REFRESH CLUSTER command to bring the repository information up to date. If the problem persists, contact your IBM support center for assistance."));
      amqMN_Ht.put("9438",new AMQMessageNumber(9438,
                  "CONNAME could not be discovered for CLUSRCVR <insert_3>.",
                  "30 : Severe error",
                  "TCP/IP CLUSRCVR <insert_3> was validly specified with a blank or absent CONNAME parameter. However when the repository process, amqrrmfa, attempted to obtain the CONNAME (IP address) for itself it was unable to. If there is an existing matching CLUSRCVR object in the cache its CONNAME is used. The CONNAME used was <insert_4>.",
                  "Check the error log for a message arising from an associated TCP/IP call (gethostname, gethostbyname or inet_ntoa). Pass all the error information to your systems administrator."));
      amqMN_Ht.put("9439",new AMQMessageNumber(9439,
                  "Repository corruption: bad CLQMGR object for channel <insert_3>.",
                  "30 : Severe error",
                  "An internal error has occurred.",
                  "Collect the items listed in the 'Problem determination' chapter in the System Administration guide and contact your IBM support center."));
      amqMN_Ht.put("9440",new AMQMessageNumber(9440,
                  "Reset command failed.",
                  "0 : Information",
                  "Reset Cluster(<insert_3>) Qmname(<insert_4>) command failed. To issue this command, queue manager <insert_5> must be a repository for cluster <insert_3>. Alter the queue manager attributes Repos, or Reposnl, to include cluster <insert_3> and retry the command.",
                  "None."));
      amqMN_Ht.put("9441",new AMQMessageNumber(9441,
                  "Reset command processed.",
                  "0 : Information",
                  "The reset Cluster(<insert_3>) Qmname(<insert_4>) command has processed on this repository and <insert_1> other queue managers have been sent notification.",
                  "None."));
      amqMN_Ht.put("9442",new AMQMessageNumber(9442,
                  "Refresh Cluster command processed.",
                  "0 : Information",
                  "The Refresh Cluster(<insert_4>) command caused <insert_1> objects to be refreshed and <insert_2> objects to be republished.",
                  "None."));
      amqMN_Ht.put("9443",new AMQMessageNumber(9443,
                  "Suspend Qmgr Cluster command processed.",
                  "0 : Information",
                  "The Suspend Qmgr Cluster command completed. <insert_1> objects suspended.I n the case of a name list the cluster name is the first name in the list.",
                  "None."));
      amqMN_Ht.put("9444",new AMQMessageNumber(9444,
                  "Resume Qmgr Cluster command processed.",
                  "0 : Information",
                  "The Resume Qmgr Cluster(<insert_4>) command completed. <insert_1> objects resumed. In the case of a name list the cluster name is the first name in the list.",
                  "None."));
      amqMN_Ht.put("9445",new AMQMessageNumber(9445,
                  "Error creating channel <insert_3>.",
                  "30 : Severe error",
                  "Channel <insert_4> tried to replace itself by creating channel <insert_3>. The attempt to create the channel was unsuccessful for the following reason: \"<insert_5>\". A previous message may give further information.",
                  "Rectify the problem which prevented successful creation of channel <insert_3>. Restart channel <insert_4>."));
      amqMN_Ht.put("9446",new AMQMessageNumber(9446,
                  "Error deleting channel <insert_3>.",
                  "30 : Severe error",
                  "Channel <insert_3> tried to delete itself after creating channel <insert_4> to replace it. The attempt to delete the channel was unsuccessful for the following reason: \"<insert_5>\".",
                  "If channel <insert_3> still exists rectify the problem which prevented its deletion and then manually delete the channel."));
      amqMN_Ht.put("9447",new AMQMessageNumber(9447,
                  "Unable to backout repository changes.",
                  "30 : Severe error",
                  "Following an error, the repository manager tried to backout some updates to the repository, but was unsuccessful. The repository manager terminates.",
                  "If the repository manager subsequently restarts successfully, this message can be ignored. If the repository manager does not restart, contact your IBM support center for assistance."));
      amqMN_Ht.put("9448",new AMQMessageNumber(9448,
                  "Repository manager stopping because of errors. Restart in <insert_1> seconds.",
                  "30 : Severe error",
                  "A severe error, as reported in the preceding messages, occurred during repository manager processing. The repository manager was unable to continue and terminates. The repository manager will try to restart after the specified interval.",
                  "Correct the problem reported in the preceding messages."));
      amqMN_Ht.put("9449",new AMQMessageNumber(9449,
                  "Repository manager restarted.",
                  "0 : Information",
                  "The repository manager restarted successfully following an error.",
                  "None."));
      amqMN_Ht.put("9450",new AMQMessageNumber(9450,
                  "Usage: <insert_3> [-m QMgrName] -f OutputFile [-v OutputFileVersion]",
                  "10 : Warning",
                  "Values passed to the channel table writer program were invalid. The parameter string passed to this program is as follows: [-m QMgrName] -f OutputFile [-v OutputFileVersion]\n where OutputFileVersion can be either 2 or 5 (5 is the default) Default values will be used for parameters not supplied.",
                  "Correct the parameters passed to the channel table writer program and retry the operation."));
      amqMN_Ht.put("9453",new AMQMessageNumber(9453,
                  "FORCEREMOVE command failed, cluster <insert_3> target <insert_4> is not unique.",
                  "0 : Information",
                  "The repository queue manager could not process a RESET ACTION(FORCEREMOVE) command for the indicated cluster and target queue manager, because there is more than one queue manager with the specified name in the cluster. The command is ignored.",
                  "Reissue the command specifying the identifier (QMID) of the queue manager to be removed, rather than its name."));
      amqMN_Ht.put("9455",new AMQMessageNumber(9455,
                  "FORCEREMOVE command failed, cluster <insert_3>, target <insert_4>, not found.",
                  "0 : Information",
                  "The repository queue manager could not process a RESET ACTION(FORCEREMOVE) command for the indicated cluster and target queue manager, because no information about that queue manager was found in the local repository. The command is ignored.",
                  "Reissue the command, specifying the correct queue manager name or identifier."));
      amqMN_Ht.put("9456",new AMQMessageNumber(9456,
                  "Update not received for queue <insert_3>, queue manager <insert_4> from full repository for cluster <insert_5>.",
                  "0 : Information",
                  "The repository manager detected a queue that has been used in the last 30 days for which updated information should have been sent from a full repository. However, this has not occurred. The repository manager will keep the information about this queue for a further 60 days.",
                  "If the queue is still required, check that: 1) The cluster channels to and from the full repository and the queue manager that hosts the queue, are able to run. 2) The repository managers running on these queue managers have not ended abnormally."));
      amqMN_Ht.put("9456",new AMQMessageNumber(9456,
                  "(iSeries) Update not received from full repository.",
                  "0 : Information",
                  "Update not received for queue <insert_3>, queue manager <insert_4> from full repository for cluster <insert_5>. The repository manager detected a queue that has been used in the last 30 days for which updated information should have been sent from a full repository. However, this has not occurred. The repository manager will keep the information about this queue for a further 60 days.",
                  "If the queue is still required, check that: 1) The cluster channels to and from the full repository and the queue manager that hosts the queue, are able to run. 2) The repository managers running on these queue managers have not ended abnormally."));
      amqMN_Ht.put("9457",new AMQMessageNumber(9457,
                  "Repository available, cluster <insert_4>, channel <insert_5>, sender <insert_3>.",
                  "0 : Information",
                  "The repository queue manager received a command from another queue manager, whose identifier is <insert_3>, reporting that it is again a repository for cluster <insert_4>. The cluster-sender channel <insert_5> is changed so that it can be used to access the other queue manager in relation to the the cluster.",
                  "None."));
      amqMN_Ht.put("9491",new AMQMessageNumber(9491,
                  "Transmission Queue <insert_3> set to NOSHARE.",
                  "20 : Error",
                  "The channel <insert_4> on queue manager <insert_5> cannot start because this queue manager has a setting for PipeLineLength greater than 1, and so multiple threads will run in this channel's MCA. Only the first thread would be able to open the Transmission Queue <insert_3> because it is set to be non-shareable.",
                  "Check the definition of the Transmission Queue <insert_3> on queue manager <insert_5> and set it to be SHARE instead of NOSHARE. Alternatively, you can set all channels on this queue manager to use only a single thread, by using the PipeLineLength parameter."));
      amqMN_Ht.put("9492",new AMQMessageNumber(9492,
                  "The <insert_3> responder program encountered an error.",
                  "30 : Severe error",
                  "The responder program was started but detected an error.",
                  "Look at previous error messages in the error files to determine the error encountered by the responder program."));
      amqMN_Ht.put("9494",new AMQMessageNumber(9494,
                  "A protocol error was detected for channel <insert_3>.",
                  "30 : Severe error",
                  "During communications with the remote queue manager, a TCP/IP read and receive call returned EINTR, indicating that it had been interrupted. Immediately after this the channel program detected a protocol error. The failure type was <insert_1> with associated data of <insert_2>.",
                  "If you are running an AIX client you will avoid problems arising from EINTRs on TCP/IP reads, by writing your application so that system calls interrupted by signals are restarted. You must establish the signal handler with sigaction(2) and set the SA_RESTART flag in the sa_flags field of the new action structure. If you are running on a platform other than AIX, an AIX server, or an AIX client with an application that adheres to the restart guidelines provided above, contact the systems administrator who should examine the error logs to determine the cause of the failure."));
      amqMN_Ht.put("9495",new AMQMessageNumber(9495,
                  "The CLWL exit <insert_3> is inconsistent with a dynamic cache.",
                  "30 : Severe error",
                  "When the CLWL exit <insert_3> was called for the ExitReason MQXR_INIT, the value <insert_1> was returned in the ExitResponse2 field. This indicates the CLWL exit is incompatible with the Queue Manager cache type which is dynamic. Either change the Queue Manager cache type to static (using the Tuning Parameter, ClusterCacheType=STATIC) or rewrite the CLWL exit to be compatible with a dynamic cache. The CLWL exit has been suppressed.",
                  "None."));
      amqMN_Ht.put("9496",new AMQMessageNumber(9496,
                  "Channel ended by a remote exit.",
                  "30 : Severe error",
                  "Channel program <insert_3> was ended because the channel exit at the remote end requested it.",
                  "Examine the error logs at the remote end of the channel to see the reason why the remote exit ended the channel."));
      amqMN_Ht.put("9498",new AMQMessageNumber(9498,
                  "The MQCD structure supplied was not valid.",
                  "30 : Severe error",
                  "The value of the <insert_3> field has the value <insert_4>. This value is invalid for the operation requested.",
                  "Change the parameter and retry the operation."));
      amqMN_Ht.put("9499",new AMQMessageNumber(9499,
                  "A WebSphere MQ listener will end shortly.",
                  "0 : Information",
                  "One listener detected in the system is scheduled for shutdown.",
                  "None."));
      amqMN_Ht.put("9500",new AMQMessageNumber(9500,
                  "No Repository storage",
                  "10 : Warning",
                  "An operation failed because there was no storage available in the repository. An attempt was made to allocate <insert_1> bytes from <insert_3>.",
                  "Reconfigure the Queue Manager to allocate a larger repository."));
      amqMN_Ht.put("9501",new AMQMessageNumber(9501,
                  "Usage: <insert_3> [-m QMgrName] -c ChlName.",
                  "10 : Warning",
                  "Values passed to the channel program are not valid. The parameter string passed to this program is as follows :- [-m QMgrName] -c ChlName Default values will be used for parameters not supplied.",
                  "Correct the parameters passed to the Channel program and retry the operation."));
      amqMN_Ht.put("9502",new AMQMessageNumber(9502,
                  "Type of channel not suitable for action requested.",
                  "30 : Severe error",
                  "The operation requested cannot be performed on channel <insert_3>. Some operations are only valid for certain channel types. For example, you can only ping a channel from the end sending the message.",
                  "Check whether the channel name is specified correctly. If it is check that the channel has been defined correctly."));
      amqMN_Ht.put("9503",new AMQMessageNumber(9503,
                  "Channel negotiation failed.",
                  "30 : Severe error",
                  "Channel <insert_3> between this machine and the remote machine could not be established due to a negotiation failure.",
                  "Tell the systems administrator, who should attempt to identify the cause of the channel failure using problem determination techniques. For example, look for FFST files, and examine the error logs on the local and remote systems where there may be messages explaining the cause of failure. More information may be obtained by repeating the operation with tracing enabled."));
      amqMN_Ht.put("9504",new AMQMessageNumber(9504,
                  "A protocol error was detected for channel <insert_3>.",
                  "30 : Severe error",
                  "During communications with the remote queue manager, the channel program detected a protocol error. The failure type was <insert_1> with associated data of <insert_2>.",
                  "Contact the systems administrator who should examine the error logs to determine the cause of the failure."));
      amqMN_Ht.put("9505",new AMQMessageNumber(9505,
                  "Channel sequence number wrap values are different.",
                  "30 : Severe error",
                  "The sequence number wrap value for channel <insert_3> is <insert_1>, but the value specified at the remote location is <insert_2>. The two values must be the same before the channel can be started.",
                  "Change either the local or remote channel definitions so that the values specified for the message sequence number wrap values are the same."));
      amqMN_Ht.put("9506",new AMQMessageNumber(9506,
                  "Message receipt confirmation failed.",
                  "30 : Severe error",
                  "Channel <insert_3> has ended because the remote queue manager did not accept the last batch of messages.",
                  "The error log for the channel at the remote site will contain an explanation of the failure. Contact the remote Systems Administrator to resolve the problem."));
      amqMN_Ht.put("9507",new AMQMessageNumber(9507,
                  "Channel <insert_3> is currently in-doubt.",
                  "30 : Severe error",
                  "The requested operation cannot complete because the channel is in-doubt with host <insert_4>.",
                  "Examine the status of the channel, and either restart a channel to resolve the in-doubt state, or use the RESOLVE CHANNEL command to correct the problem manually."));
      amqMN_Ht.put("9508",new AMQMessageNumber(9508,
                  "Program cannot connect to the queue manager.",
                  "30 : Severe error",
                  "The connection attempt to queue manager <insert_4> failed with reason code <insert_1>.",
                  "Ensure that the queue manager is available and operational."));
      amqMN_Ht.put("9509",new AMQMessageNumber(9509,
                  "Program cannot open queue manager object.",
                  "30 : Severe error",
                  "The attempt to open either the queue or queue manager object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Ensure that the queue is available and retry the operation."));
      amqMN_Ht.put("9510",new AMQMessageNumber(9510,
                  "Messages cannot be retrieved from a queue.",
                  "30 : Severe error",
                  "The attempt to get messages from queue <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "If the reason code indicates a conversion problem, for example MQRC_SOURCE_CCSID_ERROR, remove the message(s) from the queue. Otherwise, ensure that the required queue is available and operational."));
      amqMN_Ht.put("9511",new AMQMessageNumber(9511,
                  "Messages cannot be put to a queue.",
                  "30 : Severe error",
                  "The attempt to put messages to queue <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Ensure that the required queue is available and operational."));
      amqMN_Ht.put("9512",new AMQMessageNumber(9512,
                  "Ping operation is not valid for channel <insert_3>.",
                  "30 : Severe error",
                  "Ping may only be issued for SENDER, SERVER or CLUSSDR channel types. Also, it may not be issued for an SSL channel on the HP-UX or Linux platforms.",
                  "If the local channel is a receiver channel, you must issue the ping from the remote queue manager."));
      amqMN_Ht.put("9513",new AMQMessageNumber(9513,
                  "Maximum number of channels reached.",
                  "30 : Severe error",
                  "The maximum number of channels that can be in use simultaneously has been reached. The number of permitted channels is a configurable parameter in the queue manager configuration file.",
                  "Wait for some of the operating channels to close. Retry the operation when some channels are available."));
      amqMN_Ht.put("9514",new AMQMessageNumber(9514,
                  "Channel <insert_3> is in use.",
                  "30 : Severe error",
                  "The requested operation failed because channel <insert_3> is currently active.",
                  "Either end the channel manually, or wait for it to close, and retry the operation."));
      amqMN_Ht.put("9515",new AMQMessageNumber(9515,
                  "Channel <insert_3> changed.",
                  "10 : Warning",
                  "The statistics shown are for the channel requested, but it is a new instance of the channel. The previous channel instance has ended.",
                  "None."));
      amqMN_Ht.put("9516",new AMQMessageNumber(9516,
                  "File error occurred.",
                  "30 : Severe error",
                  "The filesystem returned error code <insert_1> for file <insert_3>.",
                  "Record the name of the file <insert_3> and tell the systems administrator, who should ensure that file <insert_3> is correct and available."));
      amqMN_Ht.put("9516",new AMQMessageNumber(9516,
                  "(iSeries) File error occurred.",
                  "30 : Severe error",
                  "The filesystem returned error code <insert_4> for file <insert_3>.",
                  "Record the name of the file <insert_3> and tell the systems administrator, who should ensure that file <insert_3> is correct and available."));
      amqMN_Ht.put("9517",new AMQMessageNumber(9517,
                  "File damaged.",
                  "30 : Severe error",
                  "The program has detected damage to the contents of file <insert_3>.",
                  "Record the values and tell the systems administrator who must restore a saved version of file <insert_3>. The return code was <insert_1> and the record length returned was <insert_2>."));
      amqMN_Ht.put("9518",new AMQMessageNumber(9518,
                  "File <insert_3> not found.",
                  "30 : Severe error",
                  "The program requires that the file <insert_3> is present and available.",
                  "This may be caused by invalid values for the optional environment variables MQCHLLIB, MQCHLTAB or MQDATA. If these variables are valid or not set then record the name of the file and tell the systems administrator who must ensure that file <insert_3> is available to the program."));
      amqMN_Ht.put("9519",new AMQMessageNumber(9519,
                  "Channel <insert_3> not found.",
                  "30 : Severe error",
                  "The requested operation failed because the program could not find a definition of channel <insert_3>.",
                  "Check that the name is specified correctly and the channel definition is available."));
      amqMN_Ht.put("9520",new AMQMessageNumber(9520,
                  "Channel not defined remotely.",
                  "30 : Severe error",
                  "There is no definition of channel <insert_3> at the remote location.",
                  "Add an appropriate definition to the remote hosts list of defined channels and retry the operation."));
      amqMN_Ht.put("9521",new AMQMessageNumber(9521,
                  "Host is not supported by this channel.",
                  "30 : Severe error",
                  "The connection across channel <insert_5> was refused because the remote host <insert_4> did not match the host <insert_3> specified in the channel definition.",
                  "Update the channel definition, or remove the explicit mention of the remote machine connection name."));
      amqMN_Ht.put("9522",new AMQMessageNumber(9522,
                  "Error accessing the status table.",
                  "30 : Severe error",
                  "The program could not access the channel status table.",
                  "A value of <insert_1> was returned from the subsystem when an attempt was made to access the Channel status table. Contact the systems administrator, who should examine the log files to determine why the program was unable to access the status table."));
      amqMN_Ht.put("9523",new AMQMessageNumber(9523,
                  "Remote host detected a protocol error.",
                  "30 : Severe error",
                  "During communications through channel <insert_3>, the remote queue manager channel program detected a protocol error. The failure type was <insert_1> with associated data of <insert_2>.",
                  "Tell the systems administrator, who should examine the error files to determine the cause of the failure."));
      amqMN_Ht.put("9524",new AMQMessageNumber(9524,
                  "Remote queue manager unavailable.",
                  "30 : Severe error",
                  "Channel <insert_3> cannot start because the remote queue manager is not currently available.",
                  "Either start the remote queue manager, or retry the operation later."));
      amqMN_Ht.put("9525",new AMQMessageNumber(9525,
                  "Remote queue manager is ending.",
                  "30 : Severe error",
                  "Channel <insert_3> is closing because the remote queue manager is ending.",
                  "None."));
      amqMN_Ht.put("9526",new AMQMessageNumber(9526,
                  "Message sequence number error for channel <insert_3>.",
                  "30 : Severe error",
                  "The local and remote queue managers do not agree on the next message sequence number. A message with sequence number <insert_1> has been sent when sequence number <insert_2> was expected.",
                  "Determine the cause of the inconsistency. It could be that the synchronization information has become damaged, or has been backed out to a previous version. If the situation cannot be resolved, the sequence number can be manually reset at the sending end of the channel using the RESET CHANNEL command."));
      amqMN_Ht.put("9527",new AMQMessageNumber(9527,
                  "Cannot send message through channel <insert_3>.",
                  "30 : Severe error",
                  "The channel has closed because the remote queue manager cannot receive a message.",
                  "Contact the systems administrator who should examine the error files of the remote queue manager, to determine why the message cannot be received, and then restart the channel."));
      amqMN_Ht.put("9528",new AMQMessageNumber(9528,
                  "User requested closure of channel <insert_3>.",
                  "10 : Warning",
                  "The channel is closing because of a request by the user.",
                  "None."));
      amqMN_Ht.put("9529",new AMQMessageNumber(9529,
                  "Target queue unknown on remote host.",
                  "30 : Severe error",
                  "Communication using channel <insert_3> has ended because the target queue for a message is unknown at the remote host.",
                  "Ensure that the remote host contains a correctly defined target queue, and restart the channel."));
      amqMN_Ht.put("9530",new AMQMessageNumber(9530,
                  "Program could not inquire queue attributes.",
                  "30 : Severe error",
                  "The attempt to inquire the attributes of queue <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Ensure that the queue is available and retry the operation."));
      amqMN_Ht.put("9531",new AMQMessageNumber(9531,
                  "Transmission queue specification error.",
                  "30 : Severe error",
                  "Queue <insert_4> identified as a transmission queue in the channel definition <insert_3> is not a transmission queue.",
                  "Ensure that the queue name is specified correctly. If so, alter the queue usage parameter of the queue to that of a transmission queue."));
      amqMN_Ht.put("9532",new AMQMessageNumber(9532,
                  "Program cannot set queue attributes.",
                  "30 : Severe error",
                  "The attempt to set the attributes of queue <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Ensure that the queue is available and retry the operation."));
      amqMN_Ht.put("9533",new AMQMessageNumber(9533,
                  "Channel <insert_3> is not currently active.",
                  "10 : Warning",
                  "The channel was not stopped because it was not currently active. If attempting to stop a specific instance of a channel by connection name or by remote queue manager name this message indicates that the specified instance of the channel is not running.",
                  "None."));
      amqMN_Ht.put("9534",new AMQMessageNumber(9534,
                  "Channel <insert_3> is currently not enabled.",
                  "30 : Severe error",
                  "The channel program ended because the channel is currently not enabled.",
                  "Issue the START CHANNEL command to re-enable the channel."));
      amqMN_Ht.put("9535",new AMQMessageNumber(9535,
                  "User exit not valid.",
                  "30 : Severe error",
                  "Channel program <insert_3> ended because user exit <insert_4> is not valid.",
                  "Ensure that the user exit is specified correctly in the channel definition, and that the user exit program is correct and available."));
      amqMN_Ht.put("9536",new AMQMessageNumber(9536,
                  "Channel ended by an exit.",
                  "30 : Severe error",
                  "Channel program <insert_3> was ended by exit <insert_4>.",
                  "None."));
      amqMN_Ht.put("9537",new AMQMessageNumber(9537,
                  "Usage: <insert_3> [-m QMgrName] [-q InitQ]",
                  "10 : Warning",
                  "Values passed to the Channel Initiator program are not valid. The parameters should be passed as follows: [-m QMgrName] [-q InitQ] Default values are used for parameters that are not supplied.",
                  "Correct the parameters passed to the program and retry the operation."));
      amqMN_Ht.put("9538",new AMQMessageNumber(9538,
                  "Commit control error.",
                  "30 : Severe error",
                  "An error occurred when attempting to start commitment control. Either exception <insert_3> was received when querying commitment status, or commitment control could not be started.",
                  "Refer to the error log for other messages pertaining to this problem."));
      amqMN_Ht.put("9539",new AMQMessageNumber(9539,
                  "No channels available.",
                  "30 : Severe error",
                  "The channel initiator program received a trigger message to start an MCA program to process queue <insert_3>. The program could not find a defined, available channel to start.",
                  "Ensure that there is a defined channel, which is enabled, to process the transmission queue."));
      amqMN_Ht.put("9540",new AMQMessageNumber(9540,
                  "Commit failed.",
                  "30 : Severe error",
                  "The program ended because return code <insert_1> was received when an attempt was made to commit change to the resource managers. The commit ID was <insert_3>.",
                  "Tell the systems administrator."));
      amqMN_Ht.put("9541",new AMQMessageNumber(9541,
                  "CCSID supplied for data conversion not supported.",
                  "30 : Severe error",
                  "The program ended because, either the source CCSID <insert_1> or the target CCSID <insert_2> is not valid, or is not currently supported.",
                  "Correct the CCSID that is not valid, or ensure that the requested CCSID can be supported."));
      amqMN_Ht.put("9542",new AMQMessageNumber(9542,
                  "Queue manager is ending.",
                  "10 : Warning",
                  "The program will end because the queue manager is quiescing.",
                  "None."));
      amqMN_Ht.put("9543",new AMQMessageNumber(9543,
                  "Status table damaged.",
                  "30 : Severe error",
                  "The channel status table has been damaged.",
                  "End all running channels and issue a DISPLAY CHSTATUS command to see the status of the channels. Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9544",new AMQMessageNumber(9544,
                  "Messages not put to destination queue.",
                  "10 : Warning",
                  "During the processing of channel <insert_3> one or more messages could not be put to the destination queue and attempts were made to put them to a dead-letter queue. The location of the queue is <insert_1>, where 1 is the local dead-letter queue and 2 is the remote dead-letter queue.",
                  "Examine the contents of the dead-letter queue. Each message is contained in a structure that describes why the message was put to the queue, and to where it was originally addressed. Also look at previous error messages to see if the attempt to put messages to a dead-letter queue failed. The program identifier (PID) of the processing program was <insert_4>."));
      amqMN_Ht.put("9545",new AMQMessageNumber(9545,
                  "Disconnect interval expired.",
                  "0 : Information",
                  "Channel <insert_3> closed because no messages arrived on the transmission queue within the disconnect interval period.",
                  "None."));
      amqMN_Ht.put("9546",new AMQMessageNumber(9546,
                  "Error return code received.",
                  "30 : Severe error",
                  "The program has ended because return code <insert_1> was returned from function <insert_3>",
                  "Correct the cause of the failure and retry the operation."));
      amqMN_Ht.put("9547",new AMQMessageNumber(9547,
                  "Type of remote channel not suitable for action requested.",
                  "30 : Severe error",
                  "The operation requested cannot be performed because channel <insert_3> on the remote machine is not of a suitable type. For example, if the local channel is defined as a sender the remote machine must define its channel as either a receiver or requester.",
                  "Check that the channel name is specified correctly. If it is, check that the remote channel has been defined correctly."));
      amqMN_Ht.put("9548",new AMQMessageNumber(9548,
                  "Message put to the 'dead-letter queue'.",
                  "10 : Warning",
                  "During processing a message has been put to the dead-letter queue.",
                  "Examine the contents of the dead-letter queue. Each message is contained in a structure that describes why the message was put to the queue, and to where it was originally addressed."));
      amqMN_Ht.put("9549",new AMQMessageNumber(9549,
                  "Transmission Queue <insert_3> inhibited for MQGET.",
                  "20 : Error",
                  "An MQGET failed because the transmission queue had been previously inhibited for MQGET.",
                  "None."));
      amqMN_Ht.put("9550",new AMQMessageNumber(9550,
                  "Channel program <insert_3> cannot be stopped at this time.",
                  "30 : Severe error",
                  "The channel program can not be terminated immediately but should end shortly.",
                  "If the channel does not end in a short time issue the STOP CHANNEL command again."));
      amqMN_Ht.put("9551",new AMQMessageNumber(9551,
                  "Protocol not supported by remote host",
                  "30 : Severe error",
                  "The operation you are performing over Channel <insert_3> to the host at <insert_4> is not supported by the target host.",
                  "Check that the connection name parameter is specified correctly and that the levels of the products in use are compatible."));
      amqMN_Ht.put("9552",new AMQMessageNumber(9552,
                  "Security flow not received.",
                  "30 : Severe error",
                  "During communications through channel <insert_3> the local security exit requested security data from the remote machine. The security data has not been received so the channel has been closed.",
                  "Tell the systems administrator who should ensure that the security exit on the remote machine is defined correctly."));
      amqMN_Ht.put("9553",new AMQMessageNumber(9553,
                  "The function is not supported.",
                  "30 : Severe error",
                  "The <insert_3> function <insert_4> attempted is not currently supported on this platform.",
                  "None."));
      amqMN_Ht.put("9554",new AMQMessageNumber(9554,
                  "User not authorized.",
                  "30 : Severe error",
                  "You are not authorized to perform the Channel operation.",
                  "Tell the systems administrator who should ensure that the correct access permissions are available to you, and then retry the operation."));
      amqMN_Ht.put("9555",new AMQMessageNumber(9555,
                  "File format error.",
                  "30 : Severe error",
                  "The file <insert_3> does not have the expected format.",
                  "Ensure that the file name is specified correctly."));
      amqMN_Ht.put("9556",new AMQMessageNumber(9556,
                  "Channel synchronization file missing or damaged.",
                  "30 : Severe error",
                  "The channel synchronization file <insert_3> is missing or does not correspond to the stored channel information for queue manager <insert_4>.",
                  "Rebuild the synchronization file using the rcrmqmobj command rcrmqmobj -t syncfile (-m q-mgr-name)"));
      amqMN_Ht.put("9556",new AMQMessageNumber(9556,
                  "(iSeries) Channel synchronization file missing or damaged.",
                  "30 : Severe error",
                  "The channel synchronization file <insert_3> is missing or does not correspond to the stored channel information for queue manager <insert_4>.",
                  "Rebuild the synchronization file using the RCRMQMOBJ command."));
      amqMN_Ht.put("9557",new AMQMessageNumber(9557,
                  "Queue Manager User ID initialization failed.",
                  "30 : Severe error",
                  "The call to initialize the User ID failed with CompCode <insert_1> and Reason <insert_2>.",
                  "Correct the error and try again."));
      amqMN_Ht.put("9558",new AMQMessageNumber(9558,
                  "Remote Channel is not currently available.",
                  "30 : Severe error",
                  "The channel program ended because the channel <insert_3> is not currently available on the remote system. This could be because the channel is disabled or that the remote system does not have sufficient resources to run a further channel.",
                  "Check the remote system to ensure that the channel is available to run, and retry the operation."));
      amqMN_Ht.put("9560",new AMQMessageNumber(9560,
                  "Rebuild Synchronization File - program started",
                  "0 : Information",
                  "Rebuilding the Synchronization file for Queue Manager <insert_3> .",
                  "None."));
      amqMN_Ht.put("9561",new AMQMessageNumber(9561,
                  "Rebuild Synchronization File - program completed normally",
                  "0 : Information",
                  "Rebuild Synchronization File program completed normally.",
                  "None."));
      amqMN_Ht.put("9562",new AMQMessageNumber(9562,
                  "Synchronization file in use.",
                  "30 : Severe error",
                  "The Synchronization file <insert_3> is in use and cannot be recreated.",
                  "Stop any channel activity and retry the rcrmqmobj command."));
      amqMN_Ht.put("9562",new AMQMessageNumber(9562,
                  "(iSeries) Synchronization file in use.",
                  "30 : Severe error",
                  "The Synchronization file <insert_3> is in use and cannot be recreated.",
                  "Stop any channel activity and retry the RCRMQMOBJ command."));
      amqMN_Ht.put("9563",new AMQMessageNumber(9563,
                  "Synchronization file cannot be deleted",
                  "30 : Severe error",
                  "The filesystem returned error code <insert_1> for file <insert_3>.",
                  "Tell the systems administrator who should ensure that file <insert_3> is available and not in use."));
      amqMN_Ht.put("9564",new AMQMessageNumber(9564,
                  "Synchronization File cannot be created",
                  "30 : Severe error",
                  "The filesystem returned error code <insert_1> for file <insert_3>.",
                  "Tell the systems administrator."));
      amqMN_Ht.put("9565",new AMQMessageNumber(9565,
                  "No dead-letter queue defined.",
                  "30 : Severe error",
                  "The queue manager <insert_4> does not have a defined dead-letter queue.",
                  "Either correct the problem that caused the program to try and write a message to the dead-letter queue or create a dead-letter queue for the queue manager."));
      amqMN_Ht.put("9566",new AMQMessageNumber(9566,
                  "Invalid MQSERVER value",
                  "30 : Severe error",
                  "The value of the MQSERVER environment variable was <insert_3>. The variable should be in the format 'ChannelName/Protocol/ConnectionName'.",
                  "Correct the MQSERVER value and retry the operation."));
      amqMN_Ht.put("9572",new AMQMessageNumber(9572,
                  "Message header is not valid.",
                  "30 : Severe error",
                  "Channel <insert_3> is stopping because a message header is not valid. During the processing of the channel, a message was found that has a header that is not valid. The dead-letter queue has been defined as a transmission queue, so a loop would be created if the message had been put there.",
                  "Correct the problem that caused the message to have a header that is not valid."));
      amqMN_Ht.put("9573",new AMQMessageNumber(9573,
                  "Maximum number of active channels reached.",
                  "30 : Severe error",
                  "There are too many channels active to start another. The current defined maximum number of active channels is <insert_1>.",
                  "Either wait for some of the operating channels to close or use the stop channel command to close some channels. Retry the operation when some channels are available. The maximum number of active channels is a configurable parameter in the queue manager configuration file."));
      amqMN_Ht.put("9574",new AMQMessageNumber(9574,
                  "Channel <insert_3> can now be started.",
                  "30 : Severe error",
                  "Channel <insert_3> has been waiting to start, but there were no channels available because the maximum number of active channels was running. One, or more, of the active channels has now closed so this channel can start.",
                  ""));
      amqMN_Ht.put("9575",new AMQMessageNumber(9575,
                  "DCE Security: failed to get the user's login name.",
                  "30 : Severe error",
                  "System call <insert_4> to get the login name of the user running WebSphere MQ client application process <insert_1> failed with error value <insert_2>. This occurred in security exit function create_cred. The exit will now attempt to open channel <insert_3> using the DCE default login context.",
                  "If you wish to run using the DCE default login context take no action. If you wish to run using the user's login name as the DCE security exit principal examine the documentation for the operating system on which you are running MQ clients and reconfigure the operating system as necessary to allow the <insert_4> call to succeed."));
      amqMN_Ht.put("9576",new AMQMessageNumber(9576,
                  "DCE Security: an exit could not allocate memory.",
                  "30 : Severe error",
                  "A DCE exit was unsuccessful in obtaining the memory it needed. The failure occurred in exit function <insert_4>. Channel <insert_3> is closed.",
                  "Make more memory available to the WebSphere MQ system and restart the relevant channel."));
      amqMN_Ht.put("9577",new AMQMessageNumber(9577,
                  "DCE security exit: no partner name.",
                  "30 : Severe error",
                  "Channel <insert_3> has not been opened because the DCE security exit which initiates the security context was not passed a valid partner name. When the DCE security exit is called to initiate the security context it is essential that the PartnerName field in the MQCXP structure contains a valid partner name. On this call it did not. This can arise as a result of a usage error, for instance only specifying the security exit on one end of the channel. The error was reported from security exit function savePartnerName.",
                  "Check your usage of the DCE security exit for errors, such as only specifying the exit in one of the matching channel definitions. Correct any errors found and retry."));
      amqMN_Ht.put("9578",new AMQMessageNumber(9578,
                  "DCE Security: bad return from DCE call.",
                  "30 : Severe error",
                  "Channel <insert_3> has been closed because one of the DCE channel exits received a bad return code from DCE.",
                  "Consult the appropriate DCE manuals to find out the meaning of major_status <insert_1> and minor_status <insert_2> on call <insert_5>. Then rectify the error. The exit function name is <insert_4>."));
      amqMN_Ht.put("9579",new AMQMessageNumber(9579,
                  "DCE Security: partner name does not match target.",
                  "30 : Severe error",
                  "The DCE Security exit was requested to perform a trusted channel check: target partner name <insert_4> was specified in the SCYDATA field of channel <insert_3>. The actual partner name associated with channel <insert_3> was <insert_5>, so the security exit suppressed the channel.",
                  "Examine the channel definition of channel <insert_3> and alter it so that the relevant name on the partner system matches that specified in the SCYDATA field."));
      amqMN_Ht.put("9580",new AMQMessageNumber(9580,
                  "DCE Security: invalid message received.",
                  "30 : Severe error",
                  "An IBM-supplied DCE exit on channel <insert_3> received a message that was not generated by a matching exit, or was not the expected type of message. The header.mechanism field had value <insert_1>. The header.msgtype field had value <insert_2>. The name of the exit function in which the error was discovered is <insert_4>.",
                  "Make sure that the exits at both ends of the channel generate compatible flows."));
      amqMN_Ht.put("9581",new AMQMessageNumber(9581,
                  "DCE Security: wrong exit called.",
                  "30 : Severe error",
                  "Exit <insert_4> on channel <insert_3> was called for use as a WebSphere MQ exit of the wrong type. DCE_SEC_SCY_CHANNELEXIT functions as a security exit; DCE_SEC_SRM_CHANNELEXIT functions as a send, receive or message exit. The ExitId parameter passed to the exit was <insert_1>.",
                  "Alter the exit definitions to ensure that exit <insert_4> is called correctly."));
      amqMN_Ht.put("9582",new AMQMessageNumber(9582,
                  "DCE Security: invalid exit function requested.",
                  "30 : Severe error",
                  "Exit <insert_4> on channel <insert_3> was called with an invalid ExitReason (value <insert_1>).",
                  "Check that the exit is being run with a compatible release of WebSphere MQ base code. If not then correct it. If it is, contact your IBM support center for help."));
      amqMN_Ht.put("9583",new AMQMessageNumber(9583,
                  "The DCE security exit was not run.",
                  "30 : Severe error",
                  "The DCE_SEC_SRM_CHANNELEXIT exit was called on channel <insert_3>; the value of pContext->mechanism (<insert_1>) passed was not valid.",
                  "This is probably because the DCE_SEC_SRM_CHANNELEXIT exit has been called without first calling the DCE_SEC_SCY_CHANNELEXIT security exit. Alter the system so that either both or neither are run."));
      amqMN_Ht.put("9584",new AMQMessageNumber(9584,
                  "DCE Security: message too short.",
                  "30 : Severe error",
                  "The DCE_SEC_SRM_CHANNELEXIT receive or message exit was called on channel <insert_3> to process an incoming message. The pDataLength parameter supplied to the exit indicated that the message received was too short to be a valid message for the relevant exit. The *pDataLength value was <insert_1>.",
                  "Configure the system so that compatible send/receive/message exits are run at both ends of the channel."));
      amqMN_Ht.put("9585",new AMQMessageNumber(9585,
                  "Maximum number of channel initiators reached.",
                  "30 : Severe error",
                  "The maximum number of channels initiators that can be in use simultaneously has been reached. The number of permitted channel initiators is a configurable parameter in the queue manager configuration file.",
                  "Wait for one or more channel initiators to close and retry the operation or modify the configuration file to allow more initiators and restart the Queue Manager."));
      amqMN_Ht.put("9586",new AMQMessageNumber(9586,
                  "Program cannot create queue manager object.",
                  "30 : Severe error",
                  "The attempt to create object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier. Contact your IBM support center."));
      amqMN_Ht.put("9587",new AMQMessageNumber(9587,
                  "Program cannot open queue manager object.",
                  "30 : Severe error",
                  "The attempt to open object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier. Contact your IBM support center."));
      amqMN_Ht.put("9588",new AMQMessageNumber(9588,
                  "Program cannot update queue manager object.",
                  "30 : Severe error",
                  "The attempt to update object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier. Contact your IBM support center."));
      amqMN_Ht.put("9589",new AMQMessageNumber(9589,
                  "Program cannot query queue manager object.",
                  "30 : Severe error",
                  "The attempt to query object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier. Contact your IBM support center."));
      amqMN_Ht.put("9590",new AMQMessageNumber(9590,
                  "Program cannot close queue manager object.",
                  "30 : Severe error",
                  "The attempt to close object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier. Contact your IBM support center."));
      amqMN_Ht.put("9591",new AMQMessageNumber(9591,
                  "Program cannot prepare queue manager object.",
                  "30 : Severe error",
                  "The attempt to prepare object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier. Contact your IBM support center."));
      amqMN_Ht.put("9592",new AMQMessageNumber(9592,
                  "Program cannot resolve queue manager object.",
                  "30 : Severe error",
                  "The attempt to resolve object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier. Contact your IBM support center."));
      amqMN_Ht.put("9593",new AMQMessageNumber(9593,
                  "Program cannot delete queue manager object.",
                  "30 : Severe error",
                  "The attempt to delete object <insert_4> on queue manager <insert_5> failed with reason code <insert_1>.",
                  "Use the standard facilities supplied with your system to record the problem identifier. Contact your IBM support center."));
      amqMN_Ht.put("9594",new AMQMessageNumber(9594,
                  "Usage: runmqfmt [filename].",
                  "0 : Information",
                  "Syntax for the usage of runmqfmt.",
                  "None."));
      amqMN_Ht.put("9595",new AMQMessageNumber(9595,
                  "Usage: endmqlsr [-w] [-m QMgrName]",
                  "10 : Warning",
                  "The correct usage is shown.",
                  "Correct the parameters passed to the endmqlsr program and retry the operation."));
      amqMN_Ht.put("9596",new AMQMessageNumber(9596,
                  "Queue Manager <insert_3> still running",
                  "30 : Severe error",
                  "The requested operation can not complete because queue manager <insert_3> is still running.",
                  "End the queue manager and retry the operation."));
      amqMN_Ht.put("9597",new AMQMessageNumber(9597,
                  "No WebSphere MQ listeners for Queue Manager <insert_3>.",
                  "0 : Information",
                  "No listener processes were found in the system for Queue Manager <insert_3>.",
                  "None."));
      amqMN_Ht.put("9598",new AMQMessageNumber(9598,
                  "<insert_1> WebSphere MQ listeners will end shortly.",
                  "0 : Information",
                  "<insert_1> listeners detected in the system are scheduled for shutdown.",
                  "None."));
      amqMN_Ht.put("9599",new AMQMessageNumber(9599,
                  "Program could not open queue manager object.",
                  "30 : Severe error",
                  "The attempt to open either the queue or queue manager object <insert_4> on queue manager <insert_5> by user <insert_3> failed with reason code <insert_1>.",
                  "Ensure that the queue is available and retry the operation. If the message is from a remote Queue Manager, check the Message Channel Agent User Identifier has the correct authority."));
      amqMN_Ht.put("9601",new AMQMessageNumber(9601,
                  "Program could not inquire on queues on this queue manager.",
                  "30 : Severe error",
                  "The WebSphere MQ clustering repository program was attempting to find out about the queues on queue manager <insert_3>. One of the calls failed with reason code <insert_1>. The repository command was backed out and the repository process went into a timed wait.",
                  "Correct the error. When the repository process restarts it processes the backed out command again and continues."));
      amqMN_Ht.put("9602",new AMQMessageNumber(9602,
                  "Maximum number of channel processes reached.",
                  "30 : Severe error",
                  "The channel can not start because the number of channel processes has already reached the maximum allowable value. The maximum number of channels processes is configured as <insert_1>. This value is a configurable parameter in the queue manager configuration file.",
                  "Wait for some of the operating channels to close. Retry the operation when some channels are available."));
      amqMN_Ht.put("9603",new AMQMessageNumber(9603,
                  "Error accessing the process pool shared segment.",
                  "30 : Severe error",
                  "The program could not access the process pool shared segment",
                  "A value of <insert_1> was returned from the subsystem when an attempt was made to access the Channel process pool shared memory. Contact the systems administrator, who should examine the log files to determine why the program was unable to access the process pool shared segment."));
      amqMN_Ht.put("9604",new AMQMessageNumber(9604,
                  "Channel <insert_3> terminated unexpectedly",
                  "30 : Severe error",
                  "The process or thread executing channel <insert_3> is no longer running. The check process system call returned <insert_1> for process <insert_2>.",
                  "No immediate action is required because the channel entry has been removed from the list of running channels. Inform the system administrator who should examine the operating system procedures to determine why the channel process has terminated."));
      amqMN_Ht.put("9605",new AMQMessageNumber(9605,
                  "<insert_1> WebSphere MQ listeners have been ended.",
                  "0 : Information",
                  "<insert_1> listeners detected in the system have been ended.",
                  "None."));
      amqMN_Ht.put("9606",new AMQMessageNumber(9606,
                  "A WebSphere MQ listener has ended.",
                  "0 : Information",
                  "One listener detected in the system has been ended.",
                  "None."));
      amqMN_Ht.put("9608",new AMQMessageNumber(9608,
                  "Remote resources in recovery",
                  "30 : Severe error",
                  "Channel <insert_3> could not establish a successful connection with the remote Queue Manager because resources are being recovered.",
                  "Restart the channel at a later time. If the problem persists then examine the error logs of the remote Queue Manager to see the full explanation of the cause of the problem."));
      amqMN_Ht.put("9610",new AMQMessageNumber(9610,
                  "AMQ<insert_1> messages suppressed",
                  "0 : Information",
                  "<insert_2> messages of type AMQ<insert_1> were suppressed",
                  "Message suppression is controlled by MQ_CHANNEL_SUPPRESS_MSGS and MQ_CHANNEL_SUPPRESS_INTERVAL environment variables."));
      amqMN_Ht.put("9611",new AMQMessageNumber(9611,
                  "Rebuild Client Channel Table - program completed normally",
                  "0 : Information",
                  "Rebuild Client Channel Table program completed normally.",
                  "None."));
      amqMN_Ht.put("9612",new AMQMessageNumber(9612,
                  "<insert_1> WebSphere MQ listeners could not be ended.",
                  "0 : Information",
                  "The request to the end the WebSphere MQ listeners for specified Queue Manager was completed however <insert_1> listeners could not be stopped. Reasons why listener may not be stopped are: The listener process contains channels which are still active.",
                  "Active channels may be stopped using the 'STOP CHANNEL' command or by ending the Queue Manager, and reissuing the end-listener request."));
      amqMN_Ht.put("9614",new AMQMessageNumber(9614,
                  "(iSeries) Certificate is not signed by a trusted Certificate Authority.",
                  "0 : Information",
                  "The attempt to start channel <insert_3> failed because the certificate used in the SSL handshake is not signed by a Certificate Authority (CA) listed in the certificate trust list for this queue manager. This error occurs when the SSL key repository for the queue manager is specified as '*SYSTEM' and the application definition in Digital Certificate Manager has been modified to specify a CA trust list.",
                  "Use Digital Certificate Manager to add the required Certificate Authority (CA) certificates to the application definitions CA trust list."));
      amqMN_Ht.put("9615",new AMQMessageNumber(9615,
                  "(iSeries) Queue Manager is not registered with DCM.",
                  "0 : Information",
                  "The attempt to start channel <insert_3> failed because the queue manager is not registered as a SSL server application with Digital Certificate Manager (DCM). This error occurs when the SSL key repository for the queue manager is specified as '*SYSTEM' but WebSphere MQ cannot register the queue manager as an SSL server application with DCM, or alternatively when the application definition for the queue manager has been manually removed from DCM.",
                  "Attempt to re-register the queue manager with Digital Certificate Manager by issuing CHGMQM SSLKEYR(*SYSTEM). If this is unsuccessful you may need to manually add the application definition through Digital Certificate Manager, see the WebSphere MQ Security manual for more details."));
      amqMN_Ht.put("9616",new AMQMessageNumber(9616,
                  "The CipherSpec proposed is not enabled on the SSL server.",
                  "30 : Severe error",
                  "The SSL subsystem at the SSL server end of a channel been configured in such a way that it has rejected the CipherSpec proposed by an SSL client. This rejection occured during the SSL handshake (i.e. it happened before the proposed CipherSpec was compared with the CipherSpec in the SSL server channel definition). This error most commonly occurs when the choice of acceptable CipherSpecs has been limited by setting the SSLFipsRequired attribute on the SSL server queue manager while the SSL client is trying to connect with a CipherSpec which is not FIPS-certified on the SSL server. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Analyse why the proposed CipherSpec was not enabled on the SSL server. Alter the client CipherSpec, or reconfigure the SSL server to accept the original client CipherSpec. Restart the channel."));
      amqMN_Ht.put("9617",new AMQMessageNumber(9617,
                  "Parameter requesting FIPS has an invalid value.",
                  "30 : Severe error",
                  "An SSL channel running on an MQ client has failed to start. This is because the value specified for the MQSSLFIPS environment variable, or in the MQSCO FipsRequired field, is invalid. The value specified was \"<insert_3>\"; valid values are 'YES' or 'NO'.",
                  "Set the value specified for the MQSSLFIPS environment variable, or in the MQSCO FipsRequired field, to 'YES' or 'NO'. Restart the channel."));
      amqMN_Ht.put("9618",new AMQMessageNumber(9618,
                  "SSLCRLNL attribute points to a namelist with no names.",
                  "30 : Severe error",
                  "An SSL channel has failed to start because the SSLCRLNL queue manager attribute points to a namelist with an empty list of names.",
                  "If CRL checking is required, set up the namelist referenced by SSLCRLNL with a non-empty list of authentication information object names. If no CRL checking is required, clear the SSLCRLNL queue manager attribute. Restart the failing channel."));
      amqMN_Ht.put("9619",new AMQMessageNumber(9619,
                  "SSL cannot be run from an unthreaded HP-UX MQ client.",
                  "30 : Severe error",
                  "On HP-UX, SSL cannot be run from a WebSphere MQ client which was linked with the unthreaded client libraries.",
                  "Either relink your client application with the threaded client libraries, or do not attempt to use SSL from this application."));
      amqMN_Ht.put("9620",new AMQMessageNumber(9620,
                  "Internal error on call to SSL function on channel <insert_3>.",
                  "30 : Severe error",
                  "An error indicating a software problem was returned from a function which is used to provide SSL support. The error code returned was <insert_1>. The function call was <insert_4>. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9620",new AMQMessageNumber(9620,
                  "(iSeries) Unexpected SSL error on call to <insert_4>.",
                  "0 : Information",
                  "An unexpected SSL error was returned from function <insert_4> for channel <insert_3>. The error code returned was <insert_1>. GSKit error codes are documented in the MQ manuals and also in the GSKSSL member of the H file in library QSYSINC.",
                  "Collect the items listed in the 'Problem determination' section of the System Administration manual and contact your IBM support center."));
      amqMN_Ht.put("9621",new AMQMessageNumber(9621,
                  "Error on call to SSL function ignored on channel <insert_3>.",
                  "10 : Warning",
                  "An error indicating a software problem was returned from a function which is used to provide SSL support. The error code returned was <insert_1>. The function call was <insert_4>. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. This error is not regarded as sufficiently serious to interrupt channel operation; channel operation was not affected.",
                  "None."));
      amqMN_Ht.put("9622",new AMQMessageNumber(9622,
                  "AUTHINFO object <insert_3> does not exist.",
                  "30 : Severe error",
                  "A channel or channel process has failed to start because the namelist of CRL AUTHINFO objects includes the name <insert_3>, but no AUTHINFO object of that name exists.",
                  "Ensure all the names in the namelist specified on the SSLCRLNL queue manager attribute correspond to AUTHINFO objects which are to be used on the SSL channels. Restart the failing channel or channel process."));
      amqMN_Ht.put("9623",new AMQMessageNumber(9623,
                  "Error inquiring on AUTHINFO object <insert_3>.",
                  "30 : Severe error",
                  "A channel or channel process has failed to start because reason code <insert_1> was returned when an inquire was performed on AUTHINFO object <insert_3>.",
                  "Look at the MQRC_ values in the WebSphere MQ Application Programming Reference to determine the meaning of reason code <insert_1>, correct the error, and restart the failing channel or channel process."));
      amqMN_Ht.put("9624",new AMQMessageNumber(9624,
                  "AUTHINFO object <insert_3> is not of type CRLLDAP.",
                  "30 : Severe error",
                  "A channel or channel process has failed to start because one of the AUTHINFO objects specified in the SSLCRLNL namelist is not of AUTHTYPE CRLLDAP. Instead the type value is <insert_1>.",
                  "Only include CRLLDAP AUTHINFO objects in the namelist specified on the SSLCRLNL queue manager attribute. Restart the channel or channel process."));
      amqMN_Ht.put("9625",new AMQMessageNumber(9625,
                  "AUTHINFO object <insert_3> was specified with an invalid CONNAME.",
                  "30 : Severe error",
                  "A channel or channel process has failed to start because one of the AUTHINFO objects specified in the SSLCRLNL namelist has an invalid CONNAME parameter. The invalid value is <insert_4>.",
                  "Correct the invalid parameter. Restart the channel or channel process."));
      amqMN_Ht.put("9626",new AMQMessageNumber(9626,
                  "Channel hanging while initializing SSL.",
                  "30 : Severe error",
                  "The current channel cannot start because another channel is hanging while initializing the SSL subsystem.",
                  "Investigate the reason for the hang on the other channel. Once this is rectified, restart this channel."));
      amqMN_Ht.put("9627",new AMQMessageNumber(9627,
                  "The path and stem name for the SSL key repository have not been specified.",
                  "30 : Severe error",
                  "The directory path and file stem name for the SSL key repository have not been specified. On a MQ client system there is no default location for this file. SSL connectivity is therefore impossible as this file cannot be accessed.",
                  "Use the MQSSLKEYR environment variable or MQCONNX API call to specify the directory path and file stem name for the SSL key repository."));
      amqMN_Ht.put("9628",new AMQMessageNumber(9628,
                  "An LDAP server containing CRLs was specified with an invalid CONNAME.",
                  "30 : Severe error",
                  "The WebSphere MQ client has failed to connect because an invalid CONNAME was found for one of the LDAP servers containing CRLs. The invalid value is <insert_3>.",
                  "Correct the invalid parameter. If the LDAP details were defined on a queue manager system, regenerate the client definitions. Reconnect."));
      amqMN_Ht.put("9629",new AMQMessageNumber(9629,
                  "Bad SSL cryptographic hardware parameters.",
                  "30 : Severe error",
                  "The following string was supplied to specify or control use of SSL cryptographic hardware: <insert_4>. This string does not conform to any of the MQ SSL cryptographic parameter formats. The channel is <insert_3>. The channel did not start.",
                  "Correct your SSL cryptographic hardware parameters and restart the channel."));
      amqMN_Ht.put("9630",new AMQMessageNumber(9630,
                  "An expired SSL certificate was loaded.",
                  "30 : Severe error",
                  "An SSL certificate that was loaded was not corrupt, but failed validation checks on its date fields. The certificate has either expired, or its date is not valid yet (that is, the from date is later than today), or the validity date range is incorrect (for example, the to date is earlier than the from date).",
                  "Ensure that the specified SSL certificate has a valid expiry date."));
      amqMN_Ht.put("9631",new AMQMessageNumber(9631,
                  "The CipherSpecs on the two ends of channel <insert_3> do not match.",
                  "30 : Severe error",
                  "There is a mismatch between the CipherSpecs on the local and remote ends of channel <insert_3>. The channel will not run until this mismatch is resolved.",
                  "Change the channel definitions for <insert_3> so the two ends have matching CipherSpecs and restart the channel."));
      amqMN_Ht.put("9631",new AMQMessageNumber(9631,
                  "(iSeries) The CipherSpecs at the ends of channel <insert_3> do not match.",
                  "30 : Severe error",
                  "There is a mismatch between the CipherSpecs on the local and remote ends of channel <insert_3>. The channel will not run until this mismatch is resolved. The local CipherSpec is <insert_4> and the remote CipherSpec is <insert_5>.",
                  "Change the channel definition for <insert_3> so that both ends have matching CipherSpecs and restart the channel."));
      amqMN_Ht.put("9633",new AMQMessageNumber(9633,
                  "Bad SSL certificate for channel <insert_3>.",
                  "30 : Severe error",
                  "A certificate encountered during SSL handshaking is regarded as bad for one of the following reasons: (a) it was formatted incorrectly and could not be validated, or (b) it was formatted correctly but failed validation against the Certification Authority (CA) root and other certificates held on the local system, or (c) it was found in a Certification Revocation List (CRL) on an LDAP server. (d) a CRL was specified but the CRL could not be found on the LDAP server. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Check which of the three possible causes applies on your system. Correct the error, and restart the channel."));
      amqMN_Ht.put("9634",new AMQMessageNumber(9634,
                  "SSL security context expired.",
                  "30 : Severe error",
                  "During an SSL operation to encrypt or decrypt a secured message, the SSL security context, which is used to secure communications and was previously established with the remote party, has expired because the remote party has shut down. The secured message has not been encrypted or decrypted. This failure has closed WebSphere MQ channel name <insert_3>. If the name is '????', the name is unknown. The SSL operation was <insert_4> and its completion code was <insert_5>.",
                  "Determine why the remote party has shut down and if necessary re-start the channel. The shut down might be the result of controlled termination by a system administrator, or the result of an unexpected termination due to an error. The SSL operation is described in the Windows Schannel reference manual."));
      amqMN_Ht.put("9635",new AMQMessageNumber(9635,
                  "Channel <insert_3> did not specify a valid CipherSpec.",
                  "30 : Severe error",
                  "Channel <insert_3> did not specify a valid CipherSpec.",
                  "Change channel <insert_3> to specify a valid CipherSpec."));
      amqMN_Ht.put("9635",new AMQMessageNumber(9635,
                  "(iSeries) Channel <insert_3> did not specify a valid CipherSpec.",
                  "30 : Severe error",
                  "Channel <insert_3> did not specify a valid CipherSpec, or it specified a CipherSpec that is not available from the IBM Cryptographic Access Provider product installed on this machine. CipherSpecs that use 128-bit encryption algorithms are only available in 5722-AC3 (128-bit) IBM Cryptographic Access Provider.",
                  "Change channel <insert_3> to specify a valid CipherSpec that is available from the IBM Cryptographic Access Provider product installed on this machine. Check that the CipherSpec you are using is available on this machine in either the 5722-AC2 (56-bit) IBM Cryptographic Access Provider or 5722-AC3 (128-bit) IBM Cryptographic Access Provider licensed program."));
      amqMN_Ht.put("9636",new AMQMessageNumber(9636,
                  "SSL distinguished name does not match peer name, channel <insert_3>.",
                  "30 : Severe error",
                  "The distinguished name, <insert_4>, contained in the SSL certificate for the remote end of the channel does not match the local SSL peer name for channel <insert_3>. The distinguished name at the remote end must match the peer name specified (which can be generic) before the channel can be started.",
                  "If this remote system should be allowed to connect, either change the SSL peer name specification for the local channel so that it matches the distinguished name in the SSL certificate for the remote end of the channel, or obtain the correct certificate for the remote end of the channel. Restart the channel."));
      amqMN_Ht.put("9637",new AMQMessageNumber(9637,
                  "Channel is lacking a certificate.",
                  "30 : Severe error",
                  "The channel is lacking a certificate to use for the SSL handshake. The channel name is <insert_3> (if '????' it is unknown at this stage in the SSL processing). The channel did not start.",
                  "Make sure the appropriate certificates are correctly configured in the key repositories for both ends of the channel. If you have migrated from WebSphere MQ V5.3 to V6, it is possible that the missing certificate is due to a failure during SSL key repository migration. Check the relevant error logs. If these show that an orphan certificate was encountered then you should obtain the relevant missing certification authority (signer) certificates and then import these and the orphan certificate into the WebSphere MQ V6 key repository, and then re-start the channel."));
      amqMN_Ht.put("9638",new AMQMessageNumber(9638,
                  "SSL communications error for channel <insert_3>.",
                  "30 : Severe error",
                  "An unexpected SSL communications error occurred for a channel, as reported in the preceding messages. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Investigate the problem reported in the preceding messages. Review the local and remote console logs for reports of network errors. Correct the errors and restart the channel."));
      amqMN_Ht.put("9639",new AMQMessageNumber(9639,
                  "Remote channel <insert_3> did not specify a CipherSpec.",
                  "30 : Severe error",
                  "Remote channel <insert_3> did not specify a CipherSpec when the local channel expected one to be specified. The channel did not start.",
                  "Change the remote channel <insert_3> to specify a CipherSpec so that both ends of the channel have matching CipherSpecs."));
      amqMN_Ht.put("9640",new AMQMessageNumber(9640,
                  "SSL invalid peer name, channel <insert_3>, attribute <insert_5>.",
                  "30 : Severe error",
                  "The SSL peer name for channel <insert_3> includes a distinguished name attribute key <insert_5> which is invalid or unsupported. The channel did not start.",
                  "Correct the SSL peer name for the channel. Restart the channel."));
      amqMN_Ht.put("9641",new AMQMessageNumber(9641,
                  "Remote CipherSpec error for channel <insert_3>.",
                  "30 : Severe error",
                  "The remote end of channel <insert_3> has had a CipherSpec error. The channel did not start.",
                  "Review the error logs on the remote system to discover the problem with the CipherSpec."));
      amqMN_Ht.put("9642",new AMQMessageNumber(9642,
                  "No SSL certificate for channel <insert_3>.",
                  "30 : Severe error",
                  "The channel <insert_3> did not supply a certificate to use during SSL handshaking, but a certificate is required by the remote queue manager. The channel did not start.",
                  "Ensure that the key repository of the local queue manager or MQ client contains an SSL certificate which is associated with the queue manager or client. Alternatively, if appropriate, change the remote channel definition so that its SSLCAUTH attribute is set to OPTIONAL and it has no SSLPEER value set. If you have migrated from WebSphere MQ V5.3 to V6, it is possible that the missing certificate is due to a failure during SSL key repository migration. Check the relevant error logs. If these show that an orphan certificate was encountered then you should obtain the relevant missing certification authority (signer) certificates and then import these and the orphan certificate into the WebSphere MQ V6 key repository, and then re-start the channel."));
      amqMN_Ht.put("9642",new AMQMessageNumber(9642,
                  "(iSeries) No SSL certificate for channel <insert_3>.",
                  "0 : Information",
                  "The channel <insert_3> did not supply a certificate to use during SSL handshaking, but a certificate is required by the remote queue manager. The channel did not start.",
                  "If the SSL key repository for the queue manager has been specified as '*SYSTEM' ensure that a certificate has been associated with the application description for the queue manager in Digital Certificate Manager. Alternatively, if appropriate, change the remote channel definition so that its SSLCAUTH attribute is set to OPTIONAL and it has no SSLPEER value set."));
      amqMN_Ht.put("9643",new AMQMessageNumber(9643,
                  "Remote SSL peer name error for channel <insert_3>.",
                  "30 : Severe error",
                  "The remote end of channel <insert_3> has had an SSL peer name error. The channel did not start.",
                  "Review the error logs on the remote system to discover the problem with the peer name."));
      amqMN_Ht.put("9645",new AMQMessageNumber(9645,
                  "Correctly labelled SSL certificate missing on channel <insert_3>.",
                  "30 : Severe error",
                  "The key database file in use has not been set up with a correctly labelled SSL certificate. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Add a correctly labelled SSL certificate to the current key database file. Restart the channel."));
      amqMN_Ht.put("9646",new AMQMessageNumber(9646,
                  "Channel <insert_3> could not connect to any LDAP CRL servers.",
                  "30 : Severe error",
                  "LDAP Certification Revocation List (CRL) servers were specified but a connection could not be established to any of them. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Check that the LDAP CRL server specifications are correct. If they are, check that the servers are running and that the networking to access them is working correctly. Fix any errors found and restart the channel."));
      amqMN_Ht.put("9647",new AMQMessageNumber(9647,
                  "I/O error on SSL key repository.",
                  "30 : Severe error",
                  "An I/O error was encountered when attempting to read the SSL key repository. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Analyse why there is a I/O problem when reading the key repository. Fix the error if one is found, or it may be a temporary problem. Restart the channel."));
      amqMN_Ht.put("9648",new AMQMessageNumber(9648,
                  "The SSL key repository has an invalid internal format.",
                  "30 : Severe error",
                  "The SSL key repository has an invalid internal format. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Recreate the SSL key repository and restart the channel."));
      amqMN_Ht.put("9649",new AMQMessageNumber(9649,
                  "The SSL key repository contains duplicate keys.",
                  "30 : Severe error",
                  "The SSL key repository contains two or more entries with the same key. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Use your key management tool to remove the duplicate keys. Restart the channel."));
      amqMN_Ht.put("9650",new AMQMessageNumber(9650,
                  "The SSL key repository contains entries with duplicate labels.",
                  "30 : Severe error",
                  "The SSL key repository contains two or more entries with the same label. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Use your key management tool to remove the duplicate entries. Restart the channel."));
      amqMN_Ht.put("9651",new AMQMessageNumber(9651,
                  "The SSL key repository is corrupt or has a bad password.",
                  "30 : Severe error",
                  "The SSL key repository has become corrupted or its password id is incorrect. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Use your key management tool to recreate the key repository with a new password. Restart the channel."));
      amqMN_Ht.put("9652",new AMQMessageNumber(9652,
                  "The remote SSL certificate has expired.",
                  "30 : Severe error",
                  "The SSL certificate used by MQ on the remote end of the channel has expired. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Use your key management tool to provide MQ with a current SSL certificate on the remote end of the channel. Restart the channel."));
      amqMN_Ht.put("9653",new AMQMessageNumber(9653,
                  "An SSL trace file could not be opened.",
                  "10 : Warning",
                  "An SSL trace file could not be opened. The SSL trace files are created in directory /var/mqm/trace and have names AMQ.SSL.TRC and AMQ.SSL.TRC.1. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. This error is not regarded as sufficiently serious to interrupt channel operation; channel operation was not affected.",
                  "Check that you have a directory called /var/mqm/trace and that the userid under which WebSphere MQ runs has permissions and space to create and open a file in that directory. Fix the problem and you will get SSL trace output."));
      amqMN_Ht.put("9654",new AMQMessageNumber(9654,
                  "An invalid SSL certificate was received from the remote system.",
                  "30 : Severe error",
                  "An SSL certificate received from the remote system was not corrupt but failed validation checks on something other than its ASN fields and date. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the remote system has a valid SSL certificate. Restart the channel."));
      amqMN_Ht.put("9655",new AMQMessageNumber(9655,
                  "Problem loading GSKit SSL support.",
                  "30 : Severe error",
                  "MQ SSL support is provided on this platform using a component called GSKit which is installed as part of MQ. GSKit had an internal problem loading one if its dynamic link libraries. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Uninstall MQ and reinstall. Restart the channel."));
      amqMN_Ht.put("9656",new AMQMessageNumber(9656,
                  "An invalid SSL certificate was received from the remote system.",
                  "30 : Severe error",
                  "An SSL certificate received from the remote system was not corrupt but failed validation checks on its ASN fields. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the remote system has a valid SSL certificate. Restart the channel."));
      amqMN_Ht.put("9657",new AMQMessageNumber(9657,
                  "The key repository could not be opened (channel <insert_3>).",
                  "30 : Severe error",
                  "The key repository could not be opened. The key repository either does not exist or has incorrect permissions associated with it. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the key repository you specify exists and that its permissions are such that the MQ process involved can read from it. Restart the channel."));
      amqMN_Ht.put("9658",new AMQMessageNumber(9658,
                  "An invalid SSL certificate has been encountered.",
                  "30 : Severe error",
                  "An SSL certificate has been encountered which was not corrupt but which failed validation checks on its date fields. The certificate has either expired, or its date is not valid yet (i.e. the from date is later than today), or the validity date range is incorrect (e.g. the to date is earlier than the from date). The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that both the local and remote systems have valid, current SSL certificates. Restart the channel."));
      amqMN_Ht.put("9659",new AMQMessageNumber(9659,
                  "A failure occurred during SSL handshaking.",
                  "30 : Severe error",
                  "During SSL handshaking, or associated activities, a failure occurred. The failure is <insert_4> and has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Refer to prior message in the WebSphere MQ error log for information related to this problem."));
      amqMN_Ht.put("9660",new AMQMessageNumber(9660,
                  "SSL key repository: password stash file absent or unusable.",
                  "30 : Severe error",
                  "The SSL key repository cannot be used because MQ cannot obtain a password to access it. Reasons giving rise to this error include: (a) the key database file and password stash file are not present in the location configured for the key repository, (b) the key database file exists in the correct place but that no password stash file has been created for it, (c) the files are present in the correct place but the userid under which MQ is running does not have permission to read them, (d) one or both of the files are corrupt. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the key repository variable is set to where the key database file is. Ensure that a password stash file has been associated with the key database file in the same directory, and that the userid under which MQ is running has read access to both files. If both are already present and readable in the correct place, delete and recreate them. Restart the channel."));
      amqMN_Ht.put("9661",new AMQMessageNumber(9661,
                  "Bad SSL data from peer on channel <insert_3>.",
                  "30 : Severe error",
                  "An SSL channel has stopped because bad SSL data was received from the remote end of the channel. More detail on the nature of the corruption can be found from the GSKit return value of <insert_1> (the GSKit return values are documented in the MQ manuals). The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'.",
                  "Ensure you are connecting to a version of MQ which supports SSL at the remote end of the channel. Check your network between the two ends of the channel, and consider whether any possible causes of message corruption could be present. Fix any problems which may exist and restart the channel."));
      amqMN_Ht.put("9661",new AMQMessageNumber(9661,
                  "(iSeries) Bad SSL data from peer on channel <insert_3>.",
                  "0 : Information",
                  "An SSL channel has stopped because bad SSL data was received from the remote end of the channel. More detail on the nature of the corruption can be found from the GSKit return value of <insert_1> (the GSKit return values are documented in the MQ manuals and also in the GSKSSL member of the H file in library QSYSINC). The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'.",
                  "Ensure the remote queue manager and channel listener are running and that you are connecting to a version of MQ which supports SSL at the remote end of the channel. Check your network between the two ends of the channel, and consider whether any possible causes of message corruption could be present. Fix any problems which may exist and restart the channel."));
      amqMN_Ht.put("9662",new AMQMessageNumber(9662,
                  "SSL has encountered something it does not support.",
                  "30 : Severe error",
                  "This error can arise for a number of reasons: 1) The platform does not support a given type of cryptographic hardware, e.g. Ncipher and Rainbow are/were not supported on the Linux/390 platform. 2) The cryptographic hardware cryptography has returned an error. 3) Unsupported X509 General Name format when checking the remote certificate. The GSKit SSL provider incorporated in MQ only supports formats rfc822, DNSName, directoryname, uniformResourceID, and IPAddress. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Check that your cryptographic hardware is supported on your platform and test it to see that it is working correctly. Check that the remote certificates you are using conform to the X509 General Name formats listed. Fix the problem and restart the channel."));
      amqMN_Ht.put("9663",new AMQMessageNumber(9663,
                  "An invalid SSL certificate was received from the remote system.",
                  "30 : Severe error",
                  "An SSL certificate received from the remote system failed validation checks on its signature. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the remote system has a valid SSL certificate. Restart the channel."));
      amqMN_Ht.put("9664",new AMQMessageNumber(9664,
                  "Bad userid for CRL LDAP server; SSL channel <insert_3>.",
                  "30 : Severe error",
                  "Certification Revocation List (CRL) checking on an LDAP server or servers has been configured on the local MQ system. The userid information configured for the LDAP server or servers is incorrect. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Check the userid information for the CRL LDAP server or servers you have configured locally. Correct any problems found and restart the channel."));
      amqMN_Ht.put("9665",new AMQMessageNumber(9665,
                  "SSL connection closed by remote end of channel <insert_3>.",
                  "30 : Severe error",
                  "The SSL connection was closed by the remote end of the channel during the SSL handshake. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Check the remote end of the channel for SSL-related errors. Fix them and restart the channel."));
      amqMN_Ht.put("9666",new AMQMessageNumber(9666,
                  "Error accessing CRL LDAP servers; SSL channel <insert_3>.",
                  "30 : Severe error",
                  "CRL checking on LDAP servers has been configured on the local MQ system. An error was found when trying to access the CRL LDAP servers when validating a certificate from the remote system. Possible causes are: (a) cannot connect to any of the LDAP servers, or (b) invalid login user id or password for an LDAP server, or (c) the certificate issuer's Distinguished Name (DN) is not defined in the DIT of an LDAP server. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Check access to the CRL LDAP server(s) you have configured locally. Put right any problems found and restart the channel."));
      amqMN_Ht.put("9667",new AMQMessageNumber(9667,
                  "Bad password for CRL LDAP server; SSL channel <insert_3>.",
                  "30 : Severe error",
                  "Certification Revocation List (CRL) checking on an LDAP server or servers has been configured on the local MQ system. The password information configured for the LDAP server or servers is incorrect. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Check the password information for the CRL LDAP server or servers you have configured locally. Correct any problems found and restart the channel."));
      amqMN_Ht.put("9668",new AMQMessageNumber(9668,
                  "The specified PKCS #11 shared library could not be loaded.",
                  "30 : Severe error",
                  "A failed attempt was made to load the PKCS #11 shared library specified to MQ in the PKCS #11 driver path field of the GSK_PKCS11 SSL CryptoHardware parameter. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the PKCS #11 shared library exists and is valid at the location specified. Restart the channel."));
      amqMN_Ht.put("9669",new AMQMessageNumber(9669,
                  "The PKCS #11 token could not be found.",
                  "30 : Severe error",
                  "The PKCS #11 driver failed to find the token specified to MQ in the PKCS #11 token label field of the GSK_PKCS11 SSL CryptoHardware parameter. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the PKCS #11 token exists with the label specified. Restart the channel."));
      amqMN_Ht.put("9670",new AMQMessageNumber(9670,
                  "PKCS #11 card not present.",
                  "30 : Severe error",
                  "A PKCS #11 card is not present in the slot. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the correct PKCS #11 card is present in the slot. Restart the channel."));
      amqMN_Ht.put("9671",new AMQMessageNumber(9671,
                  "The PKCS #11 token password specified is invalid.",
                  "30 : Severe error",
                  "The password to access the PKCS #11 token is invalid. This is specified to MQ in the PKCS #11 token password field of the GSK_PKCS11 SSL CryptoHardware parameter. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Ensure that the PKCS #11 token password specified on GSK_PKCS11 allows access to the PKCS #11 token specified on GSK_PKCS11. Restart the channel."));
      amqMN_Ht.put("9672",new AMQMessageNumber(9672,
                  "An SSL security call failed.",
                  "30 : Severe error",
                  "An SSPI call to the Secure Channel (Schannel) SSL provider failed. The failure has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Consult the Windows Schannel reference manual to determine the meaning of status <insert_5> for SSPI call <insert_4>. Correct the failure and if necessary re-start the channel."));
      amqMN_Ht.put("9673",new AMQMessageNumber(9673,
                  "SSL client handshaking failed.",
                  "30 : Severe error",
                  "During an SSL client's handshaking, an SSPI call to the Secure Channel (Schannel) SSL provider failed. The failure has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Consult the Windows Schannel reference manual to determine the meaning of status <insert_4> for SSPI call <insert_5>. Correct the failure and if necessary re-start the channel."));
      amqMN_Ht.put("9674",new AMQMessageNumber(9674,
                  "An unknown error occurred during an SSL security call.",
                  "30 : Severe error",
                  "An unknown error occurred during an SSPI call to the Secure Channel (Schannel) SSL provider. The error may be due to a Windows SSL problem or to a general Windows problem or to invalid WebSphere MQ data being used in the call. The WebSphere MQ error recording routine has been called. The error has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Consult the Windows Schannel reference manual to determine the meaning of status <insert_5> for SSPI call <insert_4>. If the problem can be resolved using the manual, correct the failure and if necessary re-start the channel. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9675",new AMQMessageNumber(9675,
                  "The requested certificate could not be found.",
                  "30 : Severe error",
                  "A request for a certificate identified as <insert_4> <insert_5> in the store <insert_3> has failed, because the certificate could not be found. The Windows error code has been set to <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Consult the Windows reference manual to determine the meaning of error <insert_1> if this value is non-zero. Check to see whether the specified certificate has been copied to the correct certificate store and has not been deleted. Use the amqmcert command line utility or the WebSphere MQ Explorer administration application to configure certificate store for use with WebSphere MQ. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9676",new AMQMessageNumber(9676,
                  "The Windows cryptographic services library could not be loaded.",
                  "30 : Severe error",
                  "WebSphere MQ requires crypt32.dll to be available in order to carry out cryptographic functionality. The attempt to load this library returned the Windows error code <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Consult the Windows reference manual to determine the meaning of error code <insert_1>. Check that the crypt32.dll file is available and not corrupt. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9677",new AMQMessageNumber(9677,
                  "The Windows security services library could not be loaded.",
                  "30 : Severe error",
                  "WebSphere MQ requires <insert_3> to be available in order to run or configure SSL functionality. The attempt to load this library returned the Windows error code <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Consult the Windows reference manual to determine the meaning of error code <insert_1>. Check that the <insert_3> file is available and not corrupt. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9678",new AMQMessageNumber(9678,
                  "The certificate <insert_4>/<insert_5> already exists in the store <insert_3>.",
                  "10 : Warning",
                  "The certificate store <insert_3> already contains the specified certificate, identified by the issuer name of <insert_4>, serial number <insert_5>. The existing certificate has not been replaced.",
                  ""));
      amqMN_Ht.put("9679",new AMQMessageNumber(9679,
                  "The certificate store <insert_3> could not be opened.",
                  "30 : Severe error",
                  "The certificate store <insert_3> could not be opened, and failed with the Windows error code <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Consult the Windows reference manual to determine the meaning of error <insert_1> if this value is non-zero. Check that either your MQSSLKEYR environment variable (for client connections), or SSLKEYR queue manager attribute (for WebSphere MQ queue managers) has been defined correctly, and that the file path specified is valid. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9680",new AMQMessageNumber(9680,
                  "A problem was encountered with the specified certificate file.",
                  "30 : Severe error",
                  "A problem occurred when attempting to read the certificate from the file <insert_3>. The file may be corrupt or incorrectly formatted. The Windows error code reported is <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Ensure that the certificate file is valid and complete, and in one of the file formats supported by WebSphere MQ. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9681",new AMQMessageNumber(9681,
                  "The requested functionality is not supported on this system.",
                  "30 : Severe error",
                  "An SSL function was attempted that is not supported on this system. a) importing pfx format certificate files with private key data is only supported on Windows 2000 or greater. b) a the security library installed on your system is not of the correct level and does not contain the pre-requisite functions. On pre Windows 2000 systems, Internet Explorer 4.1 or greater must be installed. The WebSphere MQ error recording routine has been called.",
                  "If pre-requisite software is missing, please install the necessary levels of software and retry the operation. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9682",new AMQMessageNumber(9682,
                  "The WebSphere MQ SSL library has not been initialized.",
                  "30 : Severe error",
                  "The WebSphere MQ SSL library 'amqcssln.dll' has been called without it first being initialized by the calling process.",
                  "Ensure that the initialization function has been called prior to issuing any amqcssln function calls."));
      amqMN_Ht.put("9683",new AMQMessageNumber(9683,
                  "The private key data for this certificate is not exportable.",
                  "30 : Severe error",
                  "An attempt has been made to export the private key data from a certificate, but the properties of the certificate will not allow this. WebSphere MQ needs to be able to export private key data when copying personal certificates between certificate stores. The Windows cryptographic API returned the error code <insert_1>.",
                  "When requesting the certificate from the certificate authority, the private key data must be marked as exportable to enable WebSphere MQ to be able to copy the certificate and private key data into a WebSphere MQ store. The certificate file may need to be requested again to resolve this problem. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9684",new AMQMessageNumber(9684,
                  "A problem occurred while attempting to access the certificate's properties.",
                  "30 : Severe error",
                  "The certificate issued by <insert_3> with serial number <insert_4>, or it's private key data, appears to be unusable and may be corrupt. The Windows return code <insert_1> was generated when attempting to use this certificate. The WebSphere MQ error recording routine has been called.",
                  "Consult the Windows reference manual to determine the meaning of error <insert_1>. Check that the certificate is valid and has not been corrupted. If it is possible that the certificate or private key data is corrupt, try to remove the certificate from your system and re-import it. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9685",new AMQMessageNumber(9685,
                  "A problem occured while accessing the registry.",
                  "30 : Severe error",
                  "An error occured while attempting to load or unload the personal registry hive (HKEY_LOCAL_USER) for the user who launched this process. The WebSphere MQ error recording routine has been called.",
                  "If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9686",new AMQMessageNumber(9686,
                  "An unexpected error occured while attempting to manage a certificate store.",
                  "30 : Severe error",
                  "The Windows cryptographic API returned error code <insert_1> when calling the function <insert_3> for certificate store <insert_4>. The error may be due to a certificate store problem or to a general Windows problem or to a problem with a certificate in the store. The WebSphere MQ error recording routine has been called.",
                  "Consult the Windows reference manual to determine the meaning of error <insert_1>. Check that the certificate store is valid and not corrupt. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9687",new AMQMessageNumber(9687,
                  "The pfx password provided is invalid.",
                  "30 : Severe error",
                  "The password supplied for importing or copying the certificate is incorrect, and the operation could not be completed.",
                  "Make sure the password is correct and try again. If the password has been forgotten or lost, the certificate will need to be regenerated or exported from the original source."));
      amqMN_Ht.put("9688",new AMQMessageNumber(9688,
                  "The private key data for this certificate is unavailable.",
                  "30 : Severe error",
                  "The private key data associated with this certificate is reported as being present on the system, but has failed, returning the Windows error code <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Consult the Windows reference manual to determine the meaning of error code <insert_1>. If the problem can be resolved using the manual, correct the failure and if necessary re-try the operation. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9689",new AMQMessageNumber(9689,
                  "An unknown error occurred deleting the store <insert_3>.",
                  "30 : Severe error",
                  "The WebSphere MQ certificate store for queue manager <insert_3> could not be deleted. The filename for the certificate store is <insert_4>. The Windows error code has been set to <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Consult the Windows reference manual to determine the meaning of error <insert_1>. If the problem can be resolved using the manual, correct the failure and if necessary re-try the operation. Check that the store file exists and that other processes (such as queue managers) that may be accessing the store are not running. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9690",new AMQMessageNumber(9690,
                  "The public key in the issuer's certificate has failed to validate the subject certificate.",
                  "30 : Severe error",
                  "The public key in the issuer's certificate (CA or signer certificate), is used to verify the signature on the subject certificate assigned to channel <insert_3>. This verification has failed, and the subject certificate therefore cannot be used. The WebSphere MQ error recording routine has been called.",
                  "Check that the issuer's certificate is valid and available, and that it is up to date. Verify with the certificate's issuer that the subject certificate and issuer certificate should still be valid. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9691",new AMQMessageNumber(9691,
                  "The WebSphere MQ MQI library could not be loaded.",
                  "30 : Severe error",
                  "The library file <insert_3> is expected to be available on your system, but attempts to load it have failed with Windows return code <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Ensure that the WebSphere MQ <insert_3> library file exists and is available on your system. Consult the Windows reference manual to determine the meaning of error code <insert_1>. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9692",new AMQMessageNumber(9692,
                  "The SSL library has already been initialized.",
                  "20 : Error",
                  "The SSL library has already been initialized once for this process, any changes to SSL attributes will not take affect, and the original values will remain in force.",
                  "If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9693",new AMQMessageNumber(9693,
                  "The password provided for the LDAP server is incorrect.",
                  "30 : Severe error",
                  "One or more of the LDAP servers used for providing CRL information to WebSphere MQ has rejected a login attempt because the password provided is incorrect. The WebSphere MQ error recording routine has been called. The error has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Ensure that the passwords specified in the AuthInfo objects are correct for each server name provided. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9694",new AMQMessageNumber(9694,
                  "The DN syntax provided for an LDAP search is invalid.",
                  "30 : Severe error",
                  "The distinguished name provided in one or more AuthInfo object definitions is invalid, and the request to a CRL LDAP server has been rejected. The WebSphere MQ error recording routine has been called. The error has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Verify that the details supplied in the AuthInfo object definitions for this channel are correct. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9695",new AMQMessageNumber(9695,
                  "The username provided for the LDAP server is incorrect.",
                  "30 : Severe error",
                  "One or more of the LDAP servers used for providing CRL information to WebSphere MQ has rejected a login attempt because the username provided does not exist. The WebSphere MQ error recording routine has been called. The error has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Ensure that the usernamed specified in the AuthInfo objects for this channel are correct for each LDAP server name provided. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9696",new AMQMessageNumber(9696,
                  "Usage: amqmcert [SERVERNAME] [-a handle]",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("9697",new AMQMessageNumber(9697,
                  "WebSphere MQ Services could not be contacted on the target server.",
                  "30 : Severe error",
                  "An attempt was made to contact the WebSphere MQ Services on the target server <insert_3>. The call failed with return code <insert_1>. The WebSphere MQ error recording routine has been called.",
                  "Ensure that the target server name specified is correct and that you have sufficient access rights on that server to be able to administer WebSphere MQ. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9698",new AMQMessageNumber(9698,
                  "An SSL security call failed during SSL handshaking.",
                  "30 : Severe error",
                  "An SSPI call to the Secure Channel (Schannel) SSL provider failed during SSL handshaking. The failure has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Consult the Windows Schannel reference manual to determine the meaning of status <insert_5> for SSPI call <insert_4>. Correct the failure and if necessary re-start the channel."));
      amqMN_Ht.put("9699",new AMQMessageNumber(9699,
                  "An unknown error occurred during an SSL security call during SSL handshaking.",
                  "30 : Severe error",
                  "An unknown error occurred during an SSPI call to the Secure Channel (Schannel) SSL provider during SSL handshaking. The error may be due to a Windows SSL problem or to a general Windows problem or to invalid WebSphere MQ data being used in the call. The WebSphere MQ error recording routine has been called. The error has caused WebSphere MQ channel name <insert_3> to be closed. If the name is '????' then the name is unknown.",
                  "Consult the Windows Schannel reference manual to determine the meaning of status <insert_5> for SSPI call <insert_4>. If the problem can be resolved using the manual, correct the failure and if necessary re-start the channel. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9710",new AMQMessageNumber(9710,
                  "SSL security refresh failed.",
                  "30 : Severe error",
                  "The request to refresh SSL security was unsuccessful.",
                  "Look at previous error messages in the error files to determine the cause of the failure."));
      amqMN_Ht.put("9711",new AMQMessageNumber(9711,
                  "SSL security refresh succeeded but channel restarts failed.",
                  "30 : Severe error",
                  "The SSL environments for this queue manager have been refreshed so current values and certificates are in use for all SSL channels. However, not all the outbound SSL channels which were running when the security refresh was initiated could be restarted after the refresh had completed.",
                  "Look at previous error messages in the error files to determine which channels could not be restarted. Restart these if necessary."));
      amqMN_Ht.put("9712",new AMQMessageNumber(9712,
                  "SSL security refresh timed out waiting for channel <insert_3>.",
                  "30 : Severe error",
                  "The system was performing a security refresh for SSL. This function requests all outbound and inbound SSL channels to stop. It then waits for these channels to actually stop. SSL channel <insert_3> did not stop within the timeout period.",
                  "Investigate why channel <insert_3> is hung. Terminate the hung channel. Rerun the SSL security refresh."));
      amqMN_Ht.put("9713",new AMQMessageNumber(9713,
                  "Channel <insert_3> ended: SSL refresh in progress.",
                  "0 : Information",
                  "The SSL support on this queue manager is in the middle of a security refresh. An attempt was made to start outbound SSL channel <insert_3>. It cannot start while the SSL security refresh is in progress. The channel is restarted automatically once the SSL security refresh is complete.",
                  "None."));
      amqMN_Ht.put("9714",new AMQMessageNumber(9714,
                  "SSL refresh on receiving queue manager: channel did not start.",
                  "30 : Severe error",
                  "An SSL security refresh is in progress on the queue manager at the receiving end of this SSL channel. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'. The channel did not start.",
                  "Restart the channel once the SSL refresh is complete. The channel will restart automatically if it is configured to retry the connection."));
      amqMN_Ht.put("9715",new AMQMessageNumber(9715,
                  "Unexpected error detected in validating SSL session ID.",
                  "30 : Severe error",
                  "This error can arise when the GSKit SSL provider is missing one or more pre-requisite PTFs on the OS/400 platform. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'.",
                  "Ensure the GSKit SSL provider is at the latest level of maintenance and restart the channel."));
      amqMN_Ht.put("9719",new AMQMessageNumber(9719,
                  "Invalid CipherSpec for FIPS mode.",
                  "30 : Severe error",
                  "The user is attempting to start a channel on a queue manager or MQ client which has been configured to run in FIPS mode. The user has specified a CipherSpec which is not FIPS-compliant. The channel is <insert_3>; in some cases its name cannot be determined and so is shown as '????'.",
                  "Redefine the channel to run with a FIPS-compliant CipherSpec. Alternatively, the channel may be defined with the correct CipherSpec and the queue manager or MQ client should not be running in FIPS mode; if this is the case, ensure that FIPS mode is not configured. Once the error is corrected, restart the channel."));
      amqMN_Ht.put("9720",new AMQMessageNumber(9720,
                  "",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9721",new AMQMessageNumber(9721,
                  "",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9722",new AMQMessageNumber(9722,
                  "",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9723",new AMQMessageNumber(9723,
                  "",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9724",new AMQMessageNumber(9724,
                  "",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9725",new AMQMessageNumber(9725,
                  "",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9726",new AMQMessageNumber(9726,
                  "A certificate failed to be migrated because it has an invalid date.",
                  "30 : Severe error",
                  "During the migration of a certificate, the certificate's date fields have been found to be invalid. The certificate has either expired or its 'from' date is later than today's date or its 'to' date is earlier than the 'from' date. The certificate has not been migrated.",
                  "If the certificate is required for migration then obtain a valid replacement before importing it into the GSKit key database <insert_5>."));
      amqMN_Ht.put("9727",new AMQMessageNumber(9727,
                  "A certificate failed to be migrated because it has an incomplete certification path.",
                  "30 : Severe error",
                  "During the migration of a certificate, the certificate's certification authority (signer) certificate could not be found. The certificate is therefore regarded as an orphan certificate. A copy of the certificate has been written to the file name <insert_4>. If file name is suffixed \".cer\" then the certificate is a certification authority (signer) certificate. If file name is suffixed \".pfx\" then the certificate is a personal certificate and it has a password which is the same as that specified for the GSKit key database <insert_5>. The certificate has not been migrated.",
                  "If the certificate is required for migration then ensure that a complete certification path exists in the GSKit key database <insert_5> before importing the certificate."));
      amqMN_Ht.put("9728",new AMQMessageNumber(9728,
                  "A certificate failed to be migrated because it could not be imported into the GSKit key database <insert_5>.",
                  "30 : Severe error",
                  "A certificate failed to be imported because there was a problem during the migration of the certificate. A copy of the certificate has been written to the file name <insert_4>. If file name is suffixed \".cer\" then the certificate is a certification authority (signer) certificate. If file name is suffixed \".pfx\" then the certificate is a personal certificate and it has a password which is the same as that specified for the GSKit key database <insert_5>. The certificate has not been migrated.",
                  "Refer to the previous message in the error log to determine the cause of the failure. If appropriate, refer to the Windows or GSKit reference documentation to determine the cause."));
      amqMN_Ht.put("9729",new AMQMessageNumber(9729,
                  "Unable to create certificate file <insert_3>.",
                  "30 : Severe error",
                  "A certificate failed to be imported because there was a problem during the migration of the certificate. In addition to this first problem, a second problem occurred when trying to create a copy of the certificate by writing it to the file <insert_3>. The certificate is located in the Microsoft Certificate Store <insert_4>. The certificate is intended for the GSKit key database <insert_5>. If file name is suffixed \".cer\" then the certificate is a certification authority (signer) certificate. If file name is suffixed \".pfx\" then the certificate is a personal certificate. The certificate has not been migrated.",
                  "Determine the cause of the 2 problems. Refer to the previous message in the error log to determine the cause of the first failure. If appropriate, refer to the Windows or GSKit reference documentation to determine the cause. The second failure occurred during a call to the Windows 'CreateFile' function with a return code of <insert_1>. For this failure, check that file does not already exist and that you have authority to create this file."));
      amqMN_Ht.put("9730",new AMQMessageNumber(9730,
                  "Certificate migration has completed with no failures. The number of certificates migrated was <insert_1>.",
                  "0 : Information",
                  "The migration of certificates from the Microsoft Certificate Store <insert_3> to the GSKit key database <insert_4> has completed and there were no migration failures. The number of certificates migrated was <insert_1>.",
                  "If any certificates were migrated, use the GSKit iKeyman GUI to verify that the GSKit key database contains all the certificates required to support the intended SSL channel. If no certificates were migrated then this is probably because <insert_3> contained only a default set of certification authority (signer) certificates. The default set is not migrated because the newly created GSKit key database will have its own set which will be the same or more up to date. Although there were no failures which caused certificates not to be migrated, there may have been other failures and these must be resolved otherwise the SSL channel may subsequently fail to start. Refer to the error log and check for any failures."));
      amqMN_Ht.put("9731",new AMQMessageNumber(9731,
                  "The Transfer Certificates (amqtcert) command has completed.",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("9732",new AMQMessageNumber(9732,
                  "A registry entry already exists for <insert_3>.",
                  "30 : Severe error",
                  "The command has been used to request automatic migration for a queue manager's or a client's Microsoft Certificate Store. However, there is already an entry in the registry for this store. If the request was for a queue manager then <insert_3> is the queue manager name, otherwise it is the name of the client's Microsoft Certificate Store.",
                  "List, and then check, the contents of the registry by running the Transfer Certificates (amqtcert) command with the options \"-a -l\". If it is necessary to replace the entry then firstly remove it, by using amqtcert with the \"-r\" option, then use amqtcert to request automatic migration."));
      amqMN_Ht.put("9733",new AMQMessageNumber(9733,
                  "The request to automatically migrate certificates has completed successfully.",
                  "0 : Information",
                  "A request was made to automatically migrate SSL certificates. This request may have been made during the installation of WebSphere MQ or by using the Transfer Certificates (amqtcert) command. The request has now been performed and the migration has completed successfully.",
                  "Use the GSKit iKeyman GUI to verify that the GSKit key database contains all the certificates required to support the intended SSL channel. If no certificates were migrated then this is because the Microsoft Certificate Store contained only a default set of certification authority (signer) certificates. The default set is not migrated because the newly created GSKit key database will have its own set which will be the same or more up to date."));
      amqMN_Ht.put("9734",new AMQMessageNumber(9734,
                  "There was a failure during the automatic migration of certificates.",
                  "30 : Severe error",
                  "A request was made to automatically migrate SSL certificates. This request may have been made during the installation of WebSphere MQ or by using the Transfer Certificates (amqtcert) command. The request has now been performed but there was a failure during the migration process.",
                  "Refer to previous messages in the error log to determine the cause of the failure. It may be the case that all certificates have successfully migrated and that the failure did not affect this part of the migration process. In this case, use the GSKit iKeyman GUI to verify that the GSKit key database contains all the certificates required to support the intended SSL channel."));
      amqMN_Ht.put("9735",new AMQMessageNumber(9735,
                  "Certificate migration has terminated unexpectedly. A failure occured during GSKit initialization.",
                  "30 : Severe error",
                  "The certificate migration process has terminated unexpectedly. The migration requires the GSKit environment to be successfully initialized. This involves the GSKit operations of initialization, creation of the key database and stashing of the key database password. There was a failure during one of these operations. No certificates have been migrated. If the stashing of the password failed then the key database <insert_4> will have been created. The failure occurred during the GSKit operation <insert_3> and the GSKit return code <insert_1> was generated.",
                  "If the key database has been created then, after the cause of the failure has been resolved, delete it, remove the relevant registry state information and then re-try the certificate migration process. Use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9736",new AMQMessageNumber(9736,
                  "The library <insert_3> was not found.",
                  "30 : Severe error",
                  "An attempt to dynamically load the library <insert_3> failed because the library was not found. If this an WebSphere MQ library, it is only available on WebSphere MQ server installations and is required when the Transfer Certificates (amqtcert) command is used to perform a queue manager operation. If this a GSKit library, it should have been installed during the WebSphere MQ installation.",
                  "Do not use the command to perform a queue manager operation on a WebSphere MQ client-only installation. If the command has been made on a WebSphere MQ server installation, or if it is a GSKit library which is missing, then record the problem identifier, save any generated output files and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9737",new AMQMessageNumber(9737,
                  "Unable to allocate memory.",
                  "30 : Severe error",
                  "An attempt to allocate memory failed.",
                  "Make more memory available to the command."));
      amqMN_Ht.put("9738",new AMQMessageNumber(9738,
                  "Unable to obtain the MQSSLKEYR environment variable value.",
                  "30 : Severe error",
                  "An attempt to obtain the MQSSLKEYR environment variable value failed. When using the command to specify all clients then the MQSSLKEYR environment variable must be defined with the name of a Microsoft Certificate Store file containing certificates for all clients.",
                  "Ensure that the MQSSLKEYR environment variable is defined with an appropriate value."));
      amqMN_Ht.put("9739",new AMQMessageNumber(9739,
                  "The certificate store <insert_3> could not be accessed.",
                  "30 : Severe error",
                  "The certificate store <insert_3> could not be accessed, and failed with Windows error code <insert_1>. If you are using the -c parameter check that the name given to amqtcert is correct. If you are using the -m parameter check the SSLKEYR value on the queue manager specified.",
                  "Consult the Windows reference manual to determine the meaning of error <insert_1> if this value is non-zero. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9740",new AMQMessageNumber(9740,
                  "The certificate store <insert_3> could not be opened.",
                  "30 : Severe error",
                  "The certificate store <insert_3> could not be opened, and failed with Windows error code <insert_1>.",
                  "Consult the Windows reference manual to determine the meaning of error <insert_1> if this value is non-zero. If the problem cannot be resolved then use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9741",new AMQMessageNumber(9741,
                  "A problem occurred during a Windows operation.",
                  "30 : Severe error",
                  "During operation <insert_3>, the Windows return code <insert_1> was generated.",
                  "Consult the Windows reference manual to determine the meaning of return code <insert_1> for operation <insert_3>."));
      amqMN_Ht.put("9742",new AMQMessageNumber(9742,
                  "A problem occured during a GSKit operation.",
                  "30 : Severe error",
                  "During operation <insert_3>, the GSKit return code <insert_1> was generated.",
                  "Use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9743",new AMQMessageNumber(9743,
                  "A certificate failed to be migrated and failed to be logged.",
                  "30 : Severe error",
                  "There was a problem trying to migrate a certificate to the GSKit key database <insert_5>.",
                  "Refer to the previous message in the error log to determine why the migration failed."));
      amqMN_Ht.put("9744",new AMQMessageNumber(9744,
                  "No matching automatic migration registry entry.",
                  "10 : Warning",
                  "There is no automatic certificate migration entry in the registry which matches the input provided.",
                  "None, if the entry to be removed was correctly specified. Otherwise, input the command again with correct parameters."));
      amqMN_Ht.put("9745",new AMQMessageNumber(9745,
                  "amqtcert: insufficient memory to migrate certificates.",
                  "30 : Severe error",
                  "An attempt to allocate memory failed while amqtcert was migrating certificate file <insert_3>.sto'. The migration did not complete successfully.",
                  "Do not delete <insert_3>.sto', but delete all other files called <insert_4>.*' (these were created as a result of the failed migration). Also, if you want to rerun this migration automatically, use the -r flag on amqtcert to remove the automatic migration registry entry for this .sto file. Then use the -a flag on amqtcert to create a new automatic migration registry entry for this .sto file. Make more memory available. Rerun the migration."));
      amqMN_Ht.put("9746",new AMQMessageNumber(9746,
                  "File <insert_3> not found.",
                  "30 : Severe error",
                  "The file specified as a command argument has not been found. The characters \".sto\" have been automatically appended to the file name.",
                  "Check that file exists and that it is specified as the absolute (rather than relative) directory path and file name (excluding the .sto suffix) of the Microsoft Certificate Store."));
      amqMN_Ht.put("9747",new AMQMessageNumber(9747,
                  "Usage: amqtcert [-a] [-c [Filename | *]] [-e ExpirationTime] [-g FileName]",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("9748",new AMQMessageNumber(9748,
                  "A problem occurred accessing the Windows registry.",
                  "30 : Severe error",
                  "An attempt to access a key or value or data field in the Windows registry key failed. The failure may be due to part of the registry being in an invalid state or may be due to insufficient authority to access that part. The WebSphere MQ error recording routine has been called.",
                  "If <insert_3> includes the name of a Windows call, consult the Windows reference manual to determine the meaning of status <insert_1> for that call. Use the standard facilities supplied with your system to record the problem identifier, and to save the generated output files. Contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9749",new AMQMessageNumber(9749,
                  "Invalid combination of command arguments.",
                  "30 : Severe error",
                  "The command syntax is incorrect because of an invalid combination of arguments.",
                  "Re-try the command using a valid combination of arguments."));
      amqMN_Ht.put("9750",new AMQMessageNumber(9750,
                  "File <insert_3> already exists.",
                  "30 : Severe error",
                  "The file <insert_3> cannot be created because it already exists.",
                  "Ensure that the file does not exist in the directory. If necessary, make a copy of the file before renaming or moving or deleting it."));
      amqMN_Ht.put("9751",new AMQMessageNumber(9751,
                  "You are not authorized to perform the requested operation.",
                  "30 : Severe error",
                  "You tried to issue a command for which you are not authorized.",
                  "Contact your system administrator to perform the command for you or to request authority to perform the command."));
      amqMN_Ht.put("9752",new AMQMessageNumber(9752,
                  "A certificate failed to be migrated because a Windows operation failed.",
                  "30 : Severe error",
                  "A personal certificate could not be migrated because there was a failure during the Windows operation <insert_3> with a return code of <insert_1>. A personal certificate is exported, with its private key data, from the Microsoft Certificate Store prior to being imported into the GSKit key database. The failure occurred during the export and is probably due to a problem with accessing or using the private key data assoicated with the personal certificate.",
                  "Check that the private key data is available and that you have authority to access it. Consult the Windows reference manual to determine the meaning of return code <insert_1> for operation <insert_3>."));
      amqMN_Ht.put("9753",new AMQMessageNumber(9753,
                  "File <insert_3> is empty.",
                  "30 : Severe error",
                  "The file <insert_3> cannot be used because it is empty.",
                  "Ensure that the correct file has been used and if necessary investigate the reason for it being empty."));
      amqMN_Ht.put("9754",new AMQMessageNumber(9754,
                  "A certificate failed to be migrated because a GSKit operation failed.",
                  "30 : Severe error",
                  "During operation <insert_3>, the GSKit return code <insert_1> was generated.",
                  "Use the standard facilities supplied with your system to record the problem identifier and save the generated output files, and then contact your IBM support center. Do not discard these files until the problem has been resolved."));
      amqMN_Ht.put("9755",new AMQMessageNumber(9755,
                  "Certificate migration has completed with some failures. The number of certificates migrated was <insert_1>.",
                  "0 : Information",
                  "The migration of certificates from the Microsoft Certificate Store <insert_3> to the GSKit key database <insert_4> has completed but there has been one or more failures. The number of certificates migrated was <insert_1>.",
                  "If any certificates were migrated, use the GSKit iKeyman GUI to verify that the GSKit key database contains all the certificates required to support the intended SSL channel. The failures must be resolved otherwise the SSL channel may subsequently fail to start. Refer to previous messages in the error log to determine the cause of such failures."));
      amqMN_Ht.put("9756",new AMQMessageNumber(9756,
                  "The number of certificates in the Microsoft Certificate Store <insert_3> is <insert_1>.",
                  "0 : Information",
                  "Provides a count of the number of certificates in the Microsoft Certificate Store <insert_3>.",
                  "None."));
      amqMN_Ht.put("9757",new AMQMessageNumber(9757,
                  "",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9758",new AMQMessageNumber(9758,
                  "Subject: <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9759",new AMQMessageNumber(9759,
                  "Issuer: <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9760",new AMQMessageNumber(9760,
                  "Valid From: <insert_3> to <insert_4>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9761",new AMQMessageNumber(9761,
                  "Certificate Usage: <All>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9762",new AMQMessageNumber(9762,
                  "Certificate Usage: <insert_3>",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9763",new AMQMessageNumber(9763,
                  "Certificate Type: Personal",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9764",new AMQMessageNumber(9764,
                  "Certificate Type: Signer",
                  "0 : Information",
                  "None.",
                  "None."));
      amqMN_Ht.put("9765",new AMQMessageNumber(9765,
                  "Personal certificate not found for the command option \"-i <insert_1>\".",
                  "30 : Severe error",
                  "The Transfer Certificates (amqtcert) command was executed using the \"-i ListNumber\" option with a value of <insert_1>. However, no personal certificate was found which corresponded to this value. Certificate migration has failed and no certificates were migrated.",
                  "Check that the option value corresponds to a correctly identified personal certificate. If it is not correct then run the command using the \"-l List\" option to determine the correct number. A GSKit key database, and its associated key database files, was created when the command was run using the \"-i ListNumber\" option. The database and associated files must be deleted before re-trying the command with the \"-i\" option."));
      amqMN_Ht.put("9766",new AMQMessageNumber(9766,
                  "A failure occurred creating the GSKit key database <insert_4>.",
                  "30 : Severe error",
                  "GSKit was unable to create the key database and its associated files. During the GSKit operation <insert_3>, the return code <insert_1> was generated. This is probably due to insufficient authority or to insufficient disk space being available.",
                  "Check that you have sufficient authority and that there is sufficient disk space available."));
      amqMN_Ht.put("9767",new AMQMessageNumber(9767,
                  "Usage: strmqikm [iKeymanWorkingDirectory]",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("9768",new AMQMessageNumber(9768,
                  "Directory <insert_3> not found.",
                  "30 : Severe error",
                  "The directory specified as a command argument has not been found.",
                  "Check that the directory exists and that it is specified as an absolute (rather than relative) directory path."));
      amqMN_Ht.put("9769",new AMQMessageNumber(9769,
                  "Usage: runmqckm",
                  "0 : Information",
                  "",
                  "None."));
      amqMN_Ht.put("9913",new AMQMessageNumber(9913,
                  "The specified local address <insert_3> cannot be resolved to an IP address. The return code is <insert_1>.",
                  "30 : Severe error",
                  "An attempt to resolve the local address hostname to an IP address has failed.",
                  "Check that the local address hostname is correct and has an entry in the DNS database."));
      amqMN_Ht.put("9914",new AMQMessageNumber(9914,
                  "The type of local address specified is incompatible with the IP protocol (<insert_3>) used.",
                  "30 : Severe error",
                  "An attempt to use a local address that is incompatible with the IP protocol used.",
                  "Make sure that the local address specified is of the same type (IPv4 or IPV6) as the IP Protocol."));
      amqMN_Ht.put("9915",new AMQMessageNumber(9915,
                  "The IP protocol <insert_3> is not present on the system.",
                  "30 : Severe error",
                  "An attempt to use an IP protocol that is not present on the system has been made.",
                  "Install the required IP protocol or use an IP protocol that is available on the system."));
      amqMN_Ht.put("9920",new AMQMessageNumber(9920,
                  "A SOAP Exception has been thrown.",
                  "30 : Severe error",
                  "A SOAP method encountered a problem and has thrown an exception. Details of the exception are: <insert_3>",
                  "Investigate why the SOAP method threw the exception."));
      amqMN_Ht.put("9921",new AMQMessageNumber(9921,
                  "An error was encountered writing to the Dead Letter Queue.",
                  "30 : Severe error",
                  "An error was encountered when an attempt was made to write a message to Dead Letter Queue <insert_3>. The message was <insert_4>.",
                  "Ensure that Dead Letter Queue <insert_3> exists and is put enabled. Ensure that the Queue Manager attribute DEADQ is set up correctly. Resend the SOAP message."));
      amqMN_Ht.put("9922",new AMQMessageNumber(9922,
                  "Maximum wait time exceeded on queue <insert_3>.",
                  "30 : Severe error",
                  "The maximum time waiting for a message to arrive on queue <insert_3> has been exceeded.",
                  "Ensure that the queue is not put inhibited. Ensure that messages are being written to the queue."));
      amqMN_Ht.put("9923",new AMQMessageNumber(9923,
                  "Insufficient parameters on command.",
                  "30 : Severe error",
                  "The SOAP command has been issued with insufficient paramaters.",
                  "Supply the correct number of parameters and reissue the command."));
      amqMN_Ht.put("9924",new AMQMessageNumber(9924,
                  "Usage: amqwSOAPNETListener -u wmqUri",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("9925",new AMQMessageNumber(9925,
                  "Cannot connect to queue manager <insert_3>.",
                  "30 : Severe error",
                  "A SOAP application or the SOAP listener cannot connect to the queue manager <insert_3> using <insert_4> bindings.",
                  "Ensure the bindings are set to the correct value and that the queue manager exists. Check any error messages from the Java MQQueueManager class."));
      amqMN_Ht.put("9926",new AMQMessageNumber(9926,
                  "Null SOAP action specified in a received SOAP message.",
                  "30 : Severe error",
                  "A NULL soap action has been specified in the SOAP message <insert_3>. The message will not be processed.",
                  "Include the appropriate SOAP action in the SOAP message."));
      amqMN_Ht.put("9927",new AMQMessageNumber(9927,
                  "MQ queue backout threshold exceeded.",
                  "30 : Severe error",
                  "The WebSphere MQ backout threshold value has been exceeded for queue <insert_3>, processing message <insert_4>.",
                  "Correct the backout threshold value for queue <insert_3> and resend the SOAP message."));
      amqMN_Ht.put("9928",new AMQMessageNumber(9928,
                  "Target service or URI is missing from a SOAP message.",
                  "30 : Severe error",
                  "The target service or the target URI is missing from SOAP message <insert_3>.",
                  "Supply a target service or the target URI in the SOAP message."));
      amqMN_Ht.put("9929",new AMQMessageNumber(9929,
                  "Message backout for message (<insert_3>) failed.",
                  "30 : Severe error",
                  "Backout for a message has failed.",
                  "Investigate the reason for the backout failure."));
      amqMN_Ht.put("9930",new AMQMessageNumber(9930,
                  "Required Option <insert_3> missing from command.",
                  "30 : Severe error",
                  "The SOAP command was issued with manadatory option <insert_3> missing.",
                  "Reissue the SOAP command supplying the missing option."));
      amqMN_Ht.put("9931",new AMQMessageNumber(9931,
                  "Invalid value <insert_3> specified for option <insert_4>.",
                  "30 : Severe error",
                  "THE SOAP command was issued with an invalid value for an option.",
                  "Reissue the SOAP command supplying the correct option value."));
      amqMN_Ht.put("9932",new AMQMessageNumber(9932,
                  "Application host class not found",
                  "30 : Severe error",
                  "Application host class <insert_3> has not been found.",
                  "Specify the correct application host class in the SOAP message."));
      amqMN_Ht.put("9933",new AMQMessageNumber(9933,
                  "Options <insert_3> and <insert_4> are mutually exclusive",
                  "30 : Severe error",
                  "The SOAP command was issued with incompatible options <insert_3> and <insert_4>.",
                  "Reissue the SOAP command supplying compatible options."));
      amqMN_Ht.put("9934",new AMQMessageNumber(9934,
                  "Could not parse URL <insert_3>. MQCC_FAILED(2) MQRC_SOAP_URL_ERROR(2212).",
                  "30 : Severe error",
                  ".Could not parse URL <insert_3>.MQCC_FAILED(2) MQRC_SOAP_URL_ERROR(2212).",
                  "Correct the URL and retry."));
      amqMN_Ht.put("9935",new AMQMessageNumber(9935,
                  "Illegal URL <insert_3>. MQCC_FAILED(2) MQRC_SOAP_URL_ERROR(2212).",
                  "30 : Severe error",
                  ".The URL <insert_3> failed validation.. MQCC_FAILED(2) MQRC_SOAP_URL_ERROR(2212).",
                  "Correct the URL and retry."));
      amqMN_Ht.put("9936",new AMQMessageNumber(9936,
                  "Cannot get connection using <insert_3> bindings. MQCC_FAILED(2) MQRC_CONNECTION_ERROR(2273).",
                  "30 : Severe error",
                  ".Cannot get connection using <insert_3> bindings. MQCC_FAILED(2) MQRC_CONNECTION_ERROR(2273).",
                  "Check that the queue manager is available and running."));
      amqMN_Ht.put("9937",new AMQMessageNumber(9937,
                  "The asyncResult is null. MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR.(2210).",
                  "30 : Severe error",
                  ".The asyncResult is null. MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR.(2210).",
                  "Check why the SOAP responses are not being received."));
      amqMN_Ht.put("9938",new AMQMessageNumber(9938,
                  "SOAP/WMQ Timeout.",
                  "30 : Severe error",
                  ".The MQGET operation timed out. MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR.(2210).",
                  "Check why the SOAP responses are not being received. MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR.(2210)."));
      amqMN_Ht.put("9939",new AMQMessageNumber(9939,
                  "SOAP/WMQ Error. MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR.(2210).",
                  "30 : Severe error",
                  ".A SOAP error was detected. MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR.(2210).",
                  "Check the WMQ logs for the reason of the failure."));
      amqMN_Ht.put("9940",new AMQMessageNumber(9940,
                  "Report message returned in MQWebResponse. MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR.(2210).",
                  "30 : Severe error",
                  ".Report message returned in MQWebResponse. MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR.(2210).",
                  "Check the report message for the reason of the failure."));
      amqMN_Ht.put("9941",new AMQMessageNumber(9941,
                  "No RFH2 header recognised. MQCC_FAILED(2) MQRCCF_MD_FORMAT_ERROR(3023).",
                  "30 : Severe error",
                  ".No RFH2 header recognised. MQCC_FAILED(2) MQRCCF_MD_FORMAT_ERROR(3023).",
                  "Check why the message is being sent with no RFH2 header."));
      amqMN_Ht.put("9942",new AMQMessageNumber(9942,
                  "Message format is not MQFMT_NONE. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "30 : Severe error",
                  ".Message format is not MQFMT_NONE. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "Correct the message format and retry."));
      amqMN_Ht.put("9943",new AMQMessageNumber(9943,
                  "Unrecognised RFH2 version. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "30 : Severe error",
                  ".Unrecognised RFH2 version. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "Correct the version in the RFH2 message and retry."));
      amqMN_Ht.put("9944",new AMQMessageNumber(9944,
                  "Invalid RFH2 length. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "30 : Severe error",
                  ".Invalid RFH2 length. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "Correct the RFH2 length and retry."));
      amqMN_Ht.put("9945",new AMQMessageNumber(9945,
                  "Illegal RFH2 <insert_3> folder length. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "30 : Severe error",
                  ".Illegal RFH2 <insert_3> folder length. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "Correct the RFH2 message and retry."));
      amqMN_Ht.put("9946",new AMQMessageNumber(9946,
                  "Invalid actual message length. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "30 : Severe error",
                  ".Invalid actual message length. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "Correct the RFH2 message and retry."));
      amqMN_Ht.put("9947",new AMQMessageNumber(9947,
                  "Illegal RFH2 Folder <insert_3> <insert_4>. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "30 : Severe error",
                  ".Illegal RFH2 Folder <insert_3> <insert_4>. MQCC_FAILED(2) MQRC_RFH_FORMAT_ERROR(2421).",
                  "Correct the RFH2 folder syntax/format and retry."));
      amqMN_Ht.put("9948",new AMQMessageNumber(9948,
                  "Backout Threshold exceeded. MQCC_FAILED(2) MQRC_BACKOUT_THRESHOLD_REACHED(2362).",
                  "30 : Severe error",
                  ".Backout Threshold exceeded. MQCC_FAILED(2) MQRC_BACKOUT_THRESHOLD_REACHED(2362).",
                  "Correct the backout threshold limit and retry."));
      amqMN_Ht.put("9949",new AMQMessageNumber(9949,
                  "<insert_3> missing from RFH2. MQCC_FAILED(2) MQRC_RFH_PARM_MISSING(2339).",
                  "30 : Severe error",
                  ".<insert_3> missing from RFH2. MQCC_FAILED(2) MQRC_RFH_PARM_MISSING(2339).",
                  "Correct the RFH2 message and retry."));
      amqMN_Ht.put("9950",new AMQMessageNumber(9950,
                  "Target service missing from SOAP URL. MQCC_FAILED(2) MQRC_SOAP_URL_ERROR(2212).",
                  "30 : Severe error",
                  ".Target service missing from SOAP URL. MQCC_FAILED(2) MQRC_SOAP_URL_ERROR(2212).",
                  "Correct the URL and retry."));
      amqMN_Ht.put("9951",new AMQMessageNumber(9951,
                  "Asynchronous request queued successfully. MQCC_OK(0).",
                  "30 : Severe error",
                  ".Asynchronous request queued successfully. MQCC_OK(0).",
                  "Wait for response if any is expected."));
      amqMN_Ht.put("9952",new AMQMessageNumber(9952,
                  "Unexpected message type received. MQCC_FAILED(2) MQRC_UNEXPECTED_MSG_TYPE.(2215).",
                  "30 : Severe error",
                  ".A message of the wrong type was received; for instance, a report message was received when one had not been requested.",
                  "If you are running WMQ SOAP using the IBM supplied SOAP/WMQ sender, please contact IBM. If you are running WMQ SOAP using a bespoke sender, please check that the SOAP/WMQ request message has the correct options."));
      amqMN_Ht.put("9953",new AMQMessageNumber(9953,
                  "Either the ContentType or the TransportVersion in the RFH2 header have the wrong value. MQCC_FAILED(2) MQRC_RFH_HEADER_FIELD_ERROR(2228)",
                  "30 : Severe error",
                  ".Either the ContentType or the TransportVersion in the RFH2 header have the wrong value. MQCC_FAILED(2) MQRC_RFH_HEADER_FIELD_ERROR(2228)",
                  "Correct the message format and retry."));
      amqMN_Ht.put("9954",new AMQMessageNumber(9954,
                  "ViaTran.Redirect called out of transaction MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR(2410)",
                  "30 : Severe error",
                  ".ViaTran.Redirect called out of transaction MQCC_FAILED(2) MQRC_SOAP_DOTNET_ERROR(2410)",
                  "Make sure ViaTran.Redirect is only called in a transaction."));
      amqMN_Ht.put("9955",new AMQMessageNumber(9955,
                  "Usage: amqswsdl [?] Uri inputFile outputFile",
                  "0 : Information",
                  "This shows the correct usage.",
                  "None."));
      amqMN_Ht.put("9990",new AMQMessageNumber(9990,
                  "(iSeries) Keyword <insert_3> not valid for this command or the command is incomplete.",
                  "40 : Stop Error",
                  "The command is incomplete, or an invalid keyword was specified, or the parameter value of the keyword was not specified.",
                  "Complete the command, or correct the keyword, or add the parameter value, and then try the command again."));
      amqMN_Ht.put("9991",new AMQMessageNumber(9991,
                  "(iSeries) The value specified is not allowed by the command.",
                  "40 : Stop Error",
                  "<insert_3> not valid for parameter <insert_4>.",
                  "Enter one of the values that is defined for the parameter, and try the command again. More information on parameters and commands can be found in the CL reference manual or the appropriate licensed program manual."));
      amqMN_Ht.put("9992",new AMQMessageNumber(9992,
                  "(iSeries) A matching parenthesis not found.",
                  "40 : Stop Error",
                  "A matching left or right parenthesis is missing.",
                  "Add the missing parenthesis or remove the extra parenthesis."));
      amqMN_Ht.put("9999",new AMQMessageNumber(9999,
                  "Channel program ended abnormally.",
                  "30 : Severe error",
                  "Channel program <insert_3> ended abnormally.",
                  "Look at previous error messages for channel program <insert_3> in the error files to determine the cause of the failure."));
   }

   /**
    * Initialize the AMQ 7X00's message number. 
    */
   public void init7Xs()
   {
      amqMN_Ht.put("7A01",new AMQMessageNumber(0x7A01,
                                               "(iSeries) Convert MQ Data Type",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A02",new AMQMessageNumber(0x7A02,
                                               "(iSeries) Display MQ Version",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A03",new AMQMessageNumber(0x7A03,
                                               "(iSeries) Create MQ Listener",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A04",new AMQMessageNumber(0x7A04,
                                               "(iSeries) Listener name",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A05",new AMQMessageNumber(0x7A05,
                                               "(iSeries) Listener control",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A06",new AMQMessageNumber(0x7A06,
                                               "(iSeries) Listener backlog",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A07",new AMQMessageNumber(0x7A07,
                                               "(iSeries) Change MQ Listener",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A08",new AMQMessageNumber(0x7A08,
                                               "(iSeries) Copy MQ Listener",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A09",new AMQMessageNumber(0x7A09,
                                               "(iSeries) From Listener",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A0A",new AMQMessageNumber(0x7A0A,
                                               "(iSeries) To Listener",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A0B",new AMQMessageNumber(0x7A0B,
                                               "(iSeries) Display MQ Listener",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A0C",new AMQMessageNumber(0x7A0C,
                                               "(iSeries) Delete MQ Listener",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A0D",new AMQMessageNumber(0x7A0D,
                                               "(iSeries) LSRNAME not allowed with PORT",
                                               "40 : Stop Error",
                                               "A listener object can not be specified with a port.",
                                               "Specify either a listener object or a port number."));
      amqMN_Ht.put("7A0E",new AMQMessageNumber(0x7A0E,
                                               "(iSeries) LSRNAME not allowed with IPADDR",
                                               "40 : Stop Error",
                                               "A listener object can not be specified with an IP address.",
                                               "Specify either a listener object or an IP address."));
      amqMN_Ht.put("7A0F",new AMQMessageNumber(0x7A0F,
                                               "(iSeries) Work with MQ Listener object",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A10",new AMQMessageNumber(0x7A10,
                                               "(iSeries) Create MQ Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A11",new AMQMessageNumber(0x7A11,
                                               "(iSeries) Change MQ Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A12",new AMQMessageNumber(0x7A12,
                                               "(iSeries) Copy MQ Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A13",new AMQMessageNumber(0x7A13,
                                               "(iSeries) Service name",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A14",new AMQMessageNumber(0x7A14,
                                               "(iSeries) Start program",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A15",new AMQMessageNumber(0x7A15,
                                               "(iSeries) Start program arguments",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A16",new AMQMessageNumber(0x7A16,
                                               "(iSeries) End program",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A17",new AMQMessageNumber(0x7A17,
                                               "(iSeries) End program arguments",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A18",new AMQMessageNumber(0x7A18,
                                               "(iSeries) Standard output",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A19",new AMQMessageNumber(0x7A19,
                                               "(iSeries) Standard error",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A1A",new AMQMessageNumber(0x7A1A,
                                               "(iSeries) Service type",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A1B",new AMQMessageNumber(0x7A1B,
                                               "(iSeries) Service control",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A1C",new AMQMessageNumber(0x7A1C,
                                               "(iSeries) From Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A1D",new AMQMessageNumber(0x7A1D,
                                               "(iSeries) To Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A1E",new AMQMessageNumber(0x7A1E,
                                               "(iSeries) Display MQ Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A20",new AMQMessageNumber(0x7A20,
                                               "(iSeries) Delete MQ Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A21",new AMQMessageNumber(0x7A21,
                                               "(iSeries) Work with MQ Service object",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A23",new AMQMessageNumber(0x7A23,
                                               "(iSeries) Start MQ Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A24",new AMQMessageNumber(0x7A24,
                                               "(iSeries) End MQ Service",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A25",new AMQMessageNumber(0x7A25,
                                               "(iSeries) Channel initiator control",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A26",new AMQMessageNumber(0x7A26,
                                               "(iSeries) Command server control",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A27",new AMQMessageNumber(0x7A27,
                                               "(iSeries) Display Queue Manager Status",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A28",new AMQMessageNumber(0x7A28,
                                               "(iSeries) Display Listener Status",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A29",new AMQMessageNumber(0x7A29,
                                               "(iSeries) Display Service Status",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A2A",new AMQMessageNumber(0x7A2A,
                                               "(iSeries) LSRNAME not allowed with OPTION",
                                               "40 : Stop Error",
                                               "A listener object can not be specified with an end option.",
                                               "Specify either a listener object or an end option."));
      amqMN_Ht.put("7A2B",new AMQMessageNumber(0x7A2B,
                                               "(iSeries) Service startup",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A2C",new AMQMessageNumber(0x7A2C,
                                               "(iSeries) Work with Connection Handles",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A2D",new AMQMessageNumber(0x7A2D,
                                               "(iSeries) Connection Identifier",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A2E",new AMQMessageNumber(0x7A2E,
                                               "(iSeries) End Queue Manager Connection",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A2F",new AMQMessageNumber(0x7A2F,
                                               "(iSeries) Work with MQ Connections",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A30",new AMQMessageNumber(0x7A30,
                                               "(iSeries) Header Compression",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A31",new AMQMessageNumber(0x7A31,
                                               "(iSeries) Message Compression",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A32",new AMQMessageNumber(0x7A32,
                                               "(iSeries) Message compression *ANY not valid for channel type.",
                                               "30 : Severe error",
                                               "The message compression value *ANY is only valid for *RCVR, *RQSTR and *SVRCN channel types.",
                                               "Specify a valid message compression list."));
      amqMN_Ht.put("7A33",new AMQMessageNumber(0x7A33,
                                               "(iSeries) Channel Monitoring",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A34",new AMQMessageNumber(0x7A34,
                                               "(iSeries) Channel Statistics",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A35",new AMQMessageNumber(0x7A35,
                                               "(iSeries) Cluster Workload Rank",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A36",new AMQMessageNumber(0x7A36,
                                               "(iSeries) Cluster Workload Priority",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A37",new AMQMessageNumber(0x7A37,
                                               "(iSeries) Cluster Channel Weight",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A38",new AMQMessageNumber(0x7A38,
                                               "(iSeries) Cluster workload channels",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A39",new AMQMessageNumber(0x7A39,
                                               "(iSeries) Cluster workload queue use",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A3A",new AMQMessageNumber(0x7A3A,
                                               "(iSeries) Queue Monitoring",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A3B",new AMQMessageNumber(0x7A3B,
                                               "(iSeries) Queue Manager Statistics",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A3C",new AMQMessageNumber(0x7A3C,
                                               "(iSeries) Cluster Sender Monitoring",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A3D",new AMQMessageNumber(0x7A3D,
                                               "(iSeries) Queue Statistics",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A3E",new AMQMessageNumber(0x7A3E,
                                               "(iSeries) Cluster Sender Statistics",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A3F",new AMQMessageNumber(0x7A3F,
                                               "(iSeries) Statistics Interval",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A40",new AMQMessageNumber(0x7A40,
                                               "(iSeries) Display MQ Route Information",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A41",new AMQMessageNumber(0x7A41,
                                               "(iSeries) Correlation Identifier",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A42",new AMQMessageNumber(0x7A42,
                                               "(iSeries) Message Persistence",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A43",new AMQMessageNumber(0x7A43,
                                               "(iSeries) Message Priority",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A44",new AMQMessageNumber(0x7A44,
                                               "(iSeries) Report Option",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A45",new AMQMessageNumber(0x7A45,
                                               "(iSeries) Reply Queue",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A46",new AMQMessageNumber(0x7A46,
                                               "(iSeries) Reply Queue Manager",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A47",new AMQMessageNumber(0x7A47,
                                               "(iSeries) Message Expiry",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A48",new AMQMessageNumber(0x7A48,
                                               "(iSeries) Expiry Report",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A49",new AMQMessageNumber(0x7A49,
                                               "(iSeries) Route Information",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A4A",new AMQMessageNumber(0x7A4A,
                                               "(iSeries) Reply Message",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A4B",new AMQMessageNumber(0x7A4B,
                                               "(iSeries) Deliver Message",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A4C",new AMQMessageNumber(0x7A4C,
                                               "(iSeries) Forward Message",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A4D",new AMQMessageNumber(0x7A4D,
                                               "(iSeries) Maximum Activities",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A4E",new AMQMessageNumber(0x7A4E,
                                               "(iSeries) Route Detail",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A4F",new AMQMessageNumber(0x7A4F,
                                               "(iSeries) Browse Only",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A50",new AMQMessageNumber(0x7A50,
                                               "(iSeries) Display Message",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A51",new AMQMessageNumber(0x7A51,
                                               "(iSeries) Target Queue Manager",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A52",new AMQMessageNumber(0x7A52,
                                               "(iSeries) Display Information",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A53",new AMQMessageNumber(0x7A53,
                                               "(iSeries) Wait Time",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A54",new AMQMessageNumber(0x7A54,
                                               "(iSeries) RTEINF(*YES) required for RPLYMSG(*YES).",
                                               "30 : Severe error",
                                               "RPLYMSG(*YES) can not be specified without RTEINF(*YES).",
                                               "If RPLYMSG(*YES) is specified then RTEINF(*YES) must also be specified."));
      amqMN_Ht.put("7A55",new AMQMessageNumber(0x7A55,
                                               "(iSeries) RPLYQ required for RPLYMQM.",
                                               "30 : Severe error",
                                               "RPLYMQM can not be specified without RPLYQ.",
                                               "If RPLYMQM is specified then RPLYQ must also be specified."));
      amqMN_Ht.put("7A56",new AMQMessageNumber(0x7A56,
                                               "(iSeries) CRRLID specified with invalid parameters.",
                                               "30 : Severe error",
                                               "The CRRLID parameter was specified with one or more of MSGPST, MSGPRTY, OPTION, RPLYQ, RPLYMQM, EXPIRY, EXPRPT, RTEINF RPLYMSG, DLVRMSG, FWDMSG, MAXACTS and DETAIL which are invalid with CRRLID.",
                                               "Specify only those parameters which are valid with CRRLID."));
      amqMN_Ht.put("7A57",new AMQMessageNumber(0x7A57,
                                               "(iSeries) DSPMSG(*NO) specified with invalid parameters.",
                                               "30 : Severe error",
                                               "DSPMSG(*NO) was specified with one or more of BROWSE, DSPINF and WAIT which are invalid with DSPMSG(*NO).",
                                               "Specify only those parameters which are valid with DSPMSG(*NO)."));
      amqMN_Ht.put("7A58",new AMQMessageNumber(0x7A58,
                                               "(iSeries) RPLYQ required for DSPMSG(*NO) and RPLYMSG(*YES).",
                                               "30 : Severe error",
                                               "DSPMSG(*NO) and RPLYMSG(*YES) can not be specified without RPLYQ.",
                                               "If DSPMSG(*NO) and RPLYMSG(*YES) are specified than RPLYQ must also be specified."));
      amqMN_Ht.put("7A59",new AMQMessageNumber(0x7A59,
                                               "(iSeries) RPLYQ required for DSPMSG(*NO) and OPTION not *NONE.",
                                               "30 : Severe error",
                                               "DSPMSG(*NO) and OPTION not *NONE can not be specified without RPLYQ.",
                                               "If DSPMSG(*NO) and OPTION not *NONE are specified than RPLYQ must also be specified."));
      amqMN_Ht.put("7A5A",new AMQMessageNumber(0x7A5A,
                                               "(iSeries) Run WebSphere MQ Commands",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A5B",new AMQMessageNumber(0x7A5B,
                                               "(iSeries) Non Persistent Message Class",
                                               "30 : Severe error",
                                               "",
                                               ""));
      amqMN_Ht.put("7A5C",new AMQMessageNumber(0x7A5C,
                                               "(iSeries) NPMCLASS not allowed with queue type *ALS or *RMT.",
                                               "40 : Stop Error",
                                               "The NPMCLASS parameter may not be specified for a queue of type *ALS or *RMT.",
                                               "Remove the NPMCLASS parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("7A5D",new AMQMessageNumber(0x7A5D,
                                               "(iSeries) MONQ not allowed with queue type *ALS or *RMT.",
                                               "40 : Stop Error",
                                               "The MONQ parameter may not be specified for a queue of type *ALS or *RMT.",
                                               "Remove the MONQ parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("7A5E",new AMQMessageNumber(0x7A5E,
                                               "(iSeries) STATQ not allowed with queue type *ALS or *RMT.",
                                               "40 : Stop Error",
                                               "The STATQ parameter may not be specified for a queue of type *ALS or *RMT.",
                                               "Remove the STATQ parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("7A5F",new AMQMessageNumber(0x7A5F,
                                               "(iSeries) ACCTQ not allowed with queue type *ALS or *RMT.",
                                               "40 : Stop Error",
                                               "The ACCTQ parameter may not be specified for a queue of type *ALS or *RMT.",
                                               "Remove the ACCTQ parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("7A60",new AMQMessageNumber(0x7A60,
                                               "(iSeries) All queue managers have been quiesced.",
                                               "0 : Information",
                                               "All queue managers have been successfully quiesced.",
                                               "None."));
      amqMN_Ht.put("7A61",new AMQMessageNumber(0x7A61,
                                               "(iSeries) MQMNAME not valid for TRCEARLY(*YES).",
                                               "40 : Stop Error",
                                               "The MQMNAME parameter may only be specified for TRCEARLY(*NO). TRCEARLY(*YES) applies to all queue managers.",
                                               "If TRCEARLY(*YES) is required remove MQMNAME from the command."));
      amqMN_Ht.put("7A62",new AMQMessageNumber(0x7A62,
                                               "(iSeries) MQMNAME not valid for SET(*END).",
                                               "40 : Stop Error",
                                               "The MQMNAME parameter may only be specified for SET(*ON) or SET(*OFF). SET(*END) applies to all queue managers.",
                                               "If SET(*END) is required remove MQMNAME from the command."));
      amqMN_Ht.put("7B00",new AMQMessageNumber(0x7B00,
                                               "(iSeries) MQI Accounting",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B01",new AMQMessageNumber(0x7B01,
                                               "(iSeries) Input file",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B02",new AMQMessageNumber(0x7B02,
                                               "(iSeries) Queue Accounting",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B03",new AMQMessageNumber(0x7B03,
                                               "(iSeries) Member containing input",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B04",new AMQMessageNumber(0x7B04,
                                               "(iSeries) Accounting Interval",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B05",new AMQMessageNumber(0x7B05,
                                               "(iSeries) Accounting Override",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B06",new AMQMessageNumber(0x7B06,
                                               "(iSeries) Trace data size",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B07",new AMQMessageNumber(0x7B07,
                                               "(iSeries) Perform replay only",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B08",new AMQMessageNumber(0x7B08,
                                               "(iSeries) Activate backup",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B09",new AMQMessageNumber(0x7B09,
                                               "(iSeries) No connection handles to display",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B0A",new AMQMessageNumber(0x7B0A,
                                               "(iSeries) Trace Route Recording",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B0B",new AMQMessageNumber(0x7B0B,
                                               "(iSeries) Activity Recording",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B0C",new AMQMessageNumber(0x7B0C,
                                               "(iSeries) No queue manager connections to display",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B0D",new AMQMessageNumber(0x7B0D,
                                               "(iSeries) No listener objects to display",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B0E",new AMQMessageNumber(0x7B0E,
                                               "(iSeries) No service objects to display",
                                               "40 : Stop Error",
                                               "",
                                               ""));
      amqMN_Ht.put("7B0F",new AMQMessageNumber(0x7B0F,
                                               "(iSeries) CLWLRANK not allowed with queue type *MDL.",
                                               "40 : Stop Error",
                                               "The CLWLRANK parameter may not be specified for a queue of type *MDL.",
                                               "Remove the CLWLRANK parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("7B10",new AMQMessageNumber(0x7B10,
                                               "(iSeries) CLWLPRTY not allowed with queue type *MDL.",
                                               "40 : Stop Error",
                                               "The CLWLPRTY parameter may not be specified for a queue of type *MDL.",
                                               "Remove the CLWLPRTY parameter from the command or, if the command is CRTMQMQ, specify a different value for QTYPE. Then try the command again."));
      amqMN_Ht.put("7B11",new AMQMessageNumber(0x7B11,
                                               "(iSeries) LSRNAME not allowed with BACKLOG",
                                               "40 : Stop Error",
                                               "A listener object can not be specified with a listener backlog.",
                                               "Specify either a listener object or a listener backlog."));
      amqMN_Ht.put("7B12",new AMQMessageNumber(0x7B12,
                                               "(iSeries) MONCHL not valid for channel type *CLTCN.",
                                               "40 : Stop Error",
                                               "The MONCHL parameter may not be specified with channel type *CLTCN.",
                                               "Remove the MONCHL parameter from the command or, if the command is CRTMQMCHL, specify a different value for CHLTYPE. Then try the command again."));
      amqMN_Ht.put("7B13",new AMQMessageNumber(0x7B13,
                                               "(iSeries) STATCHL not valid for channel type *CLTCN.",
                                               "40 : Stop Error",
                                               "The STATCHL parameter may not be specified with channel type *CLTCN.",
                                               "Remove the STATCHL parameter from the command or, if the command is CRTMQMCHL, specify a different value for CHLTYPE. Then try the command again."));
      amqMN_Ht.put("7B14",new AMQMessageNumber(0x7B14,
                                               "(iSeries) CLWLRANK only valid for channel types *CLUSSDR and *CLUSRCVR.",
                                               "40 : Stop Error",
                                               "The CLWLRANK parameter may only be specified with channel types *CLUSSDR or *CLUSRCVR.",
                                               "Remove the CLWLRANK parameter from the command or, if the command is CRTMQMCHL, specify a different value for CHLTYPE. Then try the command again."));
      amqMN_Ht.put("7B15",new AMQMessageNumber(0x7B15,
                                               "(iSeries) CLWLPRTY only valid for channel types *CLUSSDR and *CLUSRCVR.",
                                               "40 : Stop Error",
                                               "The CLWLPRTY parameter may only be specified with channel types *CLUSSDR or *CLUSRCVR.",
                                               "Remove the CLWLPRTY parameter from the command or, if the command is CRTMQMCHL, specify a different value for CHLTYPE. Then try the command again."));
      amqMN_Ht.put("7B16",new AMQMessageNumber(0x7B16,
                                               "(iSeries) CLWLWGHT only valid for channel types *CLUSSDR and *CLUSRCVR.",
                                               "40 : Stop Error",
                                               "The CLWLWGHT parameter may only be specified with channel types *CLUSSDR or *CLUSRCVR.",
                                               "Remove the CLWLWGHT parameter from the command or, if the command is CRTMQMCHL, specify a different value for CHLTYPE. Then try the command again."));
      amqMN_Ht.put("7B17",new AMQMessageNumber(0x7B17,
                                               "(iSeries) CLWLUSEQ only allowed with queue type *LCL.",
                                               "40 : Stop Error",
                                               "The CLWLUSEQ parameter may only be specified for a queue of type *LCL.",
                                               "Remove the CLWLUSEQ parameter from the command or, if the command is CRTMQMQ, specify a value of *LCL for QTYPE. Then try the command again."));
      amqMN_Ht.put("7B18",new AMQMessageNumber(0x7B18,
                                               "(iSeries) MCAUSRID not valid for channel type *CLTCN.",
                                               "40 : Stop Error",
                                               "The MCAUSRID parameter may not be specified with channel type *CLTCN.",
                                               "Remove the MCAUSRID parameter from the command or, if the command is CRTMQMCHL, specify a different value for CHLTYPE. Then try the command again."));
   }

   /**
    * Retrieve the AMQMessageNumber object.
    * @param index
    * @return AMQMessageNumber
    */
   public AMQMessageNumber getAMQMessageNumber(String index)
   {
      return (AMQMessageNumber)amqMN_Ht.get(index);
   }
}
